/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: HomeManagerImpl.java,v 1.1 2004/08/16 09:33:11 slobodan Exp $
 */

package com.lutris.airsent.business;

import com.lutris.airsent.business.messenger.*;
import com.lutris.airsent.business.delivery.*;
import com.lutris.airsent.business.customer.*;
import com.lutris.airsent.business.address.*;
import com.lutris.airsent.business.email.*;
import com.lutris.airsent.business.sms.*;
import com.lutris.airsent.spec.delivery.DeliveryManager;
import com.lutris.airsent.spec.messenger.MessengerManager;
import com.lutris.airsent.spec.sms.SMSManager;
import com.lutris.airsent.spec.customer.CustomerManager;
import com.lutris.airsent.spec.email.EmailManager;
import com.lutris.airsent.business.address.AddressManager;
import com.lutris.airsent.spec.AirSentException;
import com.lutris.airsent.spec.HomeManager;
import com.lutris.util.Config;

/**
 * 
 */
public class HomeManagerImpl implements HomeManager {

    private static HomeManagerImpl instance = null;
    private CustomerManagerImpl  customerManager = null;
    private DeliveryManagerImpl  deliveryManager = null;
    private MessengerManagerImpl messengerManager = null;
    private AddressManager   addressManager = null;
    private EmailManagerImpl     emailManager = null;
    private SMSManagerImpl       smsManager = null;
    
    /**
     *Creates an instance of HomeManager
     *
     */
    public static HomeManagerImpl getInstance() 
        throws AirSentBusinessException {

        if (instance == null) {
            throw new AirSentBusinessException("HomeManager: not initialized.");
       
        } 
        return instance;
    } 
    
    public HomeManager getInst() 
        throws AirSentBusinessException {

        if (instance == null) {
            throw new AirSentBusinessException("HomeManager: not initialized.");
        } 
        return instance;
    } 
    
    public HomeManagerImpl(){}

    /**
     *Creates an instance of HomeManager with Config
     *
     * 
     */
    private HomeManagerImpl(Config config) 
        throws AirSentBusinessException {

	try {
	    deliveryManager = new DeliveryManagerImpl();
	    customerManager = new CustomerManagerImpl();
	    messengerManager = new MessengerManagerImpl();
	    addressManager = new AddressManager();
	    emailManager = new EmailManagerImpl(config);
	    smsManager = new SMSManagerImpl(config);
	} catch (Exception e) {
	    throw new AirSentBusinessException("Exception creating factories", e);
	} 
    }


    /**
     *Initializes HomeManager with Config
     *@param Config config
     *
     */
    public synchronized void initialize(Config config) 
            throws AirSentBusinessException {
       
        if (instance != null) {
            throw new AirSentBusinessException("HomeManager: already initialized.");
        } 
        instance = new HomeManagerImpl(config);
    } 

    
    
    /**
     *Gets the DeliveryManager 
     *
     *@return the delivery Manager
     */
    public DeliveryManager getDeliveryManager() {
        return deliveryManager;
    } 

    /**
     *Gets the CustomerManager 
     *
     *@return the customer Manager
     */
    public CustomerManager getCustomerManager() {
        return customerManager;
    } 

    /**
     * Returns the MessengerManager
     * @return MessengerManager
     */
    public MessengerManager getMessengerManager() {
	return messengerManager;
    } 

    /**
     * Returns the AddressManager
     * @return AddressManager
     */
    public AddressManager getAddressManager() {
	return addressManager;
    } 

    /**
     * Returns the EmailManager
     * @return EmailManager
     */
    public EmailManager getEmailManager() {
	return emailManager;
    } 

     /**
     * Returns the EmailManager
     * @return EmailManager
     */
    public SMSManager getSMSManager() {
	return smsManager;
    } 

}







