/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: AddressImpl.java,v 1.1 2004/08/16 09:33:11 slobodan Exp $
 */

package com.lutris.airsent.business.address;

import com.lutris.airsent.business.AirSentBusinessException;
import com.lutris.airsent.data.person.AddressDO;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.DataObjectException;
import com.lutris.airsent.spec.address.Address;
/**
 * Class declaration
 * 
 * 
 * @author joseph shoop
 * @version %I%, %G%
 */
public class AddressImpl implements Address {
    protected AddressDO addressDO = null;

    /**
     * Creates an AddressImpl object.
     */
    public AddressImpl() throws AirSentBusinessException {
	try {
	    addressDO = AddressDO.createVirgin();
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Error creating Address");
	} 
    }

    /**
     * Creates an Address object.
     * @param AddressDO address
     */
    public AddressImpl(AddressDO address) throws AirSentBusinessException {
	this.addressDO = address;
    }


     /**
     * Creates an Address object with the given id
     * @param AddressDO address
     */
    public AddressImpl(String id) 
        throws AirSentBusinessException {
	try {
            addressDO = AddressDO.createExisting(id);
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Error creating Address");
	} 
    }




    /**
     * Gets the object handle.
     * @return String
     * @exception AirSentBusinessException
     */
    public String getHandle() throws AirSentBusinessException {
	try {
	    return addressDO.getHandle();
	} catch (DatabaseManagerException ex) {
	    throw new AirSentBusinessException("Error getting address's handle", ex);
	} 
    } 

    /**
     *Gets the address data object
     *@return data object
     */
    public AddressDO getDataObject() {
        return addressDO;
    }


    /**
     * Gets the Address name.
     * @return String
     * @exception AirSentBusinessException
     */
    public String getName() throws AirSentBusinessException {
	try {
	    return addressDO.getName();
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error getting address name", 
					       ex);
	} 
    } 

    /**
     * Sets the Address name.
     * @param name Address name
     * @exception AirSentBusinessException
     */
    public void setName(String name) throws AirSentBusinessException {
	try {
	    this.addressDO.setName(name);
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error setting name", ex);
	} 
    } 

    /**
     * Get street1.
     * 
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public String getStreet1() throws AirSentBusinessException {
	try {
	    return addressDO.getStreet1();
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error getting street", ex);
	} 
    } 

    /**
     * Sets street1.
     * 
     * 
     * @param street
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void setStreet1(String street) throws AirSentBusinessException {
	try {
	    this.addressDO.setStreet1(street);
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error setting street", ex);
	} 
    } 

    /**
     * Sets street2.
     * 
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public String getStreet2() throws AirSentBusinessException {
	try {
	    return addressDO.getStreet2();
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error getting street", ex);
	} 
    } 

    /**
     * Sets street2.
     * 
     * 
     * @param street
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void setStreet2(String street) throws AirSentBusinessException {
	try {
	    this.addressDO.setStreet2(street);
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error setting street", ex);
	} 
    } 

    /**
     * Gets street3.
     * 
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public String getStreet3() throws AirSentBusinessException {
	try {
	    return addressDO.getStreet3();
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error getting address street", 
					       ex);
	} 
    } 

    /**
     * Sets street3.
     * 
     * 
     * @param street
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void setStreet3(String street) throws AirSentBusinessException {
	try {
	    this.addressDO.setStreet3(street);
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error setting address street", 
					       ex);
	} 
    } 

    /**
     * Get the city.
     * 
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public String getCity() throws AirSentBusinessException {
	try {
	    return addressDO.getCity();
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error getting address city", 
					       ex);
	} 
    } 

    /**
     * Sets the city.
     * 
     * 
     * @param city
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void setCity(String city) throws AirSentBusinessException {
	try {
	    this.addressDO.setCity(city);
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error setting address street", 
					       ex);
	} 
    } 

    /**
     * Get the postal code.
     * 
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public String getPostalCode() throws AirSentBusinessException {
	try {
	    return addressDO.getPostalCode();
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error getting address postal", 
					       ex);
	} 
    } 

    /**
     * Set the postal code.
     * 
     * 
     * @param code
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void setPostalCode(String code) throws AirSentBusinessException {
	try {
	    this.addressDO.setPostalCode(code);
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error setting address postal", 
					       ex);
	} 
    } 

    /**
     * Gets the local number.
     * 
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public String getLocalNumber() throws AirSentBusinessException {
	try {
	    return addressDO.getLocalNumber();
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error getting address phone", 
					       ex);
	} 
    } 

    /**
     * Sets the local number.
     * 
     * 
     * @param number
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void setLocalNumber(String number) 
	    throws AirSentBusinessException {
	try {
	    this.addressDO.setLocalNumber(number);
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error setting address phone", 
					       ex);
	} 
    } 

    /**
     * Get the area code.
     * 
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public String getAreaCode() throws AirSentBusinessException {
	try {
	    return addressDO.getAreaCode();
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error getting address area code", 
					       ex);
	} 
    } 

    /**
     * Sets the area code.
     * 
     * 
     * @param area
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void setAreaCode(String area) throws AirSentBusinessException {
	try {
	    this.addressDO.setAreaCode(area);
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error setting address area code", 
					       ex);
	} 
    } 

    /**
     * gets the state.
     * 
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public String getState() throws AirSentBusinessException {
	try {
	    return addressDO.getState();
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error getting address state", 
					       ex);
	} 
    } 

    /**
     * Sets the state.
     * 
     * 
     * @param state
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void setState(String state) throws AirSentBusinessException {
	try {
	    this.addressDO.setState(state);
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error setting address state", 
					       ex);
	} 
    } 

    /**
     * Gets the country.
     * 
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public String getCountry() throws AirSentBusinessException {
	try {
	    return addressDO.getCountry();
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error getting address country", 
					       ex);
	} 
    } 

    /**
     * Sets the country.
     * 
     * 
     * @param country
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void setCountry(String country) throws AirSentBusinessException {
	try {
	    this.addressDO.setCountry(country);
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error setting address country", 
					       ex);
	} 
    } 

    /**
     * Gets the directions.
     * 
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public String getDirections() throws AirSentBusinessException {
	try {
	    return addressDO.getDirections();
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error getting address directions", 
					       ex);
	} 
    } 

    /**
     * Sets the directions.
     * 
     * 
     * @param directions
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void setDirections(String directions) 
	    throws AirSentBusinessException {
	try {
	    this.addressDO.setDirections(directions);
	} catch (DataObjectException ex) {
	    throw new AirSentBusinessException("Error setting address directions", 
					       ex);
	} 
    } 

    /**
     * Saves the address.
     * 
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void save() throws AirSentBusinessException {
	try {
	    this.addressDO.commit();
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Error saving address", ex);
	} 
    } 

    /**
     * Deletes the address.
     * 
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void delete() throws AirSentBusinessException {
	try {
	    this.addressDO.delete();
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Error deleting address", ex);
	} 
    } 

}

