/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: OrderFormImpl.java,v 1.1 2004/08/16 09:33:11 slobodan Exp $
 */
package com.lutris.airsent.business.delivery;

import com.lutris.airsent.spec.AirSentException;

import com.lutris.airsent.business.AirSentBusinessException;
import com.lutris.airsent.spec.address.Address;

import  com.lutris.airsent.spec.delivery.OrderForm;
import com.lutris.airsent.spec.delivery.Delivery;
/**
 * Container object holds data collected for a delivery
 */
public class OrderFormImpl implements OrderForm,java.io.Serializable {

    private String orderid;
    private String pickupname;
    private String pickupaddress;
    private String pickupphone;
    private String pickupdirections;
    private String dropoffname;
    private String dropoffaddress;
    private String dropoffphone;
    private String dropoffdirections;
    private boolean fragile;
    private boolean urgent;
    private String size;
    private String description;

    /**
     * Creates a blank object.
     */
    public OrderFormImpl() {
        // empty
    }

    /**
     * creates an object based on an exiting order
     *
     * @param deleivery existing order
     * @exception if an error occurs reading the deleivery
     */
    public OrderFormImpl(Delivery delivery) 
        throws AirSentException {
        if (delivery == null) {
            throw new AirSentBusinessException("Error delivery is null");
        }
        Address pua = delivery.getPickUp();
        setPickUpName(pua.getName());
        setPickUpAddress1(pua.getStreet1());
        setPickUpPhone(pua.getLocalNumber());
        setPickUpDirections(pua.getDirections());

        Address doa = delivery.getDropOff();
        setDropOffName(doa.getName());
        setDropOffAddress1(doa.getStreet1());
        setDropOffPhone(doa.getLocalNumber());
        setDropOffDirections(doa.getDirections());

        setOrderId(delivery.getHandle());
        if(delivery.isFragile()) {
            setFragile(true);
        }
        if(delivery.isUrgent()) {
            setUrgent(true);
        }
        setSize(delivery.getSize());
        setDescription(String.valueOf(delivery.getDescription()));
    }

    /**
     * returns the orderid
     * 
     * @return orderid
     */
    public String getOrderId() {
        return orderid;
    }

    /**
     * sets the orderid
     *
     * @param id orderid
     */
    public void setOrderId(String id) {
        orderid = id;
    }
    
    /**
     * returns the pick up name
     * 
     * @return pick up name
     */
    public String getPickUpName() {
        return pickupname;
    }

    /**
     * sets the pick up name
     *
     * @param name pick up name
     */
    public void setPickUpName(String name) 
        throws AirSentBusinessException {
        if (name.length() > Address.MAX_NAME) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
        pickupname = name;
    }

    /**
     * returns the pick up adderss
     * 
     * @return pick up adderss
     */
    public String getPickUpAddress1() {
        return pickupaddress;
    }

    /**
     * sets the pick up adderss
     * 
     * @param addr pick up adderss
     */
    public void setPickUpAddress1(String addr)
        throws AirSentBusinessException {
        if (addr.length() > Address.MAX_ADDRESS) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
        pickupaddress = addr;
    }

    /**
     * returns the pick up phone
     * 
     * @return pick up phone
     */
    public String getPickUpPhone() {
        return pickupphone;
    }
    
    /**
     * sets the pick up phone
     * 
     * @param phone pick up phone
     */
    public void setPickUpPhone(String phone) 
        throws AirSentBusinessException {
        if (phone.length() > Address.MAX_PHONE) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
        pickupphone = phone;
    }

    /**
     * returns the pick up directions
     * 
     * @return pick up directions
     */
    public String getPickUpDirections() {
        return pickupdirections;
    }

    /**
     * sets the pick up directions
     * 
     * @param dir pick up directions
     */
    public void setPickUpDirections(String dir)
        throws AirSentBusinessException {
        if (dir.length() > Address.MAX_DIRECTIONS) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
        pickupdirections = dir;
    }

    /**
     * returns the drop off name
     * 
     * @return drop off name
     */
    public String getDropOffName() {
        return dropoffname;
    }

    /**
     * sets the drop off name
     * 
     * @param name drop off name
     */
    public void setDropOffName(String name)
        throws AirSentBusinessException {
        if (name.length() > Address.MAX_NAME) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
        dropoffname = name;
    }

    /**
     * returns the drop off address
     * 
     * @return drop off address
     */
    public String getDropOffAddress1() {
        return dropoffaddress;
    }

    /**
     * sets the drop off address
     * 
     * @param addr drop off address
     */
    public void setDropOffAddress1(String addr)
        throws AirSentBusinessException {
        if (addr.length() > Address.MAX_ADDRESS) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
        dropoffaddress = addr;
    }

    /**
     * returns the 
     * 
     * @return drop off phone
     */
    public String getDropOffPhone() {
        return dropoffphone;
    }

    /**
     * sets the drop off phone
     * 
     * @param phone drop off phone
     */
    public void  setDropOffPhone(String phone)
        throws AirSentBusinessException {
        if (phone.length() > Address.MAX_PHONE) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
        dropoffphone = phone;
    }


    /**
     * returns the  drop off directions
     * 
     * @return  drop off directions
     */
    public String getDropOffDirections() {
        return dropoffdirections;
    }

    /**
     * sets the  drop off directions
     * 
     * @param dir  drop off directions
     */
    public void setDropOffDirections(String dir)
        throws AirSentBusinessException {
        if (dir.length() > Address.MAX_DIRECTIONS) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
        dropoffdirections = dir;
    }


    /**
     * returns the fragile
     * 
     * @return fragile state
     */
    public boolean getFragile() {
        return fragile;
    }


    /**
     * sets the fragile state
     * 
     * @param f fragile
     */
    public void setFragile(boolean f) {
        fragile = f;
    }

    /**
     * returns the usrgent state 
     * 
     * @return usrgent state 
     */
    public boolean getUrgent() {
        return urgent;
    }

    /**
     * sets the usrgent state 
     * 
     * @param u usrgent state 
     */
    public void setUrgent(boolean u) {
        urgent = u;
    }

    /**
     * returns the size 
     * 
     * @return size
     */
    public String getSize() {
        return size;
    }

    /**
     * sets the size
     * 
     * @param s size
     */
    public void  setSize(String s) {
        size = s;
    }

    /**
     * returns the descriptions
     * 
     * @return descriptions
     */
    public String getDescription() {
        return description;
    }

    /**
     * sets the descriptions
     * 
     * @param d descriptions
     */
    public void setDescription(String d)
        throws AirSentBusinessException {
        if (d.length() > Delivery.MAX_DESC) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
        description = d;
    }

    /*
     *
     */
    public void dump() {
        System.out.println("OrderFrom");
        System.out.println(" orderid: " + orderid);
        System.out.println(" pickupname: " + pickupname);
        System.out.println(" pickupaddress: " + pickupaddress);
        System.out.println(" pickupphone: " + pickupphone);
        System.out.println(" pickupdirections: " + pickupdirections);
        System.out.println(" dropoffname: " + dropoffname);
        System.out.println(" dropoffaddress: " + dropoffaddress);
        System.out.println(" dropoffphone: " + dropoffphone);
        System.out.println(" dropoffdirections: " + dropoffdirections);
        System.out.println(" fragile: " + fragile);
        System.out.println(" urgent: " + urgent);
        System.out.println(" size: " + size);
        System.out.println(" description: " + description);
        System.out.println("");
    }
}
