/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: UpdateImpl.java,v 1.2 2004/10/14 11:13:11 lola Exp $
 */

package com.lutris.airsent.business.delivery;

import com.lutris.airsent.spec.delivery.Update;
import com.lutris.airsent.business.AirSentBusinessException;

/**
 * Update handles the backend of the ServerPush technique
 * by keeping tabs on the delivery changes.
 * 
 * @author
 * @version %I%, %G%
 */
public class UpdateImpl implements Update {
    private long updateState;

    /**
     * Plain Constructor
     * 
     * 
     * 
     */
    public UpdateImpl() {}

    /**
     * Constructor with initial time state.
     * 
     * 
     * @param initialState
     * 
     * 
     */
    public UpdateImpl(long initialState) {
	updateState = initialState;
    }

    /**
     * Gets the update time.
     * 
     * 
     * @param browserState
     * @param wait
     * 
     * @return
     * 
     * 
     */
    public long needUpdate(long browserState, long wait) throws AirSentBusinessException {
	waitForChange(browserState, wait);

	return getUpdateState();
    } 

    /**
     * Sets the update state.
     * 
     * 
     * @param updateState
     * 
     * 
     */
    public void setUpdateState(long updateState) {
	this.updateState = updateState;
    } 

    /**
     * Gets the update time.
     * 
     * 
     * @return
     * 
     * 
     */
    public long getUpdateState() {
	return updateState;
    } 

    /**
     * Helper function. Only does the waiting. Waits till browserState
     * is not the current state, or for wait seconds, whichever comes
     * first. This function will return immediatly if no waiting is
     * needed. While waiting the thread is sleeping, not kept running
     * in a loop (which would waste processor time).
     * 
     * Written by Andy John
     */
    private void waitForChange(long browserState, long wait)
	throws AirSentBusinessException {
	
	if ((browserState != updateState) || (wait <= 0)) {
	    
	    // Already new data to report.
	    
	    return;
	} 

	long now = System.currentTimeMillis();
	
	long giveUpTime = now + (wait * 1000);
	
	// We loop here because the wait might return prematurely.
	while (browserState == updateState) {
	    long leftToGo = giveUpTime - now;

	    if (leftToGo <= 0) {

		// Either the wait time was 0 or we ran out of time.
	
		break;
	    } 

	    try {
		Thread.sleep(leftToGo);
	    } catch (InterruptedException e) {
		throw new AirSentBusinessException("Exception in waitForChange: " + e);
	    } 

	    now = System.currentTimeMillis();
	} 
    } 

}

