/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: Email.java,v 1.1 2004/08/16 09:33:11 slobodan Exp $
 */

package com.lutris.airsent.business.email;

import java.io.*;
import java.net.Socket;
import java.util.Vector;

/**
 * Utility class used to send email.
 * 
 * @author Jason Abbott <jason.abbott@lutris.com>
 * @author Joseph Shoop
 */
public class Email {
    private OutputStreamWriter out = null;
    private BufferedReader     in = null;
    private Socket	       mailserver = null;
    private InputStreamReader  isr = null;
    private String	       mailServerURL = "localhost";

    /**
     * Constructor with mail server name.
     * 
     * 
     * @param mailserverURL
     * 
     * 
     */
    public Email(String mailServer) throws IOException {
	this.mailServerURL = mailServer;
    }

    /**
     * Writes the email to output stream.
     * 
     * 
     * @param text
     * 
     * @return
     * 
     * @throws IOException
     * 
     * 
     */
    private String send(String text) throws IOException {
	
	out.write(text, 0, text.length());
	out.flush();
	
	// return in.readLine();
	String retVal = in.readLine();
	String dump = "x";
	while ((dump = in.readLine()) == null) {}
	if (retVal.startsWith("5")) {
	    
	    throw new IOException(mailServerURL + " SMTP Error: " + retVal);
	} 
	
	return retVal;
    } 
    
    /**
     * Opens a socket connection to the mail server.
     * 
     * 
     * @throws IOException
     * 
     * 
     */
    private void open() throws IOException {
	mailserver = new Socket(mailServerURL, 25);
	    isr = new InputStreamReader(mailserver.getInputStream());
	    out = new OutputStreamWriter(mailserver.getOutputStream());
	    in = new BufferedReader(isr);
    } 
    
    /**
     * Closes connection to the mail server.
     * 
     * 
     * @throws IOException
     * 
     * 
     */
    private void close() throws IOException {
	out.close();
	in.close();
    } 

    /**
     * Sends mail.
     * 
     * 
     * @param domain
     * @param fromAddress
     * @param toAddress
     * @param subject
     * @param type
     * @param text
     * 
     * @throws Exception
     * 
     * 
     */
    public void sendMail(String domain, String fromAddress, 
			    String toAddress, String subject, String type, 
			    String text) throws Exception {
	try {
	    open();
	} catch (Exception e) {
	    throw new Exception("Error opening socket" + e);
	} 

	// Lets check all of our parameters...
	// 1.fromAddress
	if (fromAddress == null) {
	    throw new Exception("We need a from-address that is not null to send email.");
	} 

	fromAddress = fromAddress.trim();

	if (fromAddress.length() == 0) {
	    throw new Exception("We need a from-address that is not empty to send email.");
	} 

	if (fromAddress.indexOf("@") == -1) {
	    throw new Exception("We need a legal from-address to send email, we do not consider \"" 
				+ fromAddress + "\" to be legal.");

	    // 2.To address
	} 

	if (toAddress == null) {
	    throw new Exception("We need to-addresses that are not null to send email.");
	} 

	toAddress = toAddress.trim();

	if (toAddress.length() == 0) {
	    throw new Exception("We need to-addresses that are not empty to send email.");
	} 

	if (toAddress.indexOf("@") == -1) {
	    throw new Exception("We need legal to-addresses to send email, we do not consider \"" 
				+ toAddress + "\" to be legal.");

	    // 3.subject
	} 

	if (subject == null) {
	    throw new Exception("We need a non-null subject to send email");
	} 

	subject = subject.trim();

	if (subject.length() == 0) {
	    throw new Exception("We need a non-empty subject to send email");
	} 

	if (type == null) {
	    type = "test/plain";
	} 

	// 4. Text body
	if (text == null) {
	    throw new Exception("We need a non-null text body to send email");
	} 

	text = text.trim();

	if (text.length() == 0) {
	    throw new Exception("We need a non-empty text body to send email");
	} 

	try {
	    send("EHLO " + domain + "\r\n");
	} catch (IOException e) {
	    throw new Exception("SMTP Mail Initialization failure");
	} 

	try {
	    send("MAIL FROM: " + fromAddress + "\r\n");
	    send("RCPT TO: " + toAddress + "\r\n");
	} catch (IOException e) {
	    throw new Exception("SMTP Mail addressing failure");
	} 

	try {
	    
	    //Multipart email
	    String boundary = "AIRSENTBOUNDARY-1";
	    StringBuffer tmp = new StringBuffer();
	    tmp.append("DATA\r\n");
	    tmp.append("MIME-Version: 1.0");
	    tmp.append("\r\n");
	    tmp.append("From: ");
	    tmp.append(fromAddress);
	    tmp.append("\r\n");
	    tmp.append("To: ");
	    tmp.append(toAddress);
	    tmp.append("\r\n");
	    tmp.append("Subject: ");
	    tmp.append(subject);
	    tmp.append("\r\n");
	    tmp.append("Content-Type: multipart/alternative; boundary=\"" + boundary + "\"");
	    tmp.append("\r\n");
	    tmp.append("--" + boundary);
	    tmp.append("\r\n");
	    tmp.append("Content-Type: text/plain; charset=us-ascii;");
	    tmp.append("\r\n");
	    tmp.append("\r\n");
	    tmp.append("AirSent delivery notification:");
	    tmp.append("\r\n");
	    tmp.append(text);
	    tmp.append("\r\n");
	    tmp.append("\r\n");
	    tmp.append("--" + boundary);
	    tmp.append("\r\n");
	    tmp.append("Content-Type: text/html; charset=us-ascii;");
	    tmp.append("\r\n");
	    tmp.append("\r\n");
	    tmp.append(text);
	    tmp.append("\r\n");
	    tmp.append("\r\n");
	    tmp.append("--" + boundary + "--");
	    tmp.append("\r\n" + ".\r\n");
	    //End multipart email
	    
	    send(tmp.toString());

	} catch (Exception e) {
	    throw new Exception("SMTP Mail message body failure");
	} 

	try {
	    send("QUIT\r\n");
	    close();
	} catch (IOException e) {
	    throw new Exception("SMTP Mail signoff failure");
	} 
    } 

}




