/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: EmailManagerImpl.java,v 1.1 2004/08/16 09:33:11 slobodan Exp $
 */

package com.lutris.airsent.business.email;

import com.lutris.util.Config;
import com.lutris.util.KeywordValueTable;
import com.lutris.airsent.business.AirSentBusinessException;
import com.lutris.airsent.spec.email.EmailManager;

/**
 * Class declaration
 * 
 * 
 * @author joseph shoop
 * @version %I%, %G%
 */
public class EmailManagerImpl implements EmailManager {
    protected final static String EMAIL_ACTIVE = "Active";
    protected final static String EMAIL_KEY = "Email";
    protected final static String MAIL_SERVER_NAME = "MailServer";
    protected final static String FROM_EMAIL_NAME = "AdminEmailName";
    protected final static String FROM_EMAIL_ADDRESS = "AdminEmailAddress";
    protected String		  mailServer = null;
    protected String		  fromName = null;
    protected String		  fromAddress = null;
    protected boolean             active = false;

    /**
     * Plain Constructor
     * 
     * 
     */
    public EmailManagerImpl() {}

    /**
     * Constructor using the Config object to set config variables.
     * 
     * 
     * @param config
     * 
     * 
     */
    public EmailManagerImpl(Config config) throws AirSentBusinessException {
	try {

	    // Get the email config.
	    KeywordValueTable section = config.getSection(EMAIL_KEY);

	    mailServer = section.getString(MAIL_SERVER_NAME);
	    fromName = section.getString(FROM_EMAIL_NAME);
	    fromAddress = section.getString(FROM_EMAIL_ADDRESS);
	    active = Boolean.valueOf(section.getString(EMAIL_ACTIVE)).booleanValue();
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Error sending mail:", ex);
	} 
    }

    
    /**
     * Sends email via smtp.
     * 
     * 
     * @param to
     * @param toAddress
     * @param subject
     * @param body
     * @param type
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public void send(String to, String toAddress, String subject, 
		     String body, 
		     String type) throws AirSentBusinessException {
	try {
	    if(active) {
		Email mail = new Email(mailServer);
		mail.sendMail(fromName, fromAddress, toAddress, subject, type, 
			      body);
	    } else {
		System.out.println("Email is not active. Edit airSent.conf to activate.");
	    }	    
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Error sending mail:", ex);
	} 
    } 

}

