/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: MessengerManagerImpl.java,v 1.1 2004/08/16 09:33:11 slobodan Exp $
 */

package com.lutris.airsent.business.messenger;

import com.lutris.airsent.business.AirSentBusinessException;

import com.lutris.airsent.data.person.MessengerDO;
import com.lutris.airsent.data.person.MessengerQuery;
import com.lutris.airsent.spec.messenger.Messenger;
import com.lutris.airsent.spec.messenger.MessengerManager;

/**
 * Messenger Manager implementation.
 * 
 * 
 * @author
 * @version %I%, %G%
 */
public class MessengerManagerImpl implements MessengerManager {

    /**
     * Creates an empty messenger 
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public Messenger create() throws AirSentBusinessException {
	try {
	    return new MessengerImpl();
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Error creating Messenger");
	} 
    } 


    public Messenger create(String id) throws AirSentBusinessException {
	try {
	    return new MessengerImpl(id);
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Error creating Messenger");
	} 
    } 


    /**
     * Finds a messenger by its badge.
     * 
     * 
     * @param badge
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public Messenger findByBadge(String badge) 
	    throws AirSentBusinessException {
	Messenger theMessenger = null;

	try {
	    MessengerQuery query = new MessengerQuery();

	    // Require that we only find ONE Messenger
	    query.setQueryBadge(badge);

	    MessengerDO[] messengerDO = query.getDOArray();

	    if (messengerDO.length == 0) {
		return null;
	    } else {
		return new MessengerImpl(messengerDO[0]);
	    } 
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Exception in findByInvoice", 
					       ex);
	} 
    } 

    /**
     * Find a messenger by its proximity to a geocode. NOT currently in
     * use.
     * 
     * 
     * @param geocode
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public Messenger findByProximity(String geocode) 
	    throws AirSentBusinessException {
	Messenger theMessenger = null;

	try {
	    MessengerQuery query = new MessengerQuery();

	    // Require that we only find ONE Messenger
	    query.requireUniqueInstance();
	    query.setQueryGeocode(geocode);

	    MessengerDO theMessengerDO = query.getNextDO();

	    theMessenger = new MessengerImpl(theMessengerDO);

	    return theMessenger;
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Exception in findByInvoice", 
					       ex);
	} 
    } 

    /**
     * Get all the messengers.
     * 
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public Messenger[] findAll() throws AirSentBusinessException {
	Messenger[] theMessengerArray = null;

	try {
	    MessengerQuery query = new MessengerQuery();

	    // To restrict the set of messengers returned
	    // to the that of the specific owner set in the query
	    MessengerDO[]  DOarray = query.getDOArray();

	    theMessengerArray = new Messenger[DOarray.length];

	    for (int i = 0; i < DOarray.length; i++) {
		theMessengerArray[i] = new MessengerImpl(DOarray[i]);
	    } 
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Exception in findByMessenger", 
					       ex);
	} 

	return theMessengerArray;
    } 

    /**
     * Validate a messenger login and password.
     * 
     * 
     * @param badge
     * @param password
     * 
     * @return
     * 
     * @throws AirSentBusinessException
     * 
     * 
     */
    public Messenger validatePassword(String badge, String password) 
	    throws AirSentBusinessException {
	Messenger theMessenger = null;

	try {
	    if ((theMessenger = findByBadge(badge)) == null) {
		return null;
	    } else {
		if (theMessenger.getPassword().equals(password)) {
		    return theMessenger;
		} else {
		    return null;
		} 
	    } 
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Exception in validateLogin", 
					       ex);
	} 
    } 

}

