/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: SMSManagerImpl.java,v 1.1 2004/08/16 09:33:11 slobodan Exp $
 */

package com.lutris.airsent.business.sms;

import com.lutris.util.Config;
import com.lutris.util.KeywordValueTable;
import com.lutris.airsent.business.AirSentBusinessException;
import com.lutris.airsent.spec.sms.SMSManager;

/**
 * Class declaration
 * 
 * 
 * @author
 * @version %I%, %G%
 */
public class SMSManagerImpl implements SMSManager {
    
    
    protected final static String SMS_ACTIVE = "Active";
    protected final static String SMS_KEY = "SMS";
    protected final static String SMS_HOSTNAME = "Hostname";
    protected final static String SMS_PORT = "Port";
    protected final static String SMS_SYSID = "Sysid";
    protected final static String SMS_SYSTYPE = "Systype";
    protected final static String SMS_SYSPASSWORD = "Password";

    protected String		  smsServer = null;
    protected String              smsPort = null;
    protected String		  smsSysid = null;
    protected String		  smsSystype = null;
    protected String		  smsSyspassword = null;
    protected boolean             active = false;

    /**
     * Plain Constructor
     * 
     * 
     * @see
     */
    public SMSManagerImpl() {}

    /**
     * Constructor using the Config object to set config variables.
     * 
     * 
     * @param config
     * 
     * @see
     */
    public SMSManagerImpl(Config config) throws AirSentBusinessException {
	try {
	    // Get the email config.
	    KeywordValueTable section = config.getSection(SMS_KEY);
	    smsServer = section.getString(SMS_HOSTNAME);
	    smsPort = section.getString(SMS_PORT);
	    smsSysid = section.getString(SMS_SYSID);
	    smsSystype = section.getString(SMS_SYSTYPE);
	    smsSyspassword = section.getString(SMS_SYSPASSWORD);
	    active = Boolean.valueOf(section.getString(SMS_ACTIVE)).booleanValue();
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Error sending mail:", ex);
	} 
    }
    
    
    /**
     * Sends sms message.
     * 
     * 
     * @param to
     * @param toAddress
     * @param subject
     * @param body
     * @param type
     * 
     * @throws AirSentBusinessException
     * 
     * @see
     */
    public void send(String id, String body) throws AirSentBusinessException {
	try {
	    if(active) {
		SMS sms = new SMS(smsServer, smsPort, smsSysid, 
					  smsSystype, smsSyspassword);
		sms.sendMessage(id, body);
	    } else {
		System.out.println("SMS is not active. Edit airSent.conf to activate.");
	    }	    
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Error sending SMS:", ex);
	} 
    } 

}

