/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: AirSent.java,v 1.1 2004/08/16 09:33:16 slobodan Exp $
 */

package com.lutris.airsent;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
import com.lutris.airsent.spec.*;

/**
 * The application object.
 * 
 * Application-wide data would go here.
 */
public class AirSent extends StandardApplication {
    
    protected boolean useServerPush = false;

    /*
     * A few methods you might want to add to.
     * See StandardApplication for more details.
     */

    /**
     * Start the application.
     * 
     * 
     * @param appConfig
     * 
     * @throws ApplicationException
     * 
     * @see
     */
    public void startup(Config appConfig) 
        throws ApplicationException {

	super.startup(appConfig);

      try {
        
         
	  HomeManager homeManager = HomeManagerFactory.getHomeManager("com.lutris.airsent.business.HomeManagerImpl");
         
          homeManager.initialize(appConfig);
          
      
          setAirSentConfig(appConfig);
	
	} catch (Exception ex) {
	     	} 
    } 

    /**
     * Gets the preprocessor.
     * 
     * 
     * @param comms
     * 
     * @return
     * 
     * @throws Exception
     * 
     * @see
     */
    public boolean requestPreprocessor(HttpPresentationComms comms) 
        throws Exception {

	return super.requestPreprocessor(comms);
    } 

    /**
     * Gets the home Manager which controlls 
     * access to all business objects.
     *
     */
    public HomeManager getHomeManager() 
        throws AirSentException {

      HomeManager homeManager = HomeManagerFactory.getHomeManager("com.lutris.airsent.business.HomeManagerImpl");
      return  homeManager.getInst();

    
    } 

    /**
     * This is an optional function, used only by the Multiserver's graphical
     * administration. This bit of HTML appears in the status page for this
     * application. You could add extra status info, for example
     * a list of currently logged in users.
     * 
     * @return HTML that is displayed in the status page of the Multiserver.
     */
    public String toHtml() {
	return "This is <I>AirSent</I>";
    } 

    /**
     *Sets AirSent Configuration parameters
     *
     *
     *
     */
    private void setAirSentConfig(Config config) throws AirSentException{
	
	try {
	    // Get the email config.
	    KeywordValueTable section = config.getSection("AirSentConfig");
	    useServerPush = Boolean.getBoolean(section.getString("ServerPush"));
	
        } catch (Exception ex) {
	    throw new AirSentException("Error configuring AirSent:", ex);
	} 
    }


    
    /**
     * Returns boolean based on if the app
     * should use the server push technique
     * for instant updates to screens.
     *
     * @param weather or not to use ServerPush.
     */
    public boolean useServerPush() {

	return useServerPush;
    }
    
}

