/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: AirSentSessionData.java,v 1.1 2004/08/16 09:33:16 slobodan Exp $
 */

package com.lutris.airsent.presentation;

import com.lutris.airsent.spec.customer.*;
import com.lutris.airsent.spec.messenger.Messenger;
import com.lutris.airsent.spec.delivery.OrderForm;

import java.util.*;

/**
 * Object that will be held in session data.  This should
 * be the only object held there.  Methods should be called
 * on this object to set and get data.
 */
public class AirSentSessionData implements java.io.Serializable {

    /**
     * Hash key to save session data for the CoreTest app in the Session
     */
    public static final String SESSION_KEY = "AirSentSessionData";
    private int	AUTHORIZATION = AirSentConstants.UNAUTH_USER;
    protected String userMessage = null;
    private Messenger messenger = null;
    private Customer customer = null;
    private OrderForm orderForm = null;
    private long browserState = System.currentTimeMillis();

    /**
     * Gets the user authentication code.
     * 
     * 
     * @return
     * 
     */
    public int getUserAuth() {
	return AUTHORIZATION;
    } 

    /**
     * Sets the user authentication code.
     * 
     * 
     * @param auth
     * 
     * 
     */
    public void setUserAuth(int auth) {
	this.AUTHORIZATION = auth;
    } 

    /**
     * Set the messenger in the session data.
     * 
     * 
     * @param messenger
     * 
     * 
     */
    public void setMessenger(Messenger messenger) {
	this.messenger = messenger;
    } 

    /**
     * Set the customer in the session data.
     * 
     * 
     * @param customer
     * 
     * 
     */
    public void setCustomer(Customer customer) {
	this.customer = customer;
    } 

    /**
     * Gets the messenger user
     * 
     */
    public Messenger getMessenger() {
	return messenger;
    } 

    /**
     * Gets the customer user
     * 
     */
    public Customer getCustomer() {
	return customer;
    } 

    /**
     * Sets the user message
     * 
     * @param msg the message to be set
     */
    public void setUserMessage(String msg) {
	this.userMessage = msg;
    } 

    /**
     * Retrieve the most recent user message and then clear it so no
     * other app tries to use it.
     */
    public String getAndClearUserMessage() {
	String msg = this.userMessage;
	
	this.userMessage = null;

	return msg;
    } 

    /**
     * Sets the browser state for 
     * server push functionality.
     * 
     * @param browserState
     * 
     * 
     */
    public void setBrowserState(long browserState) {
	this.browserState = browserState;
    } 

    /**
     * Gets the browser state for 
     * server push functionality.
     * 
     * 
     * @return
     * 
     * 
     */
    public long getBrowserState() {
	return browserState;
    } 

    public void setOrderForm(OrderForm d) {
        orderForm = d;
    }

    public OrderForm getOrderForm() {
        return orderForm;
    }
}

