/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: DeviceBasePO.java,v 1.1 2004/08/16 09:33:16 slobodan Exp $
 */

package com.lutris.airsent.presentation;

import com.lutris.airsent.presentation.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*; 
import com.lutris.appserver.server.Enhydra;
//import com.lutris.xml.xmlc.*;
import com.lutris.logging.*;
import com.lutris.util.KeywordValueException;
import com.lutris.appserver.server.user.User;
import org.w3c.dom.*;
import org.enhydra.wireless.wml.dom.WMLGoElement;
import org.enhydra.xml.xmlc.XMLObject;
import java.lang.reflect.*;
import java.util.*;
import java.lang.Throwable;

/**
 * This is the parent Presentaion object.  All presentation objects
 * should extend this class.
 * 
 * The run method looks for an event parameter and then calls
 * handle<EventName>.  If the "event" Parameter is not defined then
 * the handleDefault() method is called in your child class.
 * 
 * @author
 * @version
 */
public abstract class DeviceBasePO extends BasePO {

    /**
     * Handles default events.
     * 
     * 
     * @return
     * 
     * @throws HttpPresentationException
     * 
     */
    public abstract XMLObject handleDefault() 
	    throws HttpPresentationException;

    /**
     * This method should be implemented in the subclass so that it returns
     * the authorization level necessary to access the po.
     */
    abstract protected int getRequiredAuthLevel();

    /**
     * This rerouteForContent overrides the parent method to avoid
     * a continous loop senario of rerouting.
     */
    protected void rerouteForContent(HttpPresentationComms comms) {}

    ;

    /*
     * Uses the xmlfactory and the request content
     * type to determine the proper page type
     * to generate.
     * For example if the request type is wml and
     * the PO is MessengerLogin, then
     * using the
     * 
     * XMLObject page =
     * myComms.xmlcFactory.create(Class.forName("MessengerLogin" + "WML"));
     * 
     * @param String poName
     * @return XMLObject
     * 
     * @throws AirSentPresentationException
     */
    public XMLObject create(String poName) 
	    throws AirSentPresentationException {
	XMLObject page = null;

	try {
	    page = 
		getComms().xmlcFactory.create(DeviceUtils.getPageName(getComms(), 
		    poName));
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Trouble getting current authorization level", 
						   ex);
	} 
	return page;
    } 


    /**
     * This logs out a user from the session.  it first grabs
     * an instance of the sessionData associated with the request
     * and then sets the user to null in the session data.
     * 
     * @param HttpServletRequest request - this is the request that
     * the session data is extracted from.
     * 
     * @exception AirSentPresentationException when an error occurs or if
     * the session data for the request is null.
     */
    public XMLObject handleLogout() throws AirSentPresentationException {
	try {

	    mySessionData = null;
            SessionManager sessionManager = myComms.session.getSessionManager();
            sessionManager.deleteSession(myComms.session);
	    	    
	    throw new ClientPageRedirectException(AirSentConstants.XML_PAGE);
	} catch (Exception e) {
	    throw new AirSentPresentationException("Trouble logging out user", 
						   e);
	} 
    } 
}

