/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: DeviceUtils.java,v 1.2 2005/01/21 13:19:09 slobodan Exp $
 */

package com.lutris.airsent.presentation;

import com.lutris.airsent.presentation.*;
import com.lutris.appserver.server.httpPresentation.*;
import javax.servlet.http.HttpServletRequest;
import org.enhydra.xml.xmlc.*;
import org.w3c.dom.*;
/**
 *  Provides utilities for outputing and forewarding to devices. 
 * 
 * 
 * 
 */
public class DeviceUtils {
    /**
     * Method that determines which content to output based on
     * the accept type.
     * Current types:
     * text/html
     * text/xml
     * text/vnd.wap.wml
     * 
     * 
     */
    public static void rerouteForContent(HttpPresentationComms comms) 
	    throws AirSentPresentationException {
	String header = null;
	String userAgent = null;
	String flashClient = null;
	try {
	    flashClient =  comms.request.getParameter("flash"); 
	    userAgent = comms.request.getHeader("User-Agent"); 
	    if (flashClient != null) {
		comms.response.setEncoding("ISO8859_1");
		throw new ClientPageRedirectException(AirSentConstants.XML_PAGE);
	    } 
	    else if(userAgent != null && userAgent.indexOf("DoCoMo") != -1) {
		throw new ClientPageRedirectException(AirSentConstants.CHTML_PAGE);
	    } 
	    else if(userAgent != null && userAgent.indexOf("Pixo") != -1) {
		throw new ClientPageRedirectException(AirSentConstants.CHTML_PAGE);
	    }
	    else if(userAgent != null && userAgent.indexOf("RIM") != -1) {
		throw new ClientPageRedirectException(AirSentConstants.CHTML_PAGE);
	    }
	    else if(userAgent != null && userAgent.indexOf("Digital Paths") != -1) {
		throw new ClientPageRedirectException(AirSentConstants.CHTML_PAGE);
	    }
	    else if(userAgent != null && userAgent.indexOf("UP.") != -1) {
		throw new ClientPageRedirectException(AirSentConstants.WAP_PAGE);
	    } 
	    else if ((header = comms.request.getHeader(AirSentConstants.ACCEPT)) 
		     == null) {
		// Just display HTML
		return;
	    }else if (header != null && header.indexOf(AirSentConstants.HTML_CONTENT) != -1 ||
			     flashClient != null) {
			return;
	    } 
	    else if (header != null && header.indexOf(AirSentConstants.XML_CONTENT) != -1 ||
		     flashClient != null) {
		throw new ClientPageRedirectException(AirSentConstants.XML_PAGE);
	    } 
	    else if ( header != null && header.indexOf(AirSentConstants.WAP_CONTENT) != -1) {
		throw new ClientPageRedirectException(AirSentConstants.WAP_PAGE);
	    } 
	    else if (header != null && header.indexOf(AirSentConstants.XHTML_CONTENT) 
		     != -1) {}
	    else if (header != null && header.indexOf(AirSentConstants.CHTML_CONTENT) != -1) {
		//????
	    } 
	    else {
		if(userAgent == null) {
		    throw new ClientPageRedirectException(AirSentConstants.CHTML_PAGE);
		}
	    }
	} catch (Exception e) {
	    throw new AirSentPresentationException("Trouble rerouting header:" 
						   + header, e);
	} 
    } 
    
    /**
     * Gets the suffix of a PO name based on the
     * accept type.
     * @param String PO classname
     * @return PO new full classname with suffix.
     */
    public static String getPageName(HttpPresentationComms comms, String poName) 
	throws AirSentPresentationException {
	String header = null;
	String userAgent = null;
	String flashClient = null;
	
	try {
	    flashClient =  comms.request.getParameter("flash"); 
	    userAgent = comms.request.getHeader("User-Agent"); 
	    if (flashClient != null) {
		comms.response.setEncoding("ISO-8859-1");
		return poName + "XML";
	    } 
	    //
	    //DoCoMo code thanks to Ray Mercer
	    //
	    else if(userAgent != null && userAgent.indexOf("DoCoMo") != -1) {
		return poName + "CHTML";
	    }   else if(userAgent != null && userAgent.indexOf("Pixo") != -1) {
		return poName + "CHTML";
	    } else if(userAgent != null && userAgent.indexOf("RIM") != -1) {
		return poName + "HTML";
	    } else if(userAgent != null && userAgent.indexOf("UP.") != -1) {
		return poName + "WML";
	    } else if ((header = comms.request.getHeader(AirSentConstants.ACCEPT)) 
		       == null) {
		// Can't decipher accept type.
		return null;
	    }else if (header.indexOf(AirSentConstants.HTML_CONTENT) != -1) {
	    	//Defaulting to html.
		    return poName + "HTML";
	    } else if (header.indexOf(AirSentConstants.XML_CONTENT) != -1) {
		return poName + "XML";
	    } else if (header.indexOf(AirSentConstants.WAP_CONTENT) != -1) {
		return poName + "WML";
	    } else if (header.indexOf(AirSentConstants.XHTML_CONTENT) != -1) {
		return poName + "XHTML";
	    } else if (header.indexOf(AirSentConstants.CHTML_CONTENT) != -1) {
		return poName + "CHTML";
	    }
	    else {
		if(userAgent == null) {
		    //Defaulting to chtml.
		    return poName + "CHTML";
		}
		else {
		    //Defaulting to html.
		    return poName + "HTML";
		}
	    } 
	} catch (Exception e) {
	    throw new AirSentPresentationException("Trouble rerouting header:" 
						   + header, e);
	} 
    } 
    
    /**
     *Recursivly walk the dom and replace any url with the
     *url + a timestamp=current time in millis.
      **!!Hack for buggy phone caching!!
      */
    public static void setURLTimeStamp(org.enhydra.xml.xmlc.XMLObject dom) {
	    DeviceUtils.setNodeTimeStamp(dom.getDocumentElement());
    }

    
    /**
     *Recursivly walk the DOM replacing urls.  
     **!!Hack for buggy phone caching!!
     */
    public static void setNodeTimeStamp(Node node) {
    	if (Node.ELEMENT_NODE == node.getNodeType()){
    	    String urlValue = null;
    	    if(!(urlValue = ((Element)node).getAttribute("href")).equals("")) {
    		((Element)node).setAttribute("href", DeviceUtils.getTimeStampParam(urlValue));
    	    }
    	    NodeList nodeList = node.getChildNodes();
    	    for(int curNode = 0; curNode < nodeList.getLength(); curNode++){
    		DeviceUtils.setNodeTimeStamp(nodeList.item(curNode));
    	    }
    	}
    }
    
    /**
     **Append timestamp to force caching
     **!!Hack for buggy phone caching!!
     **
     */
    public static String getTimeStampParam(String url) {
    	String delimiter = url.indexOf("?") != -1 ? "&" : "?";
    	return url + delimiter + "timestamp=" + System.currentTimeMillis();
    }

    
    


}















