/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: AdminDetails.java,v 1.1 2004/08/16 09:33:16 slobodan Exp $
 */

package com.lutris.airsent.presentation.admin;

import com.lutris.airsent.presentation.BasePO;
import org.enhydra.xml.xmlc.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;


import com.lutris.airsent.spec.AirSentException;

import com.lutris.airsent.presentation.AirSentPresentationException;
import com.lutris.airsent.presentation.AirSentConstants;
import java.util.*;

import com.lutris.airsent.spec.delivery.Delivery;
import com.lutris.airsent.spec.messenger.Messenger;
import com.lutris.airsent.spec.*;
/**
 * Admin Details.java provides Delivery details to the admin,
 * as well as allowing the Admin to assign a messenger.
 * 
 */
public class AdminDetails extends BasePO {

    /**
     * Constants
     */
    private static final int AUTH_LEVEL = AirSentConstants.ADMINISTRATOR_USER;

    /**
     * Superclass method override.
     * returns the authorization level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * Default event. Just show the page.
     */
    public XMLObject handleDefault() throws HttpPresentationException {
	return showPage(null);
    } 

    /**
     * 
     * @param error messages
     * @return XMLObject page.
     */
    public XMLObject showPage(String errorMsg) 
        throws HttpPresentationException {
	AdminDetailsHTML  page = (AdminDetailsHTML) myComms.xmlcFactory.create(AdminDetailsHTML.class);
	HTMLInputElement  deliveryField = page.getElementDelivery();

	try {
	    String deliveryID = getComms().request.getParameter(AirSentConstants.DELIVERYID);
 
            HomeManager hf = getApplication().getHomeManager();
	    Delivery delivery = hf.getDeliveryManager().findByHandle(deliveryID);
	    
	    deliveryField.setValue(deliveryID);
	    if (null != errorMsg || null != (errorMsg = getSessionData().getAndClearUserMessage())) {
		page.setTextErrorText(errorMsg);
	    } else {
		page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
	    } 
            
           
            
	    page.setTextPickupname(delivery.getPickUp().getName());
	    page.setTextPickupaddress(delivery.getPickUp().getStreet1());
	    page.setTextDropoffname(delivery.getDropOff().getName());
	    page.setTextDropoffaddress(delivery.getDropOff().getStreet1());

            Messenger messenger = delivery.getMessenger();
            String messengerName = AirSentConstants.NOT_ASSIGNED;
            
            HTMLSelectElement messengerSelect = page.getElementMessengerSelect();
            HTMLOptionElement messengerOption = page.getElementMessengerOption();
            messengerOption.removeAttribute("id");
            messengerOption.removeChild(messengerOption.getFirstChild());
            
	    Messenger[] messengerList = hf.getMessengerManager().findAll();
            int selected = 0;
	    for (int i = 0; i < messengerList.length; i++) {
                HTMLOptionElement clonedOption = (HTMLOptionElement) messengerOption.cloneNode(true);
                String badge = messengerList[i].getBadge();
                String name  = messengerList[i].getFirstName();
 		if (messenger != null) {
                    if (badge.equals(messenger.getBadge()) == true) {
                        messengerName = name;                    
                        selected = i + 1;
                    }
                }
		clonedOption.setValue(badge);
		Node optionTextNode = clonedOption.getOwnerDocument().createTextNode(name);
		clonedOption.appendChild(optionTextNode);
		messengerSelect.appendChild(clonedOption);
	    }
            if (messenger == null) {
                HTMLOptionElement clonedOption = (HTMLOptionElement) messengerOption.cloneNode(true);
                clonedOption.setValue("none");
		Node optionTextNode = clonedOption.getOwnerDocument().createTextNode(AirSentConstants.NOT_ASSIGNED);
		clonedOption.appendChild(optionTextNode);
		messengerSelect.appendChild(clonedOption);
                selected = messengerList.length + 1;
            }
            messengerSelect.setSelectedIndex(selected);
            messengerOption.getParentNode().removeChild(messengerOption);
            page.setTextDropoffmessenger(messengerName);
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run AirSent_pres ) 
 * We need  to allow AirSent_pres to be functional ,  response
 * will be default HTML page with message
 *	
 */
           
        }catch(NullPointerException e){
                page.setTextErrorText("You cannot assign or change the messenger assigned to a given order while running AirSent_pres");
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Error adding messenger", ex);
	} 
    return page;
    
    } 

    /**
     * Process the handleEdit event.
     * 
     * @return page XMLObject
     */
    public XMLObject handleEdit() throws HttpPresentationException {
	String messengerID = getComms().request.getParameter(AirSentConstants.MESSENGER_NAME);
	String deliveryID = getComms().request.getParameter(AirSentConstants.DELIVERYID);
	Messenger messenger = null;
	Delivery  delivery = null;

	if ((messengerID != null) && (messengerID.equals("none") == true)) {
	    return showPage("Select a messenger");
	} 
        
	try {
	    if ((delivery = getApplication().getHomeManager().getDeliveryManager().findByHandle(deliveryID)) != null) {
		if ((messenger = getApplication().getHomeManager().getMessengerManager().findByBadge(messengerID)) != null) {
		    sendMessage(delivery, messenger);
		    delivery.setMessenger(messenger);
		    delivery.save();
		    
		    throw new ClientPageRedirectException(AirSentConstants.ADMIN_MAIN_PAGE);
		} 
	    } 
	}catch(NullPointerException e){return showPage("You cannot assign or change the messenger assigned to a given order , while running AirSent_pres");
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Error adding messenger", 
						   ex);
	} 

	return showPage("Error occured updating delivery.");
    } 


    
    /**
     *Informs the given messenger via sms that they have 
     *been assigned a delivery. If the delivery already
     *had a messenger and is being reassiged, send a 
     *message to both messengers.
     */
    private void sendMessage(Delivery delivery, Messenger messenger) 
	throws AirSentPresentationException {
	
	Messenger previousMessenger = null;
	String changeMessage = "You have been un-assigned delivery: ";
	String newMessage = "You have been assigned delivery: ";
	
	try {
	
	    if((previousMessenger = delivery.getMessenger()) != null) {
		getApplication().getHomeManager().getSMSManager().send(changeMessage + delivery.getHandle(), 
								       previousMessenger.getAddress().getLocalNumber());
		getApplication().getHomeManager().getSMSManager().send(newMessage + delivery.getHandle(), 
								       messenger.getAddress().getLocalNumber());
	    } else {
		getApplication().getHomeManager().getSMSManager().send(newMessage + delivery.getHandle(), messenger.getAddress().getLocalNumber());
	    }
	    
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Error adding messenger", 
						   ex);
	} 
    }

    
    /**
     * this takes an html element, finds the first text child node.
     * if this doesn't exist it creates a new text node.  it then
     * sets the text in the text node.
     *
     * @param Element e - the Parent element that has the text node child
     * @param String text - the text to set in the text node.
     */
    public static void setText(Element e, String text) {
        if (null == text) {
            text = "";
        }
	
        org.w3c.dom.Text t = null;
        try {
	    t = XMLCUtil.getFirstText(e);
        } catch (Throwable th) {
        }
        if (t != null)
	    t.setData(text);
        else {
	    t = e.getOwnerDocument().createTextNode(text);
	    e.insertBefore(t, null);
        }
    }


}






