/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights 
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for 
 * use only by licensed users of product(s) that include this source 
 * file. Use of this source file or the software that uses it is covered 
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF 
 * ANY KIND, either express or implied. See the License for the specific terms 
 * governing rights and limitations under the License.
 * 
 */

package com.lutris.airsent.presentation.admin;

import org.enhydra.xml.xmlc.XMLObject;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import java.util.*;
import com.lutris.airsent.presentation.BasePO;
import com.lutris.airsent.spec.AirSentException;

import com.lutris.airsent.presentation.AirSentPresentationException;
import com.lutris.airsent.presentation.AirSentConstants;
import com.lutris.airsent.spec.delivery.Delivery;
import com.lutris.airsent.spec.messenger.Messenger;
import com.lutris.airsent.spec.address.*;
/**
 * AdminMain is the main interactive page for the AirSent
 * administrator. AdminMain shows current(and past) delivies
 * allows for Detailed viewing of Deliveries and Messengers.
 * 
 */
public class AdminMain extends BasePO {

    /**
     * Constants
     */
    private static final int  AUTH_LEVEL = AirSentConstants.ADMINISTRATOR_USER;
    private static final int  TOP_TEN_DELIVERIES = 10;
    private static final int  TOP_TWENTY_DELIVERIES = 20;
    private static final int  ALL_DELIVERIES = 0;
    private static final long TIME_OUT = 60;

    /**
     * Superclass method override.
     * returns the authorization level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * Default event. Just show the page.
     */
    public XMLObject handleDefault() throws HttpPresentationException { 
	return showPage(null);
    } 

    /**
     * 
     * @param error messages
     * @return page.
     */
    public XMLObject showPage(String errorMsg) 
	    throws AirSentPresentationException {
	  // Build the page, checking for errors first.
	    AdminMainHTML page = 
		(AdminMainHTML) myComms.xmlcFactory.create(AdminMainHTML.class);
	
	try {
	   
	  
	    
	    if (null != errorMsg 
		|| null 
		!= (errorMsg = 
		    getSessionData().getAndClearUserMessage())) {
		page.setTextErrorText(errorMsg);
	    } else {
		page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
	    } 

	    /**
	     * If the ServerPush tecnique is specified in the
	     * conf file, then server push will be enabled. Essentially
	     * server push is a page that is continously loading, or  until
	     * rather, waiting to load until the page has a)changed, or b)
	     * the TCP/IP timeout of 60 seconds has expired.
	     */
	    
	    if(getApplication().useServerPush()) {
		if (getComms().request.getParameter(AirSentConstants.REFRESH) 
		    != null) {
		    
		    // 
		    // Do the update or wait
		    // 
		    getSessionData().setBrowserState(getApplication().getHomeManager().getDeliveryManager().getUpdate(getSessionData().getBrowserState(), 
														      TIME_OUT));
		} else {
		    getSessionData().setBrowserState(getApplication().getHomeManager().getDeliveryManager().getUpdate(0, 
														      TIME_OUT));
		} 
	    }  else {
		
		//Remove the javascript that reloads the
		//page as the mechanism fo ServerPush.
		page.getElementRefreshScript().getParentNode().removeChild(page.getElementRefreshScript());
	    }
	    	    

	    // Decide how to sort each table by delivery.
	    String pickupSort = null;
	    int    currentPickupSort = 0;

	    if ((pickupSort = getComms().request.getParameter(AirSentConstants.PICKUP_FILTER)) 
		    != null) {
		currentPickupSort = Integer.parseInt(pickupSort);
	    } 

	    String dropoffSort = null;
	    int    currentDropoffSort = 0;

	    if ((dropoffSort = getComms().request.getParameter(AirSentConstants.PICKUP_FILTER)) 
		    != null) {
		currentDropoffSort = Integer.parseInt(dropoffSort);
	    } 

	    // Build each table based on the delivery filters.
	    buildPickUpTable(page, currentPickupSort);
	    buildDropOffTable(page, currentDropoffSort);
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run AirSent_pres ) 
 * We need  to allow AirSent_pres to be functional ,  response
 * will be default HTML page with message
 *	
 */
	   }catch(NullPointerException e){
              page.setTextErrorText("You cannot delete order while runnig AirSent_pres");
	} catch (Exception e) {
	    throw new AirSentPresentationException("Trouble showing page", e);
	} 
     return page;
    } 

    /**
     * Builds the PickUp table
     * 
     * 
     */
    protected void buildPickUpTable(AdminMainHTML page, int type) 
	    throws AirSentPresentationException {
	String messenger = null;

	try {
	    HTMLTableRowElement templatePickUpRow = 
		page.getElementPickuprow();
	    Node		pickupTable = 
		templatePickUpRow.getParentNode();

	    // Get the template elements
	    HTMLElement		pickupnameCellTemplate = 
		page.getElementPickupname();
	    HTMLElement		pickupaddressCellTemplate = 
		page.getElementPickupaddress();
	    HTMLImageElement    pickupisurgentCellTemplate = 
		page.getElementPickupurgent();
	    HTMLElement		pickupmessengerCellTemplate = 
		page.getElementPickupmessenger();
	    HTMLAnchorElement   pickupeditCellTemplate = 
		page.getElementPickupedit();
	    HTMLAnchorElement   deletepickupCellTemplate = 
		page.getElementDeletepickuporder();
	    HTMLAnchorElement   messengerURL = 
		page.getElementPickupmessengerURL();

	    // Remove ids to prevent duplicates
	    // (browsers don't care, but the DOM does)
	    templatePickUpRow.removeAttribute("id");
	    pickupnameCellTemplate.removeAttribute("id");
	    pickupaddressCellTemplate.removeAttribute("id");
	    pickupisurgentCellTemplate.removeAttribute("id");
	    pickupeditCellTemplate.removeAttribute("id");
	    deletepickupCellTemplate.removeAttribute("id");
	    messengerURL.removeAttribute("id");
	    
	    // Get the array of deliveries.
	    Delivery[] deliveryList = getDeliveries(type);

	    // Create a row for each delivery
	    for (int numDeliveries = 0; numDeliveries < deliveryList.length; 
		    numDeliveries++) {
		Delivery currentDelivery = deliveryList[numDeliveries];

		// set text of new cells to values from string array
		page.setTextPickupname(currentDelivery.getPickUp().getName());
		page.setTextPickupaddress(currentDelivery.getPickUp().getStreet1());

		messenger = AirSentConstants.NOT_ASSIGNED;

		if (currentDelivery.getMessenger() != null) {
		    messenger = currentDelivery.getMessenger().getFirstName();

		   messengerURL.setHref(AirSentConstants.NEW_MESSENGER_PAGE 
					 + "?" 
					 + AirSentConstants.MESSENGER_NAME 
					 + "=" 
					 + currentDelivery.getMessenger().getBadge());
		} else {
                    messengerURL.setHref(AirSentConstants.ADMINDETAILS_PAGE
					       + "?deliveryID=" 
					       + currentDelivery.getHandle());
		}

		page.setTextPickupmessenger(messenger);
		page.setTextPickuptime(currentDelivery.getPickedUpTime());
		page.setTextPickupordernumber(currentDelivery.getHandle());
		pickupeditCellTemplate.setHref(AirSentConstants.ADMINDETAILS_PAGE 
					       + "?deliveryID=" 
					       + currentDelivery.getHandle());
		deletepickupCellTemplate.setHref(AirSentConstants.ADMIN_MAIN_PAGE 
						 + "?event=delete" 
						 + "&deliveryID=" 
						 + currentDelivery.getHandle());

		if (currentDelivery.isUrgent()) {
		    pickupisurgentCellTemplate.setSrc(AirSentConstants.URGENT_IMAGE);
		} else {
		    pickupisurgentCellTemplate.setSrc(AirSentConstants.BLANK_IMAGE);
		} 

		HTMLTableRowElement tempPickUpRow = 
		    (HTMLTableRowElement) templatePickUpRow.cloneNode(true);

		if (currentDelivery.isPickedUp()) {
		    tempPickUpRow.setBgColor("cccccc");
		} else {
		    tempPickUpRow.setBgColor("yellow");
		    page.setTextPickuptime("-");
		} 

		// Add a deep clone of the row to the DOM
		pickupTable.appendChild(tempPickUpRow);
	    } 

	    // Finally remove the template row and template select option
	    // from the page
	    templatePickUpRow.getParentNode().removeChild(templatePickUpRow);
	//same thing
        }catch(NullPointerException e){
	} catch (Exception e) {
	    throw new AirSentPresentationException("Trouble showing page", e);
	} 
    } 

    /**
     * Builds the DropOff table
     * 
     * 
     */
    protected void buildDropOffTable(AdminMainHTML page, int type) 
	    throws AirSentPresentationException {
	String messenger = AirSentConstants.NOT_ASSIGNED;

	try {
	    HTMLTableRowElement templateDropOffRow = 
		page.getElementDropoffrow();
	    Node		dropoffTable = 
		templateDropOffRow.getParentNode();

	    // Get the elements
	    HTMLElement		dropoffnameCellTemplate = 
		page.getElementDropoffname();
	    HTMLElement		dropoffaddressCellTemplate = 
		page.getElementDropoffaddress();
	    HTMLElement		dropoffmessengerCellTemplate = 
		page.getElementDropoffmessenger();
	    HTMLImageElement    dropoffisurgentCellTemplate = 
		page.getElementDropoffurgent();
	    HTMLAnchorElement   dropoffeditCellTemplate = 
		page.getElementDropoffedit();
	    HTMLAnchorElement   deletedropoffCellTemplate = 
		page.getElementDeletedropofforder();
	    HTMLAnchorElement   messengerURL = 
		page.getElementDropoffmessengerURL();

	    // Remove ids to prevent duplicates
	    // (browsers don't care, but the DOM does)
	    templateDropOffRow.removeAttribute("id");
	    dropoffnameCellTemplate.removeAttribute("id");
	    dropoffaddressCellTemplate.removeAttribute("id");
	    dropoffmessengerCellTemplate.removeAttribute("id");
	    dropoffisurgentCellTemplate.removeAttribute("id");
	    dropoffeditCellTemplate.removeAttribute("id");

	    // Get the delivery array
	    Delivery[] deliveryList = getDeliveries(type);

	    for (int numDeliveries = 0; numDeliveries < deliveryList.length; 
		    numDeliveries++) {
		Delivery currentDelivery = deliveryList[numDeliveries];

		// set text of new cells to values from string array
		page.setTextDropoffname(currentDelivery.getDropOff().getName());
		page.setTextDropoffaddress(currentDelivery.getDropOff().getStreet1());

                messenger = AirSentConstants.NOT_ASSIGNED;

		if (currentDelivery.getMessenger() != null) {
		    messenger = currentDelivery.getMessenger().getFirstName();

		    messengerURL.setHref(AirSentConstants.NEW_MESSENGER_PAGE 
					 + "?" 
					 + AirSentConstants.MESSENGER_NAME 
					 + "=" 
					 + currentDelivery.getMessenger().getBadge());
		}  else {
		    messengerURL.setHref(AirSentConstants.ADMINDETAILS_PAGE 
						+ "?deliveryID=" 
						+ currentDelivery.getHandle());

		}

		page.setTextDropoffmessenger(messenger);
		page.setTextDropoffordernumber(currentDelivery.getHandle());
		page.setTextDropofftime(currentDelivery.getDroppedOffTime());
		dropoffeditCellTemplate.setHref(AirSentConstants.ADMINDETAILS_PAGE 
						+ "?deliveryID=" 
						+ currentDelivery.getHandle());
		deletedropoffCellTemplate.setHref(AirSentConstants.ADMIN_MAIN_PAGE 
						  + "?event=delete" 
						  + "&deliveryID=" 
						  + currentDelivery.getHandle());

		if (currentDelivery.isUrgent()) {
		    dropoffisurgentCellTemplate.setSrc(AirSentConstants.URGENT_IMAGE);
		} else {
		    dropoffisurgentCellTemplate.setSrc(AirSentConstants.BLANK_IMAGE);
		} 

		HTMLTableRowElement tempDropOffRow = 
		    (HTMLTableRowElement) templateDropOffRow.cloneNode(true);

		if (currentDelivery.isDroppedOff()) {
		    tempDropOffRow.setBgColor("cccccc");
		} else {
		    tempDropOffRow.setBgColor("yellow");
		    page.setTextDropofftime("-");
		} 

		// Add a deep clone of the row to the DOM
		dropoffTable.appendChild(tempDropOffRow);
	    } 

	    // Finally remove the template row and template select option
	    // from the page
	    templateDropOffRow.getParentNode().removeChild(templateDropOffRow);
	
         }catch(NullPointerException e){       
	} catch (Exception e) {
	    throw new AirSentPresentationException("Trouble showing page", e);
	} 
    } 

    /**
     * Gets a collection of deliveries based on the type.
     * 
     * 
     */
    protected Delivery[] getDeliveries(int type) 
	    throws AirSentPresentationException {
	try {
	    switch (type) {

	    case TOP_TEN_DELIVERIES:
		return getApplication().getHomeManager().getDeliveryManager().findSubset(TOP_TEN_DELIVERIES);

	    case TOP_TWENTY_DELIVERIES:
		return getApplication().getHomeManager().getDeliveryManager().findSubset(TOP_TWENTY_DELIVERIES);

	    case ALL_DELIVERIES:
		return getApplication().getHomeManager().getDeliveryManager().findAll();

	    default:
		return getApplication().getHomeManager().getDeliveryManager().findAll();
	    }
	
	}catch(NullPointerException e){
                throw new NullPointerException();
	} catch (Exception e) {
	    throw new AirSentPresentationException("Trouble getting deliveries:", 
						   e);
	} 
    } 

    /**
     *Handles the deletion of an Order. 
     * 
     * 
     */
    public XMLObject handleDelete() throws AirSentPresentationException {
	try {
	    String   deliveryID = 
		getComms().request.getParameter(AirSentConstants.DELIVERYID);
	    Delivery delivery = null;

	    if ((delivery = getApplication().getHomeManager().getDeliveryManager().findByHandle(deliveryID)) 
		    != null) {
		delivery.delete();
		
	    } 
	//We need  to allow AirSent_pres to be functional , response will be default HTML page        
        }catch(NullPointerException e){
        
         AdminMainHTML defaultPage = (AdminMainHTML) myComms.xmlcFactory.create(AdminMainHTML.class);		
	 defaultPage.setTextErrorText("You cannot delete delivery while running AirSent_pres");
	 return defaultPage;
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Error in confimation", 
						   ex);
	} 
	throw new ClientPageRedirectException(AirSentConstants.ADMIN_MAIN_PAGE);
    } 

    /**
     * Update the AdminMain page.
     * 
     * 
     */
    public XMLObject handleUpdate() throws AirSentPresentationException {
	return showPage(null);
    } 

}


