/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: NewMessenger.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.presentation.admin;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;
import java.util.*;


import com.lutris.airsent.spec.*;
import com.lutris.airsent.presentation.*;


import com.lutris.airsent.spec.messenger.Messenger;


import com.lutris.airsent.spec.address.*;
/**
 * The login page of the AirSent app
 * 
 */
public class NewMessenger extends BasePO {
    private static final int AUTH_LEVEL = AirSentConstants.ADMINISTRATOR_USER;
    private String _firstname = null;
    private String _lastname = null;
    private String _badge = null;
    private String _password = null;
    private String _repassword = null;
    private String _email = null;
    private String _phone = null;

    /**
     * Superclass method override.
     * returns the authorization level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * Default event. Just show the page.
     */
    public XMLObject handleDefault() 
        throws AirSentPresentationException {
        try {
            Messenger messenger = null;
            String id = getComms().request.getParameter(AirSentConstants.MESSENGER_NAME); 
            if (id == null) {
                getSessionData().setMessenger(null);
            } else {
                messenger = getMessenger(id);
                if (messenger == null) {
                    return showPage("Messenger not found", null);
                }
                getSessionData().setMessenger(messenger);
            }
            return showPage(null, messenger);
        } catch (Exception e) {
            throw new AirSentPresentationException("", e);
        }
    } 

    /**
     *
     */
    public XMLObject handleNewMessenger() 
        throws HttpPresentationException {
        try {
            getParameters();
            
            Messenger messenger = getSessionData().getMessenger();
            boolean virgin = true;
            if (messenger != null) {
                virgin = false;
            }
            
            String err = validateForm(messenger);
            if (err != null) {
                return showPage(err, messenger);
            } 
            
            HomeManager hf = getApplication().getHomeManager();
            if (virgin == true) {
                messenger = hf.getMessengerManager().findByBadge(_badge);
                if (messenger != null) {
                    return showPage("Badge already in use", null);
                } else {
                    messenger = hf.getMessengerManager().create();
                }
            }
            
            if (virgin == true) {
                messenger.setBadge(_badge);
		messenger.setPassword(_password);
	    }
            messenger.setFirstName(_firstname);
            messenger.setLastName(_lastname);
            messenger.setEmail(_email);
            Address address = messenger.getAddress();
            address.setLocalNumber(_phone);
            messenger.save();
            getSessionData().setMessenger(null);
            throw new ClientPageRedirectException(AirSentConstants.ADMIN_MAIN_PAGE);
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run AirSent_pres ) 
 * We need  to allow AirSent_pres to be functional ,  response
 * will be default HTML page with message
 *	
 */ 
        }catch(NullPointerException e){
        	NewMessengerHTML page = (NewMessengerHTML) myComms.xmlcFactory.create(NewMessengerHTML.class);
        	page.setTextErrorText("You cannot Enter a New Messenger while running AirSent_pres");
              return page;
        } catch (Exception ex) {
            throw new AirSentPresentationException("Exception saving messenger in user: ", ex);
        } 
    } 
    
    /**
     * 
     */
    private XMLObject showPage(String errorMsg, Messenger messenger) throws Exception {
	NewMessengerHTML page = (NewMessengerHTML) myComms.xmlcFactory.create(NewMessengerHTML.class);
        if (errorMsg != null || (errorMsg = getSessionData().getAndClearUserMessage()) != null) {
	    page.setTextTitle("");
            page.setTextErrorText(errorMsg);
        } else {
            page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
        } 
            
        if (messenger == null) {
            page.setTextBadgeText("");
	    page.setTextPasswordText("");
	    Node n = page.getElementEditMessenger().getParentNode();
            n.removeChild(page.getElementNewMessengerLink());
            n.removeChild(page.getElementEditMessenger());
        } else {
            page.setTextTitle("Edit a messenger");
            page.getElementBadge().getParentNode().removeChild(page.getElementBadge());
            page.setTextBadgeText(_badge);
	    page.setTextPasswordText(_password);
	    page.getElementPassword().getParentNode().removeChild(page.getElementPassword());
	    Node n = page.getElementEditMessenger().getParentNode();
            n.removeChild(page.getElementNewMessenger());
        }
	populateForm(page, messenger);
	return page;
    } 

    /**
     * Populates the form.
     */
    private void populateForm(NewMessengerHTML page, Messenger messenger) {
        if (messenger == null) {
            if (isNullField(_badge) == false) {
                page.getElementBadge().setValue(_badge);
            } 
	    if (isNullField(_password) == false) {
		page.getElementPassword().setValue(_password);
	    }
	    if (isNullField(_password) == false) {
		page.getElementPasswordverify().setValue(_password);
	    }
	}
	if (isNullField(_firstname) == false) {
	    page.getElementFirstname().setValue(_firstname);
        }
	if (isNullField(_lastname) == false) {
	    page.getElementLastname().setValue(_lastname);
        }
	
	if (isNullField(_phone) == false) {
	    page.getElementPhone().setValue(_phone);
        }
	if (isNullField(_email) == false) {
	    page.getElementEmail().setValue(_email);
        }
    } 


    /**
     * Validates the form data.
     *
     * @return error String
     */
    private String validateForm(Messenger messenger) {
        if(messenger == null) {
	    if (isNullField(_password)) {
		return "Please provide a password";
	    } 
	    if (isNullField(_repassword)) {
		return "Please verify password.";
	    } 
	    
	    if (_password.equals(_repassword) == false) {
		return "Please make sure your password and password confirmation match exactly";
	    }
	}
        
	if (isNullField(_firstname)) {
	    return "Please provide a first name";
	} 
	if (isNullField(_lastname)) {
	    return "Please provide a last name";
	} 
        if (messenger == null) {
            if (isNullField(_badge)) {
		return "Please provide a badge";
            }
        }
	if (isNullField(_email)) {
	    return "Please provide an email";
	} 
	if (isNullField(_phone)) {
	    return "Please provide a phone number";
	} 
	return null;
    } 

    /**
     *
     */
    private Messenger getMessenger(String badge) throws Exception {
        HomeManager hf = getApplication().getHomeManager();
        Messenger messenger = hf.getMessengerManager().findByBadge(badge);
        if ((badge != null) && (messenger != null)) {
            _firstname = messenger.getFirstName();
            _lastname = messenger.getLastName();
            _badge = messenger.getBadge();
            _password = messenger.getPassword();
            _repassword = _password;
            _email = messenger.getEmail();
            Address address = messenger.getAddress();
            _phone = address.getLocalNumber();
        }
        return messenger;
    } 
    
    /**
     *
     */
    private void getParameters() throws Exception {
        _firstname = getComms().request.getParameter(AirSentConstants.FIRST_NAME);
        _lastname = getComms().request.getParameter(AirSentConstants.LAST_NAME);
        _badge = getComms().request.getParameter(AirSentConstants.BADGE_NAME);
        _repassword = getComms().request.getParameter(AirSentConstants.PASSWORD_VERIFY_NAME);
        _password = getComms().request.getParameter(AirSentConstants.PASSWORD_NAME);
        _email = getComms().request.getParameter(AirSentConstants.EMAIL_NAME);
        _phone = getComms().request.getParameter(AirSentConstants.PHONE_NAME);
    } 
}

