/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: AccountHistory.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.presentation.customer;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;
import java.util.*;
import com.lutris.airsent.presentation.AirSentPresentationException;
import com.lutris.airsent.presentation.AirSentConstants;
import com.lutris.airsent.presentation.BasePO;
import com.lutris.airsent.spec.delivery.Delivery;
import com.lutris.airsent.spec.customer.*;
/**
 * AccountHistory.java displays a customers past deliveries
 * and allows them to select on to prepopulate a future 
 * delivery.
 */
public class AccountHistory extends BasePO {

    /**
     * Constants
     */
    private static final int AUTH_LEVEL = AirSentConstants.CUSTOMER_USER;

    /**
     * Superclass method override.
     * returns the authorization level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * Default event. Just show the page.
     */
    public XMLObject handleDefault() throws HttpPresentationException {
	return showPage(null, null);
    } 

    /**
     * Shows the AccountHistory page
     * @param error messages
     * @return the page
     */
    public XMLObject showPage(String errorMsg, Delivery delivery) 
	    throws HttpPresentationException {
	AccountHistoryHTML page = 
		(AccountHistoryHTML) myComms.xmlcFactory.create(AccountHistoryHTML.class);

	
	try {
	    
	    Delivery[] deliveries = 
		getApplication().getHomeManager().getDeliveryManager().findByCustomer(getSessionData().getCustomer());
		
	    if (null != errorMsg 
		    || null 
		       != (errorMsg = 
			   getSessionData().getAndClearUserMessage())) {
		page.setTextErrorText(errorMsg);
	    } else {
		page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
	    } 

	    HTMLTableRowElement templateRow = page.getElementRowtemplate();
	    Node		pickupTable = templateRow.getParentNode();
	    HTMLAnchorElement   deliveryURL = page.getElementSelectdelivery();
	    HTMLTableRowElement tempRow = null;

	    // Remove ids to prevent duplicates
	    // (browsers don't care, but the DOM does)
	    templateRow.removeAttribute("id");
	    deliveryURL.removeAttribute("id");

	   
	    // Build the account history table.
	    for (int i = 0; i < deliveries.length; i++) {
		Delivery currentDelivery = deliveries[i];

		page.setTextPickupname(currentDelivery.getPickUp().getName());
		page.setTextPickupaddress(currentDelivery.getPickUp().getStreet1());
		page.setTextDropoffaddress(currentDelivery.getDropOff().getStreet1());
		page.setTextDescription(currentDelivery.getDescription());
		page.setTextOrdernumber(currentDelivery.getHandle());
		page.setTextPickuptime(currentDelivery.getPickedUpTime());
		page.setTextDropofftime(currentDelivery.getDroppedOffTime());
		deliveryURL.setHref("OrderPage1.po?" + AirSentConstants.DELIVERYID 
				    + "=" + currentDelivery.getHandle());
		
		tempRow = (HTMLTableRowElement) templateRow.cloneNode(true);

		// Add a deep clone of the row to the DOM
		pickupTable.appendChild(tempRow);
	    } 

	    // Finally remove the template row and template select option
	    // from the page
	    templateRow.getParentNode().removeChild(templateRow);

/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run AirSent_pres ) 
 * We need  to allow AirSent_pres to be functional ,  response
 * will be default HTML page 
 *	
 */	   
	 }catch(NullPointerException e){
	   	page.setTextErrorText("This is a default HTML page");
	  	HTMLAnchorElement   deliveryURL = page.getElementSelectdelivery();
	  	deliveryURL.setHref("OrderPage1.po");
	  	
	} catch (Exception e) {
	    throw new AirSentPresentationException("Exception ", e);
	}
	  return page;
    } 

}

