/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: Login.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.presentation.customer;

import com.lutris.airsent.presentation.BasePO;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;
import com.lutris.airsent.spec.AirSentException;

import com.lutris.airsent.presentation.AirSentPresentationException;
import com.lutris.airsent.presentation.AirSentConstants;
import java.util.*;
import com.lutris.airsent.spec.customer.Customer;

/**
 * The login page of the AirSent app
 * 
 */
public class Login extends BasePO {

    /**
     * Constants
     */
    private static final int AUTH_LEVEL = AirSentConstants.UNAUTH_USER;

    /**
     * Superclass method override.
     * returns the authorization level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * Default event. Just show the page.
     */
    public XMLObject handleDefault() throws HttpPresentationException {
	return showPage(null);
    } 

    /**
     * 
     * @param error messages
     * @return  page as XMLObject.
     */
    public XMLObject showPage(String errorMsg) 
	    throws AirSentPresentationException {
	LoginHTML page = 
	    (LoginHTML) myComms.xmlcFactory.create(LoginHTML.class);

	try {
	    if (null != errorMsg 
		|| null 
		!= (errorMsg = 
		    getSessionData().getAndClearUserMessage())) {
		page.setTextErrorText(errorMsg);
	    } else {
		page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
	    } 
	} catch (Exception e) {
	    throw new AirSentPresentationException("Trouble showing page", e);
	} 

	return page;
    } 

    /**
     * Process login data
     * 
     * @return error String
     */
    public XMLObject handleLogin() throws HttpPresentationException {
	String   login = 
	    getComms().request.getParameter(AirSentConstants.LOGIN_NAME);
	String   password = 
	    getComms().request.getParameter(AirSentConstants.PASSWORD_NAME);
	Customer customer = null;

	try {
	    if ((customer = 
		    getApplication().getHomeManager().getCustomerManager().validatePassword(login, 
		    password)) == null) {
		return showPage("Invalid username or password.");
	    } else {
		getSessionData().setCustomer(customer);
		getSessionData().setUserAuth(AirSentConstants.CUSTOMER_USER);
		
		this.getSessionData().setUserMessage("Welcome, " + customer.getFirstName());
		//throw new ServerPageRedirectException(AirSentConstants.ORDERSTEP1_PO);
		throw new ClientPageRedirectException(AirSentConstants.ORDERSTEP1_PO);
	    } 
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run AirSent_pres ) 
 * We need  to allow AirSent_pres to be functional ,  response
 * will be default HTML page 
 *	
 */
	}catch(NullPointerException ex){
		throw new ClientPageRedirectException(AirSentConstants.ORDERSTEP1_PO);
		 
	
	} catch (Exception ex) {
	    throw new AirSentPresentationException("System error finding user", 
						   ex);
	} 
    } 

}

