/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: OrderPage1.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.presentation.customer;

import com.lutris.airsent.presentation.BasePO;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;


import com.lutris.airsent.spec.AirSentException;

import com.lutris.airsent.presentation.AirSentPresentationException;
import com.lutris.airsent.presentation.AirSentConstants;
import java.util.*;
import com.lutris.airsent.spec.address.*;
import com.lutris.airsent.spec.customer.*;
import com.lutris.airsent.spec.*;


import com.lutris.airsent.spec.delivery.*;
/**
 * Presentation object for order page 1
 */
public class OrderPage1 
    extends BasePO {

    /**
     * Constants
     */
    private static final int AUTH_LEVEL = AirSentConstants.CUSTOMER_USER;


    /**
     * Superclass method override.
     * returns the authorization level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * displays default page
     *
     * @return html page
     * @exception if an error occurs
     */
    public XMLObject handleDefault() 
        throws HttpPresentationException {
	try {
	    String id = getComms().request.getParameter(AirSentConstants.DELIVERYID);
            OrderForm d = getOrder(id);
            getSessionData().setOrderForm(d);
            return showPage1(null, d);
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error finding user", e);
	} 
    } 
    
    /**
     * process the form submission
     *     
     * @return html page
     * @exception if an error occurs
     */
    public XMLObject handleOrder1() 
        throws AirSentPresentationException {
	try {
            String p;
            OrderForm orderForm = getOrder(null);
            
            p = getComms().request.getParameter(AirSentConstants.PICKUP_NAME);
	    if (checkField(p, Address.MAX_NAME) == false) {
		return showPage1("Please provide a pickupname", orderForm);
	    } 
            orderForm.setPickUpName(p);

            p = getComms().request.getParameter(AirSentConstants.PICKUP_ADDRESS);
	    if (checkField(p, Address.MAX_ADDRESS) == false) {
		return showPage1("Please provide a pickupaddress", orderForm);
	    } 
            orderForm.setPickUpAddress1(p);
                
            p = getComms().request.getParameter(AirSentConstants.PICKUP_PHONE);
	    if (checkField(p, Address.MAX_PHONE) == false) {
		return showPage1("Please provide a pickupphone", orderForm);
	    } 
            orderForm.setPickUpPhone(p);

            p = getComms().request.getParameter(AirSentConstants.PICKUP_DIRECTIONS);
	    if (checkField(p, Address.MAX_DIRECTIONS) == false) {
		return showPage1("Please provide a pickupdirections", orderForm);
	    } 
            orderForm.setPickUpDirections(p);
            
            getSessionData().setOrderForm(orderForm);
	    throw new ClientPageRedirectException(AirSentConstants.ORDERSTEP2_PO);
	
	//We need  to allow AirSent_pres to be functional , response will be default HTML page        
        }catch(NullPointerException e){
           OrderStep2HTML defaultPage = (OrderStep2HTML) myComms.xmlcFactory.create(OrderStep2HTML.class);		
	  defaultPage.setTextErrorText("This is a default HTML page");
	return defaultPage;
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error processing OrderStep1.", e);
	} 
    } 

    /**
     * creates the html page
     *
     * @param errorMsg error message to display
     * @param orderData data to populate the page with
     * @exception if an error occurs
     */
    private XMLObject showPage1(String errorMsg, OrderForm orderForm) 
        throws AirSentPresentationException {

	OrderStep1HTML page = (OrderStep1HTML) myComms.xmlcFactory.create(OrderStep1HTML.class);
	try {
	    if (null != errorMsg || null != (errorMsg = getSessionData().getAndClearUserMessage())) {
		page.setTextErrorText(errorMsg);
	    } else {
		page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
	    } 
            if(orderForm.getOrderId() == null) {
                Node n = page.getElementNavDestination().getParentNode();
                n.removeChild(page.getElementNavDestination());
                n.removeChild(page.getElementNavDetails());
            }
            populatePage1(page, orderForm);
//We need  to allow AirSent_pres to be functional , response will be default HTML page        
        }catch(NullPointerException e){
           
         OrderStep1HTML defaultPage = (OrderStep1HTML) myComms.xmlcFactory.create(OrderStep1HTML.class);		
	  defaultPage.setTextErrorText("This is a default HTML page");
	return defaultPage;
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error showing page 1", e);
	} 
	return page;
    } 
    
    /**
     * populates the page
     *
     * @param page to populate
     * @param orderData data to populate it with
     * @exception if an error occurs
     */
    private void populatePage1(OrderStep1HTML page, OrderForm orderForm)
        throws AirSentPresentationException {

	try {
	    if(orderForm.getOrderId() != null) {
                page.setTextOrderHeader("Order");
		page.setTextOrdernumber(orderForm.getOrderId());
		page.getElementPickupname().setValue(orderForm.getPickUpName());
		page.getElementPickupaddress().setValue(orderForm.getPickUpAddress1());
		page.getElementPickupphone().setValue(orderForm.getPickUpPhone());
	    } else {
		page.setTextOrdernumber("");
	    }
	    //  page.getElementPickupname().setValue(orderForm.getPickUpName());
//  	    page.getElementPickupaddress().setValue(orderForm.getPickUpAddress1());
//  	    page.getElementPickupphone().setValue(orderForm.getPickUpPhone());
	    
	    if (orderForm.getPickUpDirections() != null) {
		Text commentText = (page.getDocument()).createTextNode(orderForm.getPickUpDirections());
		page.getElementPickupdirections().appendChild(commentText);
	    } 
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error populating page1", e);
	} 
    } 

    /**
     * gets the order data from session or persisten store
     */
    private OrderForm getOrder(String id) 
        throws AirSentPresentationException {
        try {
            OrderForm orderForm = getSessionData().getOrderForm();
            if ((orderForm == null) && (id == null)) {
                // new order session
                // System.out.println("new order session");
                
                orderForm = OrderFormFactory.getOrderForm("com.lutris.airsent.business.delivery.OrderFormImpl");
                
            
            
            } else if (id != null) {
                // editing previous order session
                // System.out.println("editing previous order session: id = " + id);
                HomeManager homeManager = getApplication().getHomeManager();
                Delivery d = homeManager.getDeliveryManager().findByHandle(id);
              
                orderForm = OrderFormFactory.getOrderForm("com.lutris.airsent.business.delivery.OrderFormImpl",d);
            } 
            return orderForm;
 

        } catch (Exception e) {
            throw new AirSentPresentationException("System error getting order", e);
        } 
    }
}


