/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: OrderPage2.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.presentation.customer;

import com.lutris.airsent.presentation.BasePO;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;
import com.lutris.airsent.spec.AirSentException;

import com.lutris.airsent.presentation.AirSentPresentationException;
import com.lutris.airsent.presentation.AirSentConstants;
import java.util.*;
import com.lutris.airsent.spec.address.*;
import com.lutris.airsent.spec.customer.Customer;
import com.lutris.airsent.spec.delivery.OrderForm;

/**
 * Presentation object for order page 2
 */
public class OrderPage2
    extends BasePO {

    /**
     * Constants
     */
    private static final int AUTH_LEVEL = AirSentConstants.CUSTOMER_USER;

    /**
     * Superclass method override. returns the authorization 
     * level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * displays default page
     *
     * @return html page
     * @exception if an error occurs
     */
    public XMLObject handleDefault() 
        throws HttpPresentationException {
	try {
            OrderForm orderForm = getSessionData().getOrderForm();
            if (orderForm == null) {
                System.err.println("ERROR: no order form");
                throw new ClientPageRedirectException(AirSentConstants.ORDERSTEP1_PO); 
            }
            return showPage2(null, orderForm);
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error finding user", e);
	} 
    } 

    /**
     * creates the html page
     *
     * @param errorMsg error message to display
     * @param orderData data to populate the page with
     * @exception if an error occurs
     */
    private XMLObject showPage2(String errorMsg, OrderForm orderForm) 
        throws AirSentPresentationException {
	OrderStep2HTML page = (OrderStep2HTML) myComms.xmlcFactory.create(OrderStep2HTML.class);
	try {
	    if (null != errorMsg || null != (errorMsg = getSessionData().getAndClearUserMessage())) {
		page.setTextErrorText(errorMsg);
	    } else {
		page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
	    } 
            if(orderForm.getOrderId() == null) {
                Node n = page.getElementNavPickup().getParentNode();
                n.removeChild(page.getElementNavPickup());
                n.removeChild(page.getElementNavDetails());
            }
            populatePage2(page, orderForm);
	//We need  to allow AirSent_pres to be functional , responsed will be default HTML page        
        }catch(NullPointerException e){
           
         OrderStep2HTML defaultPage = (OrderStep2HTML) myComms.xmlcFactory.create(OrderStep2HTML.class);		
	  defaultPage.setTextErrorText("This is a default HTML page");
	return defaultPage;
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error showing page 1", e);
	} 
	return page;
    } 

    /**
     * populates the page
     *
     * @param page to populate
     * @param orderData data to populate it with
     * @exception if an error occurs
     */
    private void populatePage2(OrderStep2HTML page, OrderForm orderForm)
        throws AirSentPresentationException {
        
        try {
            if(orderForm.getOrderId() != null) {
                page.setTextOrderHeader("Order");
                page.setTextOrdernumber(orderForm.getOrderId());
		 page.getElementDropoffname().setValue(orderForm.getDropOffName());
		 page.getElementDropoffaddress().setValue(orderForm.getDropOffAddress1());
		 page.getElementDropoffphone().setValue(orderForm.getDropOffPhone());
            }else {
                page.setTextOrdernumber("");
            }
            
           //   page.getElementDropoffname().setValue(orderForm.getDropOffName());
//              page.getElementDropoffaddress().setValue(orderForm.getDropOffAddress1());
//              page.getElementDropoffphone().setValue(orderForm.getDropOffPhone());
            if (orderForm.getDropOffDirections() != null) {
                Text commentText = (page.getDocument()).createTextNode(orderForm.getDropOffDirections());
                page.getElementDropoffdirections().appendChild(commentText);
            } 
        } catch (Exception e) {
            throw new  AirSentPresentationException("", e);
        }
    }

    /**
     * process the form submission
     *     
     * @return html page
     * @exception if an error occurs
     */
    public XMLObject handleOrder2() 
        throws AirSentPresentationException {
	try {
            String p;
            OrderForm orderForm = getSessionData().getOrderForm();

            p = this.getComms().request.getParameter(AirSentConstants.DROPOFF_NAME);
	    if (checkField(p, Address.MAX_NAME) == false) {
		return showPage2("Please provide a drop off name", orderForm);
	    } 
            orderForm.setDropOffName(p);
            
            p = this.getComms().request.getParameter(AirSentConstants.DROPOFF_ADDRESS);
	    if (checkField(p, Address.MAX_ADDRESS) == false) {
		return showPage2("Please provide a drop off address", orderForm);
	    } 
            orderForm.setDropOffAddress1(p);
                
            p = this.getComms().request.getParameter(AirSentConstants.DROPOFF_PHONE);
	    if (checkField(p, Address.MAX_PHONE) == false) {
		return showPage2("Please provide a drop off phone", orderForm);
	    } 
            orderForm.setDropOffPhone(p);

            p = this.getComms().request.getParameter(AirSentConstants.DROPOFF_DIRECTIONS);
	    if (checkField(p, Address.MAX_DIRECTIONS) == false) {
		return showPage2("Please provide a drop off directions", orderForm);
	    } 
            orderForm.setDropOffDirections(p);
            
            getSessionData().setOrderForm(orderForm);
	    throw new ClientPageRedirectException(AirSentConstants.ORDERSTEP3_PO);
	 }catch(NullPointerException e){
           OrderStep3HTML defaultPage = (OrderStep3HTML) myComms.xmlcFactory.create(OrderStep3HTML.class);		
	  defaultPage.setTextErrorText("This is a default HTML page");
	return defaultPage;
     
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error processing OrderStep2.", e);
	} 
    } 
}



