/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: OrderPage3.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.presentation.customer;

import com.lutris.airsent.presentation.BasePO;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;
import com.lutris.airsent.spec.delivery.Delivery;
import com.lutris.airsent.spec.AirSentException;

import com.lutris.airsent.presentation.AirSentPresentationException;
import com.lutris.airsent.presentation.AirSentConstants;
import java.util.*;
import com.lutris.airsent.spec.address.*;
import com.lutris.airsent.spec.customer.Customer;
import com.lutris.airsent.spec.delivery.OrderForm;


/**
 * Presentation object for order page 3
 */
public class OrderPage3
    extends BasePO {

    private static final int AUTH_LEVEL = AirSentConstants.CUSTOMER_USER;

    /**
     * Superclass method override. returns the authorization 
     * level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * displays default page
     *
     * @return html page
     * @exception if an error occurs
     */
    public XMLObject handleDefault() 
        throws HttpPresentationException {

	try {
            OrderForm orderForm = getSessionData().getOrderForm();
            if (orderForm == null) {
                System.err.println("ERROR: no order form");
                throw new ClientPageRedirectException(AirSentConstants.ORDERSTEP1_PO); 
            }
            
            return showPage3(null, orderForm);
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error finding user", e);
	} 
    } 

    /**
     * creates the html page
     *
     * @param errorMsg error message to display
     * @param orderData data to populate the page with
     * @exception if an error occurs
     */
    private XMLObject showPage3(String errorMsg, OrderForm orderForm) 
        throws AirSentPresentationException {

	OrderStep3HTML page = (OrderStep3HTML) myComms.xmlcFactory.create(OrderStep3HTML.class);
	try {
	    if (null != errorMsg || null != (errorMsg = getSessionData().getAndClearUserMessage())) {
		page.setTextErrorText(errorMsg);
	    } else {
		page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
	    } 
            if(orderForm.getOrderId() == null) {
                Node n = page.getElementNavDestination().getParentNode();
                n.removeChild(page.getElementNavDestination());
                n.removeChild(page.getElementNavPickup());
            }
            populatePage3(page, orderForm);
	//We need  to allow AirSent_pres to be functional , responsed will be default HTML page        
        }catch(NullPointerException e){
           
         OrderStep3HTML defaultPage = (OrderStep3HTML) myComms.xmlcFactory.create(OrderStep3HTML.class);		
	  defaultPage.setTextErrorText("This is a default HTML page");
	return defaultPage;
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error finding user", e);
	} 
	return page;
    } 

    /**
     * populates the page
     *
     * @param page to populate
     * @param orderData data to populate it with
     * @exception if an error occurs
     */
    private void populatePage3(OrderStep3HTML page, OrderForm orderForm) 
        throws AirSentPresentationException {
        
        try {
            if(orderForm.getFragile() == true) {
                page.getElementFragile().setChecked(true);
            }
            if(orderForm.getUrgent() == true) {
                page.getElementUrgent().setChecked(true);
            }
            page.getElementSize().setValue(orderForm.getSize());
            if (orderForm.getDescription() != null) {
                Text commentText = (page.getDocument()).createTextNode(orderForm.getDescription());
                page.getElementDescription().appendChild(commentText);
            } 
        } catch (Exception e) {
            throw new AirSentPresentationException("", e);
        }
    }

    /**
     * process the form submission
     *     
     * @return html page
     * @exception if an error occurs
     */
    public XMLObject handleOrder3() 
        throws AirSentPresentationException {

	try {
            String p;
            OrderForm orderForm = getSessionData().getOrderForm();

            p = this.getComms().request.getParameter(AirSentConstants.FRAGILE);
            if (!isNullField(p)) {
                orderForm.setFragile(true);
	    } else {
                orderForm.setFragile(false);
	    }

            p = this.getComms().request.getParameter(AirSentConstants.URGENT);
	    if (!isNullField(p)) {
                orderForm.setUrgent(true);
	    } else {
                orderForm.setUrgent(false);
	    }

            p = this.getComms().request.getParameter(AirSentConstants.DESCRIPTION);
	    if (checkField(p, Delivery.MAX_DESC) == false) {
		return showPage3("Please provide a description.", orderForm);
	    } 
            orderForm.setDescription(p);

            p = this.getComms().request.getParameter(AirSentConstants.SIZE);
	    if (checkField(p, Delivery.MAX_SIZE) == false) {
		return showPage3("Please enter a valid size.", orderForm);
	    } 
            orderForm.setSize(p);
            
            getSessionData().setOrderForm(orderForm);
	    throw new ClientPageRedirectException(AirSentConstants.CONFIRMATION_PO);
	//We need  to allow AirSent_pres to be functional , responsed will be default HTML page        
        }catch(NullPointerException e){
       ConfirmationHTML defaultPage = (ConfirmationHTML) myComms.xmlcFactory.create(ConfirmationHTML.class);		
	  defaultPage.setTextErrorText("This is a default HTML page");
	return defaultPage;
   
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error processing OrderStep1.", e);
	} 
    } 

    /*
     * check to see if a string is a number
     */
    private boolean isNumber(String n) {
        if (n == null) {
            return false;
        }
        try {
            Integer i = Integer.valueOf(n);
        } catch (NumberFormatException e) {
            return false;
        }
        return true;
    }
}



