/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: OrderPage4.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.presentation.customer;
import com.lutris.airsent.spec.AirSentException;
import com.lutris.airsent.presentation.*;


import com.lutris.airsent.spec.customer.Customer;
import com.lutris.airsent.spec.delivery.*;
import com.lutris.airsent.spec.*;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;
import java.util.*;

/**
 * Presentation object for order page 4
 */
public class OrderPage4
    extends BasePO {

    /**
     * Constants
     */
    private static final int AUTH_LEVEL = AirSentConstants.CUSTOMER_USER;

    /**
     * Superclass method override.
     * returns the authorization level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * displays default page
     *
     * @return html page
     * @exception if an error occurs
     */
    public XMLObject handleDefault() 
        throws HttpPresentationException {

	try {
            OrderForm orderForm = getSessionData().getOrderForm();
            if (orderForm == null) {
                System.err.println("ERROR: no order form");
                throw new ClientPageRedirectException(AirSentConstants.ORDERSTEP1_PO); 
            }
            return showPage4(null, orderForm);
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error finding user", e);
	} 
    } 

    /**
     * creates the html page
     *
     * @param errorMsg error message to display
     * @param orderData data to populate the page with
     * @exception if an error occurs
     */
    private XMLObject showPage4(String errorMsg, OrderForm orderForm) 
        throws AirSentPresentationException {

        ConfirmationHTML page = (ConfirmationHTML) myComms.xmlcFactory.create(ConfirmationHTML.class);
	if (null != errorMsg || null != (errorMsg = getSessionData().getAndClearUserMessage())) {
	    page.setTextErrorText(errorMsg);
	} else {
	    page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
	} 

	try {
	    page.setTextPickupname(orderForm.getPickUpName());
	    page.setTextPickupaddress(orderForm.getPickUpAddress1());
	    page.setTextPickupphone(orderForm.getPickUpPhone());
	    page.setTextDropoffname(orderForm.getDropOffName());
	    page.setTextDropoffaddress(orderForm.getDropOffAddress1());
	    page.setTextDropoffphone(orderForm.getDropOffPhone());
            
	    if (orderForm.getUrgent() == true) {
		page.getElementPickupurgent().setSrc(AirSentConstants.URGENT_IMAGE);
		page.getElementDropoffurgent().setSrc(AirSentConstants.URGENT_IMAGE);
	    } else {
		page.getElementPickupurgent().setSrc(AirSentConstants.BLANK_IMAGE);
		page.getElementDropoffurgent().setSrc(AirSentConstants.BLANK_IMAGE);
	    }
	    
	    
	    if (orderForm.getFragile() == true) {
		page.getElementPickupfragile().setSrc(AirSentConstants.URGENT_IMAGE);
		page.getElementDropofffragile().setSrc(AirSentConstants.URGENT_IMAGE);
	    } else {
		page.getElementPickupfragile().setSrc(AirSentConstants.BLANK_IMAGE);
		page.getElementDropofffragile().setSrc(AirSentConstants.BLANK_IMAGE);
	    }
	    page.setTextPickupdescription(orderForm.getDescription());
	    page.setTextDropoffdescription(orderForm.getDescription());
	     
	    
	    
	//We need  to allow AirSent_pres to be functional , responsed will be default HTML page        
        }catch(NullPointerException e){
           
         ConfirmationHTML defaultPage = (ConfirmationHTML) myComms.xmlcFactory.create(ConfirmationHTML.class);		
	  defaultPage.setTextErrorText("This is a default HTML page");
	return defaultPage;
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Error in confimation", ex);
	} 
	return page;
    } 

    /**
     * process the form submission
     *     
     * @return html page
     * @exception if an error occurs
     */
    public XMLObject handleOrder4() 
        throws AirSentPresentationException {
	try {
            OrderForm orderForm = getSessionData().getOrderForm();
            if (orderForm == null) {
                System.err.println("ERROR: no order form");
                throw new ClientPageRedirectException(AirSentConstants.ORDERSTEP1_PO); 
            }
            Customer customer = getSessionData().getCustomer();
            HomeManager hf = getApplication().getHomeManager();
            DeliveryManager df = hf.getDeliveryManager();
            Delivery delivery = df.create(customer, orderForm); 
            delivery.save();
            if (orderForm.getOrderId() == null) {
                getSessionData().setUserMessage("Thanks for your order! Your order # is:" + delivery.getHandle());
            } else {
                getSessionData().setUserMessage("Your changes to order #:" + delivery.getHandle() + " have been saved.");
            }
            getSessionData().setOrderForm(null);
            throw new ClientPageRedirectException(AirSentConstants.ORDERSTEP1_PO);
        //We need  to allow AirSent_pres to be functional , responsed will be default HTML page        
        }catch(NullPointerException e){
         throw new ClientPageRedirectException(AirSentConstants.ORDERSTEP1_PO);
  
        } catch (AirSentException ex) {
            throw new AirSentPresentationException("Exception logging in order: ", ex);
        } 
    } 

    /**
     * process the cancel event
     *     
     * @return html page
     * @exception if an error occurs
     */
    public XMLObject handleCancel() {
        getSessionData().setOrderForm(null);
        throw new ClientPageRedirectException(AirSentConstants.ORDERSTEP1_PO);
    } 
}



