/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: Register.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.presentation.customer;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;
import com.lutris.airsent.spec.AirSentException;
import com.lutris.airsent.presentation.BasePO;
import com.lutris.airsent.presentation.AirSentPresentationException;
import com.lutris.airsent.presentation.AirSentConstants;
import com.lutris.airsent.spec.HomeManager;


import com.lutris.airsent.spec.customer.Customer;
import com.lutris.airsent.spec.address.*;

import java.util.*;

/**
 * The customer register page of the AirSent app
 * 
 */
public class Register extends BasePO {

    /**
     * Constants
     */
    private static final int AUTH_LEVEL = AirSentConstants.UNAUTH_USER;
    String		     _login = null;
    String		     _password = null;
    String		     _firstname = null;
    String		     _lastname = null;
    String		     _repassword = null;
    String		     _email = null;
    String		     _postal = null;
    String		     _street = null;
    String		     _phone = null;
    String                   _company = null;
    String                   _city = null;

    /**
     * Superclass method override.
     * returns the authorization level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * Default event. Just show the page.
     */
    public XMLObject handleDefault() throws HttpPresentationException {
	return showPage(null);
    } 

    /**
     * Shows the register page
     * @param error messages
     * @return XMLObject page.
     */
    public XMLObject showPage(String errorMsg) 
        throws AirSentPresentationException {
	RegisterHTML page = (RegisterHTML) myComms.xmlcFactory.create(RegisterHTML.class);
	try {
	    initCustomer(page);
	    if (null != errorMsg || null != (errorMsg = getSessionData().getAndClearUserMessage())) {
		page.setTextErrorText(errorMsg);
	    } else {
		page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
	    } 
            populateForm(page);
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error showing page", e);
	} 
	return page;
    } 

    /**
     * Handles the registration.
     * @return the page.
     */
    public XMLObject handleRegister() 
        throws HttpPresentationException {
        try {
            getParameters();
            Customer user = getSessionData().getCustomer();
            boolean virgin = true;
            if (user != null) {
                virgin = false;
            }

            String error = validateForm(user);
            if (error != null) {
                return showPage(error);
            } 
            
            if (_repassword != null && !_repassword.equals(_password)) {
                return showPage("Please make sure your password and password confirmation match exactly");
            } 
            
            HomeManager hf = getApplication().getHomeManager();
            if (virgin == true) {
                user = hf.getCustomerManager().findByLogin(_login);
                if (user != null) {
                    return showPage("The login name " + _login + " is already in use.");    
                } else {
                    user = hf.getCustomerManager().create();
                }
            }
            
            if (virgin == true) {
                user.setLogin(_login);
		user.setPassword(_password);
	    }
	    user.setFirstName(_firstname);
	    user.setLastName(_lastname);
	    user.setEmail(_email);
	    user.setBusiness(_company);
            Address address = user.getAddress();
	    address.setPostalCode(_postal);
	    address.setStreet1(_street);
	    address.setCity(_city);
	    address.setLocalNumber(_phone);
	    user.save();

            getSessionData().setCustomer(user);
            getSessionData().setUserAuth(AirSentConstants.CUSTOMER_USER);
            getSessionData().setUserMessage("Welcome, " + user.getFirstName());
            throw new ClientPageRedirectException(AirSentConstants.ORDERSTEP1_PO);
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run AirSent_pres ) 
 * We need  to allow AirSent_pres to be functional ,  response
 * will be default HTML page with message
 *	
 */	
	  }catch(NullPointerException e){
	  	return showPage("You cannot register for an Account while running AirSent_pres") ;
            
	} catch (Exception e) {
	    throw new AirSentPresentationException("Exception logging in user: ", e);
	} 
    } 

    /**
     *
     */
    protected void initCustomer(RegisterHTML page)
        throws AirSentPresentationException {

	try {
            Customer customer = getSessionData().getCustomer();
	    if(customer != null) {
                _login = customer.getLogin();
		_password = customer.getPassword();
		_firstname = customer.getFirstName();
		_lastname = customer.getLastName();
		_repassword = customer.getPassword();
		_email = customer.getEmail();
		_company = customer.getBusiness();
		_postal = customer.getAddress().getPostalCode();
		_street = customer.getAddress().getStreet1();
		_phone = customer.getAddress().getLocalNumber();
		_city = customer.getAddress().getCity();
            }
        } catch (AirSentException ex) {
            throw new AirSentPresentationException("Exception initializing customer: ", ex);
        } 
    } 

    /**
     * Populates the form.
     * 
     */
    public void populateForm(RegisterHTML page) {
        Customer customer = getSessionData().getCustomer();

	if ((_login != null) && (customer != null)) {
            // if customer exists in session, then poulate MyAccount
            page.setTextTitle("My Account");
            page.getElementLogin().getParentNode().removeChild(page.getElementLogin());
	    page.getElementPassword().getParentNode().removeChild(page.getElementPassword());
	    page.getElementPasswordverify().getParentNode().removeChild(page.getElementPasswordverify());
            page.getElementCancelBtn().setHref(AirSentConstants.ORDERSTEP1_PO);
            page.setTextLoginText(_login);
	    page.setTextPasswordText(_password);
	    page.setTextPasswordverifyText(_password);
            page.setTextTableHeader("My Account"); 
        } else {
            
	    // assume new registration or editing bad submittal
            page.getElementLoginText().getParentNode().removeChild(page.getElementLoginText());
	    page.getElementLogin().setValue(_login);
	    page.getElementPasswordText().getParentNode().removeChild(page.getElementPasswordText());
	    page.getElementPassword().setValue(_password);
	    page.getElementPasswordverifyText().getParentNode().removeChild(page.getElementPasswordverifyText());
	     page.getElementPasswordverify().setValue(_password);
	    
	}
	
	if(_firstname != null) {
	    page.getElementFirstname().setValue(_firstname);
        }
	if(_lastname != null) {
	    page.getElementLastname().setValue(_lastname);
        }
	if(_repassword != null) {
	    page.getElementPasswordverify().setValue(_repassword);
	 }
	if(_email != null) {
	    page.getElementEmail().setValue(_email);
        }
	if(_postal != null) {
	    page.getElementPostalcode().setValue(_postal);
        }
	if(_street != null) {
	    page.getElementStreet1().setValue(_street);
        }
	if(_phone != null) {
	    page.getElementPhone().setValue(_phone);
        }
	if(_city != null) {
	    page.getElementCity().setValue(_city);
        }
	if(_company != null) {
	    page.getElementCompany().setValue(_company);
        }

    } 
    
    /**
     * Validates the form data.
     * @retutn error String
     */
    public String validateForm(Customer user) {

        if (user == null) {

            if (isNullField(_login)) {
                return "Please provide a login";
            } 
	    
	    if (isNullField(_password)) {
		return "Please provide a password";
	    } 
	    
	    if (isNullField(_repassword)) {
		return "Please provide a password verification";
	    } 
	    
        } 

	if (isNullField(_firstname)) {
	    return "Please provide a first name";
	}
	
	if (isNullField(_lastname)) {
	    return "Please provide a last name";
	} 
	
	if (isNullField(_email)) {
	    return "Please provide an email";
	} 

	if (isNullField(_postal)) {
	    return "Please provide a postal code";
	} 

	if (isNullField(_street)) {
	    return "Please provide a street address";
	} 

	if (isNullField(_phone)) {
	    return "Please provide a phone number";
	} 

	if (isNullField(_company)) {
	    return "Please provide a company name";
	} 

	if (isNullField(_city)) {
	    return "Please provide a city";
	}

	return null;
    } 

    /**
     *
     */
    private void getParameters() 
        throws AirSentPresentationException {
        
        try {
            _login = this.getComms().request.getParameter(AirSentConstants.LOGIN_NAME);
            _password = this.getComms().request.getParameter(AirSentConstants.PASSWORD_NAME);
            _firstname = this.getComms().request.getParameter(AirSentConstants.FIRST_NAME);
            _lastname = this.getComms().request.getParameter(AirSentConstants.LAST_NAME);
            _repassword = this.getComms().request.getParameter(AirSentConstants.PASSWORD_VERIFY_NAME);
            _email = this.getComms().request.getParameter(AirSentConstants.EMAIL_NAME);
            _postal = this.getComms().request.getParameter(AirSentConstants.POSTAL_CODE_NAME);
            _street = this.getComms().request.getParameter(AirSentConstants.STREET1_NAME);
            _phone = this.getComms().request.getParameter(AirSentConstants.PHONE_NAME);
            _company = this.getComms().request.getParameter(AirSentConstants.COMPANY_NAME);
            _city = this.getComms().request.getParameter(AirSentConstants.CITY_NAME);
        } catch (Exception e) {
            throw new AirSentPresentationException("ERROR: getting parameters", e);            
        }
    }
}




