/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: Details.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.presentation.messenger;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
import org.enhydra.xml.xmlc.XMLCUtil;
import org.enhydra.xml.xmlc.XMLObject;
import org.w3c.dom.*;
import java.util.Date;
import com.lutris.airsent.presentation.AirSentPresentationException;
import com.lutris.airsent.presentation.AirSentConstants;
import com.lutris.airsent.presentation.DeviceBasePO;
import com.lutris.airsent.spec.delivery.Delivery;
import com.lutris.airsent.presentation.DeviceUtils;
import java.lang.reflect.Method;

/**
 * Details.java handles the delivery details of the messenger portion
 * of the app.
 */
public class Details extends DeviceBasePO {

    /**
     * Constants
     */
    private static final int AUTH_LEVEL = AirSentConstants.UNAUTH_USER;

    /**
     * Superclass method override.
     * returns the authorization level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * Default event. Just show the page.
     * 
     * @return wml document
     * @exception HttpPresentationException
     */
    public XMLObject handleDefault() throws HttpPresentationException {
	return showPage(null);
    } 

    /**
     * Displays the Delivery Details page
     * 
     * @param errorMsg, the error messages
     * @return wml document
     * @exception HttpPresentationException
     */
    public XMLObject showPage(String errorMsg) 
	    throws HttpPresentationException {
  
  Class tempClass=null;
    	Object page= null;
    	Method setTextErrorText = null;
    	Method setTextName = null;
    	Method setTextAddress = null;
    	Method setTextDirections = null;
    	Method getTagComplete = null;
      
    	try{
      	Class stringClass=Class.forName("java.lang.String");
      	tempClass = Class.forName(DeviceUtils.getPageName(myComms, "com.lutris.airsent.presentation.messenger.Details")); 
        page= myComms.xmlcFactory.create(tempClass);
        Class[] argTypeArr={stringClass};
        setTextErrorText= tempClass.getMethod("setTextErrorText",argTypeArr);
        setTextName= tempClass.getMethod("setTextName",argTypeArr);
        setTextAddress= tempClass.getMethod("setTextAddress",argTypeArr);
        setTextDirections= tempClass.getMethod("setTextDirections",argTypeArr);
        getTagComplete = tempClass.getMethod("getElementComplete",null);
      }catch(Exception e){}
  
	//DetailsPage page = 
	    //(DetailsPage) create("com.lutris.airsent.presentation.messenger.Details");
try{
	if (null != errorMsg 
		|| null 
		   != (errorMsg = 
		       this.getSessionData().getAndClearUserMessage())) {
	    Object [] temp={errorMsg};
	    setTextErrorText.invoke(page,temp);
	    //page.setTextErrorText(errorMsg);
	    
	    return ((XMLObject)page);
	} else {
	    Object [] temp={""};
	    setTextErrorText.invoke(page,temp);
	    //page.setTextErrorText("");
	} 
}catch(Exception e){}
	String   pageSelection = 
	    myComms.request.getParameter(AirSentConstants.SELECTION_NAME);
	String   deliveryID = 
	    myComms.request.getParameter(AirSentConstants.DELIVERYID);
	String   name = null;
	String   address = null;
	String   directions = null;
	Delivery delivery = null;

	try {
	    if ((delivery = 
		 getApplication().getHomeManager().getDeliveryManager().findByHandle(deliveryID)) 
		    == null) {
		return showPage("Could not find delivery");
	    } 

	    if (pageSelection == null) {
		return showPage("Error in pickup or delivery");
	    } 

	    if (pageSelection.equals(AirSentConstants.PICKUP)) {
		name = delivery.getPickUp().getName();
		address = delivery.getPickUp().getStreet1();
		directions = delivery.getPickUp().getDirections();
	    } else {
		name = delivery.getDropOff().getName();
		address = delivery.getDropOff().getStreet1();
		directions = delivery.getDropOff().getDirections();
	    } 
	    Object [] temp1={name};
	    setTextName.invoke(page,temp1);
	    //page.setTextName(name);
	    Object [] temp2={address};
	    setTextAddress.invoke(page,temp2);
	    //page.setTextAddress(address);
	    Object [] temp3={directions};
	    setTextDirections.invoke(page,temp3);
	    //page.setTextDirections(directions);
	    //page.getTagComplete().getAttributeNode("href").setValue(AirSentConstants.DISPLAY_PO 
  		((Element)getTagComplete.invoke(page,null)).getAttributeNode("href").setValue(AirSentConstants.DISPLAY_PO 
  					      + "?event=complete" + "&" 
  					      + "deliveryID=" 
  					      + delivery.getHandle() 
  					      + "&" 
  					      + AirSentConstants.SELECTION_NAME 
  					      + "=" + pageSelection);
	    
	} catch (Exception e) {
	    throw new AirSentPresentationException("System error showing page", e);
	} 
	DeviceUtils.setURLTimeStamp((XMLObject)page);
	return ((XMLObject)page);
    } 

}



