/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: Display.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.presentation.messenger;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
import org.enhydra.xml.xmlc.XMLCUtil;
import org.w3c.dom.*;
import org.enhydra.xml.xmlc.XMLObject;
import java.util.Date;
import com.lutris.airsent.presentation.AirSentPresentationException;
import com.lutris.airsent.presentation.AirSentConstants;
import com.lutris.airsent.presentation.DeviceBasePO;
import com.lutris.airsent.presentation.DeviceUtils;
import org.enhydra.wireless.wml.dom.WMLAElement;
import com.lutris.airsent.spec.delivery.Delivery;
import com.lutris.airsent.presentation.customer.EmailConfirmationHTML;
import java.lang.reflect.Method;

/**
 * Display.java handles the displaying of a messenger's
 * pickups and dropoffs as well as handling the actual
 * pickup and delivery and confirmation email.
 *
 * @author
 * @version
 */
public class Display extends DeviceBasePO {

    /**
     * Constants
     */
    private static final int AUTH_LEVEL = AirSentConstants.UNAUTH_USER;

    /**
     * Superclass method override.
     * returns the authorization level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * Default event. Just show the page.
     * 
     * @return XMLOBject document
     * @exception HttpPresentationException
     */
    public XMLObject handleDefault() throws HttpPresentationException {
	
	return showPage(null);
    } 

    /**
     * Displays the Display page.
     * 
     * @param errorMsg, the error messages
     * @return wml document
     * @exception HttpPresentationException
     */
    public XMLObject showPage(String errorMsg) 
	    throws HttpPresentationException {


    	Class tempClass=null;
    	Object page= null;
    	Method setTextErrorText = null;
      Method getTextErrorText = null;
      Method getTagDisplayTemplateName = null;
      Method getTagDisplayTemplateComplete = null;
      Method getTagDisplayTemplateRow = null;
      Method setTextNameField = null;
      Method setTextDisplayTemplateName = null;
      Method setTextDisplayTemplateComplete = null;
      
      
    	try{
     	Class stringClass=Class.forName("java.lang.String");
      	tempClass = Class.forName(DeviceUtils.getPageName(myComms, "com.lutris.airsent.presentation.messenger.Display")); 
        page= myComms.xmlcFactory.create((Class)tempClass);
        Class[] argTypeArr={stringClass};
        setTextErrorText= tempClass.getMethod("setTextErrorText",argTypeArr);
        setTextNameField= tempClass.getMethod("setTextNameField",argTypeArr);
        setTextDisplayTemplateName= tempClass.getMethod("setTextDisplayTemplateName",argTypeArr);
        setTextDisplayTemplateComplete= tempClass.getMethod("setTextDisplayTemplateComplete",argTypeArr);
        getTextErrorText = tempClass.getMethod("getElementErrorText",null);
        getTagDisplayTemplateName = tempClass.getMethod("getElementDisplayTemplateName",null);
        getTagDisplayTemplateComplete = tempClass.getMethod("getElementDisplayTemplateComplete",null);
        getTagDisplayTemplateRow = tempClass.getMethod("getElementDisplayTemplateRow",null);
      }catch(Exception e){}
  
  
//	DisplayPage page = 
	//    (DisplayPage) create("com.lutris.airsent.presentation.messenger.Display");
try{
	if (null != errorMsg 
		|| null 
		   != (errorMsg = 
		       this.getSessionData().getAndClearUserMessage())) {
	    Object [] temp={errorMsg};
	    setTextErrorText.invoke(page,temp);
	    //page.setTextErrorText(errorMsg);
	} else {
	    Object [] temp={""};
	    setTextErrorText.invoke(page,temp);
	    //page.setTextErrorText("");
	} 
}catch(Exception e){}  
	//handleSelection(page);	
	
		String pageSelection = 
	    myComms.request.getParameter(AirSentConstants.SELECTION_NAME);
       
	int pickupCount = 0;
		
	try {
	    Delivery[] deliveryList = getApplication().getHomeManager().getDeliveryManager().findByMessenger(getSessionData().getMessenger());
	    Element    templateName = (Element)getTagDisplayTemplateName.invoke(page,null);
	    //Element    templateName = page.getTagDisplayTemplateName();
	    Element    templateLink = (Element)getTagDisplayTemplateComplete.invoke(page,null);
	    //Element    templateLink = page.getTagDisplayTemplateComplete();
	    Element    templateRow = (Element)getTagDisplayTemplateRow.invoke(page,null);
	    //Element    templateRow = page.getTagDisplayTemplateRow();
	    
	    // Remove ids to prevent duplicates
	    templateName.removeAttribute("id");
	    templateLink.removeAttribute("id");
	    templateRow.removeAttribute("id");
	    
	    Delivery currentDelivery = null;
	    // Get collection of Deliveries and loop through collection
	    // to add each disc as a row in the table.
	    for (int numDeliveries = 0; numDeliveries < deliveryList.length; 
		 numDeliveries++) {
  		
		currentDelivery = deliveryList[numDeliveries];
		
		if (pageSelection.equals(AirSentConstants.PICKUP)) {
		    Object [] temp1={AirSentConstants.PICKUP};
		    setTextNameField.invoke(page,temp1);
		    //page.setTextNameField(AirSentConstants.PICKUP);

  		    // Check to see if this has already been picked up:    
		    if (!currentDelivery.isPickedUp()) {
			pickupCount++;			
			Object [] temp2={"PickUps"};
		  setTextNameField.invoke(page,temp2);
			//page.setTextNameField("PickUps");
			Object [] temp3={currentDelivery.getPickUp().getName() + "(" + 
							currentDelivery.getHandle() + ")"};
		  setTextDisplayTemplateName.invoke(page,temp3);
			//page.setTextDisplayTemplateName(currentDelivery.getPickUp().getName() + "(" + 
				//			currentDelivery.getHandle() + ")");
			Object [] temp4={"Done"};
		  setTextDisplayTemplateComplete.invoke(page,temp4);
			//page.setTextDisplayTemplateComplete("Done");
			templateName.getAttributeNode("href").setValue(AirSentConstants.DETAILS_PO 
								       + "?deliveryID=" 
								       + currentDelivery.getHandle() 
								       + "&" 
								       + AirSentConstants.SELECTION_NAME 
								       + "=" + pageSelection);
			templateLink.getAttributeNode("href").setValue(AirSentConstants.DISPLAY_PO 
								       + "?event=complete" + "&" 
								       + "deliveryID=" 
								       + currentDelivery.getHandle() 
								       + "&" 
								       + AirSentConstants.SELECTION_NAME 
								       + "=" + pageSelection);
			// Add a deep clone of the table row to the DOM
			Node clonedNode = templateRow.cloneNode(true);
			templateRow.getParentNode().appendChild(clonedNode);
		    }
		} else {
		    Object [] temp5={"DropOffs"};
		    setTextNameField.invoke(page,temp5);
		    //page.setTextNameField("DropOffs");
		    Object [] temp6={currentDelivery.getDropOff().getName() + "(" + 
						    currentDelivery.getHandle() + ")"};
		    setTextDisplayTemplateName.invoke(page,temp6);
		    //page.setTextDisplayTemplateName(currentDelivery.getDropOff().getName() + "(" + 
					//	    currentDelivery.getHandle() + ")");
		    Object [] temp7={"Done"};
		    setTextDisplayTemplateComplete.invoke(page,temp7);
		    //page.setTextDisplayTemplateComplete("Done");
		    templateName.getAttributeNode("href").setValue(AirSentConstants.DETAILS_PO 
								   + "?deliveryID=" 
								   + currentDelivery.getHandle() 
								   + "&" 
								   + AirSentConstants.SELECTION_NAME 
								   + "=" + pageSelection);
		    templateLink.getAttributeNode("href").setValue(AirSentConstants.DISPLAY_PO 
								   + "?event=complete" + "&" 
								   + "deliveryID=" 
								   + currentDelivery.getHandle() 
								   + "&" 
								   + AirSentConstants.SELECTION_NAME 
								   + "=" + pageSelection);
		    // Add a deep clone of the table row to the DOM
		    Node clonedNode = templateRow.cloneNode(true);
		    templateRow.getParentNode().appendChild(clonedNode);
		}
 	    } 
	    
	    /**No deliveries, just put a 'None'
	     **button.
	     **
	     **/
	    if(deliveryList.length > 0) {
		
		if(pageSelection.equals(AirSentConstants.PICKUP) && pickupCount == 0) {
		    Object [] temp8={"None"};
		    setTextDisplayTemplateName.invoke(page,temp8);
		    //page.setTextDisplayTemplateName("None");
		    templateName.getAttributeNode("href").setValue(AirSentConstants.MESSENGER_MAIN_PAGE); 
		    templateLink.getParentNode().removeChild(templateLink);
		}
		else {
		    templateRow.getParentNode().removeChild(templateRow);
		}
	    }
	    else {
		Object [] temp9={"None"};
		setTextDisplayTemplateName.invoke(page,temp9);
		//page.setTextDisplayTemplateName("None");
		templateName.getAttributeNode("href").setValue(AirSentConstants.MESSENGER_MAIN_PAGE); 
		templateLink.getParentNode().removeChild(templateLink);
	    }
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Error getting deliveries for user: ", 
						   ex);
	} 
	
	
	DeviceUtils.setURLTimeStamp((XMLObject)page);
	return ((XMLObject)page);
    } 

    /**
     * Handles the selection from the
     * main page, and based on the query string,
     * displays the list of pickups, or the list
     * of deliveries.
     * @param page
     * 
     * @throws HttpPresentationException
     * 
     * @see
     */

/**
    public void handleSelection(DisplayPage page) 
	    throws HttpPresentationException {
	String pageSelection = 
	    myComms.request.getParameter(AirSentConstants.SELECTION_NAME);
       
	int pickupCount = 0;
		
	try {
	    Delivery[] deliveryList = 
		getApplication().getHomeManager().getDeliveryManager().findByMessenger(getSessionData().getMessenger());
	    Element    templateName = page.getTagDisplayTemplateName();
	    Element    templateLink = page.getTagDisplayTemplateComplete();
	    Element    templateRow = page.getTagDisplayTemplateRow();
	    
	    // Remove ids to prevent duplicates
	    templateName.removeAttribute("id");
	    templateLink.removeAttribute("id");
	    templateRow.removeAttribute("id");
	    
	    Delivery currentDelivery = null;
	    // Get collection of Deliveries and loop through collection
	    // to add each disc as a row in the table.
	    for (int numDeliveries = 0; numDeliveries < deliveryList.length; 
		 numDeliveries++) {
  		
		currentDelivery = deliveryList[numDeliveries];
		
		if (pageSelection.equals(AirSentConstants.PICKUP)) {
		    page.setTextNameField(AirSentConstants.PICKUP);

  		    // Check to see if this has already been picked up:    
		    if (!currentDelivery.isPickedUp()) {
			pickupCount++;			
			page.setTextNameField("PickUps");
			page.setTextDisplayTemplateName(currentDelivery.getPickUp().getName() + "(" + 
							currentDelivery.getHandle() + ")");
			page.setTextDisplayTemplateComplete("Done");
			templateName.getAttributeNode("href").setValue(AirSentConstants.DETAILS_PO 
								       + "?deliveryID=" 
								       + currentDelivery.getHandle() 
								       + "&" 
								       + AirSentConstants.SELECTION_NAME 
								       + "=" + pageSelection);
			templateLink.getAttributeNode("href").setValue(AirSentConstants.DISPLAY_PO 
								       + "?event=complete" + "&" 
								       + "deliveryID=" 
								       + currentDelivery.getHandle() 
								       + "&" 
								       + AirSentConstants.SELECTION_NAME 
								       + "=" + pageSelection);
			// Add a deep clone of the table row to the DOM
			Node clonedNode = templateRow.cloneNode(true);
			templateRow.getParentNode().appendChild(clonedNode);
		    }
		} else {
		    page.setTextNameField("DropOffs");
		    page.setTextDisplayTemplateName(currentDelivery.getDropOff().getName() + "(" + 
						    currentDelivery.getHandle() + ")");
		    page.setTextDisplayTemplateComplete("Done");
		    templateName.getAttributeNode("href").setValue(AirSentConstants.DETAILS_PO 
								   + "?deliveryID=" 
								   + currentDelivery.getHandle() 
								   + "&" 
								   + AirSentConstants.SELECTION_NAME 
								   + "=" + pageSelection);
		    templateLink.getAttributeNode("href").setValue(AirSentConstants.DISPLAY_PO 
								   + "?event=complete" + "&" 
								   + "deliveryID=" 
								   + currentDelivery.getHandle() 
								   + "&" 
								   + AirSentConstants.SELECTION_NAME 
								   + "=" + pageSelection);
		    // Add a deep clone of the table row to the DOM
		    Node clonedNode = templateRow.cloneNode(true);
		    templateRow.getParentNode().appendChild(clonedNode);
		}
 	    } 
    
	    if(deliveryList.length > 0) {
		
		if(pageSelection.equals(AirSentConstants.PICKUP) && pickupCount == 0) {
		    page.setTextDisplayTemplateName("None");
		    templateName.getAttributeNode("href").setValue(AirSentConstants.MESSENGER_MAIN_PAGE); 
		    templateLink.getParentNode().removeChild(templateLink);
		}
		else {
		    templateRow.getParentNode().removeChild(templateRow);
		}
	    }
	    else {
		page.setTextDisplayTemplateName("None");
		templateName.getAttributeNode("href").setValue(AirSentConstants.MESSENGER_MAIN_PAGE); 
		templateLink.getParentNode().removeChild(templateLink);
	    }
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Error getting deliveries for user: ", 
						   ex);
	} 
    } 
*/	


    /**
     * Process either a pickup or a delivery by
     * using the query string to determine the
     * type and delivery number.
     * @return the Document as XMLObject
     */
    public XMLObject handleComplete() throws HttpPresentationException {
	String deliveryID = 
	    myComms.request.getParameter(AirSentConstants.DELIVERYID);
	String status = 
	    myComms.request.getParameter(AirSentConstants.SELECTION_NAME);
	
	try {
	    Delivery delivery = null;
	    if ( deliveryID != null && 
		 (delivery = getApplication().getHomeManager().getDeliveryManager().findByHandle(deliveryID)) 
		    != null) {
		if (status != null && status.equals(AirSentConstants.DROPOFF)) {
		    
		    if(!delivery.isPickedUp()) {
			this.getSessionData().setUserMessage("Need to pickup package first!");
			throw new ClientPageRedirectException(AirSentConstants.MESSENGER_MAIN_PAGE); 
		    }
		    
		    delivery.dropOff();
		    
		    // Generate an email message
		    // Send email confimation
		    //
		    String body = handleEmailConfirmation(delivery);
		    String subject = "Delivery notice from AirSent.";
		    String type = "text/html";
		    getApplication().getHomeManager().getEmailManager().send(delivery.getCustomer().getFirstName() 
									     + delivery.getCustomer().getLastName(), 
									     delivery.getCustomer().getEmail(), subject, body, 
									     type);
		   
		} else if (status != null && status.equals(AirSentConstants.PICKUP)) {
		    delivery.pickUp();
		} 
	    } else {
		this.getSessionData().setUserMessage("Error occured in completing delivery.");
		throw new ClientPageRedirectException(AirSentConstants.MESSENGER_MAIN_PAGE); 
	    } 
	    
	    throw new ClientPageRedirectException(AirSentConstants.MESSENGER_MAIN_PAGE);
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Error occured in completing delivery", 
						   ex);
	} 
    } 

    
    /**
     * Creates a personalized email confirmation.
     * @param error messages
     * @return HTML page as string.
     */
    public String handleEmailConfirmation(Delivery delivery) 
	    throws AirSentPresentationException {
	EmailConfirmationHTML page = 
	    (EmailConfirmationHTML) myComms.xmlcFactory.create(EmailConfirmationHTML.class);
	
	try {
	    if(delivery == null) System.out.println("DELIVERY IS NULL");
	    if(delivery.getCustomer() == null) System.out.println("CUSTOMER IS NULL");
	    page.setTextDescription(delivery.getDescription());
	    page.setTextName(delivery.getCustomer().getFirstName() + " " + 
			     delivery.getCustomer().getLastName());
	    page.setTextPickupname(delivery.getPickUp().getName());
	    page.setTextPickupaddress(delivery.getPickUp().getStreet1());
	    page.setTextPickuptime(delivery.getPickedUpTime());
	    page.setTextDropoffname(delivery.getDropOff().getName());
	    page.setTextDropoffaddress(delivery.getDropOff().getStreet1());
	    page.setTextDropofftime(delivery.getDroppedOffTime());
	    
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Error in email confimation", 
						   ex);
	} 
	
	//Write to text to insert into email.
	return page.toDocument();
    } 

}

