/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: MessengerLogin.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.presentation.messenger;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
import org.enhydra.xml.xmlc.XMLCUtil;
import org.w3c.dom.*;
import org.enhydra.xml.xmlc.XMLObject;
import java.util.Date;
import com.lutris.airsent.presentation.AirSentPresentationException;
import com.lutris.airsent.presentation.AirSentConstants;
import com.lutris.airsent.presentation.DeviceBasePO;
import com.lutris.airsent.presentation.DeviceUtils;
import com.lutris.airsent.spec.messenger.Messenger;
import java.lang.reflect.Method;

/**
 * MessengerLogin.java handles the login functionality for 
 * the messenger of the AirSent app.
 * 
 * @author
 * @version
 */
public class MessengerLogin extends DeviceBasePO {

    /**
     * Constants
     */
    private static final int AUTH_LEVEL = AirSentConstants.UNAUTH_USER;

    /**
     * Superclass method override.
     * returns the authorization level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * Default event. Just show the page.
     * 
     * @return XMLObject document
     * @exception HttpPresentationException
     */
    public XMLObject handleDefault() throws HttpPresentationException {
	return showPage(null);
    } 

    /**
     * Displays the MessengerLogin page.
     * 
     * @param errorMsg, the error messages
     * @return wml document
     * @exception HttpPresentationException
     */
    public XMLObject showPage(String errorMsg) 
    
	    throws HttpPresentationException {
	Class tempClass=null;
	Object page= null;
	Method setTextErrorText = null;
  Method getTagErrorText = null;
  
	try{
  	Class stringClass=Class.forName("java.lang.String");
  	tempClass = Class.forName(DeviceUtils.getPageName(myComms, "com.lutris.airsent.presentation.messenger.MessengerLogin")); 
    page= myComms.xmlcFactory.create(tempClass);
    Class[] argTypeArr={stringClass};
    setTextErrorText = tempClass.getMethod("setTextErrorText",argTypeArr);
    getTagErrorText = tempClass.getMethod("getElementErrorText",null);
  }catch(Exception e){}
	
	
	
//	MessengerLoginPage page = 
	//    (MessengerLoginPage) create("com.lutris.airsent.presentation.messenger.MessengerLogin");

	try {
	      Element error = (Element)getTagErrorText.invoke(page,null);
	   // Element error = page.getTagErrorText();

	    if (null != errorMsg 
		    || null 
		       != (errorMsg = 
			   this.getSessionData().getAndClearUserMessage())) {
			    
		Object [] temp={errorMsg};
		setTextErrorText.invoke(page,temp);
		//page.setTextErrorText(errorMsg);
	    } else {
		Object [] temp={""};
		setTextErrorText.invoke(page,temp);
		//page.setTextErrorText("");
	    } 
      
	    // Add a jsession key for those who requset it:
	    // Like the java client app.
	    if (error.getAttributeNode("jsessionid") != null) {
		error.getAttributeNode("jsessionid").setValue(myComms.session.getSessionKey());
	    } 
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Error in messenger login", 
						   ex);
	} 
	DeviceUtils.setURLTimeStamp((XMLObject)page);
	return ((XMLObject)page);
    } 

    /**
     * Process login data
     * 
     * @return error String
     */
    public XMLObject handleLogin() throws HttpPresentationException {
	String    badge = 
	    myComms.request.getParameter(AirSentConstants.LOGIN_NAME);
	String    password = 
	    myComms.request.getParameter(AirSentConstants.PASSWORD_NAME);
	Messenger messenger = null;

	try {
	    if (badge == null || 
		password == null ||
		(messenger = getApplication().getHomeManager().getMessengerManager().validatePassword(badge, password)) == null) {
		return showPage("Invalid login or password!!");
	    } else {
		
		getSessionData().setMessenger(messenger);
		getSessionData().setUserAuth(AirSentConstants.MESSENGER_USER);
		throw new ClientPageRedirectException(AirSentConstants.MESSENGER_MAIN_PAGE);
	   
	    } 
	} catch (Exception ex) {
	    throw new AirSentPresentationException("System error logging in Messenger.", 
						   ex);
	} 

    } 

}

