/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: MessengerMain.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.presentation.messenger;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
import org.enhydra.xml.xmlc.XMLCUtil;
import org.enhydra.xml.xmlc.XMLObject;
import org.w3c.dom.*;
import java.util.Date;
import com.lutris.airsent.presentation.AirSentConstants;
import com.lutris.airsent.presentation.DeviceBasePO;
import com.lutris.airsent.presentation.DeviceUtils;
import com.lutris.airsent.presentation.AirSentPresentationException;
import java.lang.reflect.Method;

/**
 * MessengerMain.java is the Main Directory page for the Messenger portion
 * of the AirSent app. 
 *
 * @author
 * @version
 */
public class MessengerMain extends DeviceBasePO {

    /**
     * Constants
     */
    private static final int AUTH_LEVEL = AirSentConstants.UNAUTH_USER;

    /**
     * Superclass method override.
     * returns the authorization level necessary to access this po.
     * 
     * @return int required authorization level
     */
    public int getRequiredAuthLevel() {
	return AUTH_LEVEL;
    } 

    /**
     * Default event. Just show the page.
     * 
     * @return XMLObject document
     * @exception HttpPresentationException
     */
    public XMLObject handleDefault() throws HttpPresentationException {
	return showPage(null);
    } 

    /**
     * Displays the MessengerMain page.
     * 
     * @param errorMsg, the error messages
     * @return XMLObject document
     * @exception HttpPresentationException
     */
    public XMLObject showPage(String errorMsg) 
	    throws HttpPresentationException {
	Class tempClass=null;
	Object page= null;
	Method setTextErrorText = null;
  
	try{
  	Class stringClass=Class.forName("java.lang.String");
  	tempClass = Class.forName(DeviceUtils.getPageName(myComms, "com.lutris.airsent.presentation.messenger.MessengerMain")); 
    page= myComms.xmlcFactory.create(tempClass);
    Class[] argTypeArr={stringClass};
    setTextErrorText = tempClass.getMethod("setTextErrorText",argTypeArr);
  }catch(Exception e){}
  
//	MessengerMainPage page = 
	//    (MessengerMainPage) create("com.lutris.airsent.presentation.messenger.MessengerMain");

	try {
	    if (null != errorMsg 
		    || null 
		       != (errorMsg = 
			   this.getSessionData().getAndClearUserMessage())) {
		Object [] temp={errorMsg};
	  setTextErrorText.invoke(page,temp);
		//page.setTextErrorText(errorMsg);
	    } else {
	  Object [] temp={""};
	  setTextErrorText.invoke(page,temp);
		//page.setTextErrorText("");
	    } 
	} catch (Exception e) {
	    throw new AirSentPresentationException("Exception showing main page: ", 
						   e);
	} 
	DeviceUtils.setURLTimeStamp((XMLObject)page);
	return ((XMLObject)page);
    } 

}






