/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: Customer.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.spec.customer;

import com.lutris.airsent.spec.AirSentException;
import com.lutris.airsent.spec.address.Address;

/**
 * Interface Customer
 * 
 * 
 * @author joseph shoop
 * @version %I%, %G%
 */
public interface Customer extends java.io.Serializable {

    /**
     * maximum size for business 
     */
    public static final int MAX_BUSINESS = 64;

    /**
     * maximum size for login
     */
    public static final int MAX_LOGIN = 64;

    /**
     * maximum size for password
     */
    public static final int MAX_PASSWORD = 64;

    /**
     * maximum size for first name
     */
    public static final int MAX_FIRST_NAME = 32;

    /**
     * maximum size for last name
     */
    public static final int MAX_LAST_NAME = 32;

    /**
     * maximum size for email address
     */
    public static final int MAX_EMAIL = 32;

    /**
     * Gets the customer handle.
     * 
     * @return database id
     * @exception if an error occurs
     */
    public String getHandle() 
        throws AirSentException;

    /**
     * Gets the first name.
     *
     * @exception if an error occurs
     */
    public String getFirstName() 
        throws AirSentException;

    /**
     * Sets the first name.
     *
     * @exception if an error occurs
     */
    public void setFirstName(String first) 
        throws AirSentException;

    /**
     * Gets the last name.
     *
     * @return last name
     * @exception if an error occurs
     */
    public String getLastName() 
        throws AirSentException;

    /**
     * Set the last name.
     *
     * @param last last name
     * @exception if an error occurs
     */
    public void setLastName(String last) 
        throws AirSentException;

    /**
     * Gets the email.
     *
     * @return email address
     * @exception if an error occurs
     */
    public String getEmail() 
        throws AirSentException;

    /**
     * Sets the email.
     *
     * @param email email address
     * @exception if an error occurs
     */
    public void setEmail(String email) 
        throws AirSentException;

    /**
     * Gets the address.
     *
     * @return address
     * @exception if an error occurs
     */
    public Address getAddress() 
        throws AirSentException;

    /**
     * Gets the business.
     *
     * @return business
     * @exception if an error occurs
     */
    public String getBusiness() 
        throws AirSentException;

    /**
     * Sets the business.
     *
     * @param business business
     * @exception if an error occurs
     */
    public void setBusiness(String business) 
        throws AirSentException;

    /**
     * Gets the login.
     *
     * @return login name
     * @exception if an error occurs
     */
    public String getLogin() 
        throws AirSentException;

    /**
     * Sets the login.
     *
     * @param login login name
     * @exception if an error occurs
     */
    public void setLogin(String login) 
        throws AirSentException;

    /**
     * Gets the password.
     *
     * @return password
     * @exception if an error occurs
     */
    public String getPassword() 
        throws AirSentException;

    /**
     * Sets the password.
     *
     * @param password password
     * @exception if an error occurs
     */
    public void setPassword(String password) 
        throws AirSentException;

    /**
     * Saves the customer.
     *
     * @exception if an error occurs
     */
    public void save() 
        throws AirSentException;

    /**
     * Deletes the customer.
     *
     * @exception if an error occurs
     */
    public void delete() 
        throws AirSentException;
}

