/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: Delivery.java,v 1.1 2004/05/19 18:15:19 
 */

package com.lutris.airsent.spec.delivery;

import com.lutris.airsent.spec.AirSentException;
import com.lutris.airsent.spec.customer.Customer;
import com.lutris.airsent.spec.messenger.Messenger;
import com.lutris.airsent.spec.address.Address;


/**
 * Delivery interface.
 */
public interface Delivery {

    /**
     * Maximum size of the description field
     */
    public static final int MAX_DESC = 1000;

    /**
     * Maximum size of the description field
     */
    public static final int MAX_SIZE = 64;

    /**
     * Gets the handle.
     *
     * @return handle
     * @exception if an error occurs
     */
    public String getHandle() 
        throws AirSentException;

    /**
     * Gets pick up address for the order.
     *
     * @return pick up address
     * @exception if an error occurs
     */
    public Address getPickUp() 
        throws AirSentException;

    /**
     * Gets the drop off address for the order
     *
     * @return drop off address
     * @exception if an error occurs
     */
    public Address getDropOff() 
        throws AirSentException;

    /**
     * gets the customer
     * 
     * @return customer
     * @exception if an error occurs
     */
    public Customer getCustomer() 
        throws AirSentException;
    
    /**
     * Gets the messenger.
     *
     * @return messenger
     * @exception if an error occurs
     */
    public Messenger getMessenger() 
        throws AirSentException;

    /**
     * Sets the messenger.
     *
     * @exception if an error occurs
     * @exception if a business rule is violated
     */
    public void setMessenger(Messenger messenger) 
        throws AirSentException;

    /**
     * gets the fragile flag
     *
     * @return true if fragile
     * @exception if an error occurs
     */
    public boolean isFragile() 
        throws AirSentException;

    /**
     * sets the fragile flag
     *
     * @param fragile true if fragile
     * @exception if an error occurs
     */
    public void setIsFragile(boolean fragile) 
        throws AirSentException;

    /**
     * gets the urgent flag
     * 
     * @return true if urgent
     * @exception if an error occurs
     */
    public boolean isUrgent() 
        throws AirSentException;

    /**
     * sets the urgent flag
     * 
     * @param urgent true if urgent
     * @exception if an error occurs
     */
    public void setIsUrgent(boolean urgent) 
        throws AirSentException;

    /**
     * gets the size description
     * 
     * @return size
     * @exception if an error occurs
     */
    public String getSize() 
        throws AirSentException;

    /**
     * sets the size for the order
     *
     * @param size size description
     * @exception if an error occurs
     */
    public void setSize(String size) 
        throws AirSentException;

    /**
     * gets the descriton of the order
     *
     * @return descriotn
     * @exception if an error occurs
     */
    public String getDescription() 
        throws AirSentException;

    /**
     * sets the order description
     * 
     * @param description order description
     * @exception if an error occurs
     * @exception if the description is invalid
     */
    public void setDescription(String description) 
        throws AirSentException;

    /**
     * saves the order
     *
     * @exception if an error occurs
     */
    public void save() 
        throws AirSentException;

    /**
     * deletes the order
     *
     * @exception if an error occurs
     */
    public void delete() 
        throws AirSentException;

    /**
     * returns the pick up statues
     *
     * @return true if order has been pickup
     * @exception if an error occurs
     */
    public boolean isPickedUp() 
        throws AirSentException;

    /**
     * returns the drop off status
     * 
     * @return true if the order has been dropped off
     * @exception if an error occurs
     */
    public boolean isDroppedOff() 
        throws AirSentException;

    /**
     * Chnages the order to be 'picked up'
     *
     * @exception if an error occurs
     */
    public void pickUp() 
        throws AirSentException;

    /**
     * returns the pick up time
     *
     * @return pick up time 
     * @exception if an error occurs
     */
    public String getPickedUpTime() 
        throws AirSentException;

    /**
     * Changes the order to be in the 'drop off' state
     *
     * @exception if an error occurs
     */
    public void dropOff() 
        throws AirSentException;

    /**
     * returns the drop off time
     * 
     * @return drop off time  
     * @exception if an error occurs
     */
    public String getDroppedOffTime() 
        throws AirSentException;

    /*
     *Gets the invoice number 
     */
    public int getInvoice() 
        throws AirSentException;

    /*
     *Sets the invoice number 
     */
    public void setInvoice(int size) 
        throws AirSentException;
}

