/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: Messenger.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.spec.messenger;

import com.lutris.airsent.spec.AirSentException;
import com.lutris.airsent.spec.address.Address;

/**
 * Messenger interface
 */
public interface Messenger extends java.io.Serializable {

    /**
     * maximum size for badge
     */
    public static int MAX_BADGE = 32;

    /**
     * maximum size for geocode
     */
    public static int MAX_GEOCODE = 32;
    
    /**
     * maximum size for password
     */
    public static int MAX_PASSWORD = 32;
    
    /**
     * Get messengers database handle
     *
     * @return database id
     * @exception if an error occurs
     */
    public String getHandle() 
        throws AirSentException;

    /**
     * Get first name.
     *
     * @return first name
     * @exception if an error occurs
     */
    public String getFirstName() 
        throws AirSentException;

    /**
     * Set first name.
     *
     * @param first first name
     * @exception if an error occurs
     * @exception if business rules are violated
     */
    public void setFirstName(String first) 
        throws AirSentException;

    /**
     * Get last name.
     *
     * @return last name
     * @exception if an error occurs
     */
    public String getLastName() 
        throws AirSentException;

    /**
     * Set last name.
     * 
     * @param last
     * @exception if an error occurs
     * @exception if business rules are violated
     */
    public void setLastName(String last) 
        throws AirSentException;

    /**
     * Gets the address.
     *
     * @return address
     * @exception if an error occurs
     */
    public Address getAddress() 
        throws AirSentException;

    /**
     * Get badge.
     *
     * @return badge
     * @exception if an error occurs
     */
    public String getBadge() 
        throws AirSentException;

    /**
     * Set the badge
     *
     * @exception if an error occurs
     * @exception if business rules are violated
     */
    public void setBadge(String badge) 
        throws AirSentException;

    /**
     * Get password.
     *
     * @return password
     * @exception if an error occurs
     */
    public String getPassword() 
        throws AirSentException;

    /**
     * Set password.
     *
     * @exception if an error occurs
     * @exception if business rules are violated
     */
    public void setPassword(String password) 
        throws AirSentException;

    /**
     * Get email.
     *
     * @return email address
     * @exception if an error occurs
     */
    public String getEmail() 
        throws AirSentException;

    /**
     * Set Email.
     * 
     * @param email
     * @exception if an error occurs
     * @exception if business rules are violated
     */
    public void setEmail(String email) 
        throws AirSentException;

    /**
     * Gets the geocode. NOT currently used.
     *
     * @return code
     * @exception if an error occurs
     */
    public String getGeocode() 
        throws AirSentException;

    /**
     * Saves the messenger.
     *
     * @exception if an error occurs
     */
    public void save() 
        throws AirSentException;

    /**
     * Deletes the messenger.
     *
     * @exception if an error occurs
     */
    public void delete() 
        throws AirSentException;

}

