/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: MessengerManager.java,v 1.1 2004/08/16 09:33:18 slobodan Exp $
 */

package com.lutris.airsent.spec.messenger;

import com.lutris.airsent.spec.AirSentException;
import com.lutris.airsent.spec.messenger.Messenger;

/**
 * Messenger factory class.
 * 
 * 
 * @author
 * @version %I%, %G%
 */
public interface MessengerManager {

    /**
     * Creates an empty messenger.
     * 
     * 
     * @return Messenger
     * 
     * @throws AirSentException
     * 
     * 
     */
    public Messenger create() throws AirSentException;


    public Messenger create(String id) throws AirSentException;

    /**
     * Find a Messenger by its badge.
     * 
     * 
     * @param badge
     * 
     * @return Messenger
     * 
     * @throws AirSentException
     * 
     * 
     */
    public Messenger findByBadge(String badge) 
	    throws AirSentException;

    /**
     * Validate a messengers password.
     * 
     * 
     * @param badge
     * @param password
     * 
     * @return Messenger
     * 
     * @throws AirSentException
     * 
     * 
     */
    public Messenger validatePassword(String badge, String password) 
	    throws AirSentException;

    /**
     * Find a messenger by proximity using a geocode. NOT currently used.
     * 
     * 
     * @param geocode
     * 
     * @return Messenger
     * 
     * @throws AirSentException
     * 
     * 
     */
    public Messenger findByProximity(String geocode) 
	    throws AirSentException;

    /**
     * Get all the messengers.
     * 
     * 
     * @return Messenger[]
     * 
     * @throws AirSentException
     * 
     * 
     */
    public Messenger[] findAll() throws AirSentException;
}



