/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EventConfig.java,v 1.2 2006/03/29 15:55:20 alex Exp $
 */
package org.enhydra.barracuda.config;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.log4j.*;

import org.enhydra.barracuda.config.events.*;
import org.barracudamvc.core.comp.*;
import org.barracudamvc.core.event.*;
import org.barracudamvc.core.event.helper.*;
import org.barracudamvc.core.forms.*;
import org.barracudamvc.core.util.dom.*;
import org.barracudamvc.core.util.srv.*;
import org.barracudamvc.plankton.data.*;
import org.barracudamvc.plankton.http.*;

/**
 * This class provides the methods needed to configure the event
 * screen
 */
public class EventConfig extends DefaultEventGateway {

    protected static final Logger logger = Logger.getLogger(EventConfig.class.getName());

    //event id (must be unique)
    public static final String FORM = EventConfig.class.getName()+".Form";

    //model name
    public static final String MODEL_NAME = "Event";

    //model elements (these are the data items supported)
    public static final String APPLICATION_GATEWAY_DL = "ApplicationGateway_DebugLevel";
    public static final String APPLICATION_GATEWAY_USE_EVENT_POOLING = "ApplicationGateway_UseEventPooling";
    public static final String APPLICATION_GATEWAY_FORCE_GC = "ApplicationGateway_ForceGC";
    public static final String DEFAULT_BASE_EVENT_USE_ID_ALIASES = "DefaultBaseEvent_UseIDAliases";
    public static final String DEFAULT_DOM_WRITER_DPP_FALSE = "DefaultDOMWriter_DefaultPrintPretty_False";
    public static final String DEFAULT_BASE_EVENT_LISTENER_DL = "DefaultBaseEventListener_DebugLevel";
    public static final String DEFAULT_DISPATCH_QUEUE_DL = "DefaultDispatchQueue_DebugLevel";
    public static final String DEFAULT_EVENT_BROKER_DL = "DefaultEventBroker_DebugLevel";
    public static final String DEFAULT_EVENT_CONTEXT_DL = "DefaultEventContext_DebugLevel";
    public static final String DEFAULT_EVENT_DISPATCHER_DL = "DefaultEventDispatcher_DebugLevel";
    public static final String DEFAULT_EVENT_DISPATCHER_MAX_POLY = "DefaultEventDispatcher_MaxPolyChainDepth";
    public static final String DEFAULT_EVENT_DISPATCHER_MAX_DISPATCH = "DefaultEventDispatcher_MaxDispatchQueueDepth";
    public static final String DEFAULT_EVENT_GATEWAY_DL = "DefaultEventGateway_DebugLevel";
    public static final String DEFAULT_EVENT_POOL_DL = "DefaultEventPool_DebugLevel";
    public static final String DEFAULT_EVENT_POOL_CLEANUP_LOCKED_EVENTS = "DefaultEventPool_CleanupLockedEvents";
    public static final String EVENT_FORWARDING_FACTORY_DL = "EventForwardingFactory_DebugLevel";
    public static final String EVENT_REDIRECT_FACTORY_DL = "EventRedirectFactory_DebugLevel";
    public static final String ERROR_MESSAGE = "ErrorMessage";
    public static final String SUCCESS_MESSAGE = "SuccessMessage";
    public static final String UPDATE_BUTTON = "UpdateButton";

    //private vars
    private ListenerFactory updateConfigFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new UpdateConfigHandler();}        public String getListenerID() {return getID(UpdateConfigHandler.class);}};
    private ListenerFactory forceGCFactory =             new DefaultListenerFactory() {public BaseEventListener getInstance() {return new ForceGCHandler();}                public String getListenerID() {return getID(ForceGCHandler.class);}};                
    private ListenerFactory cleanupLockedEventsFactory = new DefaultListenerFactory() {public BaseEventListener getInstance() {return new CleanupLockedEventsHandler();}    public String getListenerID() {return getID(CleanupLockedEventsHandler.class);}};                

    public EventConfig() {
        //specify generic interest
        specifyLocalEventInterests(updateConfigFactory);
        specifyLocalEventInterests(forceGCFactory);
        specifyLocalEventInterests(cleanupLockedEventsFactory);
    }

    //------------------------------------------------------------
    //                       Data Models
    //------------------------------------------------------------
    /**
     * define the template model that backs this screen
     */
    class EventModel extends AbstractTemplateModel {

        //register the model by name
        public String getName() {return MODEL_NAME;}
        
        //provide items by key
        public Object getItem(String key) {
            if (logger.isDebugEnabled()) logger.debug("Asking for key:"+key);    
            ViewContext vc = getViewContext();

            //Handle requests for data
            if (key.equals(APPLICATION_GATEWAY_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, ApplicationGateway.class);
            } else if (key.equals(APPLICATION_GATEWAY_USE_EVENT_POOLING)) {
                return ScreenUtil.getToggleButton(vc, APPLICATION_GATEWAY_USE_EVENT_POOLING, "true", ApplicationGateway.USE_EVENT_POOLING);
            } else if (key.equals(APPLICATION_GATEWAY_FORCE_GC)) {
                return ScreenUtil.getActionLink(vc, forceGCFactory);
            } else if (key.equals(DEFAULT_BASE_EVENT_USE_ID_ALIASES)) {
                BToggleButton bcomp = ScreenUtil.getToggleButton(vc, DEFAULT_BASE_EVENT_USE_ID_ALIASES, "true", DefaultBaseEvent.USE_ID_ALIASES);
                //because this is a read-only, we want to disable this field 
                //(this doesn't currently work in NN)
                bcomp.setEnabled(false);
                return bcomp;                
            } else if (key.equals(DEFAULT_BASE_EVENT_LISTENER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultBaseEventListener.class);
            } else if (key.equals(DEFAULT_DISPATCH_QUEUE_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultDispatchQueue.class);
            } else if (key.equals(DEFAULT_EVENT_BROKER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultEventBroker.class);
            } else if (key.equals(DEFAULT_EVENT_CONTEXT_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultEventContext.class);
            } else if (key.equals(DEFAULT_EVENT_DISPATCHER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultEventDispatcher.class);
            } else if (key.equals(DEFAULT_EVENT_DISPATCHER_MAX_POLY)) {
                DefaultListModel dlm = new DefaultListModel();
                for (int i=0; i<101; i++) dlm.add(new Integer(i));
                return ScreenUtil.getSelectComp(vc, key, dlm, DefaultEventDispatcher.MAX_POLY_CHAIN_DEPTH);
            } else if (key.equals(DEFAULT_EVENT_DISPATCHER_MAX_DISPATCH)) {
                DefaultListModel dlm = new DefaultListModel();
                for (int i=0; i<101; i++) dlm.add(new Integer(i));
                //because this is a read-only, we want to disable this field 
                //(this doesn't currently work in NN)
                BSelect bselect = ScreenUtil.getSelectComp(vc, key, dlm, DefaultEventDispatcher.MAX_DISPATCH_QUEUE_DEPTH);
                bselect.setEnabled(false);
                return bselect;
            } else if (key.equals(DEFAULT_EVENT_GATEWAY_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultEventGateway.class);
            } else if (key.equals(DEFAULT_EVENT_POOL_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultEventPool.class);
            } else if (key.equals(DEFAULT_EVENT_POOL_CLEANUP_LOCKED_EVENTS)) {
                BAction baction = ScreenUtil.getActionLink(vc, cleanupLockedEventsFactory);
                baction.setEnabled(ApplicationGateway.USE_EVENT_POOLING);
//System.out.println("Here we are! Getting a new BAction component!");            
                return baction;
            } else if (key.equals(EVENT_FORWARDING_FACTORY_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, EventForwardingFactory.class);
            } else if (key.equals(EVENT_REDIRECT_FACTORY_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, EventRedirectFactory.class);
            } else if (key.equals(ERROR_MESSAGE)) {
                return ScreenUtil.getErrMsg(vc, FORM, ERROR_MESSAGE);
            } else if (key.equals(SUCCESS_MESSAGE)) {
                return ScreenUtil.getSuccessMsg(vc, FORM, SUCCESS_MESSAGE);
            } else if (key.equals(UPDATE_BUTTON)) {
                return ScreenUtil.getUpdateButtonComp(vc, updateConfigFactory);
            } else return super.getItem(key);
        }
    }

    //------------------------------------------------------------
    //                  Form Mappings, Validators
    //------------------------------------------------------------
    /**
     * define the event map that backs the model
     */
    class EventForm extends DefaultFormMap {
        public EventForm() {
            //define the elements (note: these don't need any validators). Note
            //also that we set the defaults to the current values. This allows
            //us just to reset all current values down below without checking to
            //see if the elements actually got passed in from the event.
            if (logger.isDebugEnabled()) logger.debug("Defining form elements");            
            this.defineElement(new DefaultFormElement(APPLICATION_GATEWAY_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtLevelToInt(ApplicationGateway.class)), null, false));
            this.defineElement(new DefaultFormElement(APPLICATION_GATEWAY_USE_EVENT_POOLING, FormType.BOOLEAN, new Boolean(false), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_BASE_EVENT_USE_ID_ALIASES, FormType.BOOLEAN, new Boolean(false), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_BASE_EVENT_LISTENER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtLevelToInt(DefaultBaseEventListener.class)), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_DISPATCH_QUEUE_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtLevelToInt(DefaultDispatchQueue.class)), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_EVENT_BROKER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtLevelToInt(DefaultEventBroker.class)), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_EVENT_CONTEXT_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtLevelToInt(DefaultEventContext.class)), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_EVENT_DISPATCHER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtLevelToInt(DefaultEventDispatcher.class)), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_EVENT_DISPATCHER_MAX_POLY, FormType.INTEGER, new Integer(DefaultEventDispatcher.MAX_POLY_CHAIN_DEPTH), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_EVENT_DISPATCHER_MAX_DISPATCH, FormType.INTEGER, new Integer(DefaultEventDispatcher.MAX_DISPATCH_QUEUE_DEPTH), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_EVENT_GATEWAY_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtLevelToInt(DefaultEventGateway.class)), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_EVENT_POOL_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtLevelToInt(DefaultEventPool.class)), null, false));
            this.defineElement(new DefaultFormElement(EVENT_FORWARDING_FACTORY_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtLevelToInt(EventForwardingFactory.class)), null, false));
            this.defineElement(new DefaultFormElement(EVENT_REDIRECT_FACTORY_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtLevelToInt(EventRedirectFactory.class)), null, false));
        }
    }

    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * UpdateConfigHandler - handle the request to update the config 
     * screen.
     */
    class UpdateConfigHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //figure out our target locale and get the appropriate screen
            Locale locale = context.getViewCapabilities().getClientLocale();
            MasterScreen screen = new MasterScreenFactory().getInstance(EventConfig.this, context, locale);
            if (logger.isDebugEnabled()) ServletUtil.showParams(context.getRequest(), logger);            

            //create the login event
            ValidationException ve = null;
            EventForm formMap = new EventForm();
            try {
                //map/validate the event
                formMap.map(context.getRequest()).validate(true);
                
                //If there were no errors, update the data. Note that we just
                //assume that all the data is here. The reason we can do this is
                //because we prepopulated the event up above with the default values.
                //The more proper way would be to check each value, see if it's set,
                //and then only update it if the value has actually changed. Lot more
                //code to do that though, and since we're just setting statics, the
                //cost of doing it this way is minimal.
                ScreenUtil.setLevel(ApplicationGateway.class, ((Integer)formMap.getVal(APPLICATION_GATEWAY_DL)).intValue());
                ApplicationGateway.USE_EVENT_POOLING = ((Boolean)formMap.getVal(APPLICATION_GATEWAY_USE_EVENT_POOLING)).booleanValue();
                ScreenUtil.setLevel(DefaultBaseEventListener.class, ((Integer)formMap.getVal(DEFAULT_BASE_EVENT_LISTENER_DL)).intValue());
                ScreenUtil.setLevel(DefaultDispatchQueue.class, ((Integer)formMap.getVal(DEFAULT_DISPATCH_QUEUE_DL)).intValue());
                ScreenUtil.setLevel(DefaultEventBroker.class, ((Integer)formMap.getVal(DEFAULT_EVENT_BROKER_DL)).intValue());
                ScreenUtil.setLevel(DefaultEventContext.class, ((Integer)formMap.getVal(DEFAULT_EVENT_CONTEXT_DL)).intValue());
                ScreenUtil.setLevel(DefaultEventDispatcher.class, ((Integer)formMap.getVal(DEFAULT_EVENT_DISPATCHER_DL)).intValue());
                DefaultEventDispatcher.MAX_POLY_CHAIN_DEPTH = ((Integer)formMap.getVal(DEFAULT_EVENT_DISPATCHER_MAX_POLY)).intValue();
                ScreenUtil.setLevel(DefaultEventGateway.class, ((Integer)formMap.getVal(DEFAULT_EVENT_GATEWAY_DL)).intValue());
                ScreenUtil.setLevel(DefaultEventPool.class, ((Integer)formMap.getVal(DEFAULT_EVENT_POOL_DL)).intValue());
                ScreenUtil.setLevel(EventForwardingFactory.class, ((Integer)formMap.getVal(EVENT_FORWARDING_FACTORY_DL)).intValue());
                ScreenUtil.setLevel(EventRedirectFactory.class, ((Integer)formMap.getVal(EVENT_REDIRECT_FACTORY_DL)).intValue());

                //remember our success
                formMap.putState(SUCCESS_MESSAGE, new Boolean(true));

            } catch (ValidationException e) {
                ve = e;
            }

            //store the error message in the event and then put the event in the
            //the event context
            formMap.putState(ERROR_MESSAGE, ve);
            context.putState(FORM, formMap);

            //fire an update so that the screen will redraw
            ((EventModel) screen.eventModel).fireModelChanged();

            //redirect to the get screen again
            throw new ClientSideRedirectException(new GetBConfig());
        }
    }

    /**
     * ForceGCHandler - Force a System.gc()
     */
    class ForceGCHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //release locked events
            logger.debug("Forcing System.gc();");
            System.gc();
            
            //redirect to the config screen
            throw new ClientSideRedirectException(new GetBConfig());
        }
    }

    /**
     * CleanupLockedEventsHandler
     */
    class CleanupLockedEventsHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //release locked events
            logger.debug("Looking for ApplicationGateway...");
            ApplicationGateway ag = (ApplicationGateway) SimpleServiceFinder.findInstance(ApplicationGateway.class, EventConfig.this);
            if (ag!=null) {
                logger.debug("Found it! Getting event pool and manually cleaning up locked events.");
                EventPool ep = ag.getEventPool();
                if (ep!=null) ep.cleanupLockedEvents();
            }

            //figure out our target locale and get the appropriate screen
            Locale locale = context.getViewCapabilities().getClientLocale();
            MasterScreen screen = new MasterScreenFactory().getInstance(EventConfig.this, context, locale);

            //fire an update so that the screen will redraw
            ((EventModel) screen.eventModel).fireModelChanged();
            
            //redirect to the config screen
            throw new ClientSideRedirectException(new GetBConfig());
        }
    }


    //------------------------------------------------------------
    //                         Utility Methods
    //------------------------------------------------------------
    public TemplateModel getModel() {
        return new EventModel();
    }
}

