/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MasterScreenFactory.java,v 1.2 2006/03/29 15:55:20 alex Exp $
 */
package org.enhydra.barracuda.config;

import java.io.*;
import java.util.*;
import java.lang.ref.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.config.events.*;
import org.enhydra.barracuda.config.xmlc.*;
import org.barracudamvc.core.comp.*;
import org.barracudamvc.core.util.dom.*;
import org.barracudamvc.core.event.*;
import org.barracudamvc.core.event.helper.*;
import org.barracudamvc.core.forms.*;
import org.barracudamvc.core.view.*;
import org.barracudamvc.core.util.http.*;
import org.barracudamvc.plankton.data.*;


/**
 * <p>This class is used to get an instance of the MasterScreen.
 * We don't use a static getInstance() method here because it needs
 * to use an inner class to act as a ReferenceFactory. Consequently,
 * you would use the factory like this: 
 *
 * <p>new MasterScreenFactory().getInstance(...);
 */
public class MasterScreenFactory {

    public static final String KEY = MasterScreenFactory.class.getName()+".Key";

    public MasterScreen getInstance(EventGateway eg, ControlEventContext context, Locale locale) {
        //get the appropriate screen for the desired context and locale
        ReferenceFactory rf = new LocalReferenceFactory(getRootGateway(eg), locale);
        MasterScreen screen = (MasterScreen) SessionServices.getObjectFromCache(context, KEY+locale, rf);
        return screen;
    }
    
    protected EventGateway getRootGateway(EventGateway eg) {
        EventGateway egParent = eg.getParent();
        if (egParent==null) return eg;
        else return getRootGateway(egParent);    
    }
    
    class LocalReferenceFactory implements ReferenceFactory {
        EventGateway egRoot = null;
        Locale locale = null;

        public LocalReferenceFactory(EventGateway iegRoot, Locale ilocale) {
            egRoot = iegRoot;
            locale = ilocale;
        }
        
        public Reference getObjectReference() {
            return new SoftReference(new MasterScreen(egRoot, locale));
        }
    }
}



