/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ScreenUtil.java,v 1.2 2006/03/29 15:55:20 alex Exp $
 */
package org.enhydra.barracuda.config;

import java.io.*;
import java.util.*;
import java.lang.ref.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.log4j.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.barracudamvc.core.comp.*;
import org.barracudamvc.core.event.*;
import org.barracudamvc.core.forms.*;
import org.barracudamvc.plankton.l10n.*;


/**
 * This class represents the master Barracuda config screen. It is stored 
 * in the users session on a per-locale basis, and is used kind of as a 
 * master repository for objects that make up the system.
 */
public class ScreenUtil {

    //public constants
    protected static final Logger logger = Logger.getLogger(ScreenUtil.class.getName());

    /**
     * This method greats a typical debug level BSelect component
     *
     * @param vc the view context
     * @param compName the component name
     * @param selectedIndex the index to be selected
     * @return the resulting BSelect component
     */    
    public static BSelect getDebugLevelComp(ViewContext vc, String compName, int selectedIndex) {
        DefaultListModel dlm = new DefaultListModel();

/*
        dlm.add(new DefaultItemMap(0, "Debug Off"));
        dlm.add(new DefaultItemMap(1, "Debug = 1"));
        dlm.add(new DefaultItemMap(2, "Debug = 2"));
        dlm.add(new DefaultItemMap(3, "Debug = 3"));
        dlm.add(new DefaultItemMap(4, "Debug = 4"));
        dlm.add(new DefaultItemMap(5, "Debug = 5"));
*/        

        Locale curloc = vc.getViewCapabilities().getClientLocale();
        ResourceBundle rb = ResourceBundle.getBundle("org.enhydra.barracuda.config.xmlc.Config", curloc);
        String debugOffStr = Localize.getString(rb, "Config.General.Debug_Off");
        String debugStr = Localize.getString(rb, "Config.General.Debug");
        
        dlm.add(new DefaultItemMap(0, debugOffStr));
        dlm.add(new DefaultItemMap(1, debugStr+" = 1"));
        dlm.add(new DefaultItemMap(2, debugStr+" = 2"));
        dlm.add(new DefaultItemMap(3, debugStr+" = 3"));
        dlm.add(new DefaultItemMap(4, debugStr+" = 4"));
        dlm.add(new DefaultItemMap(5, debugStr+" = 5"));

        return getSelectComp(vc, compName, dlm, selectedIndex);
    }

    /**
     * This method greats a typical debug level BSelect component, using the new
     * Log4J options (Debug, Info, Warn, Error, Fatal)
     *
     * @param vc the view context
     * @param compName the component name
     * @param selectedIndex the index to be selected
     * @return the resulting BSelect component
     */    
    public static BSelect getDebugLevelComp2(ViewContext vc, String compName, Class cl) {
        DefaultListModel dlm = new DefaultListModel();

        //build the list
        Locale curloc = vc.getViewCapabilities().getClientLocale();
        ResourceBundle rb = ResourceBundle.getBundle("org.enhydra.barracuda.config.xmlc.Config", curloc);
        String logShort = Localize.getString(rb, "Config.General.Logging.Short");
        dlm.add(new DefaultItemMap(0, logShort+" "+Localize.getString(rb, "Config.General.Logging.Default")));
        dlm.add(new DefaultItemMap(1, logShort+" "+Localize.getString(rb, "Config.General.Logging.Fatal")));
        dlm.add(new DefaultItemMap(2, logShort+" "+Localize.getString(rb, "Config.General.Logging.Error")));
        dlm.add(new DefaultItemMap(3, logShort+" "+Localize.getString(rb, "Config.General.Logging.Warn")));
        dlm.add(new DefaultItemMap(4, logShort+" "+Localize.getString(rb, "Config.General.Logging.Info")));
        dlm.add(new DefaultItemMap(5, logShort+" "+Localize.getString(rb, "Config.General.Logging.Debug")));

        return getSelectComp(vc, compName, dlm, cvtLevelToInt(cl));
    }
    
    public static int cvtLevelToInt(Class cl) {
        Level l = null;
        if (cl==null) l = Logger.getRootLogger().getLevel();
        else l = Logger.getLogger(cl).getLevel();
//System.out.println ("Checking level for class "+cl+":"+cvtLevelToInt(l));        
        return cvtLevelToInt(l);
    }
    
    public static int cvtLevelToInt(Level l) {
        if (l==null) return 0;
        if (l.equals(Level.DEBUG)) return 5;
        else if (l.equals(Level.INFO)) return 4;
        else if (l.equals(Level.WARN)) return 3;
        else if (l.equals(Level.ERROR)) return 2;
        else if (l.equals(Level.FATAL)) return 1;
        else return 0;
    }
    
    public static Level cvtIntToLevel(int i) {
        if (i==5) return Level.DEBUG;
        else if (i==4) return Level.INFO;
        else if (i==3) return Level.WARN;
        else if (i==2) return Level.ERROR;
        else if (i==1) return Level.FATAL;
        else return null;
    }
    
    public static void setLevel(Class cl, int i) {
        Logger logr = Logger.getLogger(cl);
        logr.setLevel(cvtIntToLevel(i));
    }


    /**
     * This method greats a typical debug level BSelect component
     *
     * @param vc the view context
     * @param compName the component name
     * @param selectedIndex the index to be selected
     * @return the resulting BSelect component
     */    
    public static BSelect getSelectComp(ViewContext vc, String compName, ListModel lm, int selectedIndex) {
//        DefaultView dlv = new DefaultView(vc.getTemplateNode().cloneNode(true));
//        BSelect bsComp = new BSelect(lm, dlv, null);
        BSelect bsComp = new BSelect(lm);
        bsComp.setName(compName);
        bsComp.setSelectedIndex(selectedIndex);
        if (logger.isDebugEnabled()) logger.debug("created BSelect component: "+bsComp);
        return bsComp;        
    }
    
    /**
     * This method greats a typical debug level BSelect component
     *
     * @param vc the view context
     * @param lf the listener factory to be notified when the action occurs
     * @return the resulting BAction component
     */    
    public static BToggleButton getToggleButton(ViewContext vc, String compName, String value, boolean selected) {
//        DefaultView dv = new DefaultView(vc.getTemplateNode().cloneNode(true));
//        BToggleButton btbComp = new BToggleButton(null, name, value, selected, dv, null);
        BToggleButton btbComp = new BToggleButton(null, compName, value, selected);
        if (logger.isDebugEnabled()) logger.debug("created BToggleButton component: "+btbComp);
        return btbComp;        
    }


    /**
     * This method creates a BAction component and adds the specified
     * listener to it.
     *
     * @param vc the view context
     * @param lf the listener factory to be notified when the action occurs
     * @return the resulting BAction component
     */    
    public static BAction getActionLink(ViewContext vc, ListenerFactory lf) {
//        DefaultView dv = new DefaultView(vc.getTemplateNode().cloneNode(true));
//        BAction baComp = new BAction(dv);
        BAction baComp = new BAction();
        baComp.addEventListener(lf);
        baComp.setDisableBackButton(true);
        if (logger.isDebugEnabled()) logger.debug("created BAction component: "+baComp);
        return baComp;        
    }

    /**
     * This method greats a typical debug level BSelect component
     *
     * @param vc the view context
     * @param lf the listener factory to be notified when the action occurs
     * @return the resulting BAction component
     */    
    public static BAction getUpdateButtonComp(ViewContext vc, ListenerFactory lf) {
//        DefaultView dlv = new DefaultView(vc.getTemplateNode().cloneNode(true));
//        BAction baComp = new BAction(dlv);
        BAction baComp = new BAction();
        baComp.addEventListener(lf);
        //baComp.setDisableBackButton(true);
        if (logger.isDebugEnabled()) logger.debug("created BAction component: "+baComp);
        return baComp;
    }

    /**
     * This method extracts a ValidationException error message 
     * (if it exists) from a ViewContext object. This method expects
     * the ViewContext to contain an EventContext, which in turn must 
     * contain a FormMap (specified by formName), which in turn must
     * contain a ValidationException (specified by keyName).
     *
     * @param vc the view context
     * @param keyName the key name which was used to store the 
     *        ValidationException
     * @return the resulting error message (blank is it doesn't exist)
     */    
    public static String getErrMsg(ViewContext vc, String formName, String keyName) {
        String errmsg = " ";
        EventContext ec = vc.getEventContext();
        if (ec!=null) {
            FormMap formMap = (FormMap) ec.getState(formName);
            if (formMap!=null) {
                ValidationException ve = (ValidationException) formMap.getState(keyName);
                if (ve!=null) {
                    List errlist = ve.getExceptionList();
                    Iterator it = errlist.iterator();
                    StringBuffer sb = new StringBuffer(100);
                    while (it.hasNext()) {
                        sb.append(" "+((ValidationException) it.next()).getMessage());
                    }
                    errmsg = sb.toString();
                }
            }
        }
        if (logger.isDebugEnabled()) logger.debug("errmsg: "+errmsg);
        return errmsg;
    }    

    /**
     * This method creates a success message
     *
     * @param vc the view context
     * @param keyName the key name which was used to store the 
     *        ValidationException
     * @return the resulting error message (blank is it doesn't exist)
     */    
    public static String getSuccessMsg(ViewContext vc, String formName, String keyName) {
        String msg = " ";
        EventContext ec = vc.getEventContext();
        if (ec!=null) {
            FormMap formMap = (FormMap) ec.getState(formName);
            if (formMap!=null) {
                if (formMap.getState(keyName)!=null) msg = "Update successful.";
            }
        }
        return msg;
    }    
}



