/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TabsModel.java,v 1.2 2006/03/29 15:55:20 alex Exp $
 */
package org.enhydra.barracuda.config;

import java.util.Arrays;

import org.barracudamvc.core.comp.*;
import org.apache.log4j.*;

/**
 * TabsModel - this model doesn't actually return any data,
 * but instead processes directives to tell the template engine 
 * which tabs are actually visible.
 */
public class TabsModel extends AbstractTemplateModel {

    protected static final Logger logger = Logger.getLogger(TabsModel.class.getName());
    
    public final static String CUR_TAB = "CurrentTab";
    
    public final static String COMP_TAB_COMP = "CompTab_Comp";
    public final static String COMP_TAB_MODEL = "CompTab_Model";
    public final static String COMP_TAB_VIEW = "CompTab_View";
    public final static String COMP_TAB_REND = "CompTab_Renderer";
    public final static String DATA_TAB = "DataTab";
    public final static String DOM_TAB = "DomTab";
    public final static String EVENT_TAB = "EventTab";
    public final static String FORMS_TAB = "FormsTab";
    public final static String UTIL_TAB = "UtilTab";
    public final static String VIEW_TAB = "ViewTab";
    public final static String ABOUT_TAB = "AboutTab";
    
    public static String[] validTabs = new String[] {COMP_TAB_COMP, COMP_TAB_MODEL, COMP_TAB_VIEW, COMP_TAB_REND, DATA_TAB, DOM_TAB, EVENT_TAB, FORMS_TAB, UTIL_TAB, VIEW_TAB, ABOUT_TAB};
    
    protected BTemplate parent = null;
    protected String curTab = null;

    public TabsModel(BTemplate iparent) {
        parent = iparent;
        Arrays.sort(validTabs);
    }

    //register the model by name
    public String getName() {
        return "Tabs";
    }

    //set the current tab
    public void setCurrentTab(String tabName) {
        //make sure we're setting it to a valid tab
        if (tabName==null) return;
        if (tabName.equals(curTab)) return;
        int pos = Arrays.binarySearch(validTabs, tabName);
        if (pos<0 || pos>=validTabs.length || !validTabs[pos].equals(tabName)) return;
        
        //ok...
        if (logger.isDebugEnabled()) logger.debug("Setting curTab = "+tabName);
        curTab = tabName;
        if (parent!=null) parent.invalidate();
    }
    
    //process directives
    public boolean processDirective(TemplateDirective td) {
        if (td.getCommand().equals("Is_Visible")) {
            String tab = td.getKeyName();
            if (curTab==null) setCurrentTab(tab);
            if (logger.isDebugEnabled()) logger.debug("Checking to see if we can show "+tab+" ("+(tab.equals(curTab) ? "yes" : "no")+")");
            if (!tab.equals(curTab)) return false;    //skip this tab
        }
        return true;    //by default, allow all directives
    }
}