/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: BarracudaDiscRackEnhydra.java,v 1.5 2004/12/20 11:15:54 slobodan Exp $
 */

package barracudaDiscRack;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.*;

/**
 * The application object.
 *
 * Application-wide data would go here.
 */
public class BarracudaDiscRackEnhydra extends StandardApplication {

	/*
	 *  A few methods you might want to add to.
	 *  See StandardApplication for more details.
	 */
	public void startup(Config appConfig)
	throws ApplicationException {
		super.startup(appConfig);
		//  Here is where you would read application-specific settings from
		//  your config file.

		// setup the Disc Rack gateway to barracuda
		try {
			// setup the init parameters for the gateway
			// if present in the applications config file
			// default to the standard config from multiserver/enhydra
			ServletConfig servletConfig =
			this.getHttpPresentationManager().getServlet().getServletConfig();
			Config gatewayConfig = appConfig.getConfig("InitGateway");
			if ( null != gatewayConfig ) {
				servletConfig =
				new ServletConfigWrapper(
																this.getHttpPresentationManager().getServletContext(),
																gatewayConfig,
																servletConfig.getServletName() );
			}

			this.myDiscRackGateway = new DiscRackGateway();
			this.myGatewayExtension = this.myDiscRackGateway.getEventExtension();
			this.myDiscRackGateway.init( servletConfig );
		} catch ( ServletException ex ) {
			throw ( new ApplicationException( "Error initilising DiscRackGateway", ex ) );
		} catch ( KeywordValueException ex ) {
			throw ( new ApplicationException( "Error initilising DiscRackGateway", ex ) );
		}
	}
	
	/*
	* destroy Barracuda Disc Rack gateway
	*/
	public void shutdown () {
		super.shutdown();
		this.myDiscRackGateway.destroy();
	}

	/**
	 * Default method used by <CODE>requestPreprocessor</CODE>
	 * to ensure that a session exists and initialize the
	 * <CODE>session</CODE> and <CODE>sessionData</CODE> fields
	 * in the <CODE>HttpPresentationComms</CODE> object.
	 * New sessions are only created on requests to presentation
	 * objects, not requests for other types of files (such as gifs).
	 * This avoids allocating multiple sessions when a browser makes
	 * multiple requests for HREFs.  If the session already exist, it
	 * is alwasy set in <CODE>comms</CODE><P>
	 *
	 * This normally looks up the session using a session key from
	 * a cookie.
	 *
	 * @param comms
	 *   Object containing request, response and redirect objects.
	 * @exception ApplicationException
	 *   If an error occurs setting up the session.
	 */
	protected void ensureSession(HttpPresentationComms comms)
	throws ApplicationException {

		try {
			boolean isOkForSession = presentationManager.isPresentationRequest(comms.request) ||
															 comms.request.getRequestURI().indexOf(this.myGatewayExtension) > -1;

			if ( isOkForSession ) {
				// FIX - verify OK
				comms.session = StandardAppUtil.getRequestSession(comms);
			}
			if ( comms.session != null &&
					 sessionManager.sessionExists(comms.session.getSessionKey()) ) {
				/*
				 * InitializeNewSession() sets up comms.sessionData, so
				 * that it may be used if an application defines it's own
				 * version of that method (after calling the super method,
				 * it has a normal comms object to work with).
				 * Therefore, if we did not call initializeNewSession(),
				 * we have to initialize comms.sessionData here.
				 */
				comms.sessionData = comms.session.getSessionData();
			} else {

				if ( isOkForSession ) {
					initializeNewSession(comms);
				}
			}
		} catch ( HttpPresentationException except ) {
			throw new ApplicationException(except);
		} catch ( SessionException except ) {
			throw new ApplicationException(except);
		}
	}

	public boolean requestPreprocessor(HttpPresentationComms comms)
	throws Exception{
		boolean retVal = super.requestPreprocessor(comms);
		if ( !retVal ) {
			String target = comms.request.getRequestURI();
			// if has the event extension in the URI then treat it as a barracuda event
			if ( target.indexOf(this.myGatewayExtension) > -1 ) {
				this.myDiscRackGateway.handleDefaultExt(
																							 comms.request.getHttpServletRequest(),
																							 comms.response.getHttpServletResponse(),
																							 comms );
				retVal = true;
			}
		}
		return retVal;
	}

	/**
	 * This is an optional function, used only by the Multiserver's graphical
	 * administration. This bit of HTML appears in the status page for this
	 * application. You could add extra status info, for example
	 * a list of currently logged in users.
	 *
	 * @return HTML that is displayed in the status page of the Multiserver.
	 */
	public String toHtml() {
		return "This is <I>DiscRack</I>";
	}

	private DiscRackGateway myDiscRackGateway;
	private String myGatewayExtension;
}


class ServletConfigWrapper implements ServletConfig {

	private ServletContext myContext;
	private Config myConfig;
	private String myName;

	public ServletConfigWrapper( ServletContext context, Config config, String name ) {
		this.myContext = context;
		this.myConfig = config;
		this.myName = name;
	}

	public ServletContext getServletContext() {
		return this.myContext;
	}

	public String getInitParameter(String name) {
		String retVal = null;
		try {
			retVal = this.myConfig.getString(name);
		} catch ( KeywordValueException ex ) {
		}
		return retVal;
	}

	public Enumeration getInitParameterNames() {
		return( Collections.enumeration(Arrays.asList(this.myConfig.keys())) );
	}

	public String getServletName() {
		return this.myName;
	}

}
