/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PersonFactory.java,v 1.4 2004/12/03 14:12:34 slobodan Exp $
 */



package barracudaDiscRack.business.person;

import barracudaDiscRack.data.person.*;
import barracudaDiscRack.business.DiscRackBusinessException;
import com.lutris.dods.builder.generator.query.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.*;


/**
 * Used to find the instance of Person.
 */
public class PersonFactory {
    
    /** 
     * The findPerson method performs a database query to
     * return a <CODE>Person</CODE> object
     * representing the row in the <CODE>person</CODE> table
     * that matches login name with the username. 
     *
     * @param username, the login name of the person.
     * @return
     *    the person. null if there isn't a person associated
     *    the username
     * @exception DiscRackBusinessException
     *    if there is a problem retrieving person information.
     */
     public static Person findPerson(String username)
        throws DiscRackBusinessException {
        try {
            PersonQuery query = new PersonQuery();
            // set query.
            query.setQueryLogin(username);
            // Throw an exception if more than one user by this name is found
            query.requireUniqueInstance();
            
            PersonDO[] foundPerson = query.getDOArray();
            if(foundPerson.length != 0) {
                return new Person(foundPerson[0]);
            } else {
                return null;
            }
        } catch(NonUniqueQueryException ex) {
            Enhydra.getLogChannel().write(Logger.DEBUG,
                                          "Non-unique user found in database: " +
                                          ex.getMessage());
            throw new DiscRackBusinessException("More than one user found with username: " +
                                                username);
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Database error retrieving user: ", ex);
        } catch(QueryException ex) {
            throw new DiscRackBusinessException("Query exception retrieving user: ", ex);
        }

    }
}
