
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: BaseDiscRackEventGateway.java,v 1.5 2006/03/29 15:55:20 alex Exp $
 */

package barracudaDiscRack.presentation;

// import disc rack specifics
import barracudaDiscRack.presentation.personMgmt.events.*;
import barracudaDiscRack.business.person.*;

// import Enhydra specifics
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.util.KeywordValueException;
import com.lutris.logging.*;

// import barracuda specifics
import org.barracudamvc.core.event.*;

public class BaseDiscRackEventGateway extends DefaultEventGateway {

	//------------------------------------------------------------
	//  BasePO functions
	//------------------------------------------------------------

	/**
	 * Initialise the Enhydra PO session data from the event context
	 *
   * @param EventContext
	 */
	protected void initUsingContext( EventContext context ) throws EventException {
		try {
			this.myComms =
			(HttpPresentationComms)context.getState( ApplicationGateway.EXTERNAL_CONTEXT_OBJ_NAME );
			this.initSessionData(this.myComms);
		} catch ( ClassCastException ex ) {
			String errMsg = "ERROR: External Context object is NOT a HttpPresentationComms object. Cannot function";
			throw ( new EventException( errMsg, ex ) );
		} catch ( DiscRackPresentationException ex ) {
			String errMsg = "ERROR: Could not get session data from session:";
			throw ( new EventException( errMsg, ex ) );
		}
	}

	/**
	 * Gets HttpPresentation object
	 *
	 * @return The saved comms objects
	 * to whichever subclass needs it
	 */
	public HttpPresentationComms getComms() {
		return this.myComms;
	}

	/**
	 * Gets the session data
	 *
	 * @return session data
	 */
	public DiscRackSessionData getSessionData() {
		return this.mySessionData;
	}

	/**
	 * Sets the user into the session
	 *
	 * @param thePerson, the person to be set in the session
	 * @exception DiscRackPresentationException
	 */
	public void setUser(Person thePerson)
	throws DiscRackPresentationException {
		this.getSessionData().setUser(thePerson);
	}

	/**
	 * Gets the user from the session
	 *
	 * @return the person object in the session
	 */
	public Person getUser() {
		return this.getSessionData().getUser();
	}

	/**
	 * Method to remove the current user from the session
	 */
	public void removeUserFromSession() {
		this.getSessionData().removeUser();
	}

	/**
	 * Method to get or create the AgSessionData object from the user session
	 * This object is saved in the EbrokerPresentation object
	 *
	 * @param HttpPresentationComms
	 * @exception Exception
	 */
	protected void initSessionData(HttpPresentationComms comms)
	throws DiscRackPresentationException {
		this.myComms = comms;

		try {
			Object obj = comms.sessionData.get(DiscRackSessionData.SESSION_KEY);
			// If we found the session data, save it in a private data member
			if ( null != obj ) {
				this.mySessionData = (DiscRackSessionData)obj;
			} else {
				// If no session data was found, create a new session data instance
				this.mySessionData = new DiscRackSessionData();
				comms.sessionData.set(DiscRackSessionData.SESSION_KEY, this.mySessionData);
			}
		} catch ( KeywordValueException ex ) {
			writeDebugMsg("Problem getting session data from session: " +
										ex.getMessage());
		}
	}

	/**
	 * Method to write a debugging message to the debug log
	 * channel when the DEBUG flag is turned on
	 *
	 * @param msg The message to write to the DEBUG log channel
	 */
	public static void writeDebugMsg(String msg) {
		Enhydra.getLogChannel().write(Logger.DEBUG,msg);
	}

	/**
	 * Saved input and output context, and session data
	 */
	private HttpPresentationComms myComms = null;
	private DiscRackSessionData mySessionData = null;

	/**
	 * Reference to the person logged in to the session
	 */
	private Person myPerson = null;

}
