/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Edit.java,v 1.4 2004/12/03 14:12:34 slobodan Exp $
 */

package barracudaDiscRack.presentation.discMgmt;

import barracudaDiscRack.presentation.BasePO;
import barracudaDiscRack.business.person.*;
import barracudaDiscRack.business.disc.*;
import barracudaDiscRack.presentation.DiscRackPresentationException;
import barracudaDiscRack.business.DiscRackBusinessException;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;

/**
 * Edit class handles the disc management (add, edit or delete)
 * of the DiscRack app
 *
 * @author
 * @version
 */
public class Edit extends BasePO {
    
    /**
     * Constants representing HTTP parameters passed in from 
     * the form submission
     */
    private static String TITLE_NAME = "title";
    private static String ARTIST_NAME = "artist";
    private static String GENRE_NAME = "genre";
    private static String DISC_ID = "discID";
    private static String IS_LIKED = "like";
    private static String INVALID_ID = "invalidID";
    private static String EDIT_COMMAND = "edit";

    /**
     * Superclass method override
     */
    public boolean loggedInUserRequired() {
        return true;
    }

    /** 
     * Default event. Just show the page populated with any disc 
     * parameters that were passed along.
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleDefault() 
        throws HttpPresentationException {
        return showEditPage(null);
    }
    
    /** 
     * handle show add disc page event. 
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleShowAddPage() 
        throws HttpPresentationException {
        return showAddPage(null);
    }
    
    /*
     * Edits an existing disc
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleEdit()
        throws HttpPresentationException {		   
        String discID = this.getComms().request.getParameter(DISC_ID);        
        Disc disc = null;
        
        // Try to get the disc by its ID
        try {
            disc = DiscFactory.findDiscByID(discID);
        } catch(Exception ex) {
            this.getSessionData().setUserMessage("Please choose a valid disc to edit");
            throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
        }
        
        try {
            saveDisc(disc);
            throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
        } catch(Exception ex) {
            return showEditPage("You must fill out all fields to edit this disc");
        }    
    }
    
    /*
     * Adds a disc to the disc database
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleAdd() 
        throws HttpPresentationException {       
        try {
            Disc disc = new Disc();
            saveDisc(disc);
            throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
        } catch(Exception ex) {
            return showAddPage("You must fill out all fields to add this disc");
        }
    }
    
    /*
     * Deletes a Disc from the Disc database
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleDelete()
        throws HttpPresentationException, DiscRackPresentationException {
        String discID = this.getComms().request.getParameter(DISC_ID);
        
        try {
            Disc disc = DiscFactory.findDiscByID(discID);
            String title = disc.getTitle();
            disc.delete();
            this.getSessionData().setUserMessage("The disc, " + title +
                                                 ", was deleted");
            // Catch any possible database exception as well as the null pointer
            //  exception if the disc is not found and is null after findDiscByID
        } catch(Exception ex) {
            this.getSessionData().setUserMessage("Please choose a valid disc to delete");
        }
        // Redirect to the catalog page which will display the error message,
        //  if there was one set by the above exception
        throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
    }
    
    /** 
     * Produce HTML for this PO, populated by the disc information
     * that the user wants to edit
     *
     * @param errorMsg, the error messages
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject showEditPage(String errorMsg) 
        throws HttpPresentationException, DiscRackPresentationException {        
        String title = this.getComms().request.getParameter(TITLE_NAME);
        String artist = this.getComms().request.getParameter(ARTIST_NAME);
        String genre = this.getComms().request.getParameter(GENRE_NAME);
        String discID = this.getComms().request.getParameter(DISC_ID);
        // Instantiate the page object
        EditHTML page = (EditHTML)myComms.xmlcFactory.create(EditHTML.class);

        Disc disc = null;
        
        try {
            disc = DiscFactory.findDiscByID(discID);
            // Catch any possible database exception in findDiscByID()
        } catch(DiscRackBusinessException ex) {
            this.getSessionData().setUserMessage("Please choose a valid disc to edit");
            throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
        }
        
        try {
            // If we received a valid discID then try to show the disc's contents,
            //  otherwise try to use the HTML input parameters
            page.getElementDiscID().setValue(disc.getHandle());
            
            if(null != title && title.length() != 0) {
                page.getElementTitle().setValue(title);
            } else {
                page.getElementTitle().setValue(disc.getTitle());
            }
            
            if(null != artist  && artist.length() != 0) {
                page.getElementArtist().setValue(artist);
            } else {
                page.getElementArtist().setValue(disc.getArtist());
            }
            
            if(null != genre && genre.length() != 0) {
                page.getElementGenre().setValue(genre);
            } else {
                page.getElementGenre().setValue(disc.getGenre());
            }
            
            if(null != this.getComms().request.getParameter(IS_LIKED)) {
                page.getElementLikeBox().setChecked(true);
            } else {
                page.getElementLikeBox().setChecked(disc.isLiked());
            }
            
            if(null == errorMsg) {       
                page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
            } else {
                page.setTextErrorText(errorMsg);
            }
        } catch(DiscRackBusinessException ex) {
            throw new DiscRackPresentationException("Error populating page for disc editing", ex);
        }
        
        page.getElementEventValue().setValue(EDIT_COMMAND);
        return page;
    }
    
    /** 
     * Produce HTML for this PO
     * 
     * @param errorMsg, the error messages
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject showAddPage(String errorMsg) 
        throws HttpPresentationException, DiscRackPresentationException {        
        String title = this.getComms().request.getParameter(TITLE_NAME);
        String artist = this.getComms().request.getParameter(ARTIST_NAME);
        String genre = this.getComms().request.getParameter(GENRE_NAME);
        String discID = this.getComms().request.getParameter(DISC_ID);
        // Instantiate the page object
        EditHTML page = (EditHTML)myComms.xmlcFactory.create(EditHTML.class);
        
        if(null != this.getComms().request.getParameter(TITLE_NAME)) {
            page.getElementTitle().setValue(this.getComms().request.getParameter(TITLE_NAME));
        }
        if(null != this.getComms().request.getParameter(ARTIST_NAME)) {
            page.getElementArtist().setValue(this.getComms().request.getParameter(ARTIST_NAME));
        }
        if(null != this.getComms().request.getParameter(GENRE_NAME)) {
            page.getElementGenre().setValue(this.getComms().request.getParameter(GENRE_NAME));
        }
        if(null != this.getComms().request.getParameter(IS_LIKED)) {
            page.getElementLikeBox().setChecked(true);
        } else {
            page.getElementLikeBox().setChecked(false);
        }
        
        if(null == errorMsg) {       
            page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
        } else {
            page.setTextErrorText(errorMsg);
        }
        
        return page;
    }
    
    /**
     * Method to save a new or existing disc to the database
     *
     * @param disc, the disc to be saved
     * @return html document
     * @exception HttpPresentationException
     */
    protected void saveDisc(Disc theDisc)
        throws HttpPresentationException {        
        String title = this.getComms().request.getParameter(TITLE_NAME);
        String artist = this.getComms().request.getParameter(ARTIST_NAME);
        String genre = this.getComms().request.getParameter(GENRE_NAME);
        
        try {
            theDisc.setTitle(title);
            theDisc.setArtist(artist);
            theDisc.setGenre(genre);
            
            if(null != this.getComms().request.getParameter(IS_LIKED)) {
                theDisc.setLiked(true);   
            } else {
                theDisc.setLiked(false);   
            }
            theDisc.setOwner(this.getUser());
            theDisc.save();	
        } catch(Exception ex) {
            throw new DiscRackPresentationException("Error adding disc", ex);
        }    
    }
}






