/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: Login.java,v 1.6 2006/03/31 09:50:32 alex Exp $
 */

package barracudaDiscRack.presentation.personMgmt;

import barracudaDiscRack.business.person.*;
import barracudaDiscRack.presentation.BasePO;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;
import barracudaDiscRack.business.DiscRackBusinessException;
import barracudaDiscRack.presentation.DiscRackPresentationException;

// import barracuda specifics
import org.barracudamvc.core.comp.*;
import org.barracudamvc.core.forms.*;
import org.barracudamvc.core.forms.validators.*;
import org.barracudamvc.plankton.data.*;
import org.barracudamvc.core.view.ViewCapabilities;
import org.barracudamvc.core.event.*;
import org.barracudamvc.core.comp.model.*;
import org.barracudamvc.plankton.data.CollectionsUtil;

// import java specifics
import javax.servlet.http.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.List;
import java.util.Iterator;
import java.util.Collection;

import org.apache.log4j.*;

/**
 * Login.java handles the login functionality of the DiscRack app.
 *
 * @author
 * @version
 */
public class Login extends BasePO {

  /**
   * Constants
   */
  private static String SUBMIT_NAME = "submit";
  private static String LOGIN_NAME = "login";
  private static String PASSWORD_NAME = "password";
  private static String ERROR_NAME = "ERROR_NAME";

  /**
   * Superclass method override
   */
  public boolean loggedInUserRequired() {
    return false;
  }

  /**
   *  Default event. Just show the page.
   */
  public XMLObject handleDefault() throws HttpPresentationException {
    return showPage(null);
  }

  /**
   * Process login data
   *
   * @return wml document
   * @exception HttpPresentationException
   */
  public XMLObject handleLogin() throws HttpPresentationException {
    String login = this.getComms().request.getParameter(LOGIN_NAME);
    String password = this.getComms().request.getParameter(PASSWORD_NAME);
    Person user = null;
    try {
      user = PersonFactory.findPerson(login);
      if (null == user || !user.getPassword().equals(password)) {
        return showPage("Invalid username or password");
        // Show error message that user not found (bad username/password)
      }
      else {
        this.setUser(user);
                throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
      }
    }
    catch (DiscRackBusinessException ex) {
      this.writeDebugMsg("System error finding user: " + ex.getMessage());
      throw new DiscRackPresentationException("System error finding user", ex);
    }
  }

  /**
   * handle logout event
   *
   * @return html document
   * @exception HttpPresentationException
   */
  public XMLObject handleLogout() throws HttpPresentationException {
    this.removeUserFromSession();
    return new ExitHTML();
  }

  /**
   * handle throw exception event.
   *
   * @return html document
   * @exception Exception
   */
  public XMLObject handleThrowException() throws Exception {
    throw new Exception(
        "This is a test exception thrown from Login.java handleThrowException()");
  }

  /**
   * display page
   *
   * @param errorMsg, the error messages
   * @return html document
   */
  public XMLObject showPage(String errorMsg) {

    LoginHTML page = (LoginHTML) myComms.xmlcFactory.create(LoginHTML.class);

    if (null != errorMsg ||
        null != (errorMsg = this.getSessionData().getAndClearUserMessage())) {
      page.setTextErrorText(errorMsg);
    }
    else {
      page.getElementErrorText().getParentNode().removeChild(page.
          getElementErrorText());
    }

    return page;
  }

  //------------------------------------------------------------
  //               Use Barracuda Component Model
  //------------------------------------------------------------

  /**
   *  handle special event to show page using Barracuda
   */
  public XMLObject handleShowUsingBarracuda() throws HttpPresentationException {
    return this.showPageUsingBarracuda(new DefaultStateMap());
  }

  /**
   *  handle special event to login using Barracuda
   */
  public XMLObject handleLoginUsingBarracuda() throws HttpPresentationException {

    //create the login form
    ValidationException ve = null;
    if (logger.isDebugEnabled())
      logger.debug("Creating login form");
    LoginForm lf = new LoginForm(getComms().session.getHttpSession());
    try {
      //map/validate the form
      if (logger.isDebugEnabled())
        logger.debug("Mapping/Validating login form");
      lf.map(this.getComms().request.getHttpServletRequest()).validate(true);
    }
    catch (ValidationException e) {
      removeUserFromSession();
      if (logger.isDebugEnabled())
        logger.debug("Validation Exception: " + e);

      if (logger.isDebugEnabled()){
        CollectionsUtil.printStackTrace(e.getExceptionList(), 0, logger, null);
      }

      ve = e;
    }

    //if no errors redirect to the catalog screen
    if (ve == null) {
      if (logger.isDebugEnabled())
        logger.debug("Redirecting to " + BasePO.DISC_CATALOG_PAGE);
      throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
    }

    if (logger.isDebugEnabled())
      logger.debug("Displaying login page");

      // create a context for the rendering
    StateMap context = new DefaultStateMap();

    //store a copy of the form and any errors in the context
    if (logger.isDebugEnabled())
      logger.debug("Saving form, errors");
    context.putState(LOGIN_FORM, lf);
    context.putState(LOGIN_ERR, ve);

    return this.showPageUsingBarracuda(context);
  }

  /**
   * display page using Barracuda
   *
   * @param context, the error messages
   * @return html document
   */
  public XMLObject showPageUsingBarracuda(StateMap context) {

    LoginHTML page = (LoginHTML) myComms.xmlcFactory.create(LoginHTML.class);

    //create a template component and render it
    Node node = page.getDocument().getElementById("LoginForm");

    TemplateView tv = new DefaultTemplateView(node);
    TemplateModel tm = new LoginModel(context);
    BTemplate templateComp = new BTemplate(tm);
    templateComp.setView(tv);

    try {
      HttpServletRequest ireq = this.getComms().request.getHttpServletRequest();
      HttpServletResponse ires = this.getComms().response.getHttpServletResponse();

      ViewContext vc = new DefaultViewContext(new ViewCapabilities(),
                                              ireq,
                                              ires);
      templateComp.render(vc);

    }
    catch (RenderException re) {
      this.writeDebugMsg("Render err:" + re);
    }

    return page;
  }

  //=================================================================
  // NOTE - the following are duplicated from LoginEventGateway.java
  //			  (with modifications to remove any event model dependencies)
  //        purely for illustration of the standalone use of the
  //				Barracuda component/form models.
  //=================================================================

  // setup a logger for the class
  protected static Logger logger = Logger.getLogger(Login.class.getName());

  //private constants
  private static final String LOGIN_FORM = LoginEventGateway.class.getName() +
      ".LoginForm"; //LoginForm
  private static final String LOGIN_ERR = LoginEventGateway.class.getName() +
      ".LoginErr"; //ValidationException

  //------------------------------------------------------------
  //               Template Models
  //------------------------------------------------------------
  /**
   * LoginModel
   */
  class LoginModel extends AbstractTemplateModel {

    LoginForm fm = null;
    ValidationException ve = null;

    //constructor (extract form, exception from context)
    public LoginModel(StateMap context) {
      fm = (LoginForm) context.getState(LOGIN_FORM);
      ve = (ValidationException) context.getState(LOGIN_ERR);
    }

    //register the model by name
    public String getName() {
      return "Login";
    }

      public Object getItem(String key) {
        ViewContext vc = getViewContext();
        if (key.equals("Username")) {
          String username = null;
          try
          {
        	  username = fm.getVal(LoginForm.USER, "").toString();
          }
          catch (Exception ex) {}
          return (username != null ? username : "");
        }
        else if (key.equals("Password")) {
        	String pass = null;
            try
            {
          	  pass = fm.getVal(LoginForm.PASSWORD, "").toString();
            }
            catch (Exception ex) {}
          return (pass != null ? pass : "");
        }
        else if (key.equals("Errors")) {
          if (ve == null) return "";
          List errlist = ve.getExceptionList();
          StringBuffer sb = new StringBuffer(errlist.size() > 1 ?
                                             "There were several errors:" : "");
          Iterator it = errlist.iterator();
          while (it.hasNext()) {
            sb.append(" " + ( (ValidationException) it.next()).getMessage());
          }
          return sb.toString();
        }
        else {
          return super.getItem(key);
        }

      }
  }

  //------------------------------------------------------------
  //               HTML Form Mappings, Validators
  //------------------------------------------------------------

  /**
   * Login form - define the login form
   */
  class LoginForm extends DefaultFormMap {
    //login form constants (these values correspond to the HTML param names)
    static final String USER = "login";
    static final String PASSWORD = "password";

    HttpSession session = null;

    public LoginForm(HttpSession isession) {
      session = isession;

      //define the elements
      if (logger.isDebugEnabled())
        logger.debug("Defining Login form elements");
      this.defineElement(new DefaultFormElement(USER, FormType.STRING, null,
                                                new
                                                NotNullValidator(
          "You must enter a Login.")));
      this.defineElement(new DefaultFormElement(PASSWORD, FormType.STRING, null,
                                                new
                                                NotNullValidator(
          "You must enter a Password.")));

      //define a form validator
      if (logger.isDebugEnabled())
        logger.debug("Defining Registration form validator");
      this.defineValidator(new LoginValidator(session));
    }

    /**
     * we override this method because instead of using the default validation
     * process (validate elements, then form), we don't want to validate at all
     * if they are already logged in...we just assume they're valid and return.
     */
    public FormMap validate(boolean deferExceptions) throws ValidationException {
      //see if they are already logged in. If so, just consider them valid
      if (logger.isDebugEnabled())
        logger.debug("Checking to see if already logged in (prevalidated)");
      Person p = getUser();
      if (null != p) {
        try {
          String fuser = this.getVal(LoginForm.USER, "").toString();
          String fpassword = this.getVal(LoginForm.PASSWORD, "").toString();
          if (fuser == null && fpassword == null)
            return this; //if user/pwd = null, just use the current settings
          if (p.getLogin().equals(fuser) &&
              p.getPassword().equals(fpassword))
            return this; //if they're != null, make sure they match, otherwise, we'll want to force a complete revalidate
        }
        catch (DiscRackBusinessException e) {
        }
      }

      //if not, go ahead and validate
      return super.validate(deferExceptions);
    }
  }

  /**
   * Login validator - define a custom form validator
   */
  class LoginValidator extends DefaultFormValidator {

    HttpSession session = null;

    public LoginValidator(HttpSession isession) {
      session = isession;
    }

    public void validateForm(FormMap imap, boolean deferExceptions) throws
        ValidationException {
      if (logger.isDebugEnabled())
        logger.debug("Validating login form");

        //validate the login information
      if (logger.isDebugEnabled())
        logger.debug("Validate the user/pwd info");
      LoginForm map = (LoginForm) imap;
      String user = map.getVal(LoginForm.USER, "").toString();
      String password = map.getVal(LoginForm.PASSWORD, "").toString();
      String errMsg = "Invalid username or password from LoginValidator. Please check your information and try again.";
      try {
        //make sure the login exists and the passwords match
        Person person = PersonFactory.findPerson(user);
        if (person == null)
          throw new ValidationException(map.getElement(LoginForm.USER), errMsg);
        if (!person.getPassword().equals(password))
          throw new ValidationException(map.getElement(LoginForm.PASSWORD),
                                        errMsg);

        //log them in
        setUser(person);
      }
      catch (DiscRackBusinessException e) {
        removeUserFromSession(); //just to be sure we don't get stuck in a loop
        throw new ValidationException(map.getElement(LoginForm.USER),
                                      "System error finding user: " +
                                      e.getMessage() + ". Please try again. If the problem persists, contact your system administrator.",
                                      e);
      }
      catch (DiscRackPresentationException e) {
        throw new ValidationException(map.getElement(LoginForm.USER),
                                      "System error setting user: " +
                                      e.getMessage() + ". Please try again. If the problem persists, contact your system administrator.",
                                      e);
      }
    }
  }
}