/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: Register.java,v 1.4 2004/12/03 14:12:34 slobodan Exp $
 */

package barracudaDiscRack.presentation.personMgmt;

import barracudaDiscRack.business.person.*;
import barracudaDiscRack.presentation.BasePO;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;
import barracudaDiscRack.presentation.DiscRackPresentationException;
import barracudaDiscRack.business.DiscRackBusinessException;

/**
 * Register.java handles the user registration functionality
 * of the DiscRack app.
 *
 * @author
 * @version
 */
public class Register extends BasePO {

    /**
     * Constants
     */
    private static String LOGINNAME = "login";
    private static String PASSWORD = "password";
    private static String REPASSWORD = "repassword";
    private static String FIRSTNAME = "firstname";
    private static String LASTNAME = "lastname";

    /**
     * Superclass method override
     */
    public boolean loggedInUserRequired() {
        return false;
    }

    /**
     *  Default event. Just show the page.
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleDefault()
        throws HttpPresentationException {
        return showPage(null);
    }

    /**
     *  process login data, save fields to PersonManager if correct,
     *  otherwise output error messages
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleRegister()
        throws HttpPresentationException {

        String login = this.getComms().request.getParameter(LOGINNAME);
        String password = this.getComms().request.getParameter(PASSWORD);
        String firstname = this.getComms().request.getParameter(FIRSTNAME);;
        String lastname = this.getComms().request.getParameter(LASTNAME);
        String repassword = this.getComms().request.getParameter(REPASSWORD);

        // if login or password field is empty, generate error and redirect to this PO
        if (login.length() == 0 || password.length() ==0 ||
            firstname.length() == 0 || lastname.length() == 0) {
            return showPage("Missing information. Please make sure all fields are filled out exactly");
        }

        //Check that password was duplicated correctly
        if(!repassword.equals(password)) {
            return showPage("Please make sure your password and password confirmation match exactly");
        }

        try {
            // Now check that the login name is not taken.
            if(null == PersonFactory.findPerson(login)) {
                Person user = new Person();
                user.setLogin(login);
                user.setPassword(password);
                user.setFirstname(firstname);
                user.setLastname(lastname);
                //Add the user to the database.
                user.save();
                this.getSessionData().setUserMessage("You are registered, " +
                                                     user.getFirstname() + ", please log in");
                throw new ClientPageRedirectException(getComms().request.getApplicationPath() + LOGIN_PAGE);
            } else {
                // Login name already taken
                // Redirect to this same PO with informative error message
                return showPage("The login name " + login + " is already taken. Please try another.");
            }
        } catch(DiscRackBusinessException ex) {
            throw new DiscRackPresentationException("Exception logging in user: ", ex);
        }
    }

    /**
     * display page
     *
     * @param errorMsg, the error messages
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject showPage(String errorMsg)
        throws HttpPresentationException {

        RegisterHTML page = (RegisterHTML)myComms.xmlcFactory.create(RegisterHTML.class);

        if(null == errorMsg) {
            page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
        } else {
            page.setTextErrorText(errorMsg);
        }

        if(null != this.getComms().request.getParameter(LOGINNAME)) {
            page.getElementLogin().setValue(this.getComms().request.getParameter(LOGINNAME));
        }
        if(null != this.getComms().request.getParameter(FIRSTNAME)) {
            page.getElementFirstname().setValue(this.getComms().request.getParameter(FIRSTNAME));
        }
        if(null != this.getComms().request.getParameter(LASTNAME)) {
            page.getElementLastname().setValue(this.getComms().request.getParameter(LASTNAME));
        }

        return page;
    }
}
