/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Cactus", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * formatted with JxBeauty (c) johann.langhofer@nextra.at
 */


package  org.hamletsoft.enhydra.cactus.business;

import  java.io.*;
import  java.lang.reflect.*;
import  java.net.*;
import  java.util.*;
import  javax.servlet.*;
import  javax.servlet.http.*;
import  org.apache.cactus.*;
import  org.apache.log4j.Logger;
import  org.hamletsoft.enhydra.cactus.data.*;
import  org.apache.cactus.server.AbstractWebTestCaller;
import  com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import  javax.servlet.http.HttpServletRequest;
import  org.hamletsoft.enhydra.cactus.spec.*;

/**
 * Responsible for instanciating the <code>TestCase</code> class on the server
 * side, set up the implicit objects and call the test method.
 *
 * @author <a href="mailto:hattori@hitachizosen.co.jp">Ryuji "The Hamlet" Hattori</a>
 *
 * @version $Id: EnhydraAppTestCaller.java,v 1.1 2004/08/16 09:35:01 slobodan Exp $
 */
public class EnhydraAppTestCaller extends AbstractWebTestCaller {
  /**
   * The logger
   */
  protected static Logger logger = Logger.getLogger(EnhydraAppTestCaller.class.getName());

  /**
   * @param theObjects the implicit objects coming from the redirector
   */
  public EnhydraAppTestCaller (EnhydraAppImplicitObjectsImpl theObjects) {
  super(theObjects);
  }

  /**
   * @return a <code>Writer</code> object that will be used to return the
   *         test result to the client side.
   * @exception IOException if an error occurs when retrieving the writer
   */
  protected Writer getResponseWriter () throws IOException {
    HttpPresentationComms comms = ((EnhydraAppImplicitObjects)this.webImplicitObjects).getPresentationComms();
    return  comms.response.getHttpServletResponse().getWriter();
  }

  /**
   * Sets the test case fields using the implicit objects (using reflection).
   * @param theTestInstance the test class instance
   */
  protected void setTestCaseFields (AbstractTestCase theTestInstance) throws Exception {
    logger.info("setTestCaseFields([" + theTestInstance + "])");
    EnhydraAppTestCase servletInstance = (EnhydraAppTestCase)theTestInstance;
    EnhydraAppImplicitObjects servletImplicitObjects = (EnhydraAppImplicitObjects)this.webImplicitObjects;
    // Sets the request field of the test case class
    // ---------------------------------------------
    // Extract from the HTTP request the URL to simulate (if any)
    HttpPresentationComms comms = servletImplicitObjects.getPresentationComms();
    // Set the application field of the test case class
    // -------------------------------------------
    Field commsField = servletInstance.getClass().getField("comms");
    commsField.set(servletInstance, comms);
    Field requestField = servletInstance.getClass().getField("request");
    requestField.set(servletInstance, comms.request);
    // Set the response field of the test case class
    // ---------------------------------------------
    Field responseField = servletInstance.getClass().getField("response");
    responseField.set(servletInstance, comms.response);
    // Set the application field of the test case class
    // -------------------------------------------
    Field applicationField = servletInstance.getClass().getField("application");
    applicationField.set(servletInstance, comms.application);
    logger.info("setTestCaseFields");
  }

  /**
   * Overwrite Methods. because of need to load class by presentation class loader.
   *
   * @param theClassName the name of the test class
   * @param theTestCaseName the name of the current test case
   * @return the class object the test class to call
   */
  protected Class getTestClassClass (String theClassName) throws ServletException {
    logger.info("getTestClassClass([" + theClassName + "])");
    // Get the class to call and build an instance of it.
    Class testClass = null;
    try {
      EnhydraAppImplicitObjects servletImplicitObjects = (EnhydraAppImplicitObjects)this.webImplicitObjects;
      HttpPresentationComms comms = servletImplicitObjects.getPresentationComms();
      testClass = Class.forName(theClassName, true, comms.application.getHttpPresentationManager().getAppClassLoader());
    } catch (Exception e) {
      e.printStackTrace();
      String message = "Error finding class [" + theClassName + "] in classpath";
      logger.error(message, e);
      throw  new ServletException(message, e);
    }
    logger.info("getTestClassClass");
    return  testClass;
  }
}



