/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Cactus", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * formatted with JxBeauty (c) johann.langhofer@nextra.at
 */


package  org.hamletsoft.enhydra.cactus.business;

import  java.lang.reflect.*;
import  javax.servlet.*;
import  javax.servlet.http.*;
import  java.net.*;
import  junit.framework.*;
import  org.apache.cactus.client.*;
import  org.apache.cactus.server.*;
import  com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import  com.lutris.appserver.server.Application;
import  com.lutris.appserver.server.httpPresentation.HttpPresentationResponse;
import  com.lutris.appserver.server.httpPresentation.HttpPresentationRequest;
import  org.hamletsoft.enhydra.cactus.business.*;
import  org.apache.cactus.AbstractTestCase;
import  org.apache.cactus.AbstractWebTestCase;
import  javax.servlet.http.HttpServletRequest;
import  org.apache.cactus.ServletTestCase;
import  java.net.MalformedURLException;
import  java.net.URL;
import  com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import  java.util.Hashtable;


/**
 * Test classes that need access to valid EnhydraApp implicit objects (such as the
 * the HTTP request, the HTTP response, the servlet config, ...) must subclass
 * this class.
 *
 * @author <a href="mailto:hattori@hitachizosen.co.jp">Ryuji "The Hamlet" Hattori</a>
 *
 * @version $Id: EnhydraAppTestCase.java,v 1.1 2004/08/16 09:35:01 slobodan Exp $
 */
public class EnhydraAppTestCase extends AbstractWebTestCase {
  /**
   * Valid <code>HttpPresentationComms</code> object that you can access from
   * the <code>testXXX()</code>, <code>setUp</code> and
   * <code>tearDown()</code> methods. If you try to access it from either the
   * <code>beginXXX()</code> or <code>endXXX()</code> methods it will
   * have the <code>null</code> value.
   */
  public HttpPresentationComms comms;
  /**
   * Valid <code>Application</code> object that you can access from
   * the <code>testXXX()</code>, <code>setUp</code> and
   * <code>tearDown()</code> methods. If you try to access it from either the
   * <code>beginXXX()</code> or <code>endXXX()</code> methods it will
   * have the <code>null</code> value.
   */
  public Application application;
  /**
   * Valid <code>HttpPresentationRequest</code> object that you can access from
   * the <code>testXXX()</code>, <code>setUp</code> and
   * <code>tearDown()</code> methods. If you try to access it from either the
   * <code>beginXXX()</code> or <code>endXXX()</code> methods it will
   * have the <code>null</code> value.
   */
  public HttpPresentationRequest request;
  /**
   * Valid <code>HttpPresentationResponse</code> object that you can access from
   * the <code>testXXX()</code>, <code>setUp</code> and
   * <code>tearDown()</code> methods. If you try to access it from either the
   * <code>beginXXX()</code> or <code>endXXX()</code> methods it will
   * have the <code>null</code> value.
   */
  public HttpPresentationResponse response;

  /**
   * Constructs a JUnit test case with the given name.
   *
   * @param theName the name of the test case
   */
  public EnhydraAppTestCase (String theName) {
    super(theName);
  }

  /**
   * Runs a test case. This method is overriden from the JUnit
   * <code>TestCase</code> class in order to seamlessly call the
   * Cactus redirection servlet.
   */
  protected void runTest () throws Throwable {
    runGenericTest(new EnhydraAppHttpClient());
  }

  /**
   * Overwrite Methods. because of need to handle ClinetSideRedirection
   *
   * Run the test that was specified in the constructor on the server side,
   */
  protected void runServerTest () throws Throwable {
    try {
      super.runServerTest();
    } catch (ClientPageRedirectException cpre) {
      clientSideRedirect(cpre.getUrl());
    }
  }

  private void clientSideRedirect (String url) throws Throwable {
    url = makeAbsolute(url, comms.request.getHttpServletRequest());
    hookForClientPageRedirect(url, getRedirectParameters(url));
    comms.response.setHeader("Location", url);
    comms.response.setStatus(HttpPresentationResponse.SC_MOVED_TEMPORARILY,
        "Redirected to new location.");
  }

  private String makeAbsolute (String location, HttpServletRequest request) {
    URL url = null;
    try {
      // Try making a URL out of the location
      // Throws an exception if the location is relative
      url = new URL(location);
    } catch (MalformedURLException e) {
      String requrl = HttpUtils.getRequestURL(request).toString();
      try {
        url = new URL(new URL(requrl), location);
      } catch (MalformedURLException ignored) {
        return  location;       // Give up
      }
    }
    return  url.toString();
  }

  /**
   * getting Parameter table for the url of client page redirection.
   *
   * @param url URL of Page Redirection.
   * @return HTTP parameter table for the URL of Page Redirection.<br>
   *          (Class of the Hashtable's value is <b>not String but String[]</b>)<br>
   *         if the url has no parameter, this returns null.
   */
  protected Hashtable getRedirectParameters (String url) {
    Hashtable ht = null;
    if (url.indexOf("?") > -1)
      ht = HttpUtils.parseQueryString(url.substring(url.indexOf("?") + 1));
    return  ht;
  }

  /**
   * Hook method for all Test on server side for client page redirection.
   *
   * @param url URL of Page Redirection.
   * @param ht HTTP parameter table for the URL of Page Redirection.<br>
   *          (Class of the Hashtable's value is <b>not String but String[]</b>)
   */
  protected void hookForClientPageRedirect (String url, Hashtable ht) {
    ;
  }
}



