/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Cactus", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * formatted with JxBeauty (c) johann.langhofer@nextra.at
 */


package  org.hamletsoft.enhydra.cactus.business.sample;

import  org.hamletsoft.enhydra.cactus.business.EnhydraAppTestCase;
import  org.hamletsoft.enhydra.cactus.presentation.sample.SamplePO;
import  junit.framework.Test;
import  junit.framework.TestSuite;
import  org.apache.cactus.WebResponse;
import  java.io.IOException;
import  org.apache.cactus.WebRequest;
import  java.util.Map;
import  org.apache.cactus.client.authentication.BasicAuthentication;
import  com.lutris.http.BasicAuth;
import  com.lutris.http.BasicAuthResult;
import  org.apache.log4j.PropertyConfigurator;


// import junit.ui.*;
/**
 * SampleEnhydraAppTestCase.java
 *
 *
 * Created: Thu Jan 31 11:39:48 2002
 *
 * @author <a href="mailto:hattori@hitachizosen.co.jp">Ryuji Hattori</a>
 * @version 1.0
 */
public class SampleEnhydraAppTestCase extends EnhydraAppTestCase {
  static {
    // instructs PropertyConfigurator to parse a configuration file and set up logging accordingly
    PropertyConfigurator.configure("log_client.properties");
  }

  public SampleEnhydraAppTestCase (String name) {
    super(name);
  }

  /**
   * Start the tests.
   *
   * @param theArgs the arguments. Not used
   */
  public static void main (String[] theArgs) {
    junit.awtui.TestRunner.main(new String[] {
      SampleEnhydraAppTestCase.class.getName()
    });
  }

  /*
   public static void main(String[] theArgs) {
   TestRunner.main(new String[] {
   SampleEnhydraAppTestCase.class.getName()});
   }
   */
  /**
   * @return a test suite (<code>TestSuite</code>) that includes all methods
   *         starting with "test"
   */
  public static Test suite () {
    // All methods starting with "test" will be executed in the test suite.
    return  new TestSuite(SampleEnhydraAppTestCase.class);
  }

  /**
   * Verify that we can assert the servlet output stream.
   */
  public void testReadOutputStream () throws Exception {
    SamplePO samplePO = new SamplePO();
    samplePO.run(comms);
  }

  /**
   * Verify that we can assert the servlet output stream.
   *
   * @param theResponse the response from the server side.
   */
  public void endReadOutputStream (WebResponse theResponse) throws IOException {
    String expected = SamplePO.Answer1;
    //"<html><head/><body>A GET request</body></html>";
    String result = theResponse.getText();
    assertEquals(expected, result);
  }

  /**
   * Verify that we can simulate a GET request to a PO.
   *
   * @param theRequest the request object that serves to initialize the
   *                   HTTP connection to the server redirector.
   */
  public void beginPostMethod (WebRequest theRequest) {
    theRequest.addParameter("param1", "value1", WebRequest.POST_METHOD);
  }

  /**
   * Verify that we can simulate a POST request to a servlet. Note that
   * the POST method is the default method so we don't need to initialize
   * any parameter in <code>beginPostMethod()</code>.
   */
  public void testPostMethod () throws Exception {
    SamplePO samplePO = new SamplePO();
    samplePO.run(comms);
    assertEquals("POST", samplePO.checkMethod());
  }

  //-------------------------------------------------------------------------
  /**
   * Verify that we can simulate a GET request to a PO.
   *
   * @param theRequest the request object that serves to initialize the
   *                   HTTP connection to the server redirector.
   */
  public void beginGetMethod (WebRequest theRequest) {}

  /**
   * Verify that we can simulate a GET request to a PO
   */
  public void testGetMethod () throws Exception {
    SamplePO samplePO = new SamplePO();
    samplePO.run(comms);
    assertEquals("GET", samplePO.checkMethod());
  }

  /**
   * Verify that we can simulate HTTP parameters in the HTTP request.
   *
   * @param theRequest the request object that serves to initialize the
   *                   HTTP connection to the server redirector.
   */
  public void beginSendParams (WebRequest theRequest) {
    theRequest.addParameter("param1", "value1");
    theRequest.addParameter("param2", "value2");
  }

  /**
   * Verify that we can send several parameters in the HTTP request.
   */
  public void testSendParams () throws Exception {
    SamplePO samplePO = new SamplePO();
    samplePO.run(comms);
    Map params = samplePO.getRequestParameters();
    assertTrue(params.get("param1") != null);
    assertTrue(params.get("param2") != null);
    assertEquals("value1", params.get("param1"));
    assertEquals("value2", params.get("param2"));
  }

  /**
   * Verify that we can simulate HTTP headers in the HTTP request.
   *
   * @param theRequest the request object that serves to initialize the
   *                   HTTP connection to the server redirector.
   */
  public void beginSendHeader (WebRequest theRequest) {
    theRequest.addHeader("testheader", "this is a header test");
  }

  /**
   * Verify that we can simulate HTTP headers in the HTTP request.
   */
  public void testSendHeader () throws Exception {
    SamplePO samplePO = new SamplePO();
    samplePO.run(comms);
    String headerValue = samplePO.getRequestHeader("testheader");
    assertEquals("this is a header test", headerValue);
  }

  //-------------------------------------------------------------------------
  /**
   * Verify that we can simulate a single cookie sent in the HTTP request.
   *
   * @param theRequest the request object that serves to initialize the
   *                   HTTP connection to the server redirector.
   */
  public void beginSendCookie (WebRequest theRequest) {
    // Note: The cookie value that was chosen is a string without spaces
    // because there is a problem with Resin 1.2.1 which does not support
    // quoted cookies. It has been fixed since the 15/12/2000 release of
    // Resin.
    theRequest.addCookie("testcookie", "thisisacookie");
  }

  /**
   * Verify that we can simulate a single cookie sent in the HTTP request.
   */
  public void testSendCookie () throws Exception {
    SamplePO samplePO = new SamplePO();
    samplePO.run(comms);
    Map cookies = samplePO.getRequestCookies();
    assertTrue("Cannot find [testcookie] cookie in request", cookies.get("testcookie")
        != null);
    assertEquals("thisisacookie", cookies.get("testcookie"));
  }

  /**
   * Verify that we can simulate a HTTP Basic Auth in the HTTP request.
   */
  public void beginBasicAuthentication (WebRequest theRequest) {
    theRequest.setAuthentication(new BasicAuthentication("testuser", "testpwd"));
  }

  /**
   * Verify that we can simulate a HTTP Basic Auth in the HTTP request.
   */
  public void testBasicAuthentication () throws Exception {
    BasicAuthResult authResult = BasicAuth.getAuthentication(comms.request);
    assertEquals("testuser", authResult.username);
    assertEquals("testpwd", authResult.password);
  }
}               // SampleEnhydraAppTestCase



