/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Cactus", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * formatted with JxBeauty (c) johann.langhofer@nextra.at
 */


package  org.hamletsoft.enhydra.cactus.presentation;

import  java.io.*;
import  java.lang.reflect.*;
import  java.net.*;
import  java.util.*;
import  javax.servlet.*;
import  javax.servlet.http.*;
import  java.lang.Thread;
import  org.apache.cactus.*;
// Import log4j classes.
import  org.apache.log4j.Logger;
import  org.apache.log4j.PropertyConfigurator;
// Enhydra SuperServlet imports
import  com.lutris.appserver.server.httpPresentation.HttpPresentation;
import  com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import  com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import  javax.servlet.ServletException;
import  org.hamletsoft.enhydra.cactus.spec.*;

/**
 * Generic Enhydra Super Servlet redirector that calls a test method on the server side.
 *
 * @author <a href="mailto:hattori@hitachizosen.co.jp">Ryuji The "Hamlet" Hattori</a>
 *
 * @version $Id: EnhydraAppTestRedirector.java,v 1.1 2004/08/16 09:35:03 slobodan Exp $
 * @see EnhydraAppTestCaller
 */
public class EnhydraAppTestRedirector
    implements HttpPresentation {
  /**
   * Initialize the logging subsystem so that it can get it's configuration
   * details from the correct properties file. Initialization is done here
   * as this servlet is the first point of entry to the server code.
   */
  /**
   * The logger
   */
  private static Logger logger;
  static {
    // instructs PropertyConfigurator to parse a configuration file and set up logging accordingly
    PropertyConfigurator.configure("log_server.properties");
    
    logger = Logger.getLogger(EnhydraAppTestRedirector.class.getName());
  }

  /**
   * Handle requests.
   *
   * @param comms Presentation Communication Object for Test Client.
   */
  public void run (HttpPresentationComms comms) throws HttpPresentationException,
      IOException {
    try {
      logger.info("doRun(...)");
      // Create implicit object holder
    EnhydraAppImplicitObjects objects = EnhydraAppImplicitObjectsFactory.
                 getEnhydraAppImplicitObjects("org.hamletsoft.enhydra.cactus.data.EnhydraAppImplicitObjectsImpl");
      objects.setHttpServletRequest(comms.request.getHttpServletRequest());
      objects.setHttpServletResponse(comms.response.getHttpServletResponse());
      objects.setHttpPresentationComms(comms);
      objects.setServletContext(comms.application.getHttpPresentationManager().getServletContext());
      
      
      EnhydraAppTestController controller = EnhydraAppTestControllerFactory.
            getEnhydraAppTestController("org.hamletsoft.enhydra.cactus.business.EnhydraAppTestControllerImpl");
      Thread.currentThread().sleep(5);
      controller.handleRequest(objects);
      logger.info("doPost");
    } catch (Exception e) {
      throw  new HttpPresentationException(e.getMessage(), e);
    }
  }
}



