/*
 * EncodingSampleApp 
 *
 * This type of presentation object could be the target of a form's 
 * submit button, process the data, and then decide where to send 
 * the user (back or on to the next page).
 *  
 */

package org.enhydra.encodingsample.presentation;

import org.enhydra.encodingsample.EncodingSampleSessionData;

import com.lutris.appserver.server.httpPresentation.*;

public class ClientRedirectPresentation implements HttpPresentation {
	
	private EncodingSampleSessionData sessionData = null;
	
	/*
	 * There is the only function needed in order to be a presentation object.
	 */
	public void run(HttpPresentationComms comms) throws Exception {
		initSessionData(comms);
		
		if (sessionData.getEncoding() != null) {
			comms.request.getHttpServletRequest().setCharacterEncoding(
					sessionData.getEncoding());
		}
		
		/*
		 * ClientPageRedirectException option
		 */
		ClientPageRedirectException pageRedirect = new ClientPageRedirectException(comms.request
				.getAppFileURIPath("EncodingPresentation.po"),
				sessionData.getEncoding());

		/*
		 * When using ClientPageRedirect option
		 * your parameters will get lost.
		 * Therefore, insure parameter propagation!
		 * Note that in this case you parameters will 
		 * be displayed in PO URL path (GET method) 
		 */
		String requestMethod = null;
		String requestRedirect = null;
		String requestContentType = null;
		String requestEncoding = null;
		String textParam = null;
		String requestFileEncoding = null;
		String uploadFile = null;
		String event = null;
		
		try {
			requestMethod = comms.request.getParameter("requestMethod");
			requestRedirect = comms.request.getParameter("requestRedirect");
			requestContentType = comms.request
					.getParameter("requestContentType");
			requestEncoding = comms.request.getParameter("requestEncoding");
			textParam = comms.request.getParameter("textParam");
			requestFileEncoding = comms.request
					.getParameter("requestFileEncoding");
			uploadFile = comms.request.getParameter("uploadFile");
			event = comms.request.getParameter("event");
		} catch (HttpPresentationException ex) {
			requestMethod = null;
			requestRedirect = null;
			requestContentType = null;
			requestEncoding = null;
			textParam = null;
			requestFileEncoding = null;
			uploadFile = null;
			event = null;
		}
		
		if (requestMethod != null && !"".equals(requestMethod) && !"null".equals(requestMethod))
			pageRedirect.addArgument("requestMethod", requestMethod);
		
		if (requestRedirect != null && !"".equals(requestRedirect) && !"null".equals(requestRedirect))
			pageRedirect.addArgument("requestRedirect", requestRedirect);
		
		if (requestContentType != null && !"".equals(requestContentType) && !"null".equals(requestContentType))
			pageRedirect.addArgument("requestContentType", requestContentType);
		
		if (requestEncoding != null && !"".equals(requestEncoding) && !"null".equals(requestEncoding))
			pageRedirect.addArgument("requestEncoding", requestEncoding);
		
		if (textParam != null && !"".equals(textParam) && !"null".equals(textParam))
			pageRedirect.addArgument("textParam", textParam);
		
		if (requestFileEncoding != null && !"".equals(requestFileEncoding) && !"null".equals(requestFileEncoding))
			pageRedirect.addArgument("requestFileEncoding", requestFileEncoding);
		
		if (uploadFile != null && !"".equals(uploadFile))
			pageRedirect.addArgument("uploadFile", uploadFile);
		
		if (event != null && !"".equals(event))
			pageRedirect.addArgument("event", event);
		
		throw pageRedirect;
	}
	
	/**
	 * Method to get or create the SessionData object from the user session
	 * 
	 * @param comms
	 *            HttpPresentationComms
	 */
	protected void initSessionData(HttpPresentationComms comms) {
		try {
			Object obj = comms.sessionData
					.get(EncodingSampleSessionData.SESSION_KEY);
			// If we found the session data, save it in a private data member
			if (null != obj) {
				this.sessionData = (EncodingSampleSessionData) obj;
			} else {
				// If no session data was found, create a new session data
				// instance
				this.sessionData = new EncodingSampleSessionData();
				comms.sessionData.set(EncodingSampleSessionData.SESSION_KEY,
						this.sessionData);
			}
		} catch (Exception ex) {
			this.sessionData = new EncodingSampleSessionData();
		}
	}

}
