/*
 *  EncodingSampleApp
 *
 *  Enhydra super-servlet presentation object
 *  
 */

package org.enhydra.encodingsample.presentation;

// Enhydra SuperServlet imports
import com.lutris.appserver.server.httpPresentation.HttpPresentation;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.mime.ContentHeader;
import com.lutris.mime.MimeException;
import com.lutris.mime.MimeHeader;
import com.lutris.mime.MultipartMimeInput;
import com.lutris.mime.MultipartMimeInputStream;

// Standard imports
import java.io.IOException;

import org.enhydra.encodingsample.EncodingSampleApp;
import org.enhydra.encodingsample.EncodingSampleSessionData;
import org.w3c.dom.Node;
import org.w3c.dom.html.HTMLFormElement;
import org.w3c.dom.html.HTMLOptionElement;
import org.w3c.dom.html.HTMLSelectElement;
import org.w3c.dom.html.HTMLTableElement;
import org.w3c.dom.html.HTMLTableRowElement;

public class EncodingPresentation implements HttpPresentation {
	private EncodingSampleSessionData sessionData = null;

	public void run(HttpPresentationComms comms)
			throws HttpPresentationException, IOException {

		initSessionData(comms);

		String[] encodings = EncodingSampleApp.encodings;

		EncodeHTML encodePage;
		encodePage = (EncodeHTML) comms.xmlcFactory.create(EncodeHTML.class);

		if (sessionData.getEncoding() != null) {
			comms.request.getHttpServletRequest().setCharacterEncoding(
					sessionData.getEncoding());
		}

		String requestMethod = null;
		String requestRedirect = null;
		String requestContentType = null;
		String requestEncoding = sessionData.getEncoding();
		String textParam = null;
		String requestFileEncoding = sessionData.getFileEncoding();
		String event = null;

		String filePath = null;
		String fileContent = null;

		// PARAMETER INITIALIZATION
		if ("POST".equalsIgnoreCase(comms.request.getMethod())
				&& !"application/x-www-form-urlencoded"
						.equalsIgnoreCase(comms.request.getContentType())) {

			String CONTENT_TYPE = "Content-Type";
			String CONTENT_DISPOSITION = "Content-Disposition";
			try {
				String header = comms.request.getHeader(CONTENT_TYPE);
				if (header != null && !"".equals(header)) {
					ContentHeader contentHeader = new ContentHeader(header);
					MultipartMimeInput mInput = new MultipartMimeInput(
							comms.request.getInputStream(), contentHeader);

					MultipartMimeInputStream mime = mInput
							.nextPart(requestEncoding);
					while (mime != null) {
						MimeHeader mh = mime.getHeader(CONTENT_DISPOSITION);
						String paramName = mh.getParameter("name");
						if ("uploadFile".equals(paramName)) {
							filePath = mh.getParameter("filename");
							fileContent = getString(mime, requestFileEncoding);
						} else if ("requestMethod".equals(paramName)) {
							requestMethod = getString(mime, requestEncoding);
						} else if ("requestRedirect".equals(paramName)) {
							requestRedirect = getString(mime, requestEncoding);
						} else if ("requestContentType".equals(paramName)) {
							requestContentType = getString(mime,
									requestEncoding);
						} else if ("requestEncoding".equals(paramName)) {
							requestEncoding = getString(mime, requestEncoding);
							if (requestEncoding != null
									&& !"".equals(requestEncoding)
									&& !"null".equals(requestEncoding)) {
								sessionData.setEncoding(requestEncoding);
							} else {
								requestEncoding = sessionData.getEncoding();
							}
						} else if ("textParam".equals(paramName)) {
							textParam = getString(mime, requestEncoding);
						} else if ("requestFileEncoding".equals(paramName)) {
							requestFileEncoding = getString(mime,
									requestEncoding);
							if (requestFileEncoding != null
									&& !"".equals(requestFileEncoding)
									&& !"null".equals(requestFileEncoding)) {
								sessionData
										.setFileEncoding(requestFileEncoding);
							} else {
								requestFileEncoding = sessionData
										.getFileEncoding();
							}
						} else if ("event".equals(paramName)) {
							event = getString(mime, requestEncoding);
						}
						mime = mInput.nextPart(requestEncoding);
					}
				}
			} catch (MimeException mexc) {
				mexc.printStackTrace();
			}
		} else {
			try {
				requestMethod = comms.request.getParameter("requestMethod");
				requestRedirect = comms.request.getParameter("requestRedirect");
				requestContentType = comms.request
						.getParameter("requestContentType");
				requestEncoding = comms.request.getParameter("requestEncoding");
				if (requestEncoding != null && !"".equals(requestEncoding)
						&& !"null".equals(requestEncoding)) {
					sessionData.setEncoding(requestEncoding);
				} else {
					requestEncoding = sessionData.getEncoding();
				}
				textParam = comms.request.getParameter("textParam");
				requestFileEncoding = comms.request
						.getParameter("requestFileEncoding");
				if (requestFileEncoding != null
						&& !"".equals(requestFileEncoding)
						&& !"null".equals(requestFileEncoding)) {
					sessionData.setFileEncoding(requestFileEncoding);
				} else {
					requestFileEncoding = sessionData.getFileEncoding();
				}
				event = comms.request.getParameter("event");
			} catch (HttpPresentationException ex) {
				requestMethod = null;
				requestRedirect = null;
				requestContentType = null;
				requestEncoding = null;
				textParam = null;
				requestFileEncoding = null;
				event = null;
				ex.printStackTrace();
			}
		}

		// REQUEST TABLE PREPARATIONS
		HTMLOptionElement encodingOption = encodePage
				.getElementRequestEncodingOption();
		HTMLOptionElement fileEncodingOption = encodePage
				.getElementRequestFileEncodingOption();

		for (int i = 0; i < encodings.length; i++) {
			HTMLOptionElement clonedOption = (HTMLOptionElement) encodingOption
					.cloneNode(true);
			HTMLOptionElement clonedFileOption = (HTMLOptionElement) encodingOption
					.cloneNode(true);
			
			Node optionTextNode = clonedOption.getOwnerDocument()
			.createTextNode(encodings[i]);

			clonedOption.appendChild(optionTextNode.cloneNode(true));
			clonedOption.setValue(encodings[i]);
			
			clonedFileOption.appendChild(optionTextNode);
			clonedFileOption.setValue(encodings[i]);

			// clonedOption.setSelected(encodings[i].equals(requestEncoding));
			if (encodings[i].equals(requestEncoding))
				clonedOption.setAttribute("selected", "selected");

			encodingOption.getParentNode().appendChild(
					clonedOption);

			// clonedFileOption.setSelected(encodings[i].equals(requestFileEncoding));
			if (encodings[i].equals(requestFileEncoding))
				clonedFileOption.setAttribute("selected", "selected");

			fileEncodingOption.getParentNode().appendChild(clonedFileOption);
		}
		encodingOption.getParentNode().removeChild(encodingOption);
		fileEncodingOption.getParentNode().removeChild(fileEncodingOption);

		HTMLFormElement requestForm = encodePage.getElementRequestForm();

		HTMLSelectElement redirectSelection = encodePage
				.getElementRequestRedirect();
		if ("SERVER".equals(requestRedirect)) {
			redirectSelection.setSelectedIndex(2);
			requestForm.setAction("ServerRedirectPresentation.po");
		} else if ("CLIENT".equals(requestRedirect)) {
			redirectSelection.setSelectedIndex(1);
			requestForm.setAction("ClientRedirectPresentation.po");
		} else {
			redirectSelection.setSelectedIndex(0);
		}

		// DISABLE CLIENT REDIRECTION IF POST IS REQUESTED METHOD
		if ("POST".equals(requestMethod)) {
			HTMLOptionElement clientRedirectOption = encodePage
					.getElementClientRedirectOption();
			clientRedirectOption.getParentNode().removeChild(
					clientRedirectOption);
		}

		if ("POST".equals(requestMethod)) {
			// POST
			requestForm.setMethod("POST");

			HTMLSelectElement methodSelection = encodePage
					.getElementRequestMethod();
			methodSelection.setSelectedIndex(1);

			if ("multipart".equals(requestContentType)) {
				requestForm.setEnctype("multipart/form-data");

				HTMLSelectElement contentSelection = encodePage
						.getElementRequestContentType();
				contentSelection.setSelectedIndex(1);

				HTMLTableRowElement tr = encodePage.getElementParamRow();
				tr.getParentNode().removeChild(tr);
			} else {
				requestForm.setEnctype("application/x-www-form-urlencoded");

				HTMLTableRowElement tr = encodePage.getElementFileEncodingRow();
				tr.getParentNode().removeChild(tr);
				tr = encodePage.getElementFileRow();
				tr.getParentNode().removeChild(tr);
			}
		} else {
			// GET
			requestForm.setMethod("GET");

			HTMLSelectElement methodSelection = encodePage
					.getElementRequestMethod();
			methodSelection.setSelectedIndex(0);

			HTMLTableRowElement tr = encodePage.getElementContentTypeRow();
			tr.getParentNode().removeChild(tr);
			tr = encodePage.getElementFileEncodingRow();
			tr.getParentNode().removeChild(tr);
			tr = encodePage.getElementFileRow();
			tr.getParentNode().removeChild(tr);

		}

		// RESULT TABLE PREPARATIONS
		if ("submit".equals(event)) {
			if (filePath != null && fileContent != null) {
				setFileDisplay(encodePage, filePath, requestEncoding,
						fileContent, requestFileEncoding);
			} else if (textParam != null) {
				setParameterDisplay(encodePage, textParam, requestEncoding);
			} else {
				setDefaultDisplay(encodePage);
			}
		} else {
			setDefaultDisplay(encodePage);
		}

		if (sessionData.getEncoding() != null) {
			comms.response.setEncoding(sessionData.getEncoding());
		}

		comms.response.writeDOM(encodePage);
	}

	private String getString(MultipartMimeInputStream part, String ecoding)
			throws IOException {
		byte[] buf = new byte[1024];
		StringBuffer res = new StringBuffer(1024);
		int b = 0;
		while ((b = part.read(buf, 0, buf.length)) != -1) {
			try {
				res.append(new String(buf, 0, b, ecoding));
			} catch (Exception exc) {
				res.append(new String(buf, 0, b));
			}
		}
		return res.toString();
	}

	private void setParameterDisplay(EncodeHTML encodePage, String parameter,
			String encoding) {
		encodePage.setTextParamVal(parameter);
		encodePage.setTextEncodingVal(encoding);

		HTMLTableRowElement contentRow = encodePage
				.getElementResultContentRow();
		contentRow.getParentNode().removeChild(contentRow);

		HTMLTableRowElement fileRow = encodePage.getElementResultFileRow();
		fileRow.getParentNode().removeChild(fileRow);

		HTMLTableRowElement fileEncodingRow = encodePage
				.getElementResultFileEncodingRow();
		fileEncodingRow.getParentNode().removeChild(fileEncodingRow);
	}

	private void setFileDisplay(EncodeHTML encodePage, String file,
			String encoding, String content, String fileEncoding) {
		encodePage.setTextFilePathVal(file);
		encodePage.setTextFileContentVal(content);
		encodePage.setTextEncodingVal(encoding);
		encodePage.setTextFileEncodingVal(fileEncoding);
		HTMLTableRowElement paramRow = encodePage.getElementResultParamRow();
		paramRow.getParentNode().removeChild(paramRow);
	}

	private void setDefaultDisplay(EncodeHTML encodePage) {
		HTMLTableElement table = encodePage.getElementResultTable();
		table.getParentNode().removeChild(table);
		table = encodePage.getElementResultHead();
		table.getParentNode().removeChild(table);
	}

	/**
	 * Method to get or create the SessionData object from the user session
	 * 
	 * @param comms
	 *            HttpPresentationComms
	 */
	protected void initSessionData(HttpPresentationComms comms) {
		try {
			Object obj = comms.sessionData
					.get(EncodingSampleSessionData.SESSION_KEY);
			// If we found the session data, save it in a private data member
			if (null != obj) {
				this.sessionData = (EncodingSampleSessionData) obj;
			} else {
				// If no session data was found, create a new session data
				// instance
				this.sessionData = new EncodingSampleSessionData();
				comms.sessionData.set(EncodingSampleSessionData.SESSION_KEY,
						this.sessionData);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			this.sessionData = new EncodingSampleSessionData();
		}
	}

}
