package fops.business;

import fops.spec.RenderManager;

import org.apache.fop.apps.XSLTInputHandler;
import org.apache.avalon.framework.logger.Logger;
import org.apache.fop.apps.Driver;
import org.apache.fop.apps.Options;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import java.io.*;

public class RenderManagerImpl implements RenderManager{

 /**
     * renders a HTML fields and Applet contents inputsource
     * into a PDF file which is rendered directly to the response
     * object's OutputStream
     */
  public byte[] renderFO(boolean noParams,String DateHTML,String NameHTML,String SubjHTML,String AppletHTML,Logger log) throws Exception {
         
		ByteArrayOutputStream out = new ByteArrayOutputStream();
        
				
		
		StringReader str;
		if(noParams)
			str = defaultText();
		else
			str = inputText(DateHTML,NameHTML,SubjHTML,AppletHTML);
		Driver driver = new Driver(new InputSource(str), out);
		driver.setLogger(log);
		driver.setRenderer(Driver.RENDER_PDF);

		driver.run();

		byte[] content = out.toByteArray();
		
		return content;
		
		   }

  public byte[] renderXML(String xmlParam,String xslParam,Logger log) throws Exception {
        
        XSLTInputHandler input = new XSLTInputHandler(new File(xmlParam), new File(xslParam));
		ByteArrayOutputStream out = new ByteArrayOutputStream();

		
		Driver driver = new Driver();
		driver.setLogger(log);
		driver.setRenderer(Driver.RENDER_PDF);
		driver.setOutputStream(out);
		driver.render(input.getParser(), input.getInputSource());

		byte[] content = out.toByteArray();
		
		return content;
		
		
   }

   /**
    * renders an FO inputsource into a PDF file which is rendered
    * directly to the response object's OutputStream
    */
   public byte[] renderFOFile(String foParam,Logger log) throws Exception {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
        FileInputStream foFile = new FileInputStream(foParam);
		
		Driver driver = new Driver();
		driver.setLogger(log);
		driver.setRenderer(Driver.RENDER_PDF);
		driver.setInputSource(new InputSource(foFile));
		driver.setOutputStream(out);
		driver.run();

		byte[] content = out.toByteArray();
		
		return content;
		
		
   }

    /**
     * creates a SAX parser, using the value of org.xml.sax.parser
     * defaulting to org.apache.xerces.parsers.SAXParser
     *
     * @return the created SAX parser
     */
   static XMLReader createParser() throws Exception {
		String parserClassName = System.getProperty("org.xml.sax.parser");

		if (parserClassName == null) {
			parserClassName = "org.apache.xerces.parsers.SAXParser";
		}
		return (XMLReader) Class.forName(parserClassName).newInstance();
   }

    /**
     * Method inputText
     * Creates a XSLFO file with inserted parameters from HTML.
     * Noncomptibility: FOP 0.19 -     <fo:page-sequence master-name="simple">
     *                  FOP 0.20.4 -   <fo:page-sequence master-reference="simple">
     * @return   a StringReader
     *
     */
	public StringReader inputText(String DateHTML,String NameHTML,String SubjHTML,String AppletHTML) {
      String text = new String();
      text += "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
      text += "<fo:root xmlns:fo=\"http://www.w3.org/1999/XSL/Format\">\n";
      text += "  <fo:layout-master-set>\n";
      text += "     <fo:simple-page-master master-name=\"simple\" page-height=\"29.7cm\" page-width=\"21cm\" " + "margin-top=\"1cm\" margin-bottom=\"2cm\" margin-left=\"2.5cm\" margin-right=\"2.5cm\">\n";
      text += "        <fo:region-body margin-top=\"3cm\"/>\n";
      text += "        <fo:region-before extent=\"3cm\"/>\n";
      text += "        <fo:region-after extent=\"1.5cm\"/>\n";
      text += "     </fo:simple-page-master>\n";
      text += "  </fo:layout-master-set>\n";
      text += "  <fo:page-sequence master-reference=\"simple\">\n";
      text += "     <fo:flow flow-name=\"xsl-region-body\">\n";
      text += "        <fo:block font-size=\"18pt\" font-family=\"sans-serif\" line-height=\"24pt\" " + "space-after.optimum=\"15pt\" background-color=\"blue\" color=\"white\" " + "text-align=\"center\" padding-top=\"3pt\">\n";
      text += "           Demo PDF file from applet \n";
      text += "        </fo:block>\n";
      text += "        <fo:table font-family=\"Helvetica\" space-after.optimum=\"20pt\">\n";
      text += "           <fo:table-column column-width=\"20mm\" />\n";
      text += "           <fo:table-column column-width=\"150mm\" />\n";
      text += "           <fo:table-body>\n";
      text += "              <fo:table-row>\n";
      text += "                 <fo:table-cell>\n";
      text += "                       <fo:block color=\"blue\" font-weight=\"bold\">Date:</fo:block>\n";
      text += "                 </fo:table-cell>\n";
      text += "                 <fo:table-cell>\n";
      text += "                    <fo:block>" + DateHTML + "</fo:block>\n";
      text += "                 </fo:table-cell>\n";
      text += "              </fo:table-row>\n";
      text += "              <fo:table-row>\n";
      text += "                 <fo:table-cell>\n";
      text += "                    <fo:block color=\"blue\" font-weight=\"bold\">Name:</fo:block>\n";
      text += "                 </fo:table-cell>\n";
      text += "                 <fo:table-cell>\n";
      text += "                    <fo:block>" + NameHTML + "</fo:block>\n";
      text += "                 </fo:table-cell>\n";
      text += "              </fo:table-row>\n";
      text += "              <fo:table-row>\n";
      text += "                 <fo:table-cell>\n";
      text += "                    <fo:block color=\"blue\" font-weight=\"bold\">Subject:</fo:block>\n";
      text += "                 </fo:table-cell>\n";
      text += "                 <fo:table-cell>\n";
      text += "                    <fo:block>" + SubjHTML + "</fo:block>\n";
      text += "                 </fo:table-cell>\n";
      text += "              </fo:table-row>\n";
      text += "           </fo:table-body>\n";
      text += "        </fo:table>\n";
      text += AppletHTML;
      text += "     </fo:flow>\n";
      text += "  </fo:page-sequence>\n";
      text += "</fo:root>\n";

      StringReader foText = new StringReader(text);

      return foText;
   }
   
   public StringReader defaultText(){
      String text = new String();
      text += "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
      text += "<fo:root xmlns:fo=\"http://www.w3.org/1999/XSL/Format\">\n";
      text += "  <fo:layout-master-set>\n";
      text += "     <fo:simple-page-master master-name=\"simple\" page-height=\"29.7cm\" page-width=\"21cm\" " + "margin-top=\"1cm\" margin-bottom=\"2cm\" margin-left=\"2.5cm\" margin-right=\"2.5cm\">\n";
      text += "        <fo:region-body margin-top=\"3cm\"/>\n";
      text += "        <fo:region-before extent=\"3cm\"/>\n";
      text += "        <fo:region-after extent=\"1.5cm\"/>\n";
      text += "     </fo:simple-page-master>\n";
      text += "  </fo:layout-master-set>\n";
      text += "  <fo:page-sequence master-reference=\"simple\">\n";
      text += "     <fo:flow flow-name=\"xsl-region-body\">\n";
      text += "        <fo:block font-size=\"18pt\" font-family=\"sans-serif\" line-height=\"24pt\" " + "space-after.optimum=\"15pt\" background-color=\"blue\" color=\"white\" " + "text-align=\"center\" padding-top=\"3pt\">\n";
      text += "           Demo PDF file \n";
      text += "        </fo:block>\n";
      text += "        <fo:block font-size=\"12pt\" font-family=\"sans-serif\" line-height=\"18pt\" space-after.optimum=\"3pt\" text-align=\"center\" white-space-collapse=\"false\" language=\"en_US\" hyphenate=\"true\">\n";
      text += "           <fo:inline font-family=\"sans-serif\" font-size=\"18pt\" font-weight=\"bold\" >There is no any request parameter given!!!</fo:inline>\n";
      text += "        </fo:block>\n";
      text += "        <fo:block font-size=\"12pt\" font-family=\"sans-serif\" line-height=\"18pt\" space-after.optimum=\"3pt\" text-align=\"center\" white-space-collapse=\"false\" language=\"en_US\" hyphenate=\"true\">\n";
      text += "           <fo:inline font-family=\"sans-serif\" font-size=\"16pt\" ></fo:inline>\n";
      text += "        </fo:block>\n";
      text += "        <fo:block font-size=\"12pt\" font-family=\"sans-serif\" line-height=\"18pt\" space-after.optimum=\"3pt\" text-align=\"center\" white-space-collapse=\"false\" language=\"en_US\" hyphenate=\"true\">\n";
      text += "           <fo:inline font-family=\"sans-serif\" font-size=\"12pt\" >Please, see documentation for details.</fo:inline>\n";
      text += "        </fo:block>\n";
      text += "     </fo:flow>\n";
      text += "  </fo:page-sequence>\n";
      text += "</fo:root>\n";
   	
      StringReader defaultText = new StringReader(text);

      return defaultText;
   }
}