/**
 * Applet TextEditor
 * <P>
 * @authors: Stefanovic Nenad, Puskas Vladimir, Bojanic Sasa
 * @Date:   July-November 2001.
 */

import java.io.StringReader;
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.beans.*;
import java.net.URL;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import java.io.*;




/** This applet class is used as a <b>text editor</b> that can return a
 * <b>String</b> that represent a content of applet and is already formatted
 * as a XSL:FO file. That String can be transformed into PDF file, which
 * is achived by the use of FOP (FOP is Java application that reads a form
 * of an XSL:FO and then turns it into a PDF document.).<p>
 * Applet's text can be formatted in a different ways - every character can
 * have their own font family and size and different style (bold, italic,
 * underline), and every paragraph can have different alignment (left, center,
 * right). Content of applet is transformed in a form of XSL:FO formatted string
 * by calling applet's method FormatFO().<p>
 * It is important to mention that applet's look and functionality is fully
 * determined by the property files.
*/
 public class TextEditor extends JApplet{
   int cnt=0;
   /** String to use in methods accessing the properties file */
   public static final String _fontfamily = "fontfamily";
   /** String to use in methods accessing the properties file */
   public static final String _fontsize = "fontsize";
   /** String to use in methods accessing the properties file */
   public static final String _family = "family";
   /** String to use in methods accessing the properties file */
   public static final String _size = "size";
   /** String to use in methods accessing the properties file */
   public static final String _label = "Label";
   /** String to use in methods accessing the properties file */
   public static final String _action = "Action";
   /** String to use in methods accessing the properties file */
   public static final String _tooltip = "Tooltip";
   /** Limit on items in combobox */
   public static final int _maxComboItems = 20;

   /** Applet's toolbar */
   private JToolBar toolbar;
   /** Applet's menubar */
   private JMenuBar MenuBar;
   /** Font family combo box */
   private JComboBox comboFontFamilies;
   /** Font size combo box */
   private JComboBox comboFontSizes;
   /** Action Listener for combo boxes */
   private ActionListener comboFontListener=new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        comboFontActions(e);
      }
   };
	/** Action Listener for toggle buttons */
   private ActionListener toggleButtonListener=new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        toggleButtonActions(e);
      }
   };
   public static final String copyExtAction = "copyExt";
   /** AttributeSet used to correct behaviour of new paragraph */
   private SimpleAttributeSet currentAS;
   /** boolean used to correct behaviour of new paragraph */
   private boolean isEnterPressed=false;
   /** boolean used to correct behaviour of new paragraph */
   private boolean isBSValid=false;
   /** An instance of text editor */
   private MyTextPane editor;
   /** Contains possible editor actions */
   private Hashtable commands;
  /** Declaration of "main" resource file */
   private static ResourceBundle resources;
   /** Texts for "language" default resource bundle **/
   private static ResourceBundle resdef;
   /** Texts for resource bundle **/
   private static ResourceBundle resText;
   /** Name of resource(property) file */
   String res;
   /** String for identifing using language **/
   String resLan;
   /** Editor's document root elements (section and paragraphs) */
   private Element[] rootElements;
	/** Popup menu for TextEditor**/
   private JPopupMenu PopUp;
   /** Toolbar's toggle button for getting bold character style */
   private JToggleButton boldButton;
   /** Toolbar's toggle button for getting italic character style */
   private JToggleButton italicButton;
   /** Toolbar's toggle button for getting underline character style */
   private JToggleButton underButton;
   /** Toolbar's toggle button for getting left paragraph alignment */
   private JToggleButton leftButton;
   /** Toolbar's toggle button for getting center paragraph alignment */
   private JToggleButton centerButton;
   /** Toolbar's toggle button for getting right paragraph alignment */
   private JToggleButton rightButton;
   /** Group for togglebuttons left,cenetr,right */
   private ButtonGroup group=new ButtonGroup();
   /**
    * Predefined default line-height value used when formating applet context
    * into a form of XSL:FO format. If there is no such value in property file
    * this one will be used
    */
   private String defLineHeight="18";
   /**
    * Predefined default language specification value for hyphenation. If there is no
    * such value in property file, this one will be used
    */
   private String defHyph="en";
   // There are some predefined default font values if no one exist in property file.
   /**
    * Predefined default font family value. Possible values for fontfamily are:
    * 'SansSerif'(Helvetica), 'Serif'(TimesRoman) and 'Monospaced'(Courier).
    */
   private String defFontFamily="Serif";
   /** Predefined default font size value. Possible values for font size are:
   * 8, 10, 12, 14, 16, 18, 24, 36 and 48.
   */
   private String defFontSize="16";
   /** Predefined default bold font style value. Possible values are true and false */
   private boolean defbold=false;
   /** Predefined default italic font style value. Possible values are true and false */
   private boolean defitalic=false;
   /** Predefined default underline font style value. Possible values are true and false */
   private boolean defunder=false;
   /**
    * Returns the value of parameter <b>key</b> in the HTML tag, or
    * default value <b>def</b> if doesn't find it. For example, if
    * HTML document referencing this applet contains line:<BR>
    * <CENTER>&ltPARAM NAME="Resource" VALUE="TextEditor"&gt</CENTER>
    * then a call to getParameter("Resource","TEX") returns the value
    * "TextEditor". If this line doesn't exist in HTML document,
    * the same call to this method will return the value "TEX".
    * @param key a parameter name
    * @param def default value for the parameter
    * @return the value of parameter, or default value if not set
    */
   public String getParameter(String key, String def) {
      if (getParameter(key) != null) {
         return getParameter(key);
      }
      return def;
   }

   /** There's nothing to do within the constructor */
   public TextEditor() {
   }
   /**
    * Initializes the state of this instance. Gets the name of the "main" property file
    * to use and language to use from HTML tag, sets the 'look and feel' of applet
    * to the system one, gets the "main" and "language" property files declaration,
    * and calls the function jbInit for further creation and initialization of the applet.
    */
   public void init() {
      // taking parametars from HTML document
      res = getParameter("Resource", "TextEditor");
      resLan=getParameter("Language","");
      // setting 'look and feel'
      try {
         UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      }
      catch(Exception e) {
         e.printStackTrace();
      }
      // reading property files
      try {
         resources = ResourceBundle.getBundle(res);
      }
      catch (MissingResourceException MRe) {
         System.err.println(res+".properties not found");
         System.exit(0);
      }
		try {
         resdef=ResourceBundle.getBundle("Text");
      }
      catch (MissingResourceException MRe) {
         System.err.println("Text.properties not found");
         System.exit(0);
      }
      try {
      	resText=ResourceBundle.getBundle("Text_"+resLan);
      }
      catch (MissingResourceException MRe) {
         resText=ResourceBundle.getBundle("Text");
         System.err.println("Text_"+resLan+".properties not found, using default Text.properties !!!");
      }
      super.init();
      try  {
         jbInit();
      }
      catch (Exception e) {
         e.printStackTrace();
      }

   }
   /** Main method for creating and initializing applet. Creates the
    * Editor, adds him a Mouse, Caret and Key listener. Sets Editor's
    * look and menubar and toolbar
   */
   protected void jbInit() throws Exception {
      // Dimensions of applet
      setSize(new Dimension(600, 400));
      // Seting up for main jPanel
      JPanel Glavni = new JPanel();
      Glavni.setBorder(BorderFactory.createEtchedBorder());
      Glavni.setLayout(new BorderLayout());
      //Creating a editor for TextEditor
      editor=createEditor();
      editor.addMouseListener(new PopMouseListener());
      editor.addCaretListener(new CaretListener(){
         public void caretUpdate(CaretEvent e) {
            caretChange(e);
         }
      });
      editor.addKeyListener(new KeyAdapter(){
         public void keyPressed(KeyEvent e){
            int keyCode=e.getKeyCode();
            if ((keyCode==KeyEvent.VK_ENTER) || (keyCode==KeyEvent.VK_BACK_SPACE)) {
               StyledDocument theDocument = getEditor().getStyledDocument();
               int dot = getEditor().getCaret().getDot();
               //current paragraph
               Element prgel=theDocument.getParagraphElement(dot);
               // start and end of current paragraph
               int stof=prgel.getStartOffset();
               int enof=prgel.getEndOffset();

               // if back space is pressed at the place that will not empty the
               // paragraph, return
               if (keyCode==KeyEvent.VK_BACK_SPACE && !(dot==stof+1 && enof==stof+2)) {
                  return;
               }

               // current paragraph and character attributes
               AttributeSet cpas=getEditor().getParagraphAttributes();
               AttributeSet ccas=editor.getInputAttributes();

               currentAS=new SimpleAttributeSet(cpas);

               StyleConstants.setFontFamily(currentAS,StyleConstants.getFontFamily(ccas));
               StyleConstants.setFontSize(currentAS,StyleConstants.getFontSize(ccas));
               StyleConstants.setBold(currentAS,StyleConstants.isBold(ccas));
               StyleConstants.setItalic(currentAS,StyleConstants.isItalic(ccas));
               StyleConstants.setUnderline(currentAS,StyleConstants.isUnderline(ccas));

               if (keyCode==KeyEvent.VK_ENTER) {
                  isEnterPressed=true;
               }
               if (keyCode==KeyEvent.VK_BACK_SPACE) {
                  isBSValid=true;
               }

               // if Enter is pressed at an empty paragraph, change it's setting
               if ((dot==stof) && (enof-stof==1) && (keyCode==KeyEvent.VK_ENTER)) {
                  editor.setParagraphAttributes(currentAS,true);
               }
            }
         }

         public void keyTyped(KeyEvent e){
            int keyCode=e.getKeyChar();
            // in the case of 'backspacing' to the begining, attribute
            // setup mustn't change to something different
            if (keyCode==KeyEvent.VK_BACK_SPACE) {
               if (isBSValid) {
                  int dot = getEditor().getCaret().getDot();
                  StyledDocument theDocument = getEditor().getStyledDocument();
                  //current paragraph
                  Element prgel=theDocument.getParagraphElement(dot);
                  // start and end of current paragraph
                  int stof=prgel.getStartOffset();
                  int enof=prgel.getEndOffset();
                  if (enof > theDocument.getLength()) {
                    enof=theDocument.getLength();
                  }
                  editor.moveCaretPosition(enof);
                  editor.setParagraphAttributes(currentAS,true);
                  editor.setCharacterAttributes(currentAS,true);
                  editor.moveCaretPosition(dot);
                  isBSValid=false;
               }
            }

            if (keyCode==KeyEvent.VK_ENTER && isEnterPressed==true) {
               editor.setParagraphAttributes(currentAS,true);
               editor.setCharacterAttributes(currentAS,true);
               isEnterPressed=false;
            }
         }

      });
      // Install the command table in Hashtable
      commands = new Hashtable();
      Action[] actions = getActions();
      for (int i = 0; i < actions.length; i++) {
         Action a = actions[i];
         commands.put(a.getValue(Action.NAME), a);
      }
      JScrollPane scroll = new JScrollPane();
      JViewport port = scroll.getViewport();
      port.add(editor);
      if (getResourceString("menubar")!=null)
         MenuBar=createMenuBar();
      JPanel panel=new JPanel();
      Glavni.add(scroll,BorderLayout.CENTER);
      if (getResourceString("toolbar")!=null)
         Glavni.add(createToolbar(),BorderLayout.NORTH);
      //Adding a JMenuBar to the applet
      this.setJMenuBar(MenuBar);
      this.getContentPane().add(Glavni);
      setContentPane(Glavni);
      editor.requestFocus();
   }
   /**
    * Creates an editor for TextEditor applet. Sets the default values of fonts
    * and colors for editor.
    * @return created editor (instance of class MyTextPane).
    */
   protected MyTextPane createEditor(){
      editor=new MyTextPane();
      // Setting a default values for document
		StyleContext sc = new StyleContext();
      DefaultStyledDocument doc = new DefaultStyledDocument(sc);
      editor.setStyledDocument(doc);

      // Setting a default values for editor
      setDefaultFonts();
      setDefaultColors();

      // Setting default style to match default values
      Style def = sc.getStyle(StyleContext.DEFAULT_STYLE);
      StyleConstants.setFontFamily(def,defFontFamily);
      StyleConstants.setFontSize(def,Integer.parseInt(defFontSize));
      StyleConstants.setBold(def,defbold);
      StyleConstants.setItalic(def,defitalic);
      StyleConstants.setUnderline(def,defunder);

      return editor;
   }
   /**
    * Setting up a default font values for editor. The default values are
    * read from "main" property file. If no default values is defined in
    * the resource file, there are some predefined values.
    */
   protected void setDefaultFonts(){
      MutableAttributeSet defAtt=editor.getInputAttributes();
      try{
         String[] defKeys = tokenize(getResourceString("defvalues"));
         String[] keyval=new String[defKeys.length];
         for (int i = 0; i < defKeys.length; i++) {
            keyval[i]=getResourceString(defKeys[i]);
            if (defKeys[i].equals("defFontFamily")){
               String ff=keyval[i].toString();
               if (validateDefaults(_family,ff))
                  defFontFamily=ff;
            }
            else if (defKeys[i].equals("defFontSize")){
               String fs=keyval[i].toString();
               if (validateDefaults(_size,fs))
                  defFontSize=fs;
            }
            else if (defKeys[i].equals("defbold")){
               String bld=keyval[i].toString();
               if (bld.equals("true"))
                  defbold=true;
            }
            else if (defKeys[i].equals("defitalic")){
               String itl=keyval[i].toString();
               if (itl.equals("true"))
                  defitalic=true;
            }
            else if (defKeys[i].equals("defunder")){
               String und=keyval[i].toString();
               if (und.equals("true"))
                  defunder=true;
            }
         }
      }
      catch(Exception e){
         System.err.println("There is no default values for editor in property file");
      }
      int defFSi=Integer.parseInt(defFontSize);
      // Defining fontfamily
      StyleConstants.setFontFamily(defAtt,defFontFamily);
      // Defining fontsize
      StyleConstants.setFontSize(defAtt, defFSi);
      // Defining fontstyle
      StyleConstants.setBold(defAtt,defbold);
      StyleConstants.setItalic(defAtt,defitalic);
      StyleConstants.setUnderline(defAtt,defunder);
   	// reading value for line-height
   	String strLH=getResourceString("lineHeight");
   	int intLH;
   	// some checking
   	try {
   		intLH=Integer.parseInt(strLH);
   		if (intLH>=8 && intLH<=96) {
   			defLineHeight=strLH;
   		}
   		else {
   			if (intLH<8) defLineHeight="8";
   			else defLineHeight="96";
   			System.err.println("Warning, line height value should be beetwen 8 and 96, using value of "+defLineHeight);
   		}
   	}
   	catch (Exception e) {
   		System.err.println("Error reading lineHeight parameter, using predefined value of "+defLineHeight);
   	}
      //reading lanuage specification value for hyphenation
      String strHyph=getResourceString("defHyphen");
      if (strHyph==null)
         System.out.println("Error while reading defHyphen parameter, using predefined value: "+defHyph);
      else
         defHyph=strHyph;
   }
   /** Method for validating default values of font family and font size
    * from "main" property file (and programs default values). This method is
    * caled from method setDefaultFonts().<BR>
    * Default value specified in the property file must be one of possible
    * values, also specified in property file. If this value matches the one of
    * possible values, method returns <b>true</b>, otherwise, if default value
    * isn't specified correctly or isn't specified at all, this method returns
    * <b>false</b>, which tells program to use the default value specified by
    * the program itself. In that case this method, before returning <b>false</b>,
    * also verifies the program default value, and if such doesn't exist in the
    * property file, the program default value becomes the first of possible values
    * specified in property file.
    * @param key name of default value to validate ("family" or "size")
    * @param value default value attained from property file
    * @return wether property file default value is OK (<b>true</b>) or not (<b>false</b>
    */
   private boolean validateDefaults (String key,String value) {
// reading possible values of font-size or font-family from
// property file and putting it into vector
		int i = 1;
      Vector vecSizesOrFamilies=new Vector();
      int substrstart;
      if (key.equals(_size)) substrstart=10;
      else substrstart=12;
		for( String action; null !=( action = getResourceString( key + i + _action )); ++i ) {
// must remove 'font-size-' or 'font-family-' from action string
// to get the size or family of the font
         action=action.substring(substrstart);
			vecSizesOrFamilies.addElement(action);
      }
// validating value
      if (vecSizesOrFamilies.contains(value)) {
         return true;
      }
      else {
// validating if default program values are corresponding to
// those possibile in property file, if not, changing them
         if (key.equals(_size)) {
            if(!(vecSizesOrFamilies.contains(defFontSize)) &&
               !(vecSizesOrFamilies.isEmpty()))
               defFontSize=(String)vecSizesOrFamilies.firstElement();
         }
         else {
            if(!(vecSizesOrFamilies.contains(defFontFamily)) &&
               !(vecSizesOrFamilies.isEmpty()))
               defFontFamily=(String)vecSizesOrFamilies.firstElement();
         }
         return false;
      }
   }
   /**
    * Setting up a default color values for editor. These values are read
    * from "main" property file. If no default values is defined in property
    * file, there are some predefined values.
    */
   protected void setDefaultColors(){
      /** Predefined default color values **/
      Color foreground=Color.green;
      Color background=Color.black;
      Color caretColor=Color.yellow;
      Color selectedTextColor=Color.yellow;
      Color selectionColor=Color.lightGray;
      Color border=SystemColor.info;
      /**
       * Standard colors are:
		 * black,blue,cyan,darkGray,gray,green,lightGray,magenta,orange,pink,red,white,yellow
       * Also, there are many system colors like:
       * activeCaption,control,controlShadow,controlText,desktop,inactiveCaptionText,info,
       * infoText,menu,menuText,scrollbar,text,window,windowBorder,windowText etc...
       */
      /** string array containing names of possible colors to set **/
      String strCol[]={"Color.black","Color.blue","Color.cyan","Color.darkGray","Color.gray",
                        "Color.green","Color.lightGray","Color.magenta","Color.orange",
                        "Color.pink","Color.red","Color.white","Color.yellow",
                        "SystemColor.desktop","SystemColor.activeCaption",
                        "SystemColor.activeCaptionText","SystemColor.activeCaptionBorder",
                        "SystemColor.inactiveCaption","SystemColor.inactiveCaptionText",
                        "SystemColor.inactiveCaptionBorder","SystemColor.window",
                        "SystemColor.windowBorder","SystemColor.windowText",
                        "SystemColor.menu","SystemColor.menuText","SystemColor.text",
                        "SystemColor.textText","SystemColor.textHighlight",
                        "SystemColor.textHighlightText","SystemColor.textInactiveText",
                        "SystemColor.control","SystemColor.controlText",
                        "SystemColor.controlHighlight","SystemColor.controlLtHighlight",
                        "SystemColor.controlShadow","SystemColor.controlDkShadow",
                        "SystemColor.scrollbar","SystemColor.info","SystemColor.infoText"};
      /** Color array containing possible colors to set **/
      Color colCol[]={Color.black,Color.blue,Color.cyan,Color.darkGray,Color.gray,
                        Color.green,Color.lightGray,Color.magenta,Color.orange,
                        Color.pink,Color.red,Color.white,Color.yellow,
                        SystemColor.desktop,SystemColor.activeCaption,
                        SystemColor.activeCaptionText,SystemColor.activeCaptionBorder,
                        SystemColor.inactiveCaption,SystemColor.inactiveCaptionText,
                        SystemColor.inactiveCaptionBorder,SystemColor.window,
                        SystemColor.windowBorder,SystemColor.windowText,
                        SystemColor.menu,SystemColor.menuText,SystemColor.text,
                        SystemColor.textText,SystemColor.textHighlight,
                        SystemColor.textHighlightText,SystemColor.textInactiveText,
                        SystemColor.control,SystemColor.controlText,
                        SystemColor.controlHighlight,SystemColor.controlLtHighlight,
                        SystemColor.controlShadow,SystemColor.controlDkShadow,
                        SystemColor.scrollbar,SystemColor.info,SystemColor.infoText};
      // Vectors for manipulating previous arrays
      Vector vStrCol=new Vector();
      Vector vColCol=new Vector();
      for (int i=0; i<strCol.length;i++) {
         vStrCol.addElement(strCol[i]);
         vColCol.addElement(colCol[i]);
      }
      // Reading the default values from resource file
      try{
         String[] defKeys = tokenize(getResourceString("defvalues"));
         String[] keyval=new String[defKeys.length];
         for (int i = 0; i < defKeys.length; i++) {
            keyval[i]=getResourceString(defKeys[i]);
            if (defKeys[i].equals("foreground")){
               foreground=getColor(keyval[i],foreground,vStrCol,vColCol);
            }
            else if (defKeys[i].equals("background")){
               background=getColor(keyval[i],background,vStrCol,vColCol);
            }
            else if (defKeys[i].equals("caretColor")){
               caretColor=getColor(keyval[i],caretColor,vStrCol,vColCol);
            }
            else if (defKeys[i].equals("selectedTextColor")){
               selectedTextColor=getColor(keyval[i],selectedTextColor,vStrCol,vColCol);
            }
            else if (defKeys[i].equals("selectionColor")){
               selectionColor=getColor(keyval[i],selectionColor,vStrCol,vColCol);
            }
            else if (defKeys[i].equals("border")){
               border=getColor(keyval[i],border,vStrCol,vColCol);
            }
         }
      }
      catch(Exception e){
         System.err.println("There is no default values for editor in property file");
      }
      // Defining foreground color
      editor.setForeground(foreground);
      // Defining background color
      editor.setBackground(background);
      // Defining caret color
      editor.setCaretColor(caretColor);
      // Defining color of selected text
      editor.setSelectedTextColor(selectedTextColor);
      // Defining color of selection
      editor.setSelectionColor(selectionColor);
      // Defining border color
      editor.setBorder(BorderFactory.createLineBorder(border,1));
   }
   /**
    * Method for determining default color values from property file.
    * Method returns default color value specified in the property file
    * if that value is one of possibile values (Color.black, Color.blue, ...),
    * or program predefined value otherwise.
    * @param strColor default color from property file to check
    * @param defColor predefined color to set if <b>strColor</b> doesn't pass the test
    * @param vStrColor vector of possible color names
    * @param vColColor vector of possible colors (corresponds to the vector vStrCol)
    * @return <b>Color</b> to be set (either one from property file, either predefined one)
    */
   private Color getColor(String strColor,Color defColor,Vector vStrCol,Vector vColCol) {
      Color colorVal=defColor;
      if (vStrCol.contains(strColor)) {
         int ind=vStrCol.indexOf(strColor);
         colorVal=(Color)vColCol.elementAt(ind);
      }
      return colorVal;
   }
   /**
    * Creates the menubar for the applet. Reads the property file to
    * determine which menus to create, then creates and puts
    * them on menubar.
    * @return created menubar.
    */
   protected JMenuBar createMenuBar(){
      JMenuItem mi;
      JMenuBar mb = new JMenuBar();
      String[] menuKeys = tokenize(getResourceString("menubar"));
      for (int i = 0; i < menuKeys.length; i++) {
         JMenu m = createMenu(menuKeys[i]);
         if (m != null)
            mb.add(m);
      }
      return mb;
   }
   /**
    * Creates a menu for the menubar of the applet. Reads the property file to
    * set the menu name, to determine which menu items to create, then
    * creates and puts them in the menu.
    * @param key name of menu to search for in property file and create.
    * @return created menu.
    */
   protected JMenu createMenu(String key) {
      String[] itemKeys = tokenize(getResourceString(key));
      JMenu menu = new JMenu(getResourceString(key + "Label"));
      menu.setMnemonic(menu.getText().charAt(0));
      for (int i = 0; i < itemKeys.length; i++) {
         if (itemKeys[i].equals("-")) {
            menu.addSeparator();
         }
         else {
            JMenuItem mi = createMenuItem(itemKeys[i]);
            menu.add(mi);
         }
      }
      return menu;
   }
   /**
    * Creates a menuitems for the menu. Reads the property file to set
    * the menu item name, icon and action. For info menu items, defines
    * the action listener.
    * @param cmd the name of menu item.
    * @return created menu item.
    */
   protected JMenuItem createMenuItem(String cmd) {
      JMenuItem mi = new JMenuItem(getResourceString(cmd + "Label"));
      if (cmd.equals("about")){
         // Action listener for About window
         mi.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
               helpAbout(e);
            }
         });
      }
      else if (cmd.equals("parametar")){
         // Action listener for ParametarInfo window
         mi.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
               helpParam(e);
            }
         });
      }
      else if (cmd.equals("save")){
          // Action listener for open window
          mi.addActionListener(new ActionListener(){
             public void actionPerformed(ActionEvent e){
                saveToFile(e);
             }
          });
       }
      else if (cmd.equals("open")){
          // Action listener for open window
          mi.addActionListener(new ActionListener(){
             public void actionPerformed(ActionEvent e){
                openFapFile(e);
             }
          });
       }
      else if (cmd.equals("close")){
          // Action listener for open window
          mi.addActionListener(new ActionListener(){
             public void actionPerformed(ActionEvent e){
                closeFapFile(e);
             }
          });
       }
      else{
         URL url = getResource(cmd + "ImageMenu");
         if (url != null) {
            mi.setHorizontalTextPosition(JButton.RIGHT);
            mi.setIcon(new ImageIcon(url));
         }
         String astr = getResourceString(cmd + "Action");
         if (astr == null) astr = cmd;
         final String astrf=astr;
         final String cmdf=cmd;
         mi.setActionCommand(astrf);
         final Action a = getAction(astrf);
         if (a != null) {
// linking format menu items with toolbar buttons and combos - must define Action Listener
            if (astrf.substring(0,4).equals("font")) {
               if (cmdf.equals("bold")) {
                  mi.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent ae) {
                        a.actionPerformed(ae);
                        if (boldButton!=null) {
                           boldButton.setSelected(!boldButton.isSelected());
                        }
                     }
                  });
               }
               else if (cmdf.equals("italic")) {
                  mi.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent ae) {
                        a.actionPerformed(ae);
                        if (italicButton!=null) {
                           italicButton.setSelected(!italicButton.isSelected());
                        }
                     }
                  });
               }
               else if (cmdf.equals("underline")) {
                  mi.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent ae) {
                        a.actionPerformed(ae);
                        if (underButton!=null) {
                           underButton.setSelected(!underButton.isSelected());
                        }
                     }
                  });
               }
               else if (astrf.substring(5,9).equals(_size)) {
                  mi.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent ae) {
                        a.actionPerformed(ae);
                        if (comboFontSizes!=null) {
                           comboFontSizes.removeActionListener(comboFontListener);
                           comboFontSizes.setSelectedItem(getResourceString(cmdf + _label));
                           comboFontSizes.addActionListener(comboFontListener);
                        }
                     }
                  });
               }
               else if (astrf.substring(5,11).equals(_family)) {
                  mi.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent ae) {
                        a.actionPerformed(ae);
                        if (comboFontFamilies!=null) {
                           comboFontFamilies.removeActionListener(comboFontListener);
                           comboFontFamilies.setSelectedItem(getResourceString(cmdf + _label));
                           comboFontFamilies.addActionListener(comboFontListener);
                        }
                     }
                  });
               }
            }
            else {
               mi.addActionListener(a);
            }
            mi.setEnabled(a.isEnabled());
         }
         else mi.setEnabled(false);
      }
      return mi;
   }
  /**
	 * Gets a resource string from the resource bundle.<p> Resource bundle
	 * represents the <i>property file</i>. For example, if property file
	 * contains something like this:<BR><CENTER>menubar=edit format info</CENTER>
    * method call getResourceString("menubar") will give the string
    * <i>edit format info</i> as a result. <BR> This method reads information
    * from "main" property file, if can't find desired resource, searchs the "language"
    * property file, and if still can't find it searchs the default "language" property file.
    * If can't find desired resource in either of these files, returns <b>null</b>.
	 * @param nm name of the resource to fetch.
    * @return String value of named resource.
	 */
   protected String getResourceString(String nm) {
      String str;
      try {
         str = this.resources.getString(nm);
      }
      catch (MissingResourceException mre) {
         try {
            str = this.resText.getString(nm);
         }
         catch (MissingResourceException mree) {
            try {
               str = this.resdef.getString(nm);
            }
            catch (MissingResourceException mreee) {
               str = null;
            }
         }
      }
      if (str != null) str=str.trim();
      return str;
   }
   /**
    * Gets a url from a resource string.
    * @param key the string key to the url in the resource bundle.
    * @return the resource location.
    * @see java.lang.Class#getResource
    */
   protected URL getResource(String key) {
   	String name = getResourceString(key);
	   if (name != null) {
	      URL url = this.getClass().getResource(name);
	      return url;
	   }
	   return null;
   }
   /**
    * Take the given string and chop it up into a series
    * of strings on whitespace boundaries.
    * @param input string to 'tokenize'
    * @return 'tokenized' string array
    */
   public static String[] tokenize(String input) {
      Vector v = new Vector();
      StringTokenizer t = new StringTokenizer(input);
      String cmd[];
      while (t.hasMoreTokens())
         v.addElement(t.nextToken());
      cmd = new String[v.size()];
      for (int i = 0; i < cmd.length; i++)
         cmd[i] = (String) v.elementAt(i);
      return cmd;
   }
   /**
    * Method to get all actions suported by the class JTextPane.
    * @return Returns all supported editor's actions.
    **/
   public Action[] getActions() {
      Action[] defaultActions = {
         new CopyExtAction()};
      Action[] a = TextAction.augmentList(editor.getActions(), defaultActions);
/*
        Action[] defaultActions = {
            new NewAction(),
            new OpenAction(),
            new SaveAction(),
            new StyledEditorKit.FontFamilyAction("font-family-LucidaSans",
                                                 "Lucida Sans"),
        };
        Action[] a = TextAction.augmentList(super.getActions(), defaultActions);

 */
   	// return editor.getActions();
      return a;
   }
   /**
    * Method to get editor's action corresponding to the given string.
    * @param cmd String representation of editor's action.
    * @return action specified by the string cmd.
    **/
   protected Action getAction(String cmd) {
      return (Action) commands.get(cmd);
   }
   /**
    * Public method for accessing private class member editor
    * @return an editor (instance of class MyTextPane).
    **/
   public JTextPane getEditor() {
      return editor;
   }
   /**
    * Creates the toolbar for editor. Reads the property file to
    * determine which tool buttons to create, then creates and puts
    * them on toolbar.
    * @return created toolbar.
    */
   protected Component createToolbar() {
      toolbar = new JToolBar();
      String[] toolKeys = tokenize(getResourceString("toolbar"));
      for (int i = 0; i < toolKeys.length; i++) {
         if (toolKeys[i].equals("-")) {
            toolbar.add(Box.createHorizontalStrut(10));
         }
         else {
            toolbar.add(createTool(toolKeys[i]));
         }
      }
      toolbar.add(Box.createHorizontalGlue());
      JButton but=new JButton("Dump");
      but.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            testing(e);
         }
      });
      /** For testing purposes only **/
      //toolbar.add(but);
      /** By default, alignment is left**/
      leftButton.setSelected(true);
      return toolbar;
   }
   /**
    * Hook through which every toolbar item is created. This method calls
    * other methods for creating toolbar items, depending of what item is
    * to be created.
    * @param key name of item to be created.
    * @return created component (JComboBox, JToggleButton, JToggleGroup or JButton).
    */
   protected Component createTool(String key) {
      if (key.equals("fontfamily") || key.equals("fontsize"))
         return createComboBoxes(key);
      else if(key.equals("bold") || key.equals("italic") || key.equals("underline")){
         return createToggleButton(key);
      }
      else if(key.equals("left") || key.equals("center") || key.equals("right")){
         return createToggleGroup(key);
      }
      else
      	return createToolbarButton(key);
   }
   /** Creates the toggle button's group. Within this group,
    * only one button at a time is active. This method creates toggle button
    * by the call to the method everyToggle(), and then sets the button's
    * action and tooltip text.
    * @param key name of the toggle button that belongs to the toggle group and
    * is to be created.
    * @return created toggle button.
    */
   protected JToggleButton createToggleGroup(String key){
      JToggleButton Tb = everyToggle(key);
      Tb.setRequestFocusEnabled(false);
      Tb.setMargin(new Insets(1,1,1,1));
      String astr = getResourceString(key + "Action");
      if (astr == null) {
         astr = key;
      }
      Action a = getAction(astr);
      if (a != null) {
         Tb.setActionCommand(astr);
         Tb.addActionListener(a);
      }
      else {
         Tb.setEnabled(false);
      }
      String tip = getResourceString(key + "Tooltip");
      if (tip != null) {
         Tb.setToolTipText(tip);
      }
      group.add(Tb);
      return Tb;
   }
   /**
    * This method is called as a result of selecting menu item <i>About</i> from
    * <i>Info</i> menu. Shows message dialog filled with 'about' data.
    * @param e ActionEvent to react upon
    */
   private void helpAbout(ActionEvent e){
      JOptionPane.showMessageDialog(this, new InfoPanel(), getResourceString("aboutLabel"), JOptionPane.INFORMATION_MESSAGE);
   }
   /**
    * This method is called as a result of selecting menu item <i>Parameter info</i> from
    * <i>Info</i> menu. Shows message dialog filled with 'parameter info' data.
    * @param e ActionEvent to react upon
    */
   private void helpParam(ActionEvent e){
      JOptionPane.showMessageDialog(this, new InfoPanel(resources,resLan,res,defHyph), getResourceString("parametarLabel"), JOptionPane.INFORMATION_MESSAGE);
   }
   
   
   /**
    * This method is called as a result of selecting menu item <i>Save</i> from
    * <i>File</i> menu. Shows file dialog assuming name of file, and then
    *  save content into this file.
    * @param e ActionEvent to react upon
    */  
   private void saveToFile(ActionEvent e){
	  
	      Frame parent = new Frame();
	      FileDialog fd=new FileDialog(parent,"Save");
	      fd.setMode(1);
	      fd.setFile("*.fap"); 
	      fd.show();
	      String fname=fd.getDirectory()+File.separator+fd.getFile();
	      String fotext=((MyTextPane)getEditor()).prepareTextForSave();    
	      StringReader strfo=new StringReader(fotext);
	      
	    try{  
	      FileOutputStream fout =new FileOutputStream(new File(fname)); 
	      
    	     if (strfo.ready()){
			
	    		int c = strfo.read();
//	    		 write in file
	            while(c != -1)
	            { 
	            fout.write(c);
	            c = strfo.read();
	            }
	            fout.flush();
	            fout.close();
	            strfo.close();
	      	}
	     }
	    catch(IOException ex) {}
	  
	   }
   
   
   /**
    * This method is called as a result of selecting menu item <i>Open</i> from
    * <i>File</i> menu. Shows file dialog  assuming name of file, and then
    *  shows content from this file.
    * @param e ActionEvent to react upon
    */
   
   private void openFapFile(ActionEvent e){
	   int arrlen = 10000;
		byte[] infile = new byte[arrlen];
		Frame parent = new Frame();
		FileDialog fd = new FileDialog(parent, "Please choose a file:",
		           FileDialog.LOAD);
		fd.setFile("*.fap"); 
		fd.show();
		String selectedItem = fd.getFile();
		if (selectedItem == null) {
			// no file selected
		} else {
			
			
			File ffile = new File( fd.getDirectory() + File.separator +
			        fd.getFile());
			// read the file
			boolean overwritten=doOverWrite();
			try {
				FileInputStream fis = new FileInputStream(ffile); 
				BufferedInputStream bis = new BufferedInputStream(fis);
				DataInputStream dis = new DataInputStream(bis);
				try {
					int filelength = dis.read(infile);
					String filestring = new String(infile, 0, filelength);
					((MyTextPane) getEditor()).loadTextf(filestring,overwritten);
												
				} catch(IOException iox) {
					iox.printStackTrace();
				}
			} catch (FileNotFoundException fnf) {
				fnf.printStackTrace();
			}
		}		  
	      
	   }
   
   /**
    * This method is called as a result of selecting menu item <i>Close</i> from
    * <i>File</i> menu.If document is not empty,create and show confirm dialog for save changes.
    * If type of choises is YES then call to the method saveToFile(e).                             
    * @param e ActionEvent to react upon
    */
    
   private void closeFapFile(ActionEvent e){
	if(((MyTextPane) getEditor()).getText().length()>0){
	   Frame frame = new Frame();
	   int n = JOptionPane.showConfirmDialog(
			    frame,
			    "Do you want to save the changes?",
			    "FopApplet",
			    JOptionPane.YES_NO_OPTION);
  
	   if (n == JOptionPane.YES_OPTION){
		 saveToFile(e);
		 ((MyTextPane) getEditor()).setText("");
	    }
	   else {
		  ((MyTextPane) getEditor()).setText("");  
	   }
	}  
   }
   
   /**
    * This method is called from the metod openFapFile(e).
    * If current document is not empty,this metod create the confirm dialog for overwrite and show it.
    * @return true if current document need overwrite; false otherwise 
    */
   
   private boolean doOverWrite(){
	   
	   if(((MyTextPane) getEditor()).getText().length()>0){   
	      Frame frame = new Frame();
	      int n = JOptionPane.showConfirmDialog(
			    frame,
			    "Do you want to overwrite this text?",
			    "FopApplet",
			    JOptionPane.YES_NO_OPTION);
  
	      if (n == JOptionPane.YES_OPTION){
		    return true;  
	      }
	      else {
		    return false;  
	      }
	   }
	   else
	   {
	     return true;  
	   }   
   }
   /**
    * Creates instances of ToggleButtons. For each button sets the icon and
    * default state determined by property file or predefined.
    * @param name name of toggle buton to be created.
    * @return created toggle button.
    */
	protected JToggleButton everyToggle(String name){
   	URL url = getResource(name + "ImageTool");
      if (name.equals("bold")){
      	try {
      		boldButton=new JToggleButton(new ImageIcon(url),defbold){
      			public float getAlignmentY() { return 0.5f; }
      		};
      	}
      	catch (Exception e) {
      		boldButton=new JToggleButton(name,defbold){
      			public float getAlignmentY() { return 0.5f; }
      		};
      	}
      	return boldButton;
      }
      else if (name.equals("italic")){
      	try {
      		italicButton=new JToggleButton(new ImageIcon(url),defitalic){
      			public float getAlignmentY() { return 0.5f; }
      		};
      	}
      	catch (Exception e) {
      		italicButton=new JToggleButton(name,defitalic){
      			public float getAlignmentY() { return 0.5f; }
      		};
      	}
      	return italicButton;
      }
      else if(name.equals("underline")){
      	try {
      		underButton=new JToggleButton(new ImageIcon(url),defunder){
      			public float getAlignmentY() { return 0.5f; }
      		};
      	}
      	catch (Exception e) {
      		underButton=new JToggleButton(name,defunder){
      			public float getAlignmentY() { return 0.5f; }
      		};
      	}
      	return underButton;
      }
      else if(name.equals("left")){
      	try {
      		leftButton=new JToggleButton(new ImageIcon(url),false){
      			public float getAlignmentY() { return 0.5f; }
      		};
      	}
      	catch (Exception e) {
      		leftButton=new JToggleButton(name,false){
      			public float getAlignmentY() { return 0.5f; }
      		};
      	}
      	return leftButton;
      }
      else if(name.equals("center")){
      	try {
      		centerButton=new JToggleButton(new ImageIcon(url),false){
      			public float getAlignmentY() { return 0.5f; }
      		};
      	}
      	catch (Exception e) {
      		centerButton=new JToggleButton(name,false){
      			public float getAlignmentY() { return 0.5f; }
      		};
      	}
      	return centerButton;
      }
      else {
      	try {
      		rightButton=new JToggleButton(new ImageIcon(url),false){
      			public float getAlignmentY() { return 0.5f; }
      		};
      	}
      	catch (Exception e) {
      		rightButton=new JToggleButton(name,false){
      			public float getAlignmentY() { return 0.5f; }
      		};
      	}
      	return rightButton;
      }
   }
   /**
    * Creating a ToggleButtons for actions: Bold, Italic, Underline.
    * This method creates toggle button by the call to the method everyToggle(),
    * and then sets the button's action and tooltip text.
    * @param key name of the toggle button that will be created.
    * @return created toggle button.
    */
   protected JToggleButton createToggleButton(String key){
      JToggleButton Tb = everyToggle(key);
      Tb.setRequestFocusEnabled(false);
      Tb.setMargin(new Insets(1,1,1,1));
      String astr = getResourceString(key + "Action");
      if (astr == null) {
         astr = key;
      }
      Action a = getAction(astr);
      if (a != null) {
         Tb.setActionCommand(astr);
         Tb.addActionListener(toggleButtonListener);
      }
      else {
         Tb.setEnabled(false);
      }
      String tip = getResourceString(key + "Tooltip");
      if (tip != null) {
         Tb.setToolTipText(tip);
      }
      return Tb;
   }
   /*
    * Creating a ComboBoxes for fontfamily and for fontsize
    */
   /**
    * Creates JComboBox ComboBoxes for fontfamily and for fontsize based on the
    * key String, and assigns it to the corresponding field.<p>
    * Combobox items are read from property file using getResourceString method
    * @see #_fontsize
    * @see #_fontfamily
    * @param key name of category combobox is created for <br>valid values are contained in static Strings _fontfamily and _fontsize
    * @return newly created JComboBox (which is assigned to corresponding field, either comboFontFamilies or comboFontSizes)
    **/
   protected JComboBox createComboBoxes(String key){
// property file labels are somewhat different to key, so, key is now set to the proper value
		key =( key.equals( _fontfamily ))?( _family ):( _size );
// setting up variables for loop which would read values from property file
		int i = 1;
		String[] tmp = new String[ _maxComboItems ];
		for( String label; null !=( label = getResourceString( key + i + _label )); ++i )
			tmp[ i ]= label;
// at first we did not know how many values there are, now we do it is a value of i, we use it to
// create properly sized itemlist
		String[] itemlist = new String[ i ];
		itemlist[ i - 1 ]= "";
// loop copies values from tmp array into itemlist
		while( --i > 0 )
			itemlist[ i - 1 ]= tmp[ i ];
// eventually we arrive at creating JComboBox adding an action listener and tooltip to it
		JComboBox cb = new JComboBox( itemlist );

// inserting default values into combos
// must define what to set (defFontFamily or defFontSize) and how to get
// corresponding label to select(must take substring from Action label in
// .property file to get defFontFamilyOrSize)
      String action;
      String selection;
      String defFamilyOrStyle;
      int substrstart;
      if (key.equals(_family)) {
         defFamilyOrStyle=defFontFamily;
         substrstart=12;
      }
      else {
         defFamilyOrStyle=defFontSize;
         substrstart=10;
      }
// loop through actions in property file to find right one
      for( i = 1;  null !=( action = getResourceString( key + i + _action )); ++i ){
// must remove 'font-family-' from action string to get the family of the font
         action=action.substring(substrstart);
         if( defFamilyOrStyle.equals( action )){
// when found perform a selection
            selection = getResourceString( key + i + _label );
            cb.setSelectedItem(selection);
            break;
         }
      }
// enf of inserting default values in combos

		cb.addActionListener( comboFontListener );
		tmp[ 1 ]= getResourceString( key + _tooltip );
		if( tmp[ 1 ]!= null )
			cb.setToolTipText( tmp[ 1 ]);
// this method does assign newly crated JComboBox to the proper field, the key should tell which one
		if( key.equals( _family ))
         return( comboFontFamilies = cb );
      else
         return( comboFontSizes = cb );
   }
   /**
    * Performs action of either changing font family or size.
    * Invoked by anonymous ActionListener assigned to both comboboxes.<p>
    * @param e ActionEvent to react upon
    */
   protected void comboFontActions(ActionEvent e){
   // selectedItem and key are crucial, so, set them first
		JComboBox source =( JComboBox )e.getSource();
		String selectedItem = source.getSelectedItem().toString();
		String key =( comboFontFamilies.equals( source ))?( _family ):( _size );
		Action act;
		String label;
// loop through labels in property file to find right one
		for( int i = 1;  null !=( label = getResourceString( key + i + _label )); ++i ){
			if( selectedItem.equals( label )){
// when found perform a new action and
				act = getAction( getResourceString( key + i + _action ));
				act.actionPerformed( e );
// return focus to the editor
				editor.requestFocus();
			}
		}
	}
   /**
    * Creates a toolbar button. Sets button's icon, action and tooltip text.
    * @param key name of button to be created.
	 * @return created button.
    */
   protected JButton createToolbarButton(String key) {
   	URL url = getResource(key + "ImageTool");
      JButton b;
      try {
         b = new JButton(new ImageIcon(url)) {
            public float getAlignmentY() { return 0.5f; }
	      };
	   }
	   catch (Exception e) {
	   	b = new JButton(key) {
            public float getAlignmentY() { return 0.5f; }
	      };
	   }
      b.setRequestFocusEnabled(false);
      b.setMargin(new Insets(1,1,1,1));
   	String astr = getResourceString(key + "Action");
   	if (astr == null) {
	      astr = key;
   	}
   	Action a = getAction(astr);
      if (a != null) {
	       b.setActionCommand(astr);
   	    b.addActionListener(a);
      }
      else {
   	   b.setEnabled(false);
      }
   	String tip = getResourceString(key + "Tooltip");
   	if (tip != null) {
	      b.setToolTipText(tip);
      }
      return b;
   }
   /**
    * Performs action of changing font style.
    * Invoked by anonymous ActionListener assigned to all toggle buttons.
    * @param e ActionEvent to react upon
    */
   protected void toggleButtonActions(ActionEvent e) {
// find which button is pressed
      JToggleButton source=(JToggleButton)e.getSource();
      String key;
      if (source.equals(boldButton))
         key="bold";
      else if (source.equals(italicButton))
         key="italic";
      else
         key="underline";
// look in property file to find action
      String astr = getResourceString(key + "Action");
      if (astr == null) {
         astr = key;
      }
// when found perform a new action
      Action a = getAction(astr);
      if (a != null) {
         a.actionPerformed(e);
      }
// return focus to the editor
      editor.requestFocus();
   }
   /**
    * Method for testing purposes only
	 * @param e ActionEvent to react upon
    **/
   public void testing(ActionEvent e){
      java.util.List listDoc=loading();
      Iterator iterDoc=listDoc.iterator();
      while(iterDoc.hasNext()) {
         Object nextDoc = iterDoc.next();
         java.util.List listParag=(java.util.List)nextDoc;
         Iterator iterParag=listParag.iterator();
         while(iterParag.hasNext()){
            Object nextAlign=iterParag.next();
            int align=Integer.parseInt(nextAlign.toString());
            Object nextSadrzaj=iterParag.next();
            System.out.println("Paragraf: Alignment="+align);
            java.util.List listSad=(java.util.List)nextSadrzaj;
            Iterator iterSad=listSad.iterator();
            while(iterSad.hasNext()){
               Object nextSad=iterSad.next();
               java.util.List listCon=(java.util.List)nextSad;
               Iterator iterCon=listCon.iterator();
               while(iterCon.hasNext()){
                  String Text=new String(iterCon.next().toString());
                  String FontFam=new String(iterCon.next().toString());
                  int FontSize=Integer.parseInt(iterCon.next().toString());
                  String bold=new String(iterCon.next().toString());
                  String italic=new String(iterCon.next().toString());
                  String underline=new String(iterCon.next().toString());
                  int Start=Integer.parseInt(iterCon.next().toString());
                  int End=Integer.parseInt(iterCon.next().toString());
                  System.out.println("     text: "+Text+"["+Start+","+End+"] "+FontFam+" "+FontSize+" "+"Bold="+bold+", Italic="+italic+", Underline="+underline);
               }
            }
         }
      }
   }
   /**
    * Method for loading a List called listDoc with all elements of editor.
    * Editor elements (written text elements) are:<br>   *section-the main
    * root element which contains paragraphs<br>   *paragraph-element that
    * contains alignment and character elements<br>   *character-element
    * whose attributes are font family, font size and font style.
    * @return list that represents the writen text (document)
    **/
   public java.util.List loading(){
      StyledDocument document = getEditor().getStyledDocument();
      rootElements = document.getRootElements();
      //Debug
      Element sect=rootElements[0];
      int sectCnt=sect.getElementCount();
      //Debug
      Element parag=null;
      java.util.List listDoc = new ArrayList();
      //Loop for each paragraph
      for (int i=0;i<sectCnt;i++){
         java.util.List listPrg = new ArrayList();
         // 'i'th element of paragraph-content
         parag=sect.getElement(i);
         int paragCnt=parag.getElementCount();
         if (i==sectCnt-1 && sectCnt>1) paragCnt--;
         AttributeSet ParAS = parag.getAttributes();
         int intAlign=StyleConstants.getAlignment(ParAS);
         Element content=null;
         java.util.List listSad = new ArrayList();
         //Loop for each content of given paragraph
         for(int j=0;j<paragCnt;j++){
            java.util.List listCont = new ArrayList();
            content=parag.getElement(j);
            AttributeSet ParASL = content.getAttributes();
            String text;
            try{
 			   text=document.getText(content.getStartOffset(),content.getEndOffset()-content.getStartOffset());
               String chngText="";
               for (int k=0; k<text.length(); k++) {
                  if (text.charAt(k)=='&')
                     chngText=chngText+"&amp;";
                  else if (text.charAt(k)=='<')
                     chngText=chngText+"&lt;";
                  else if (text.charAt(k)=='>')
                     chngText=chngText+"&gt;";
                  else
                     chngText=chngText+text.charAt(k);
               }
               text=chngText;
               listCont.add(text);
            }
            catch(BadLocationException e){
               listCont.add(new String());
               System.err.println("Exception "+e+" in range of  given part of document");
            }
            listCont.add(new String(StyleConstants.getFontFamily(ParASL)));
            listCont.add(new Integer(StyleConstants.getFontSize(ParASL)));
            listCont.add(new Boolean(StyleConstants.isBold(ParASL)));
            listCont.add(new Boolean(StyleConstants.isItalic(ParASL)));
            listCont.add(new Boolean(StyleConstants.isUnderline(ParASL)));
            listCont.add(new Integer(content.getStartOffset()));
            listCont.add(new Integer(content.getEndOffset()));
            //Adding a element to a ArraysList-listSad
            listSad.add(listCont);
         }
         // Adding a elements to a ArraysList-listPrg so it will have the structure
         // similar like this: listPrg{Align,Sadrzaj}
         listPrg.add(new Integer(intAlign));
         listPrg.add(listSad);
         // Adding a elements to a ArraysList-listDoc
         listDoc.add(listPrg);
      }
      return listDoc;
   }
        
     /**
    * Method for creating string in a form of XSL FO file that represent a content of applet.
    * This method format applet's text, and transforms it into XSL FO form.
    * @return String formatted as XSL FO file
    **/
   public String FormatFO(){
      String fo=new String();
      String FOalign="start";
      String FObold="";
      String FOunder="";
      String FOital="";
      String Text=" ";
      String FontFam="sans-serif";
      int FontSize=12;
      java.util.List listDoc=loading();
      Iterator iterDoc=listDoc.iterator();
      while(iterDoc.hasNext()) {
         Object nextDoc = iterDoc.next();
         java.util.List listParag=(java.util.List)nextDoc;
         Iterator iterParag=listParag.iterator();
         while(iterParag.hasNext()){
            Object nextAlign=iterParag.next();
            int align=Integer.parseInt(nextAlign.toString());
            if (align==0)
               FOalign="start";
            else if(align==1)
               FOalign="center";
            else if(align==2)
               FOalign="end";
            Object nextSadrzaj=iterParag.next();
            fo+="<fo:block font-size=\"12pt\" font-family=\"sans-serif\" line-height=\""+defLineHeight+"pt\" space-after.optimum=\"3pt\" text-align=\""+FOalign+"\" white-space-collapse=\"false\" language=\""+defHyph+"\" hyphenate=\"true\">";
            java.util.List listSad=(java.util.List)nextSadrzaj;
            Iterator iterSad=listSad.iterator();
            while(iterSad.hasNext()){
               Object nextSad=iterSad.next();
               java.util.List listCon=(java.util.List)nextSad;
               Iterator iterCon=listCon.iterator();
               while(iterCon.hasNext()){
                  Text=new String(iterCon.next().toString());
                  if (Text.equals("\n")){
                     Text=" ";
                  }
                  FontFam=new String(iterCon.next().toString());
                  FontSize=Integer.parseInt(iterCon.next().toString());
                  String bold=new String(iterCon.next().toString());
                  String italic=new String(iterCon.next().toString());
                  String underline=new String(iterCon.next().toString());
                  int Start=Integer.parseInt(iterCon.next().toString());
                  int End=Integer.parseInt(iterCon.next().toString());
                  if (FontFam.equals("Monospaced"))
                     FontFam="Courier";
                  else if (FontFam.equals("SansSerif"))
                     FontFam="Helvetica";
                  else
                     FontFam="Times Roman";
                  if (bold.equals("true"))
                     FObold="font-weight=\"bold\" ";
                  else
                     FObold="";
                  if (italic.equals("true"))
                     FOital="font-style=\"italic\" ";
                  else
                     FOital="";
                  if (underline.equals("true"))
                     FOunder="text-decoration=\"underline\" ";
                  else
                     FOunder="";
                  fo+="<fo:inline font-family=\""+FontFam+"\" font-size=\""+FontSize+"pt\" "+FObold+FOital+FOunder+">"+Text;
                  fo+="</fo:inline>";
               }
            }
            fo+="</fo:block>";
         }
      }
   return fo;
   }
   /**
    * Behavior for curetEvent. With every caret movement, updates the
    * state and selection of toolbar buttons and combo boxes.
    * States and selections visualize the attribute settings of
    * the next character to be written at the current position of the caret.
    * @param e caret event to react upon.
    **/
   protected void caretChange(CaretEvent e){
      // if Enter or Back Space was pressed, return
      if (isEnterPressed==true) {
         return;
      }
      if (isBSValid==true) {
         return;
      }
      // if there is a selection, update combos and return
      int dot = e.getDot();
      int mark = e.getMark();
		if(dot != mark){
         if (comboFontFamilies != null) {
            comboFontFamilies.removeActionListener(comboFontListener);
	   		comboFontFamilies.setSelectedItem("");
            comboFontFamilies.addActionListener(comboFontListener);
         }
         if (comboFontSizes != null) {
            comboFontSizes.removeActionListener(comboFontListener);
	   		comboFontSizes.setSelectedItem("");
            comboFontSizes.addActionListener(comboFontListener);
         }
			return;
		}

		StyledDocument theDocument = getEditor().getStyledDocument();
      // current paragraph and left and right char.
      Element prgel=theDocument.getParagraphElement(dot);
      Element leftel=theDocument.getCharacterElement(dot-1);
      Element rightel=theDocument.getCharacterElement(dot);
      Element curel;

      // start and end of current paragraph
      int stof=prgel.getStartOffset();
      int enof=prgel.getEndOffset();

      // always look to the left except at begining of non empty paragraph
      if (((dot==stof) && ((enof-stof)>1))){
         curel=rightel;
      }
      else {
         curel=leftel;
      }

      //Attributes for character
      AttributeSet AttChar;

      if ((dot==stof) && ((enof-stof)==1)) {
         AttChar=editor.getParagraphAttributes();
      }
      else {
         AttChar=curel.getAttributes();
      }

      String ff=StyleConstants.getFontFamily(AttChar);
      int fs=StyleConstants.getFontSize(AttChar);
      boolean b=StyleConstants.isBold(AttChar);
      boolean i=StyleConstants.isItalic(AttChar);
      boolean u=StyleConstants.isUnderline(AttChar);
      // updating style buttons
      if ((boldButton != null) && (boldButton.isSelected()!=b)) {
         boldButton.removeActionListener(toggleButtonListener);
         boldButton.setSelected(b);
         boldButton.addActionListener(toggleButtonListener);
      }
      if ((italicButton != null) && (italicButton.isSelected()!=i)) {
         italicButton.removeActionListener(toggleButtonListener);
         italicButton.setSelected(i);
         italicButton.addActionListener(toggleButtonListener);
      }
      if ((underButton != null) && (underButton.isSelected()!=u)) {
         underButton.removeActionListener(toggleButtonListener);
         underButton.setSelected(u);
         underButton.addActionListener(toggleButtonListener);
      }
      // updating combo boxes selections (must remove ActionListener)
      String key;
      String action;
      String selection;
      if (comboFontFamilies != null) {
         comboFontFamilies.removeActionListener(comboFontListener);
         key = _family;
         // loop through actions in property file to find right one
		   for( int j = 1;  null !=( action = getResourceString( key + j + _action )); ++j ){
            // must remove 'font-family-' from action string to get the family of the font
            action=action.substring(12);
            if( ff.equals( action )){
               // when found perform a selection
				   selection = getResourceString( key + j + _label );
				   comboFontFamilies.setSelectedItem(selection);
               break;
            }
         }
         comboFontFamilies.addActionListener(comboFontListener);
      }
      if (comboFontSizes != null) {
         comboFontSizes.removeActionListener(comboFontListener);
         key = _size;
         // loop through actions in property file to find right one
		   for( int j = 1;  null !=( action = getResourceString( key + j + _action )); ++j ){
            // must remove 'font-size-' from action string to get the size of the font
            action=action.substring(10);
            if( String.valueOf(fs).equals( action )){
               // when found perform a selection
				   selection = getResourceString( key + j + _label );
				   comboFontSizes.setSelectedItem(selection);
               break;
            }
         }
         comboFontSizes.addActionListener(comboFontListener);
      }
      //Values for paragraph
      AttributeSet AttPar=getEditor().getParagraphAttributes();
      // updating alignment buttons
      int intAlign=StyleConstants.getAlignment(AttPar);
      switch (intAlign){
         case 0:
            if ((leftButton != null) && (leftButton.isSelected()!=true))
               leftButton.setSelected(true);
            break;
         case 1:
            if ((centerButton != null) && (centerButton.isSelected()!=true))
               centerButton.setSelected(true);
            break;
         case 2:
            if ((rightButton != null) && (rightButton.isSelected()!=true))
               rightButton.setSelected(true);
            break;
      }
   }
	/**Listens for mouse clicks and shows the popup menu when the right mouse buton is pressed.*/
	private final class PopMouseListener extends MouseAdapter {
      /** leaves all the job to super class */
		PopMouseListener() {
			super();
		}
      /** When right mouse button is pressed, builds up and shows the popup menu */
		public void mousePressed(MouseEvent e) {
			if ( (e.getModifiers() & InputEvent.BUTTON1_MASK) != 0 ) return;
			if (PopUp ==null) buildPop();
			PopUp.show(editor,e.getX(),e.getY());
		}
      /** Creates the popup menu */
      private void buildPop() {
			PopUp = new JPopupMenu();
			String[] itemKeys = {"cut","copy","copyExt","paste"};
			for (int i = 0; i < itemKeys.length; i++) {
			   switch (itemKeys[i].charAt(0)) {
				   case '-' :
					   PopUp.addSeparator(); break;
			   	default:
				   JMenuItem mi = createMenuItem(itemKeys[i]);
				   PopUp.add(mi);
			   }
		   }
		}
	}
   /** Inner class for external copy **/
   class CopyExtAction extends AbstractAction{

      public CopyExtAction() {
         super(copyExtAction);
      }

      /* (non-Javadoc)
       * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
       */
      public void actionPerformed(ActionEvent e) {
         //editor.pickClipboardToUse();
         editor.copyExt();
      }

   }
   
   /** Inner class for Filefilter **/
   class FapFilter implements FilenameFilter{ 
      public FapFilter(){} 


      public boolean accept(File dir,String name) 
      {    if(name.endsWith(".fap")) 
               return true; 
           return false; 
      } 
   }
   
}

