 /*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: InventoryImpl.java,v 1.1 2004/08/16 09:44:52 slobodan Exp $
 *-----------------------------------------------------------------------------
 */




package golfShop.business.inventory;

import java.util.Vector;
import com.lutris.appserver.server.user.*;
import golfShop.data.item.ItemDO;
import golfShop.data.item.CategoryDO;
import golfShop.spec.inventory.Inventory;
// TODO - handle null returns 
// 

/**
 * The inventory and/or heirarchy of items for sale.
 *
 * @author Scott Pirie
 * @version $Revision: 1.1 $
 */
public class InventoryImpl implements Inventory{
    // Instance Variables
    private boolean     isCategoryObj;  // Is this a category?
    private long	objectid;	// Items object id
    private String	desc;		// Human Readable
    private String	ref;		// Where

    /*
     * Constructor
     */ 
    public InventoryImpl(){}
   
    public InventoryImpl(boolean IsCategoryObj, long ObjectId, String Desc,
                     String Ref) {
        isCategoryObj = IsCategoryObj;
	objectid = ObjectId;
	desc = Desc;
	ref = Ref;
    }

    /**
     * Convert name into description.  This really should be in the database.
     */
    private static String getCatDesc(CategoryDO c) {
        String desc = c.getName();
        if (desc.equals("Main Menu")) {
            desc = "Store Directory";
        } else if (desc.equals("MensApparel")) {
            desc = "Men's Apparel";
        } else if (desc.equals("WomensApparel")) {
            desc = "Women's Apparel";
        }
        return desc;
    }
    
    /* Returns an Inventory list of items and categories for the provided 
     * ojectid
     */ 
    public Vector getItems(long thisid) {
	Vector retlist = new Vector();

	// Retreive the instance (ie details) of this category.
	CategoryDO thecategory = CategoryDO.getCategory(thisid);

	// Retreive the subcategories of this category
	long[] catlist = thecategory.getSubCategories();
	for ( int count = 0; count < catlist.length ; count++)
	{
	    CategoryDO c = CategoryDO.getCategory((int)catlist[count]);

	    InventoryImpl newinv = new InventoryImpl(true,
                                             c.getObjectId(), 
					     getCatDesc(c),
					     c.getName());
	    retlist.addElement(newinv);
	}

	// Retreive the items for this category
	long[] itemlist = thecategory.getItems();
	for ( int count = 0; count < itemlist.length ; count++)
	{
	    // TODO - fix casting uglyness
	    ItemDO theitem = ItemDO.getItemByObjectId(itemlist[count]);

	    InventoryImpl newinv = new InventoryImpl(false,
                                             theitem.getObjectId(),
					     theitem.getName(), 
					     theitem.getDescription());
	    retlist.addElement(newinv);
	}

	return retlist;
    }
    public Inventory getParent(long childid)
    {
	CategoryDO cc = CategoryDO.getCategory(childid);
	long pid = cc.getParentId();
	CategoryDO pc = CategoryDO.getCategory(pid);
	// Return null if parent is the same as the child ie top of tree
	if (pc.getObjectId() == cc.getObjectId())
	    return null;

	InventoryImpl pinv = new InventoryImpl(true,
                                       pc.getObjectId(), 
                                       getCatDesc(pc),
                                       pc.getName());
	return pinv;
    }

    /* Returns an Inventory instance for the specified objectid
     */
    public Inventory getInventoryById(long thisid) {
	// Handle Inventory objects
	CategoryDO c = CategoryDO.getCategory(thisid);
	InventoryImpl newinv = new InventoryImpl(true,
                                         c.getObjectId(), 
					 getCatDesc(c),
					 c.getName());
	return newinv;
    }

    public boolean isCategory() {
        return isCategoryObj;
    }

    public long getObjectId() {
	return objectid;
    }
    public String getDesc() {
	return desc;
    }
    public String getRef() {
	return ref;
    }
}
