/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.data.item;

import java.util.Vector;
import java.util.Enumeration;
import java.lang.Object;


// TODO - convert longs to ulong
// TODO - autogenerate objectID 

/**
 * Categorize the items we have for sale.
 *
 * @author Scott Pirie
 * @version $Revision: 1.1 $
 */
public class CategoryDO
{
    /*----------------------------------------------------------------------*/
    // Class Data
    /*----------------------------------------------------------------------*/
    private static CategoryStore storage = null;

    /*----------------------------------------------------------------------*/
    // Class Methods
    /*----------------------------------------------------------------------*/

    /* Determine the storage media selected for the category and item
     * database.
     */
 
public static void InitializeStorageOption(String opt,String dir)         {
        	
	try
	{	
  
          if (opt.equalsIgnoreCase("memory")){
	   storage = new CategoryStoreMemory();
	   storage.initializeCategoryStore();
	   } 
	    else if (opt.equalsIgnoreCase("file"))
	    {
	   storage = new CategoryStoreFile();
	   storage.initializeCategoryStore(dir);
	    
	   }
	    else
	    {
          	storage = new CategoryStoreMemory();
          	storage.initializeCategoryStore();
	    }
	}
	catch (Exception e)
	{
	}
            
}

   
   
   
    /* Constructor.  This is private so non item classes must go
     * through the provided interfaces below to add new objects.
     */ 
    protected CategoryDO(long objectId, long parentId, String name)
    {
	this.objectId = objectId;
	this.parentId = parentId;
	this.name = name;
	this.subList = new Vector();
	this.itemList = new Vector();
	this.isDirty = false;

	// Update parent with subcategory.
	CategoryDO parent = getCategory(parentId);
	if ( parent != null )
	    parent.addSubCategory(objectId);
    }

    /*
     * Find the CategoryDO given the object id.
     */
    public static CategoryDO getCategory(long objid)
    {
	return(storage.findCategoryInStore(objid));
    }

    /*----------------------------------------------------------------------*/
    // Instance Data
    /*----------------------------------------------------------------------*/
    private long 	objectId;	// Application index
    private long	parentId;	// Parent category index
    private String 	name;		// Human read-able
    private Vector	subList;	// Sub Category List
    private Vector	itemList;	// Items in this Category
    private boolean	isDirty;	// Set if instance has been modified.

    /*----------------------------------------------------------------------*/
    // Instance Methods
    /*----------------------------------------------------------------------*/

    public long getObjectId()
    {
	return this.objectId;
    }
    // This is the key so it shouldn't be changed by anyone.
    protected void setObjectId(long newId)
    {
	this.objectId = newId;
    }

    public long getParentId()
    {
	return this.parentId;
    }
    public void setParentId(long newId)
    {
	this.parentId = newId;
    }

    public String getName()
    {
	return this.name;
    }
    public void setName(String newName)
    {
	this.name = newName;
    }

    public void addSubCategory(long subObjectId)
    {
	// Verify the id isn't already a sub category
	for ( Enumeration e = this.subList.elements(); e.hasMoreElements() ; )
	{
	    Long o = (Long) e.nextElement() ;
	    if (o.longValue() == subObjectId)
		return;
	}

	// Add the sub category
	Long newsub = new Long(subObjectId);
	this.subList.addElement(newsub);
    }
    public void removeSubCategory(long subObjectId)
    {
	for ( Enumeration e = this.subList.elements(); e.hasMoreElements() ; )
	{
	    Long o = (Long) e.nextElement() ;
	    if (o.longValue() == subObjectId)
		this.subList.removeElement((Object) o);
	}
    }

    /* 
     * Return an array of subcategory object ids.
     */
    public long[] getSubCategories()
    {
	    Enumeration		e;
	    int			count;

	long[] list = new long[this.subList.size()];

	for ( e = this.subList.elements(), count = 0 ; 
		e.hasMoreElements() ; 
		    count++ )
	{
	    Long o = (Long) e.nextElement() ;
	    list[count] = o.longValue();
	}
	return list;
    }

    /* 
     * Add an item to this category.
     */
    public void addItem(long objectId)
    {
	// Verify the id isn't already in the list
	for ( Enumeration e = this.itemList.elements(); e.hasMoreElements() ; )
	{
	    Long o = (Long) e.nextElement() ;
	    if (o.longValue() == objectId)
		return;
	}

	// Add the item 
	Long newitem = new Long(objectId);
	this.itemList.addElement(newitem);
    }
    public void removeItem(long objectId)
    {
	for ( Enumeration e = this.itemList.elements(); e.hasMoreElements() ; )
	{
	    Long o = (Long) e.nextElement() ;
	    if (o.longValue() == objectId)
		this.itemList.removeElement(o);
	}
    }
    /*
     * Return an array of Item object ids.
     */
    public long[] getItems()
    {
	Enumeration	e;
	int		count;

	long[] list = new long[this.itemList.size()];

	for ( e = this.itemList.elements(), count = 0 ; 
		e.hasMoreElements() ; 
		    count ++)
	{
	    Long o = (Long) e.nextElement() ;
	    list[count] = o.longValue();
	}
	return list;
    }
}
