/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.data.user;

import java.io.*;
import java.util.*;
import com.lutris.util.Config;
import com.lutris.util.ConfigException;
import com.lutris.logging.Logger;
import com.lutris.logging.LogChannel;
import com.lutris.appserver.server.Enhydra;
import golfShop.data.user.UserDOImpl;
import golfShop.data.user.UserStore;


/**
 * This is one flavor of a UserStore. The storage medium in this case
 * is a text file. To save the users, use serialization to write the whole
 * vector of them to the file. To read users, un-serialize from the file.
 * All other operations occur on the copy in memory.
 *
 * @author Andrew John
 * @version $Revision: 1.1 $
 */
public class FileUserStore extends UserStore {
    
    /*
     *  This is the storage medium. Just keep a set of users in memory.
     */
    private Vector allUsers = new Vector();
    private String fileName="";
    /**
     *    Initialize the set of users.
     */
      protected void initializeUserStore() {}
     
    protected void initializeUserStore(String fn) {
        fileName=fn;
        readAllUsersFromFile(fn);
    }


    /**
     *   Just search the copy of the objects in memory.
     */
    protected boolean usernameInUserStore(String username) {
        Enumeration e = allUsers.elements(); 
        while (e.hasMoreElements()) {
            UserDOImpl u = (UserDOImpl) e.nextElement();
            if (username.equals(u.username))
                return true;
        }
        return false;
    }

    
    /**
     *   We already read all the users. Just lookup from memory.
     */
    protected UserDOImpl lookupUserFromUserStore(String username) {
        Enumeration e = allUsers.elements(); 
        while (e.hasMoreElements()) {
            UserDOImpl u = (UserDOImpl) e.nextElement();
            if (username.equals(u.username))
                return u;
        }
        return null;
    }

    
    /**
     *   Add to memory, then write out the whole set of users to the file.
     */  
    protected void addUserToUserStore(UserDOImpl user) {
        allUsers.addElement(user);
        writeAllUsersToFile();
    }

    /**
     *   Even though only the one user has changed, we must write out
     *   the whole vector of all users.
     */
    protected void updateUserInUserStore(UserDOImpl user) {
        writeAllUsersToFile();
    }

    /////////////////////////////////////////////////////////////////
    
    private void writeAllUsersToFile() {

        try {
            FileOutputStream ostream = new FileOutputStream(fileName);
            ObjectOutputStream p = new ObjectOutputStream(ostream);
            p.writeObject(allUsers);
            p.flush();
            ostream.close();
        } catch (Exception e) {
                        LogChannel chan = Enhydra.getLogChannel();
                        if (chan != null)
                            chan.write(Logger.ERROR, 
                                       "Error writing user list to file " + 
                                        fileName + ".", e);
        }
    }

    private void readAllUsersFromFile(String fn) {
     
        try {
           
            FileInputStream istream = new FileInputStream(fn);
           
            ObjectInputStream p = new ObjectInputStream(istream);
            
            allUsers = (Vector)p.readObject();
            istream.close();
        } catch (Exception e) {
            LogChannel chan = Enhydra.getLogChannel();
            if (chan != null)
                chan.write(Logger.WARNING, "Creating a new user list with initial accout username \"enhydra\", password \"lutris\"");
            allUsers = new Vector();
            UserDOImpl u = new UserDOImpl("enhydra", "lutris", "", "", "", "", "", "" ,"");
            allUsers.addElement(u);
            
        }
    }
    
}
