/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.data.user;

import java.util.Vector;
import java.util.Enumeration;
import golfShop.data.user.UserDOImpl;
import golfShop.data.user.UserStore;



/**
 * This is one flavor of a UserStore. The storage medium in this case
 * is memory. A set of users is kept in memory. When the application exits,
 * all the users are lost.
 *
 * @author Andrew John
 * @version $Revision: 1.1 $
 */
public class MemoryUserStore extends UserStore {

    /**
     *   This is the storage medium. Just keep a set of users in memory.
     */
    private Vector allUsers = new Vector();

    /**
     *    Initialize the set of users.
     */
      protected void initializeUserStore(String dir) {}
     
    protected void initializeUserStore() {
        UserDOImpl u = new UserDOImpl("enhydra", "lutris", "", "", "", "", "", "" ,"");
        allUsers.addElement(u);
    }

    protected boolean usernameInUserStore(String username) {
        Enumeration e = allUsers.elements(); 
        while (e.hasMoreElements()) {
            UserDOImpl u = (UserDOImpl) e.nextElement();
            if (username.equals(u.username))
                return true;
        }
        return false;
    }

    protected UserDOImpl lookupUserFromUserStore(String username) {
        Enumeration e = allUsers.elements(); 
        while (e.hasMoreElements()) {
            UserDOImpl u = (UserDOImpl) e.nextElement();
            if (username.equals(u.username))
                return u;
        }
        return null;
    }
    
    protected void addUserToUserStore(UserDOImpl user) {
        allUsers.addElement(user);
    }

    /**
     *   There is nothing to do! A pointer to the object is already in the
     *   storage vector. There is really only one copy of the object. So in
     *   effect, it is already modified in the storeage medium.
     */
    protected void updateUserInUserStore(UserDOImpl user) {
    }
    
}
