/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop;

import java.io.*;
import java.math.BigDecimal;
import java.util.Enumeration;
import com.lutris.util.*;
import com.lutris.logging.Logger;
import com.lutris.logging.LogChannel;
import com.lutris.appserver.server.session.*;
import com.lutris.appserver.server.user.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.httpPresentation.*;
import golfShop.spec.user.*;

import golfShop.spec.cart.*;
import golfShop.spec.*;
/**
 * Application class for this specific application.  This implements
 * standard startup and shutdown methods and well as application specific
 * ones.
 */
public class GolfShop extends StandardApplication {
    /*
     * This is the global Config object that is shared by the 
     * whole application. If there was an error reading the file, this
     * will be null.
     *
     * @see Config
     */
    public static Config config = null;

    /**
     * User manager for this application.
     */
    private GolfShopUserManager userManager = null;

    /**
     * Return the UserManager for this server.
     */
    public GolfShopUserManager getUserManager() {
        return userManager;
    }
    
    private final String DEFAULT_FILENAME = "user_store";
    private final String DEFAULT_DIRECTORY = 
                   "golfShop/presentation/xmlc/contents/itemdb";
    private final String DEFAULT_OPTION ="File";
    
    
    /**
     * Start the application.
     * 
     * @param appConfig Configuration object.
     * @exception HarmonyAppException If an error occurs starting the
     *  application.
     */
    public void startup(Config appConfig) throws ApplicationException {
        /*
         * This will create the config and sessionManager objects.
         */
        super.startup(appConfig);
        if (logChannel != null) {
            logChannel.write(Logger.INFO, 
                             "Welcome to the Lutris GolfShop application!");
        }
        config = appConfig;
         
              
        try {
        
          userManager = GolfShopUserManagerFactory.getGolfShopUserManager("golfShop.business.user.GolfShopUserManagerImpl");
                   
          //Set the default values to parameters that we need to initialize storages        
          String userStoreOption = DEFAULT_OPTION;
          String storeOption = DEFAULT_OPTION;
          String fileName = DEFAULT_FILENAME;
          String dir = DEFAULT_DIRECTORY;
         try{
         
          //Take the values from configuration file
          userStoreOption = config.getString("GolfShop.UserStoreOption");
          storeOption = config.getString("GolfShop.ItemStore.Option");
         } catch (ConfigException e) {
         	
         }
         
         File tempFile = null;
         String tempString = null;
         
         // GolfShop.ItemStore.Directory parameter initialization
          
         try{
          dir = config.getString("GolfShop.ItemStore.Directory");
          tempFile = new File(dir);
          if (!tempFile.exists()||!tempFile.isDirectory()){
          	// in case a relative path is given (relative from configuration file)
          	tempString = config.getConfigFile().getFile().getParent();
          	dir = tempString+File.separator+dir;
          	tempFile = new File(dir);
          	
          	if (!tempFile.exists()||!tempFile.isDirectory()){
          		tempString = this.getClass().getClassLoader().getResource(DEFAULT_DIRECTORY).getPath();
         	 		dir = tempString;
         	 		tempFile = new File(dir);
          	}
          }
         } catch (ConfigException e) {
         	 tempString = this.getClass().getClassLoader().getResource(DEFAULT_DIRECTORY).getPath();
         	 dir = tempString;
         	 tempFile = new File(dir);
         }
         
         if (!tempFile.exists()||!tempFile.isDirectory()){
	       	 dir = DEFAULT_DIRECTORY;
	       	 if (logChannel != null) {
	            logChannel.write(Logger.INFO,
	                             "GolfShop.ItemStore.Directory application parameter not initialized properly!");
	         }
	       }
	       
	       // GolfShop.UserStoreFilename parameter initialization
          
         try{
          fileName = config.getString("GolfShop.UserStoreFilename");
          tempFile = new File(fileName);
          if (!tempFile.exists()){
          	// in case a relative path is given (relative from configuration file)
          	tempString = config.getConfigFile().getFile().getParent();
          	fileName = tempString+File.separator+fileName;
          	tempFile = new File(fileName);
          	
          	if (!tempFile.exists()){
          		tempString = this.getClass().getClassLoader().getResource(DEFAULT_FILENAME).getPath();
         	 		fileName = tempString;
         	 		tempFile = new File(fileName);
          	}
          }
         } catch (ConfigException e) {
         	 tempString = this.getClass().getClassLoader().getResource(DEFAULT_FILENAME).getPath();
         	 fileName = tempString;
         	 tempFile = new File(fileName);
         }
         
         if (!tempFile.exists()){
	       	 fileName = DEFAULT_FILENAME;
	       	 if (logChannel != null) {
	            logChannel.write(Logger.INFO,
	                             "GolfShop.UserStoreFilename application parameter not initialized properly!");
	         }
	       }
         
         //This will initialize all storages with given parameters
         Initialize initialize = InitializeFactory.getInitialize("golfShop.business.InitializeImpl");
         initialize.storages(userStoreOption,fileName,storeOption,dir);
        
/*
 * Catch null pointer exception ( we canot make a instances of classes from business layer  when we run GolfShop_pres ) 
 * We need  to allow GolfShop_pres to be functional 
 */ 
         }catch(NullPointerException e){         
         } catch (Exception e) {
            // N.B. If shutdown methods are available for session and user
            // manager, call them.
            userManager = null;

            // This application isn't designed to be restarted.
            state = DEAD;
            if (e instanceof ApplicationException) {
                throw (ApplicationException)e;
            } else {
                throw new ApplicationException("failure to initialize GolfShop", e);
            }
        }
    }

    /**
     * Shutdown the application.
     */
    public void shutdown() {
        userManager = null;
        super.shutdown();
    }


    public boolean requestPreprocessor(HttpPresentationComms comms)
        throws Exception {
        
        if (logChannel != null) {
            logChannel.write(Logger.DEBUG,
                             "URI=\"" + comms.request.getRequestURI() + "\"");
        }
        
        super.requestPreprocessor(comms);
        if (comms.session == null) {
            // Session not setup; so its not a presentation object.
            return false;
        }

        String pp = comms.request.getApplicationPath();
        if (!pp.endsWith("/"))
            pp += "/";
        String loginDir = pp + "login/";
        String mediaDir = pp + "media/";
        String page = comms.request.getRequestURI();
        if (page.startsWith(loginDir) || page.startsWith(mediaDir)) {
            /*
             *   Let them go to the login and logout areas whether or not
             *   they are logged in.
             */
            return false;
        } else {
            	//We need  to allow poker_pres to be dynamic , so if the requested url is GolfShop_pres user dont need to be logged
      	String uri = comms.request.getRequestURI();
      	 boolean is= uri.startsWith("/GolfShop_pres");
          
         if(is)
         
          return false;
        
          else{  
            if (comms.session.getUser() != null)
                // It's ok, they are logged in already.
                return false;
            else {
                // They are not logged in! Redirect to login page.
                String url = pp + "login/Login.po";
                throw new ClientPageRedirectException(url);
            }
          }
        }
    }

    /*
     * Generate an HTML table of sessions.
     */
    private String makeSessionTable(Enumeration allSessions) {
        StringBuffer results = new StringBuffer();

        results.append("<TABLE><TR><TD ALIGN=CENTER><B>Username</B></TD>" +
                       "<TD ALIGN=CENTER><B>Current total</B></TD></TR>\n");
        while (allSessions.hasMoreElements()) {
            String sessionKey = (String)allSessions.nextElement();
            Session session;
            try {
                session = sessionManager.getSession(sessionKey);
            } catch (SessionException except) {
                // Skip session that may have just become invalid
                continue;
            }
            Cart cart = null;
            String username = null;
            String totalStr = null;
            
            User user = session.getUser();
            if (user != null) {
                username = user.getName();
            } else {
                username = "<I>Not logged in yet</I>";
            }

            SessionData sd = session.getSessionData();
            try {
                cart = (Cart)sd.get("cart");
            } catch (KeywordValueException kve) {
            }
            if (cart != null) {
                double total = cart.getTotal();
                BigDecimal total2 = new BigDecimal(total);
                total2 = total2.setScale(2, BigDecimal.ROUND_HALF_UP);
                totalStr = "$" + total2;
            } else {
                totalStr = "<I>No cart</I>";
            }
            results.append("<TR><TD ALIGN=CENTER>");
            results.append(username);
            results.append("</TD><TD ALIGN=RIGHT>");
            results.append("</TD></TR>\n");
        }
        results.append("</TABLE>\n");
        return results.toString();
    }

    /*
     * The MultiServer's admin app will notice that this function exists
     * and will call it when showing the state of this application.
     * This is my chance to display info specific to this application.
     */
    public String toHtml() {
        Enumeration allSessions;
        try {
            allSessions = sessionManager.getSessionKeys();
        } catch (SessionException except) {
            // Return error as HTML.
            return except.getClass().getName() + " " + except.getMessage();
        }
        if (!allSessions.hasMoreElements()) {
            return "There are no current sessions.";
        } else {
            return makeSessionTable(allSessions);
        }
    }
}
