/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.cart;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import com.lutris.appserver.server.httpPresentation.*;
import java.io.*;
import java.net.URLEncoder;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import golfShop.presentation.xmlc.utilities.*;
import java.math.BigDecimal;
import java.util.Enumeration;
import golfShop.spec.cart.*;
/**
 * This presentation object dynamically creates an HTML page showing the 
 * cart contents in an tabulated form or, if the user wants to go to the
 * Checkout PO, they will be redirected appropriately.
 */
public class Contents implements HttpPresentation {
    //FIXME: Make this work...
    /**
     * Active and inactive buttons images and alt text.
     */
    private static final String ADD_INACTIVE_GIF = "";

    /**
     * URL to reference the this presentation object.
     */
    private static final String CONTENTS_URL = "Contents.po";

    /**
     * URL to reference the checkout presentation object.
     */
    private static final String CHECKOUT_URL = "../checkout/Main.po";

    /**
     * URL of the undo-inactive images.
     */
    private static final String UNDO_INACTIVE_IMG = "../media/Undo_d.gif";

    /**
     * URL of the redo-inactive images.
     */
    private static final String REDO_INACTIVE_IMG = "../media/Redo_d.gif";

    /**
     * Requested action.  If null, just display.
     */
    String action;

    /**
     * Cart object for session.
     */
    Cart cart;

    /**
     * Process the submission of an update.
     */
    private void processUpdate(HttpPresentationComms comms)
            throws HttpPresentationException {
	// Variable to track cart modifications.
	boolean modified = false;
        
        // Search parameters for form update names.
        Enumeration params = comms.request.getParameterNames();
        while (params.hasMoreElements()) {
            String name = (String)params.nextElement();
            if (name.startsWith(ContentsTableFormatter.QTY_PREFIX)) {
                // Change the quantities of the cart items
                long objectId =  Long.parseLong(name.substring(ContentsTableFormatter.QTY_PREFIX.length()));
                String qtyString = comms.request.getParameter(name);
         
         int oldQty=0;
          try{
               oldQty  = cart.getQuantity(objectId);
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run GolfShop_pres )
 * if cart is null ( we set it in CartUtils when we catch NullPointerExceptiont ) the response will be default HTML page 
 *We need  to allow GolfShop_pres to be functional
 */ 
    } catch(NullPointerException ex) {
    	 throw new ClientPageRedirectException(CONTENTS_URL);
     }
                try {
                    int newQty = Integer.valueOf(qtyString).intValue();
                    // Only change if there is a new value.
                    if (newQty != oldQty && newQty >= 0 && newQty < 1000) {
                        cart.setQuantity(objectId, newQty);
                        modified = true;

                    }
                } catch (NumberFormatException nfe) {
                    // Ignore bogus numbers
                }
	    } else if (name.startsWith(ContentsTableFormatter.DEL_PREFIX)) {
                // Delete items from the cart, if requested.
                long objectId =  Long.parseLong(name.substring(ContentsTableFormatter.DEL_PREFIX.length()));
                if (cart.getQuantity(objectId) != -1) {
		    cart.removeItem(objectId);
		    modified = true;
                }
            }
        }

	// If we made any changes above, notify the cart we are done.
	if (modified) {
            cart.doneModifying();
        }

        // Must redirect to this page without the CGI arguments, or reload
        // will result in page being updated again.
        throw new ClientPageRedirectException(CONTENTS_URL);
    }

    /**
     * Disable one either the undo or redo anchor and image.  It is expected
     * that the button is in the enabled state by default.
     * 
     * @param anchor The anchor element.
     * @param img The contained img element.
     * @param inactiveImgUrl The URL for the inactive image.
     */
    private void disableButton(HTMLAnchorElement anchor, HTMLImageElement img,
                               String inactiveImgUrl) {
        // Disable anchor by clearing href and mouseOver attributes.
        anchor.removeAttribute("href");
        anchor.removeAttribute("onmouseover");
        anchor.removeAttribute("onmouseout");

        // Set image to the inactive version.
        img.setSrc(inactiveImgUrl);
    }

    /**
     * Configure the undo/redo `buttons' based on the state of the cart.
     */
    private void configureUndoRedo(ContentsHTML htmlObj) {
        if (!cart.canUndo()) {
            disableButton(htmlObj.getElementUndoAnchor(),
                          htmlObj.getElementUndoImg(),
                          UNDO_INACTIVE_IMG);
        }
        if (!cart.canRedo()) {
            disableButton(htmlObj.getElementRedoAnchor(),
                          htmlObj.getElementRedoImg(),
                          REDO_INACTIVE_IMG);
        }
    }

    /**
     * Output the page, setting dynamic values.
     */
    private void outputPage(HttpPresentationComms comms)
            throws HttpPresentationException {
        ContentsHTML htmlObj
            = (ContentsHTML)comms.xmlcFactory.create(ContentsHTML.class);
      
        ContentsTableFormatter.fillInTable(htmlObj, cart, true);
     try{  
        configureUndoRedo(htmlObj);
        //same thing
      } catch(NullPointerException ex) {
     }    comms.response.writeDOM(htmlObj);
    }

    /**
     * Parse CGI arguments into fields.
     */
    private void parseArgs(HttpPresentationRequest request)
        throws HttpPresentationException {
        action = request.getParameter("action");
        if (action == null) {
            action = "display";
        }
    }

    /**
     * Entry.
     */
    public void run(HttpPresentationComms comms)
        throws HttpPresentationException {

        parseArgs(comms.request);
        cart = CartUtils.getCart(comms.session);
        
        // Process specified action, if any.
        if (action.equals("update")) {
            processUpdate(comms);
        } else if (action.equals("undo")) {
 try{ 	   
            if (cart.canUndo()) {
                cart.undo();
                }
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run GolfShop_pres )
 * if cart is null ( we set it in CartUtils when we catch NullPointerExceptiont ) the response will be default HTML page 
 *We need  to allow GolfShop_pres to be functional
 */ 
    } catch(NullPointerException ex) { throw new ClientPageRedirectException(CONTENTS_URL);
     }
           
	} else if (action.equals("redo")) {
	try{   
             if (cart.canRedo()) {
                cart.redo();}
  //same thing
  } catch(NullPointerException ex) { throw new ClientPageRedirectException(CONTENTS_URL);
     }
       
        }

        outputPage(comms);
    }
}
