/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.checkout;

import java.io.*;
import golfShop.spec.user.UserDO;
import com.lutris.appserver.server.session.Session;
import com.lutris.appserver.server.httpPresentation.*;

/**
 * Presentation object that processes the checkout request. If the user 
 * data is OK, it redirects to the confirmation page.  If any of the 
 * required fields are missing, it redirects back to the checkout page.
 *
 * @author Andrew John
 * @version $Revision: 1.1 $
 */
public class CheckoutProcessor implements HttpPresentation, java.io.Serializable {

    public void run(HttpPresentationComms comms)
		    throws IOException, PageRedirectException, Exception {

	// Update the user info to match what was entered in the form.
	UserDO user = (UserDO) comms.session.getUser();

	String param = comms.request.getParameter("address1");
	if (param != null) user.setAddress1(param);

	param = comms.request.getParameter("address2");
	if (param != null) user.setAddress2(param);

	param = comms.request.getParameter("city");
	if (param != null) user.setCity(param);

	param = comms.request.getParameter("state");
	if (param != null) user.setState(param);

	param = comms.request.getParameter("zip");
	if (param != null) user.setZip(param);

	
	boolean badEmail = false;
	boolean badCC = false;
	param = comms.request.getParameter("email");
	if (param != null) 
		user.setEmail(param);
	else 
		badEmail = true;

	param = comms.request.getParameter("creditcard");
	if (param != null) 
		user.setCreditCard(param);
	else
		badCC = true;

	// Save the changes, if some were made.
	user.commitChanges();

	// If input is bad, redirect to Main.po, else redirect to Confirm.po.
	String url = null;
	if (badEmail || badCC) {
            url = comms.request.getAppFileURIPath("checkout/Main.po");
	} else {
            url = comms.request.getAppFileURIPath("checkout/Confirm.po");
        }
 	ClientPageRedirectException e = new ClientPageRedirectException(url);
	if (badEmail) 
	    e.addArgument("badEmail", "true");
	if (badCC)
	    e.addArgument("badCC", "true");
        throw e;
    }
}
