/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.checkout;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import com.lutris.util.*;
import com.lutris.appserver.server.httpPresentation.*;
import java.io.*;
import java.math.BigDecimal;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import golfShop.presentation.xmlc.utilities.*;
import golfShop.presentation.xmlc.cart.*;
import golfShop.spec.user.UserDO;

import golfShop.spec.cart.Cart;

import golfShop.presentation.xmlc.utilities.CartUtils;
import com.lutris.appserver.server.session.Session;

/**
 * This presentation object dynamically creates an HTML page showing
 * confirmation information from the user's order.
 */
public class Confirm implements java.io.Serializable, HttpPresentation {
    /**
     * Cart object for session.
     */
    private Cart cart;

    /**
     * User data object.
     */
    private UserDO user;

    /**
     * Fill in the user and order information in the page.
     */
    private void addInfoToTable(ConfirmHTML htmlObj) {
	try{
	BigDecimal total = new BigDecimal(cart.getTotal());    
	total = total.setScale(2, BigDecimal.ROUND_HALF_UP);
        htmlObj.setTextTotal(total.toString());

        String email = user.getEmail();
        if ((email == null) || (email.length() == 0)) {
            email = "(no email address configured)";
        }
        htmlObj.setTextUserName(HtmlEncoder.encode(user.getName()));
        htmlObj.setTextEmail(HtmlEncoder.encode(email));
    //same thing
    } catch(NullPointerException ex) { 
       }
    
    }

    /**
     * Output the page, setting dynamic values.
     */
    private void outputPage(HttpPresentationComms comms)
            throws HttpPresentationException {
        ConfirmHTML htmlObj
            = (ConfirmHTML)comms.xmlcFactory.create(ConfirmHTML.class);
        ContentsTableFormatter.fillInTable(htmlObj, cart, false);
        addInfoToTable(htmlObj);
        comms.response.writeDOM(htmlObj);
    }

    /**
     * Entry.
     */
    public void run(HttpPresentationComms comms)
        throws HttpPresentationException {

        cart = CartUtils.getCart(comms.session);
        
        
       user = (UserDO)comms.session.getUser();
   
        outputPage(comms);
    try{
    	 // Empty out their cart.
	cart.reset(); 
	//if cart is null ( GolfShop_pres )
     } catch(NullPointerException ex) { 
       }
    }
}
