/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.checkout;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import com.lutris.util.*;
import com.lutris.appserver.server.httpPresentation.*;
import java.io.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import golfShop.presentation.xmlc.utilities.*;
import golfShop.presentation.xmlc.cart.*;
import golfShop.spec.user.UserDO;

import golfShop.spec.cart.*;
import golfShop.presentation.xmlc.utilities.CartUtils;
import com.lutris.appserver.server.session.Session;

/**
 * This presentation object dynamically creates an HTML page showing an order
 * form.  The order form contains the shopping cart information and the user
 * information.
 */
public class Main implements HttpPresentation {
    /**
     * Cart object for session.
     */
    private Cart cart;

    /**
     * User data object.
     */
    private UserDO user;

    /**
     * Fill in the user information in the page.
     */
    private void addUserToTable(MainHTML htmlObj) {
	// set the username, this field will always be set
     try{  
        htmlObj.setTextUserName(HtmlEncoder.encode(user.getName()));
  
    
	// set any other fields that may have been set if the user established a new account
	HTMLInputElement address1 = htmlObj.getElementAddress1();
	address1.setValue(user.getAddress1());
	
	HTMLInputElement address2 = htmlObj.getElementAddress2();
	address2.setValue(user.getAddress2());

	HTMLInputElement city = htmlObj.getElementCity();
	city.setValue(user.getCity());

	HTMLInputElement state = htmlObj.getElementState();
	state.setValue(user.getState());

	HTMLInputElement zip = htmlObj.getElementZip();
	zip.setValue(user.getZip());

	HTMLInputElement email = htmlObj.getElementEmail();
	email.setValue(user.getEmail());

	HTMLInputElement cardnum = htmlObj.getElementCreditCard();
	if((user.getCreditCard() != null) && (user.getCreditCard().length() > 0))
	    cardnum.setValue(user.getCreditCard());
  
   
    } catch(NullPointerException ex) { 
       }
    }
        

    /**
     * Output the page, setting dynamic values.
     */
    private void outputPage(HttpPresentationComms comms)
            throws HttpPresentationException {
        MainHTML htmlObj
            = (MainHTML)comms.xmlcFactory.create(MainHTML.class);
        ContentsTableFormatter.fillInTable(htmlObj, cart, false);
        addUserToTable(htmlObj);
        comms.response.writeDOM(htmlObj);
    }

    /**
     * Entry.
     */
    public void run(HttpPresentationComms comms)
        throws HttpPresentationException {

        cart = CartUtils.getCart(comms.session);
        user = (UserDO)comms.session.getUser();
        outputPage(comms);
    }
}
