/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.login;

import java.io.*;
import java.net.*;
import java.util.*;
import com.lutris.http.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.user.*;
import com.lutris.util.*;
import golfShop.GolfShop;
import golfShop.spec.user.*;
import com.lutris.appserver.server.session.*;
import golfShop.spec.LoginException;
/**
 * Presentation Object that processes the login request and redirects
 * to the appropriate page.
 *
 * @author Shawn McMurdo
 * @version $Revision: 1.1 $
 */
public class LoginProcessor implements HttpPresentation {
    /**
     * State object in session.
     */
    private LoginState loginState;

    private static final String nousername =
	"You must specify a username and password to login.";
    private static final String badlogin = 
	"Incorrect username or password.";
    private static final String unknownhost = 
	"Login failed: Unknown host name.";
    private static final String noremotehost = 
	"Login failed: No remote host header.";
    private static final String loginfailed = 
	"Login failed for an unknown reason!";
    private static final String toomanylogins = 
	"Login failed: You are already logged in! " +
	"Logout or wait for the session to expire.";
    private static final String permdenied = 
	"Login failed: Permission denied.";
    private static final String disabled = 
	"Login failed: Account is disabled.";
    private static final String nopassword =
	"Login failed: You must enter a valid password to login.";
    private static final String badpassword =
	"Login failed: You must enter a valid password to login.";
    private static final String badusername =
	"Login failed: Invalid user name.";

    /**
     * Entry point for presentation.
     */
    public void run(HttpPresentationComms comms)
		    throws IOException, PageRedirectException,Exception {

	GolfShop application = (GolfShop) comms.application;
	String fail_url = comms.request.getAppFileURIPath("login/Login.po");
	String success_url = comms.request.getAppFileURIPath("main/Main.po");

        loginState = LoginState.get(comms.session);

	//
	// Get the username and password from the html form.
	// These are mandatory fields. Redirect if nothing was entered,
	// or no data was sent, or the data is bad.
	//
	String username = comms.request.getParameter("username");
	if (username == null) {
	    myRedirect(fail_url, true, nousername, null, comms);
        }
	if (username.length() == 0) {
	    myRedirect(fail_url, true, badusername, null, comms);
        }
	String password = comms.request.getParameter("password");
	if (password == null) {
	    myRedirect(fail_url, true, nopassword, username, comms);
        }
	if (password.length() == 0) {
	    myRedirect(fail_url, true, badpassword, username, comms);
        }

	try {
	    //
	    // Do the actual login. The LBS will have created a session
            // object and assigned a cookie already. If the login is
            // sucessfull, the user manager will locate the user data 
            // object that represents the user and set it into the session
            // object. 
	    //
          GolfShopUserManager userManager = application.getUserManager();

         
   try{
	    userManager.login(username, password, comms.session);
 }catch (NullPointerException e){
}
	    // We have successfully logged in!
	    myRedirect(success_url, false, null, username, comms);
	    
	} catch (LoginException le) {
	    switch (le.reason) {
		case GolfShopUserManager.UNKNOWN_ERROR:
		    myRedirect(fail_url, true, loginfailed, username, comms);
		    break;
		case GolfShopUserManager.IO_ERROR:
		    myRedirect(fail_url, true, badlogin, username, comms);
		    break;
		case GolfShopUserManager.AUTH_FAILED:
		    myRedirect(fail_url, true, badlogin, username, comms);
		    break;
		case GolfShopUserManager.MULTIPLE_LOGIN:
		    myRedirect(fail_url, true, toomanylogins, username, comms);
		    break;
		case GolfShopUserManager.PERMISSION_DENIED:
		    myRedirect(fail_url, true, permdenied, username, comms);
		    break;
		case GolfShopUserManager.ACCOUNT_DISABLED:
		    myRedirect(fail_url, true, disabled, username, comms);
		    break;
		default:
		    myRedirect(fail_url, true, loginfailed, username, comms);
		    break;
         }
        }
    }

    /*
     * Redirect to a new page. If deny is true, then the denied message and
     * username is set in the state object.
     */
    private void myRedirect(String url, boolean deny, String msg,
		String username, HttpPresentationComms comms)
		throws HttpPresentationException  {
	ClientPageRedirectException e = new ClientPageRedirectException(url);
	if (deny) {
            loginState.lastError = msg;
	    if ((username != null) && (username.length() > 0)) {
		loginState.userName = username;
            }
	}
	throw e;
    }

}
