/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.login;

import com.lutris.util.*;
import com.lutris.appserver.server.session.*;

/**
 * Login related data kept in the session.  This is used to pass information
 * between login-related pages instead of passing values in CGI args.  This
 * is a considerably easier approach, but can cause problems if the user
 * is accessing the same set of screens from two different browser windows.
 * In this case, this issues is not significant.
 */
class LoginState implements java.io.Serializable {
    /**
     * Name for object in sessionData.
     */
    private static final String sessionDataName = "golfShop.login";

    /**
     * Last error message.  If null, there is no pending error.
     */
    public String lastError = null;

    /**
     * The username that was last entered.
     */
    public String userName = null;

    /**
     * Get the LoginState object from the session, creating one if
     * if doesn't exist.
     */
    public static LoginState get(Session session) {
        try {
            LoginState obj = (LoginState)session.getSessionData().get(sessionDataName);
            if (obj == null) {
                obj = new LoginState();
                session.getSessionData().set(sessionDataName, obj);
            }
            return obj;
        } catch (KeywordValueException except) {
            throw new FatalExceptionError(except);
        }
    }


}
