/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.login;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import com.lutris.appserver.server.httpPresentation.*;
import java.io.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import golfShop.presentation.xmlc.utilities.*;

/**
 * This presentation object dynamically creates an HTML page for
 * creating a new user account.
 */
public class NewAccount implements HttpPresentation {
    /**
     * State object in session.
     */
    private LoginState loginState;

    /**
     * Generate DOM nodes for the account creation error message and add
     * as the first children of the message paragraph element.
     */
    
    private void addErrorMsg(HTMLDocument doc, HTMLElement msgElem,
                             String errorMsg) {
        // Resulting HTML is:
        //   <FONT SIZE=+3>!&nbsp;</FONT><FONT COLOR=red>errorMsg</FONT><BR>

        // Create <FONT SIZE=+3>!&nbsp;</FONT>
        HTMLFontElement fontSize = (HTMLFontElement)doc.createElement("font");
        fontSize.setSize("+3");
        fontSize.appendChild(doc.createTextNode("! "));


        // Create <FONT COLOR=red>errorMsg</FONT>
        HTMLFontElement fontColor = (HTMLFontElement)doc.createElement("font");
        fontColor.setColor("red");
        fontColor.appendChild(doc.createTextNode(errorMsg));

        // finally <BR>
        HTMLBRElement br = (HTMLBRElement)doc.createElement("br");

        // Assemble, bottom-up so its before template text in message.
        msgElem.insertBefore(br,
                             msgElem.getFirstChild());
        msgElem.insertBefore(fontColor,
                             msgElem.getFirstChild());
        msgElem.insertBefore(fontSize,
                             msgElem.getFirstChild());
        
        // Indicate error reported
        loginState.lastError = null;
    }


    /*
     * Displays a username and two password fields.  The url parameter 
     * create.deny is used to signal that a previous invoation had an error. 
     * If true, then the error string is stored in the url parameter 
     * create.message.  If the url paramater create.initial is set, then use 
     * that as the initial username.  This presentation object just gathers 
     * data. It sends the data to the AccountProcessor presentation object 
     * to actually create the new account and log the user in.
     */
    private void outputPage(HttpPresentationComms comms)
            throws HttpPresentationException {
        NewAccountHTML htmlObj
            = (NewAccountHTML)comms.xmlcFactory.create(NewAccountHTML.class);

        // If new account has just failied, modify message to user.
        if (loginState.lastError != null) {
            addErrorMsg(htmlObj, htmlObj.getElementMessage(),
                        loginState.lastError);
        }

        // If we have a user name, set it as the default value.
        HTMLInputElement userName = htmlObj.getElementUsername();
        if (loginState.userName != null) {
            userName.setDefaultValue(loginState.userName);
        }
        comms.response.writeDOM(htmlObj);
    }

    /**
     * Entry.
     */
    public void run(HttpPresentationComms comms)
        throws HttpPresentationException {

        loginState = LoginState.get(comms.session);
        outputPage(comms);
    }
}
