/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.main;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import com.lutris.appserver.server.httpPresentation.*;
import java.io.*;
import java.net.URLEncoder;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import golfShop.presentation.xmlc.utilities.*;

/**
 * This presentation object that dynamically creates the buy (and
 * other actions) button menu.
 *
 * Parameters:
 * <UL>
 * <LI> add - If specified, this identifies the item on the screen to associate
 *      with the add button.
 * </UL>
 */
public class Buy implements HttpPresentation {
    /**
     * Active and inactive button images and alt text.
     */
    private static final String ADD_INACTIVE_GIF = "../media/AddItem_d.gif";
    private static final String ADD_INACTIVE_TEXT = "You Must Select An Item First";
    private static final String ADD_ACTIVE_GIF = "../media/AddItem_n.gif";
    private static final String ADD_ACTIVE_GIF_MOUSEOVER = "../media/AddItem_b.gif";
    private static final String ADD_ACTIVE_GIF_MOUSEOUT = "../media/AddItem_n.gif";
    private static final String ADD_ACTIVE_TEXT = "Click To Add Item to Your Shopping Cart";

    /**
     * URL to reference the item presentation object.
     */
    private static final String ITEM_URL = "Item.po";

    /**
     * Item identifier for adding to the cart.  Maybe null.
     */
    private String add;

    /**
     * Output the page, setting dynamic values.
     */
    private void outputPage(HttpPresentationComms comms)
            throws HttpPresentationException {
        BuyHTML htmlObj = (BuyHTML)comms.xmlcFactory.create(BuyHTML.class);
        HTMLImageElement img = htmlObj.getElementAddImage();
        HTMLAnchorElement anchor = htmlObj.getElementAddAnchor();
        
        if (add == null) {
            img.setSrc(ADD_INACTIVE_GIF);
            img.setAlt(ADD_INACTIVE_TEXT);
            anchor.removeAttribute("href");
        } else {
            img.setSrc(ADD_ACTIVE_GIF);
            img.setAlt(ADD_ACTIVE_TEXT);

            // When the add button is pressed, it goes back to Item.po, without
            // the add parameter, but with the page being referenced as
            // the shopping cart.
            anchor.setHref(ITEM_URL + "?newItem=" + add);
            anchor.setTarget("item");

            // Set on/out javascript.; must URL encode the contents due to quoting.
            anchor.setAttribute("onmouseover", 
                                "add.src='" + ADD_ACTIVE_GIF_MOUSEOVER + "'");
            anchor.setAttribute("onmouseout",
                                "add.src='" + ADD_ACTIVE_GIF_MOUSEOUT + "'");
        }
        comms.response.writeDOM(htmlObj);
    }

    /**
     * Parse CGI arguments into fields.
     */
    private void parseArgs(HttpPresentationRequest request)
        throws HttpPresentationException {

        add = request.getParameter("add");
    }

    /**
     * Entry.
     */
    public void run(HttpPresentationComms comms)
        throws HttpPresentationException {

        parseArgs(comms.request);
        outputPage(comms);
    }
}
