/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.main;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import com.lutris.util.*;
import com.lutris.appserver.server.httpPresentation.*;
import java.io.*;
import java.net.URLEncoder;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import golfShop.presentation.xmlc.utilities.*;
import golfShop.spec.LoginException;
/**
 * This presentation object dynamically creates an HTML page with two frames,
 * consisting of the Buy presentation object and the URL of the item
 * in question.  <P>
 *
 * Parameters:
 * <UL>
 * <LI> add - If specified, the Buy presentation object is passed this value,
 *      identifying the item on the screen.
 * <LI> ref - The value is the URL used for the lower frame, otherwise a default
 *      HTML file is loaded.
 * <LI> newItem - If specified, this item is added to the cart and the cart
 *      object displayed.
 * </UL>
 */
public class Item implements HttpPresentation {
    /**
     * Buy presentation object.
     */
    private static final String buyPO = "Buy.po";

    /**
     * Cart contents PO.
     */
    private static final String CART_CONTENTS_URL = "../cart/Contents.po";

    /**
     * Default item html page.
     */
    private static final String defaultItem = "noItem.html";

    /**
     * Item identifier for adding to the cart.  Maybe null.
     */
    private String add;

    /**
     * URL of page for the item.  Maybe null.
     */
    private String ref;

    /**
     * Item to add to the cart.  Maybe null.
     */
    private Long newItem;

    /**
     * Output the page, setting dynamic values.
     */
    private void outputPage(HttpPresentationComms comms)
            throws HttpPresentationException {
        ItemHTML htmlObj = (ItemHTML)comms.xmlcFactory.create(ItemHTML.class);
        
        HTMLFrameElement buyFrame = htmlObj.getElementBuy();
        String buyUrl = buyPO;
        if (add != null) {
            buyUrl += "?add=" + URLEncoder.encode(add);
        }
        buyFrame.setSrc(buyUrl);

        htmlObj.getElementItem().setSrc(ref);

        comms.response.writeDOM(htmlObj);
    }

    /**
     * Parse CGI arguments into fields.
     */
    private void parseArgs(HttpPresentationRequest request)
        throws HttpPresentationException {

        add = request.getParameter("add");
        ref = request.getParameter("ref");
        if (ref == null) {
            ref = defaultItem;
        }
        String newItemStr = request.getParameter("newItem");
        if (newItemStr != null) {
            newItem = new Long(newItemStr);
        }
    }

    /**
     * Entry.
     */
    public void run(HttpPresentationComms comms)
        throws HttpPresentationException {

        parseArgs(comms.request);
        if (newItem != null) {
            CartUtils.addItem(comms.session, newItem.longValue());
            ref = CART_CONTENTS_URL;
        }
        outputPage(comms);
    }
}
