/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.utilities;

import java.math.BigDecimal;
import java.util.Enumeration;


import golfShop.spec.cart.*;

import com.lutris.util.*;
import com.lutris.appserver.server.session.*;
import com.lutris.appserver.server.httpPresentation.*;

/**
 * This class provides static functions for formatted output of the
 * cart contents.
 */
public class CartUtils {
    
    private static String FF = "<FONT FACE=\"Arial,Helvetica,C Univers 57 Condensed,Futura Book\" SIZE=\"2\">";

    /**
     * A CartUtils constructor.  Prevent instantiation with a private
     * constructor.
     */
    private CartUtils() {
    }

    /**
     * Find the cart object in session.  Creating it if it doesn't
     * exist.
     */
    public static Cart getCart(Session session)
        throws HttpPresentationException {
        try {
            Cart cart = (Cart)session.getSessionData().get("cart");
            if (cart == null) {
                
               Cart car = CartFactory.getCart("golfShop.business.cart.CartImpl");
    
                session.getSessionData().set("cart", car);
            
            }
            return cart;
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run GolfShop_pres )
 * We need  to allow GolfShop_pres to be functional 
 * so the cart will be null
 */ 
 
        } catch(NullPointerException ex) {
             return null;
        } catch (KeywordValueException except) {
            throw new HttpPresentationException(except);
        }
    }

    /**
     * Add an item to the cart.
     */
    public static void addItem(Session session, long objectId)
        throws HttpPresentationException {

        Cart cart = getCart(session);
        cart.addItem(objectId);
        cart.doneModifying();
    }

}
