/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PersonImpl.java,v 1.2 2005/12/28 08:49:58 slobodan Exp $
 */

package transactionsDiscRack.business.person;

import transactionsDiscRack.data.person.*;
import transactionsDiscRack.spec.Person;
import transactionsDiscRack.business.TransactionsDiscRackBusinessException;

import com.lutris.appserver.server.sql.DBTransaction;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.dods.builder.generator.query.DataObjectException;
import org.enhydra.dods.exceptions.AssertionDataObjectException;

/**
 * Represents a person.
 */
public class PersonImpl implements Person {
	/**
	 * The DO of the Person.
	 */
	protected String handle = null;

	protected String login = null;

	protected String password = null;

	protected String firstName = null;

	protected String lastName = null;

	/**
	 * The public constructor.
	 */
	public PersonImpl() {
	}

	/**
	 * The public constructor
	 * 
	 * @param thePerson
	 *            The data object of the person.
	 */
	public PersonImpl(PersonDO thePerson)
			throws TransactionsDiscRackBusinessException {
		try {
			handle = thePerson.get_Handle();
			login = thePerson.getLogin();
			password = thePerson.getPassword();
			firstName = thePerson.getFirstname();
			lastName = thePerson.getLastname();
		} catch (DatabaseManagerException ex) {
			throw new TransactionsDiscRackBusinessException(
					"Error creating Person", ex);
		} catch (DataObjectException ex) {
			throw new TransactionsDiscRackBusinessException(
					"Error creating Person", ex);
		}
	}

	/**
	 * Gets the object id for the person
	 * 
	 * @return the object id.
	 * @exception DiscRackBusinessException
	 *                if an error occurs retrieving data (usually due to an
	 *                underlying data layer error).
	 */
	public String getHandle() {
		return handle;
	}

	/**
	 * Gets the login name for the person
	 * 
	 * @return the login name.
	 * @exception DiscRackBusinessException
	 *                if an error occurs retrieving data (usually due to an
	 *                underlying data layer error).
	 */
	public String getLogin() {
		return login;
	}

	/**
	 * Gets the password for the person
	 * 
	 * @return the password.
	 * @exception DiscRackBusinessException
	 *                if an error occurs retrieving data (usually due to an
	 *                underlying data layer error).
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * Gets the firstname for the person
	 * 
	 * @return the firstname.
	 * @exception DiscRackBusinessException
	 *                if an error occurs retrieving data (usually due to an
	 *                underlying data layer error).
	 */
	public String getFirstname() {
		return firstName;
	}

	/**
	 * Gets the lastname for the person
	 * 
	 * @return the lastname.
	 * @exception DiscRackBusinessException
	 *                if an error occurs retrieving data (usually due to an
	 *                underlying data layer error).
	 */
	public String getLastname() {
		return lastName;
	}

	/**
	 * Sets the login name for the person.
	 * 
	 * @param login
	 *            login name.
	 * @exception DiscRackBusinessException
	 *                if an error occurs setting the data (usually due to an
	 *                underlying data layer error).
	 */
	public void setLogin(String login) {
		this.login = login;
	}

	/**
	 * Sets the password for the person.
	 * 
	 * @param password
	 * @exception DiscRackBusinessException
	 *                if an error occurs setting the data (usually due to an
	 *                underlying data layer error).
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * Sets the firstname for the person.
	 * 
	 * @param firstname
	 * @exception DiscRackBusinessException
	 *                if an error occurs setting the data (usually due to an
	 *                underlying data layer error).
	 */
	public void setFirstname(String firstname) {
		this.firstName = firstname;
	}

	/**
	 * Sets the lastname for the person.
	 * 
	 * @param lastname
	 * @exception DiscRackBusinessException
	 *                if an error occurs setting the data (usually due to an
	 *                underlying data layer error).
	 */
	public void setLastname(String lastname) {
		this.lastName = lastname;
	}

	/**
	 * Commits all changes to the database.
	 * 
	 * @exception DiscRackBusinessException
	 *                if an error occurs retrieving data (usually due to an
	 *                underlying data layer error).
	 */
	public void save(DBTransaction transaction) throws TransactionsDiscRackBusinessException,
			AssertionDataObjectException {
		try {
			PersonDO myDO = PersonDO.createExisting(this.handle,transaction);
			if (myDO == null) {
				// person creation
				myDO = PersonDO.createVirgin(transaction);
			}

			myDO.setLogin(this.login);
			myDO.setPassword(this.password);
			myDO.setFirstname(this.firstName);
			myDO.setLastname(this.lastName);

			myDO.save(transaction);
		} catch (AssertionDataObjectException ex) {
			throw new AssertionDataObjectException(
					"Read-only table: DML operations not allowed", ex);
		} catch (Exception ex) {
			throw new TransactionsDiscRackBusinessException(
					"Error saving person", ex);
		}
	}

}
