package org.enhydra.wlh.business;

import java.io.OutputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.reflect.Array;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmc.wapi.WMAttribute;
import org.enhydra.shark.api.client.wfmc.wapi.WMConnectInfo;
import org.enhydra.shark.api.client.wfmc.wapi.WMFilter;
import org.enhydra.shark.api.client.wfmc.wapi.WMSessionHandle;
import org.enhydra.shark.api.client.wfmodel.WfActivity;
import org.enhydra.shark.api.client.wfmodel.WfAssignment;
import org.enhydra.shark.api.client.wfmodel.WfProcess;
import org.enhydra.shark.api.client.wfmodel.WfProcessMgr;
import org.enhydra.shark.api.client.wfmodel.WfResource;
import org.enhydra.shark.api.client.wfservice.AdminMisc;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.shark.api.client.wfservice.WMEntity;
import org.enhydra.shark.api.client.wfservice.XPDLBrowser;
import org.enhydra.shark.api.common.DeadlineInfo;
import org.enhydra.wlh.business.xml.ActivityInfo;
import org.enhydra.wlh.business.xml.ActivityInfos;
import org.enhydra.wlh.business.xml.AssignmentInfo;
import org.enhydra.wlh.business.xml.AssignmentInfos;
import org.enhydra.wlh.business.xml.DeadlineInfos;
import org.enhydra.wlh.business.xml.ExtendedAttribute;
import org.enhydra.wlh.business.xml.ExtendedAttributes;
import org.enhydra.wlh.business.xml.ObjectFactory;
import org.enhydra.wlh.business.xml.TWSInfo;
import org.enhydra.wlh.business.xml.VariableInfo;
import org.enhydra.wlh.business.xml.VariableInfos;
import org.enhydra.wlh.spec.User;
import org.enhydra.wlh.spec.XMLBuilder;


public class XMLBuilderImpl implements XMLBuilder {

	protected User user = null;

	public  String proId=null;
	public  String  actId=null;
	public  ObjectFactory factory=null; 
	
	public XMLBuilderImpl(User usr,String proID,String actID) {
		this.user = usr;
		this.proId=proID;
		this.actId=actID;
		this.factory=new ObjectFactory();
		
	}
	
	

	public String getResult(int xmlType) throws Exception {

		try {
			// create a JAXBContext
			JAXBContext jc = JAXBContext.newInstance("org.enhydra.wlh.business.xml");
			// create a Marshaller and marshal to write at writer
			
			
			Marshaller m = jc.createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

			Writer wr = new StringWriter();
			
			TWSInfo twsInfo = (TWSInfo)createTWSInfo(xmlType);
		
			m.marshal(twsInfo, wr);

			return wr.toString();

		} catch (Exception ex) {
			ex.printStackTrace();

			throw ex;
		}

	}

	public void writeResult(OutputStream os,int xmlType) throws Exception {
		try {
			// create a JAXBContext
			JAXBContext jc = JAXBContext.newInstance("org.enhydra.wlh.business.xml");
			// create a Marshaller and marshal to write at outputstream
			Marshaller m = jc.createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			TWSInfo twsInfo = createTWSInfo(xmlType);
 
			m.marshal(twsInfo, os);

		} catch (Exception ex) {
			ex.printStackTrace();
			throw ex;
		}

	}

	private TWSInfo createTWSInfo(int xmlType) throws Exception {

		TWSInfo twsInfo = null;
		try {
			SharkConnection sc = Shark.getInstance().getSharkConnection();
			WMConnectInfo wmci = new WMConnectInfo(user.getUsername(), user.getPassword(), "", "");
			sc.connect(wmci);
			twsInfo = fillXMLElements(sc,xmlType);
			sc.disconnect();

			return twsInfo;
		} catch (Exception e) {

			throw e;
		}

	}

	private AssignmentInfos getAssignmentInfos(WMSessionHandle wmSH, WfAssignment[] ass) throws Exception {
          
		AssignmentInfos assInfos = factory.createAssignmentInfos();
		List infos = assInfos.getAssignmentInfo();

		if (ass != null && ass.length > 0) {
			for (int i = 0; i < ass.length; i++) {
				infos.add(getAssignmentInfo(wmSH, ass[i]));
			}

		} 
		return assInfos;

	}

	private AssignmentInfo getAssignmentInfo(WMSessionHandle wmSH, WfAssignment ass) throws Exception {

		AssignmentInfo info =factory.createAssignmentInfo();
		try {
			WfActivity act = ass.activity();
			WfResource res = ass.assignee();
			WfProcess pro = act.container();

			AdminMisc am = Shark.getInstance().getAdminMisc();
			WMEntity ent = am.getActivityDefinitionInfo(null, pro.key(), act.key());

			info.setActivityDefinitionId(ent.getId());
			info.setActivityDefinitionName(ent.getName());

			ent = am.getProcessDefinitionInfo(null, pro.key());

			info.setProcessDefinitionId(ent.getId());
			info.setProcessDefinitionName(ent.getName());

			info.setActivityId(act.key());
			info.setActivityName(act.name());

			info.setProcessId(pro.key());
			info.setProcessName(pro.name());

			info.setIsAccepted(ass.get_accepted_status());
			info.setResourceUsername(res.resource_key());
			
			
			if(act.description()!=null && !act.description().equals(""))
		    info.setDescription(act.description());
			info.setPriority(Short.toString(act.priority()));

		} catch (Exception ex) {

			throw ex;

		}
		return info;

	}

	
	/**
	 * if xmlType = 
	 * 
	 * 0 this methode return only XML document which fill with all assigment form this user
	 * 1 this methode return only XML document which fill with all activity details
	 * 
	 * @param sc
	 * @param xmlType
	 * @return TWSInfo object whic represent XML structure
	 * @throws Exception
	 */
	
	private TWSInfo fillXMLElements(SharkConnection sc, int xmlType) throws Exception {
		try {
			WfResource res = sc.getResourceObject();
			TWSInfo tws = factory.createTWSInfo();
			
			tws.setName("Worklist Handler XML");
			Date date=new Date(System.currentTimeMillis());
			GregorianCalendar gC=new GregorianCalendar();
			gC.setTime(date);
			
			tws.setCreated(gC);
			tws.setProcessInfos(factory.createProcessInfos());
			tws.setRequester(user.getUsername());

			WfAssignment[] ass;
			try {

				ass = res.get_sequence_work_item(0);
				if (xmlType == 0) {
					tws.setAssignmentInfos(getAssignmentInfos(sc.getSessionHandle(), ass));
					
				} else if (xmlType == 1) {
				
					tws.setActivityInfos(getActivityInfos(sc.getSessionHandle(), sc, proId, actId));
					
				    
				} else {
					tws.setAssignmentInfos(getAssignmentInfos(sc.getSessionHandle(), ass));
					tws.setActivityInfos(getActivityInfos(sc.getSessionHandle(), sc, proId, actId));
				}
			} catch (Exception e) {
			}

			return tws;

		} catch (Exception ex) {
			ex.printStackTrace();
			throw ex;
		}
	}

	private ActivityInfos getActivityInfos(WMSessionHandle wmSH, SharkConnection sConn, String procesId, String actId) throws Exception {

		ActivityInfos actInfos = factory.createActivityInfos();
		List list = actInfos.getActivityInfo();

		list.add(getActivityInfo(wmSH, sConn, procesId, actId));

		return actInfos;

	}

	private ActivityInfo getActivityInfo(WMSessionHandle shandle, SharkConnection sConn, String procesId, String actId) throws Exception {

		ActivityInfo info = factory.createActivityInfo();
		try {
            if(procesId!=null && actId!=null){
			
			WfAssignment ass = sConn.getAssignment(procesId, actId, user.getUsername());

			WfActivity act = ass.activity();
			WfResource res = ass.assignee();
			WfProcess pro = act.container();

			AdminMisc am = Shark.getInstance().getAdminMisc();

			WMEntity ent = am.getActivityDefinitionInfo(null, pro.key(), act.key());

			ent = am.getProcessDefinitionInfo(null, pro.key());
			
			info.setDefinitionId("");
			info.setDefinitionName("");
			info.setState("");
			info.setDescription("");
		

			info.setProcessDefinitionId(ent.getId());
			info.setProcessDefinitionName(ent.getName());

			info.setId(act.key());
			info.setName(act.name());
			if(act.description()!=null && !act.description().equals(""))
			 info.setDescription(act.description());
			info.setPriority(Short.toString(act.priority()));

			info.setProcessId(pro.key());
			info.setProcessName(pro.name());
			info.setResourceUsername(res.resource_key());
		

			Date date = new Date(am.getActivityCreatedTime(shandle, procesId, actId));
			
			
			SimpleDateFormat sd=new SimpleDateFormat("yyyy-MM-dd");
			GregorianCalendar gC =  new GregorianCalendar();
			
		     gC.setTime(date);

			info.setCreated(gC);
			date = new Date(am.getActivityStartedTime(shandle, procesId, actId));
			
			
			try {
				info.setStarted(gC);
			} catch (Exception ex) {
			}

			date = new Date(am.getActivityFinishTime(shandle, procesId, actId));
			try {
				info.setFinished(gC);
			} catch (Exception ex) {
			}

			Map varMap = act.process_context();

		
			 // Id->Java Class Name
			// if Boolean, String, Date ili Long stavi u varinfos

			List variables = info.getVariableInfos();
			getVariableInfos(am, shandle, varMap, pro, variables);

			info.setExtendedAttributes(getExtendedAttributes(shandle, ent));
			List dedlines = info.getDeadlineInfos();

			getDeadlineInfos(shandle, am, pro, act, dedlines);
            }
            else
            	return null;

		} catch (Exception ex) {
			ex.printStackTrace();
			throw ex;

		}
		return info;

	}

	protected String[][] getExtAttribNVPairs(WMSessionHandle sh, WMEntity ent) throws Exception {
		XPDLBrowser xpdlb = Shark.getInstance().getXPDLBrowser();
		WMFilter filter = new WMFilter("Type", WMFilter.EQ, "ExtendedAttribute");

		WMEntity[] enti = xpdlb.listEntities(sh, ent, filter, true).getArray();
		String[][] eaNVP = new String[enti.length][2];

		for (int i = 0; i < enti.length; i++) {
			WMEntity ea = enti[i];
			WMAttribute[] attrs = xpdlb.listAttributes(sh, ea, null, true).getArray();
			eaNVP[i][0] = attrs[0].getValue().toString();
			eaNVP[i][1] = attrs[1].getValue().toString();
		}

		return eaNVP;
	}

	private void getVariableInfos(AdminMisc am, WMSessionHandle shandle, Map varMap, WfProcess pro, List variables) throws Exception {

		VariableInfos infos =factory.createVariableInfos();

		List infosList = infos.getVariableInfo();

		Iterator iter = varMap.keySet().iterator();
		while (iter.hasNext()) {

			Object key = (Object) iter.next();
			Object object = (Object) varMap.get(key);
			VariableInfo vinfo = getVariableInfo(am, shandle, key, object, pro);
			if (vinfo != null)
				infosList.add(vinfo);

		}

	
		variables.add(infos);

	}

	private VariableInfo getVariableInfo(AdminMisc am, WMSessionHandle shandle, Object key, Object value, WfProcess pro) throws Exception {

		VariableInfo info = factory.createVariableInfo();
		try {

			WfProcessMgr mgr = pro.manager();
			Map cs = mgr.context_signature(); // Id->Java Class Name
			// if Boolean, String, Date ili Long stavi u varinfos

			Object obj = cs.get(key);

			if (obj != null) {

				if (obj.equals("java.lang.String") || obj.equals("java.lang.Double") || obj.equals("java.lang.Long") || obj.equals("java.util.Date") || obj.equals("java.lang.Boolean")) {

					info.setId(key.toString());
					
					
					info.setJavaType(obj.toString());

					
					WMEntity varEnt = am.getVariableDefinitionInfo(shandle, pro.key(), key.toString());

					if (varEnt.getName() != null)
						info.setName(varEnt.getName());
					else
						info.setName(varEnt.getName());
					if (value != null)
						info.setStringValue(value.toString());
					else
						info.setStringValue("");

					XPDLBrowser xpdlb=Shark.getInstance().getXPDLBrowser();
					WMFilter filter = new WMFilter("Type", WMFilter.EQ, "Description");
					WMAttribute desc=xpdlb.listAttributes(shandle,varEnt,filter,true).getArray()[0];
					info.setDescription(desc.getValue().toString());

					info.setExtendedAttributes(getExtendedAttributes(shandle, varEnt));

				} else {
					return null;
				}
			} else {
				return null;
			}

		} catch (Exception ex) {
			ex.printStackTrace();
			throw ex;

		}
		return info;

	}

	private ExtendedAttributes getExtendedAttributes(WMSessionHandle shandle, WMEntity varEnt) throws Exception {

		try {
			ExtendedAttributes extAtts =factory.createExtendedAttributes();

			List attrs = extAtts.getExtendedAttribute();

			String[][] red = getExtAttribNVPairs(shandle, varEnt);

			if (red.length == 0)
				return null;

			for (int i = 0; i < red.length; i++) {

				getExtendedAttributes(attrs, red[i][0], red[i][1]);
			}

			return extAtts;
		} catch (Exception ex) {

			ex.printStackTrace();
			throw ex;
		}
	}

	private void getExtendedAttributes(List attributes, String name, String value) throws Exception {

		try {
			ExtendedAttribute extAtt = factory.createExtendedAttribute();

			extAtt.setName(name);
			extAtt.setName(value);

			attributes.add(extAtt);
		} catch (Exception ex) {

			ex.printStackTrace();
			throw ex;
		}
	}

	private void getDeadlineInfos(WMSessionHandle shandle, AdminMisc am, WfProcess pro, WfActivity act, List dedlines) throws Exception {

		DeadlineInfo[] dia = am.getDeadlineInfo(shandle, pro.key(), act.key());

		DeadlineInfos infos =factory.createDeadlineInfos();

		List infosList = infos.getDeadlineInfo();

		for (int i = 0; i < dia.length; i++) {

			org.enhydra.wlh.business.xml.DeadlineInfo vinfo = getDeadlineInfo(dia[i]);
			if (vinfo != null)
				infosList.add(vinfo);

		}

		dedlines.add(infos);

	}

	private org.enhydra.wlh.business.xml.DeadlineInfo getDeadlineInfo(DeadlineInfo dia) throws Exception {

		org.enhydra.wlh.business.xml.DeadlineInfo info =factory.createDeadlineInfo();
		try {

			info.setExceptionName(dia.exceptionName);
			info.setIsExecuted(dia.isExecuted);
			info.setIsSynchronous(dia.isSynchronous);

			Date date = new Date(dia.timeLimit);
			
			SimpleDateFormat sd=new SimpleDateFormat("yyyy-MM-dd");
			
			
		    GregorianCalendar gC =new GregorianCalendar();
		    gC.setTime(date);

			info.setTimeLimit(gC);

		} catch (Exception ex) {
			ex.printStackTrace();
			throw ex;

		}
		return info;

	}

}
