/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: WorkListHandler.java,v 1.1 2006/06/07 13:14:25 alex Exp $
 */

package org.enhydra.wlh;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.transaction.UserTransaction;

import org.enhydra.shark.Shark;
import org.enhydra.wlh.presentation.PresentationException;
import org.enhydra.wlh.spec.UserTransactionJNDI;

import com.lutris.appserver.server.ApplicationException;
import com.lutris.appserver.server.Enhydra;
import com.lutris.appserver.server.StandardApplication;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.logging.Logger;
import com.lutris.util.Config;
import com.lutris.util.ConfigException;

/**
 * The application object.
 * 
 * Application-wide data would go here.
 */
public class WorkListHandler extends StandardApplication {

	/**
	 * Shark config file
	 */
	public static String SHARK_CONFIG_FILE = "";

	public static String appPath = "";

	private UserTransaction transaction = null;

	protected static String xslDirectory;

	/*
	 *  A few methods you might want to add to.
	 *  See StandardApplication for more details.
	 */
	public void startup(Config appConfig) throws ApplicationException {
		super.startup(appConfig);
		try {
		Enhydra.register(this);
		//  Here is where you would read application-specific settings from
		//  your config file.

		String propertyName = "DatabaseManager.DB." + this.databaseManager.getDefaultDB() + ".XaUserTransactionLookupName";
	
		try {
			org.enhydra.wlh.spec.UserTransactionJNDI.jNDIName = appConfig.getString(propertyName);
		} catch (ConfigException ex) {
			try {
				propertyName = "DatabaseManager.DB." + this.databaseManager.getDefaultDB() + ".XATransactionManagerLookupName";
				UserTransactionJNDI.jNDIName = appConfig.getString(propertyName, "java:comp/env/UserTransaction");
			} catch (ConfigException exc) {
				UserTransactionJNDI.jNDIName = "java:comp/env/UserTransaction";
			}
		}

		try {
			WorkListHandler.SHARK_CONFIG_FILE = appConfig.getString("WorkListHandler.Shark.confFile", "");
		} catch (Exception ex) {
		}

		try {
			xslDirectory = appConfig.getString("WorkListHandler.Xsl.Directory");
			File temp = new File(xslDirectory);

			if (!temp.isDirectory()) {
				temp = new File(appConfig.getConfigFile().getFile().getParentFile().getParentFile().getParentFile().getParent() + File.separator + xslDirectory);

				if (!temp.isDirectory()) {
					if (logChannel != null)
						logChannel.write(Logger.INFO, "WorkListHandler.xsl.Directory application parameter isn't properly initialized!");

				} else {
					try {
						xslDirectory = temp.getCanonicalPath();
					} catch (Exception e) {
						logChannel.write(Logger.INFO, "WorkListHandler.xsl.Directory application parameter isn't properly initialized!");

					}
				}
			}

		} catch (ConfigException e) {
			if (logChannel != null) {
				logChannel.write(Logger.INFO, "WorkListHandler.xsl.Directory application parameter isn't properly initialized!");

			}
		}

		try {

			// calling Shark.configure(java.util.Properties) using
			// reflection

			initTransaction();

			Properties props = null;

			if (SHARK_CONFIG_FILE != null && !SHARK_CONFIG_FILE.equals("")) {
				File cFile = new File(SHARK_CONFIG_FILE);
				//if file don't exist, try to resolve relative path
				if (!cFile.exists()) {
					cFile = new File(config.getConfigFile().getFile().getParentFile().getCanonicalPath() + "/" + SHARK_CONFIG_FILE);
				}

				if(!cFile.exists())
					throw new ApplicationException("Don't exist config file");
				props = new Properties();
				props.load(new FileInputStream(cFile));

			} else {

				throw new ApplicationException("NO defined shark conf file");
			}

			Shark.configure(props);

			commitTransaction();

		} catch (Exception ex) {
			throw new ApplicationException("Shark initialization failed!", ex);
		}
		
		} finally{
			Enhydra.unRegister();
		}

	}

	public boolean requestPreprocessor(HttpPresentationComms comms) throws Exception {
		return super.requestPreprocessor(comms);
	}

	/**
	 * This is an optional function, used only by the Multiserver's graphical
	 * administration. This bit of HTML appears in the status page for this
	 * application. You could add extra status info, for example
	 * a list of currently logged in users.
	 *
	 * @return HTML that is displayed in the status page of the Multiserver.
	 */
	public String toHtml() {
		return "This is <I>WorkListHandler</I>";
	}

	/**
	 * Method initializes DB transation instance
	 */

	private void initTransaction() throws Exception {
		Context ctx = new InitialContext();
		transaction = (UserTransaction) ctx.lookup(UserTransactionJNDI.jNDIName);
		if (transaction != null) {
			transaction.begin();
		}
	}

	/**
	 * Method commits DB transation instance
	 */
	private void commitTransaction() throws Exception {
		if (transaction != null) {
			transaction.commit();
		}

	}

	/**
	 * This methode return directory where placed xsl file 
	 */

	public static String getXslDirectory() {
		return xslDirectory;
	}

}
