/*
 *  calculator
 *
 *  Enhydra super-servlet specification object
 *  
 */

package calculator.business;


import calculator.spec.CalculatorManager;

import java.math.BigDecimal;
import com.lutris.appserver.server.session.*;

import java.io.IOException;

// Enhydra SuperServlet imports

import com.lutris.util.KeywordValueException;

public class CalculatorManagerImpl implements CalculatorManager{


 /**
   *  Add a digit to the current display. We might be looking at a
   *  "stale" number (if overwrite is set). If so, then clear the
   *  display and save the current number for later use.
   */
  public void addDigit(SessionData sd, String button) throws KeywordValueException {
    
    String digits = (String) sd.get("digits");
    if (digits == null){
      digits = "0";
    }
    if (sd.get("overwrite") != null) {
      try {
        sd.set("pending_num", new Float(digits));
      } catch (NumberFormatException e) {
      }
      digits = "";
      sd.remove("overwrite");
    }
    digits += button;
    sd.set("digits", digits);
  }

 /**
   *  Add a decimal point to the current display. If it already has a
   *  decimal point, then do nothing.
   */
  public void addPoint(SessionData sd)
      throws KeywordValueException {
    String digits = (String) sd.get("digits");
    if (digits == null)
      digits = "0";
    if (sd.get("overwrite") != null) {
      try {
        sd.set("pending_num", new Float(digits));
      } catch (NumberFormatException e) {
      }
      digits = "";
      sd.remove("overwrite");
    }
    if (digits.indexOf(".") != -1)
      return;
    digits += ".";
    sd.set("digits", digits);
  }

  /**
   *  Flip the sign of the number currently in the display.
   */
  public void negate(SessionData sd)
      throws KeywordValueException {
    String digits = (String) sd.get("digits");
    if (digits == null)
      return;
    if (digits.startsWith("-"))
      digits = digits.substring(1);
    else
      digits = "-" + digits;
    sd.set("digits", digits);
  }

  /**
   *  If there is a "pending number" (not in the current display),
   *  and there is a "pending operation" (add, subtract etc...),
   *  then use them and the current contents of the display and do some
   *  math. Leave the result in the current display.
   */
  public void doEquals(SessionData sd)
      throws KeywordValueException {
    String op = (String) sd.get("pending_op");
    if (op == null)
      return;
    Float f = (Float) sd.get("pending_num");
    if (f == null)
      f = new Float("0.0");
    float f2 = f.floatValue();

    String digits = (String) sd.get("digits");
    if (digits == null)
      digits = "0";
    Float f3 = new Float(0);
    try {
      f3 = new Float(digits);
    } catch (NumberFormatException e) {
    }
    float f4 = f3.floatValue();
    double result = 0.0;

    if (op.equals("+")) {
      result = f2 + f4;
    } else if (op.equals("-")) {
      result = f2 - f4;
    } else if (op.equals("*")) {
      result = f2 * f4;
    } else if (op.equals("/")) {
      result = f2 / f4;
    }

    BigDecimal bd = new BigDecimal(result);
    sd.set("digits", bd.toString());
    sd.remove("pending_op");
    sd.remove("pending_num");
    sd.set("overwrite", Boolean.TRUE);
  }

  /**
   *  Remember that a math function was pushed.
   */
  public void doFunction(SessionData sd, String op)
      throws KeywordValueException {
    if (sd.get("pending_num") != null) {
      doEquals(sd);
    } else {
      String digits = (String) sd.get("digits");
      if (digits == null)
        digits = "0";
    }
    sd.set("pending_op", op);
    sd.set("overwrite", Boolean.TRUE);
  }

public void clear(SessionData sd) throws KeywordValueException{

    sd.set("digits", "0");
    sd.remove("pending_op");
    sd.set("pending_num", new Float(0.0));
    sd.set("overwrite", Boolean.TRUE);

   }

}