package calculator.presentation;

// Standard imports
import java.math.BigDecimal;
import com.lutris.appserver.server.session.*;
import calculator.CalcApp;
import java.io.IOException;

// Enhydra SuperServlet imports
import com.lutris.appserver.server.httpPresentation.HttpPresentation;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.util.KeywordValueException;
import org.w3c.dom.html.HTMLImageElement;
import org.w3c.dom.Node;

import calculator.spec.*;


public class CalculatorPresentation implements HttpPresentation {

  private static CalculatorHTML calculator;

  public void run(HttpPresentationComms comms) throws HttpPresentationException, IOException {

    try{
      processButton(comms);
    
      getState(comms);
      
    
    }catch(Exception e){
      e.printStackTrace();
    }
  }

  /**
   *  A sample Enhydra application. A simple calculator is simulated using
   *  a single presentation object. Every time a button is pressed this
   *  presentation object is notified via the querry string
   *  "?button=...". <P>
   *
   *  The responsability of this object is to emit an HTML string that
   *  shows the current state of the calculator. If a button is being
   *  passed to us in the query string, we process that button first,
   *  then show the current state of the calculator. <P>
   *
   *  Because the SessionData object is being used to store the state of
   *  the calculator, multiple users may  use the calculator simultaneously
   *  "for free". <P>
   *
   *  This method is called to process a button push. If no button
   *  was sent in the URL, do nothing.
   *
   *  @author Andy John andy@enhydra.org
   */

  static public void processButton(HttpPresentationComms comms) throws Exception {
    // Get the session data for the user who issued this request.
    
    
    SessionData sd = comms.session.getSessionData();
    
    
    String button =  (String) comms.request.getParameter("button");

    CalculatorManager calculatorManager = CalculatorManagerFactory.getCalculatorManager("calculator.business.CalculatorManagerImpl");


    if (button != null) {
      // Increment the counter stored in the application object.
      CalcApp myApplication = (calculator.CalcApp) comms.application;
      myApplication.buttonsPushed++;



/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer when we run calculator_pres )
 * We need  to allow calculator_pres to be functional 
 */ 
      if (button.equals("0") || button.equals("1") ||
          button.equals("2") || button.equals("3") ||
          button.equals("4") || button.equals("5") ||
          button.equals("6") || button.equals("7") ||
          button.equals("8") || button.equals("9"))
      {
        try{
               calculatorManager.addDigit(sd, button);
        
        }catch(NullPointerException ex){
        }catch(Exception e){
          e.printStackTrace();
        }
      }
      else if (button.equals("point"))
      {
        try{
            calculatorManager.addPoint(sd);
        }catch(NullPointerException ex){
        }catch(Exception e){
          e.printStackTrace();
        }
      }
      else if (button.equals("negate"))
      {
        try{
            calculatorManager.negate(sd);
        }catch(NullPointerException ex){
        }catch(Exception e){
          e.printStackTrace();
        }
      }
      else if (button.equals("clear")) {
        try{
            calculatorManager.clear(sd);
        }catch(NullPointerException ex){
        }catch(Exception e){
          e.printStackTrace();
        }
        } else if (button.equals("equals"))
        {
          try{
              calculatorManager.doEquals(sd);
        }catch(NullPointerException ex){
        }catch(Exception e){
            e.printStackTrace();
          }
        }
        else if (button.equals("plus"))
        {
          try{
              calculatorManager.doFunction(sd, "+");
        }catch(NullPointerException ex){
        }catch(Exception e){
            e.printStackTrace();
        }
        }
        else if (button.equals("minus"))
        {
          try{
              calculatorManager.doFunction(sd, "-");
        }catch(NullPointerException ex){
        }catch(Exception e){
            e.printStackTrace();
          }
        }
        else if (button.equals("times"))
        {
          try{
              calculatorManager.doFunction(sd, "*");
        }catch(NullPointerException ex){
        }catch(Exception e){
            e.printStackTrace();
          }
        }
        else if (button.equals("divide"))
        {
          try{
              calculatorManager.doFunction(sd, "/");
        }catch(NullPointerException ex){
        }catch(Exception e){
            e.printStackTrace();
          }
        }
    }
  }

   
  /**
   *  This method initializes an XMLC field with the contents of
   *  the current number in the display. The string of digits is converted
   *  into a sequence of HTML image tags.
   *
   *  This is only a demo program. It does not handle exceptional cases
   *  well, like dividing by zero, overflow, or infinties. The point is to
   *  show off one way to use Enhydra, not be a full-on HP calculator.
   */
  
  
  
  public static void getState(HttpPresentationComms comms) throws Exception {
    // Convert current state to HTML
    SessionData sd = comms.session.getSessionData();
    
    
    State state = StateFactory.getState("calculator.business.StateImpl");
    calculator = (CalculatorHTML)comms.xmlcFactory.create(CalculatorHTML.class);  



/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer when we run calculator_pres )
 * We need  to allow calculator_pres to be functional , so if the requested url is /calculator_pres/..... the response
 * will be default HTML page
 */ 

try{  
//DigitsChack  
    String digits = state.checkDigits(sd);
    
// Convert the string of digits to HTML.
   String digitHtml = state.convertDigitsToHTML(digits);

    // Make the result available to the HTML page.
   
    HTMLImageElement digitsDummy = calculator.getElementDigits();
    Node parent = digitsDummy.getParentNode();

    int resultStart = 0;
    int resultEnd = 0;
    while((resultStart=digitHtml.indexOf("<")) != -1){
      resultEnd = digitHtml.indexOf(">");
      String txt = digitHtml.substring(resultStart+9, resultEnd);
      digitHtml = digitHtml.substring(resultEnd+1);

      HTMLImageElement image = (HTMLImageElement) digitsDummy.cloneNode(false);
      image.setSrc(txt);
      parent.appendChild(image);
    }
    parent.removeChild(digitsDummy);
}catch(NullPointerException ex){
} 
    
    comms.response.writeDOM(calculator);
  }
}




