package chat.business;

import chat.spec.*;

public class DiscussionManagerImpl implements DiscussionManager {

   
    public DiscussionManagerImpl() {
    }


    /**
     *  Add a message to the discussion. Any waiting clients will be
     *  instantly notified.
     */
    public void addMessage(String name, String txt) {
      Discussion.addMessage(name,txt);
    }






    /**
     *  Throw out all the current messages. 
     *  All the waiting clients will be immediatly notified.
     */
    public void clear() {
      Discussion.clear();
    }

 


    /**
     *  Get a snapshot of the current state. Might block.
     *
     *  @param currentState
     *    The state identifier that was returned last time this was called.
     *    This is the id of the state that the client currently has. It is
     *    asking for a snapshot of the state after things change and
     *    this id is not longer current. If this has already happened, the
     *    call will immediatly return. If it has not happned, the call will
     *    block (not return) until things change.
     *  @param wait
     *    The maxmimum number of seconds this call is allowed to block for.
     *    Send 0 for an instant response. If the call blocks and then
     *    runs out of time, a snapshot is returned.
     *  @return
     *    A Snapshot object. This is just a way to return two things at
     *    once: a Vector of Message objects and a state identifier.
     *    The client should use the state identifier for the next call to
     *    this method. The client should call this method again as soon as
     *    it is done displaying the results.
     */
    public Snapshot getContents(long currentState, long wait) {
      
        return  Discussion.getContents(currentState, wait);
    }



    /**
     *  How may clients are blocked on a read? This is, effectivly, the
     *  number of people participating in the discussion.
     */
    public int getNumWaiting() {
        return Discussion.getNumWaiting();
    }

   public long getTotalReceived(){
        return Discussion.getTotalReceived();	
   }

   public long getCurrentSize(){
       return Discussion.getCurrentSize();
   }
   
       

  public void setMaxQueueSize(int maxQueueSize){
    Discussion.maxQueueSize=maxQueueSize;	
   }
  



    /**
     *  Call this function (only once) if you want to start the
     *  harverter thread. It runs in the background and periodically
     *  deletes messages that are too old. Most of the time the thread
     *  is sleeping. If this method is not called, then no age limit on
     *  messages will be enforced. <P>
     *
     *  The longest a message can live is lifetimeSec + intervalSec seconds
     *  (in the extreme case). <P>
     *
     *  The default value for the interval is a little shorter than the
     *  default value for the browser's timeout. This is so that if the
     *  browser is left idle (and messages start being deleted), the
     *  refresh cycle will sync up with this threads interval, and
     *  fewer updates will happen (delete, delete, delete... instead of
     *  timout, delete, timeout, delete, timeout, delete...).  It's not
     *  a big deal, but hey...
     *
     *  @param lifetimeSec
     *    How long should messages be kept for (seconds).
     *  @param 
     */ 
    public void startHarvester(int lifetimeSec, int intervalSec) {
       
           Discussion.startHarvester(lifetimeSec,intervalSec);
        
    }
public void stopHarvester(){
   Discussion.stopHarvester();
  }

}

     
