package chat.business;

import java.util.Vector;
import com.lutris.util.HtmlEncoder;
import chat.spec.*;

/**
 *  This class holds all the data about one message (what someone said).
 *  The message is broken down into lines, and each line is HTML encoded.
 *  This makes displaying the message faster. The message is created once,
 *  but displayed over and over to all the listeners.
 */
public class MessageImpl implements Message{

  /**
   *  Who sent in this bit of text?
   */
  public String name;

  /**
   *  The HTML encoded version of name.
   *  This string is safe to stick into a page.
   */
  public String htmlName;

  /**
   *  The text of the message, in one un-encoded string.
   *  The user may have entered invalid HTML, so use the htmlChunks
   *  when emitting as HTML.
   */
  public String text;

  /**
   *  The text of the message, broken down by lines, then HTML encoded.
   *  These strings are safe to stick into a page.
   */
  public Vector htmlChunks = new Vector();

  /**
   *  The time when this message was added to the discussion.
   *  This is used to delete messages that are too old.
   */
  private long when;

  /**
   *  Create a new message. This does not add it to the discussion.
   *  The HTML fields and time are initialized, though.
   *  @see chat.business.Discussion
   */
  public MessageImpl(String name, String text) {
    when = System.currentTimeMillis();
    this.name = name;
    this.htmlName = HtmlEncoder.encode(name);
    this.text = text;
        /*
    *  Break the text into lines, and HTML encode each line.
         */
    if (text.length() == 0)
      htmlChunks.addElement("");
    while (text.length() != 0) {
      int end = text.indexOf("\n");
      if (end != -1) {
        htmlChunks.addElement(HtmlEncoder.encode(text.substring(0,end)));
        text = text.substring(end+1);
      } else {
        htmlChunks.addElement(HtmlEncoder.encode(text));
        text = "";
      }
    }
  }


  /**
   *  Get the time when the message was created. The result is in
   *  milliseconds, see System.currentTimeMillis().
   */
  public long getWhen() {
    return when;
  }
  public String getHtmlName(){
    return htmlName;	
  }
  public Vector getHtmlChunks(){
    return htmlChunks;
  }

}

