/**
 *  Andy John
 *  2/99
 *  chat room
 *  andy@enhydra.org
 *  www.enhydra.org
 *
 *   This is a "by hand" presentation object (Enhydra Jolt was not used).
 *  This is because there is so much java code and so little static HTML that
 *  it is easier to write it by hand.
 *    This page and the Discussion business object are the core of the
 *  "on hold push" algorithm (see the Discussion object). The browser has
 *  asked for a snapshot. They may have sent in a state id. If so, then they
 *  already have something on their screen, and they are really asking for
 *  something new. This state id (or 0 if none was sent), is passed on to
 *  the Discussion object when we ask for a snapshot. It uses it to (maybe)
 *  block (not return) until there is some new state (a message has been
 *  added or deleted from the list).
 *  The browser also probably specified a time limit it is willing to wait.
 *  The browser will time out before TCP/IP will, so we use a one minute
 *  timeout. If the browser times out, it breaks the reload cycle, and the
 *  user stops seeing messages.
 *    Becuase the call to the Discussion object might block for up to a
 *  minute, there may be several threads waiting around in this object.
 */

package chat.presentation;

import java.util.Vector;
import com.lutris.appserver.server.httpPresentation.*;
import chat.ChatApplication;
import chat.spec.*;

/**
 *  Because this page has alot more code than html content, and because
 *  so much of the page is generated in Java anyway, I chose to write this
 *  presentation object by hand, ranther than using Jolt.
 */
public class ContentsPresentation implements HttpPresentation {

    /*
     *  How long can we make the browsers wait till they time out.
     *  This is a conservative guess (one minute).
     */
    private static final int refreshWait = 60;

    /*
     *  This is used with the "unique" query string parameter. It is a
     *  psued-hack to make every page look unique. Otherwise sometimes the
     *  browser will think that a request looks like an old one and try
     *  to slip you results from the cache. By using the counter, each
     *  request is guarenteed to look unique. The actual value is not
     *  used by the program.
     */
    private static long counter = 0;


    /**
     *  The main body of a presentation object. See the class comments above.
     */
    public void run(HttpPresentationComms comms) throws Exception {
        // Try to get the current state, default to zero.
        long state = 0;
        try {
            String stateStr = comms.request.getParameter("state");
            if (stateStr != null)
                state = Long.parseLong(stateStr);
        } catch (Exception e) {
        }
        // Try to get the wait timeout. Default to zero.
        long wait = 0;
        try {
            String waitStr = comms.request.getParameter("wait");
            if (waitStr != null)
                wait = Long.parseLong(waitStr);
        } catch (Exception e) {
        }
        /*
         *  Get a snapshot! This may take up to wait seconds.
         */
     
           DiscussionManager discussionManager = DiscussionManagerFactory.getDiscussionManager("chat.business.DiscussionManagerImpl");
   
   
   

  
      
      
        /*
         *  Start writing out the page.
         */
        ChatApplication myApp = (ChatApplication) comms.application;

        comms.response.setContentType("text/html");
        comms.response.setHeader("Expires", "Tue, 01 Jan 1980 1:00:00 GMT");
        comms.response.setHeader("Pragma", "no-cache");
        HttpPresentationOutputStream out = comms.response.getOutputStream();

        out.println("<HTML>");
        out.println("<HEAD></HEAD>");
        out.println("<BODY onload=\"loadOn();\" BGCOLOR=" + myApp.getBgColor() +">");
        out.println("<h3><center><font color=purple>" +
                    myApp.getRoomName() +
                    "</font></center></h3>");

/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer when we run chat_pres )
 * We need  to allow chat_pres to be functional , so if the requested url is /chat_pres/..... the response
 * will be default HTML page
 */ 
   
 try{  
        Snapshot snap = discussionManager.getContents(state, wait);
      
        if ((snap.getContents() == null) || (snap.getContents().size() == 0)) {
            out.println("<center><i><br><br><br><font color=purple><b>" +
                        "There are currently no messages." +
                        "</b></font></i></center>\n");
        } else {
            for (int i=snap.getContents().size()-1; i>=0; i--) {
                /*
                 *  I used to display the messages in the "normal" order,
                 *  but ran into browser incompatabilites with scrolling,
                 *  named anchors, meta refresh tags etc...
                 *  This way it works on all browsers (which is a good
                 *  thing for a demo).
                 */
                
            Message msg = (Message) snap.getContents().elementAt(i);
                                 
                if (msg == null)
                    continue;
                out.print("<b>");
                if ((msg.getHtmlName() == null) || (msg.getHtmlName().length() == 0))
                     out.print("<i>Anonymous</i>");
                else
                     out.print(msg.getHtmlName());
                out.print(":</b> ");
                if (msg.getHtmlChunks().size() > 1)
                    out.print("<br>\n&nbsp;&nbsp;&nbsp;&nbsp;");
                for (int j=0; j<((Vector)msg.getHtmlChunks()).size(); j++) {
                    if (j != 0)
                        out.print("&nbsp;&nbsp;&nbsp;&nbsp;");
                    out.print((String) ((Vector)msg.getHtmlChunks()).elementAt(j));
                    out.println("<br>");
                }
            }
        }
  
        
        /*
         *  A key part of the "on hold push" algorithm is that the client
         *  asks for a new snapshot as soon as it is done processing the
         *  current snapshot. This synamic Javascript triggers the browser to
         *  reload, passing on the current state id.
         *  I used to use a refresh HTTP header, but IE has problems with
         *  that when you add a query string.
         */
        
        out.println("<SCRIPT TYPE=\"text/javascript\" LANGUAGE=\"JavaScript\" id=\"RealScript\">\n"
        		+ "<!--\n"
				+ "  function loadOn() {\n"
				+ "    window.location.href = \"ContentsPresentation.po?state=" + snap.getState() + "&wait=" + refreshWait + "&uniqe=" + counter +"\"\n"
				+ "  }\n"
				+ "//-->\n"
				+ "</SCRIPT>");
   
  }catch(NullPointerException e){}
        /*
         *  Close out the page.
         */
        out.println("</BODY>");
        out.println("</HTML>");
    }

}

