/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DiscGeneratorImpl.java,v 1.1 2004/08/16 09:39:19 slobodan Exp $
 */

package discRack.business.disc;


import discRack.data.disc.*;
import discRack.spec.*;
import discRack.business.DiscRackBusinessException;
import discRack.data.person.PersonDO;

import com.lutris.dods.builder.generator.query.*;
import com.lutris.appserver.server.sql.ObjectId;
import org.enhydra.dods.exceptions.AssertionDataObjectException;
/**
 * Used to find the instances of disc.
 */
public class DiscGeneratorImpl implements DiscGenerator {
    
    /** 
     * The findDiscsForPerson method performs a database query to
     * return an array of <CODE>Disc</CODE> objects
     * representing all the rows in the <CODE>disc</CODE> table
     * that have and owner matching <CODE>Person owner</CODE>. 
     *
     * @param owner The owner of the discs
     * @return
     *     array of discs. 
     * @exception DiscRackBusinessException
     *   If there is a problem retrieving disc information.
     */
    public Disc[] findDiscsForPerson(Person owner) 
        throws DiscRackBusinessException {
        DiscImpl[] theDiscArray = null;
        
        try {
            DiscQuery query = new DiscQuery();
            //set query
            query.setQueryOwner(PersonDO.createExisting(owner.getHandle()));
            // Order discs alphabetically by artist
            query.addOrderByArtist();
            DiscDO[] DOarray = query.getDOArray();
            theDiscArray = new DiscImpl[ DOarray.length ];
            for ( int i = 0; i < DOarray.length; i++ )
	    	theDiscArray[i] = new DiscImpl(DOarray[i]);
        }catch(Exception ex) {
            throw new DiscRackBusinessException("Exception in findDiscsForPerson()", ex);
        }
        
        return theDiscArray;
    }
    
    /** 
     * The findDiscByID method performs a database query to
     * return a <CODE>Disc</CODE> object
     * representing the row in the <CODE>disc</CODE> table
     * that matches the object id. 
     *
     * @param id , the object id of the disc table.
     * @return
     *    the disc. null if there isn't a person associated
     *    the id
     * @exception DiscRackBusinessException
     *    if there is a problem retrieving person information.
     */
    public Disc findDiscByID(String id) 
        throws DiscRackBusinessException {
        DiscImpl theDisc = null;
        
        try {
            DiscQuery query = new DiscQuery();
            //set query
            query.setQueryOId(new ObjectId(id));
            // Throw an exception if more than one user by this name is found
            query.requireUniqueInstance();
            DiscDO theDiscDO = query.getNextDO();
            theDisc = new DiscImpl(theDiscDO);
            return theDisc;
        }catch(Exception ex) {
            throw new DiscRackBusinessException("Exception in findDiscsByID()", ex);
        }
    }
    
   
    
}

