/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: Edit.java,v 1.1 2004/08/16 09:39:21 slobodan Exp $
 */

package discRack.presentation.discMgmt;

import discRack.presentation.BasePO;
import discRack.presentation.DiscRackPresentationException;
import discRack.spec.*;


import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;
import org.enhydra.dods.exceptions.AssertionDataObjectException;

/**
 * Edit class handles the disc management (add, edit or delete)
 * of the DiscRack app
 *
 */
public class Edit extends BasePO {

    /**
     * Constants representing HTTP parameters passed in from
     * the form submission
     */
    private static String TITLE_NAME = "title";
    private static String ARTIST_NAME = "artist";
    private static String GENRE_NAME = "genre";
    private static String DISC_ID = "discID";
    private static String IS_LIKED = "like";
    private static String INVALID_ID = "invalidID";
    private static String EDIT_COMMAND = "edit";

    /**
     * Superclass method override
     */
    public boolean loggedInUserRequired() {
        return true;
    }

    /**
     * Default event. Just show the page populated with any disc
     * parameters that were passed along.
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleDefault()
        throws HttpPresentationException {
        return showEditPage(null);
    }

    /**
     * handle show add disc page event.
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleShowAddPage()
        throws HttpPresentationException {
        return showAddPage(null);
    }

    /*
     * Edits an existing disc
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleEdit()
        throws HttpPresentationException {
        String discID = this.getComms().request.getParameter(DISC_ID);
        Disc disc = null;

        // Try to get the disc by its ID

 
        try {
       
    disc = ((DiscGenerator)DiscGeneratorFactory.getDiscGenerator("discRack.business.disc.DiscGeneratorImpl")).findDiscByID(discID);
       
        } catch(Exception ex) {
            this.getSessionData().setUserMessage("Please choose a valid disc to edit");
            throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
        }

        try {
            saveDisc(disc);
            throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
        } catch(Exception ex) {
            return showEditPage("You must fill out all fields to edit this disc");
        }
    }

    /*
     * Adds a disc to the disc database
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleAdd()
        throws HttpPresentationException {
        try {
        
           
         Disc disc= DiscFactory.getDisc("discRack.business.disc.DiscImpl");
         saveDisc(disc);
          
         throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
        
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run discRack_pres )
 * We need  to allow discRack_pres to be functional ,  response
 * will be default HTML page
 */ 
       
        }catch(NullPointerException ex) {
            return showAddPage("You cannot add disc when you in DiscRack_pres");
        }
        catch(AssertionDataObjectException ex) {
            return showAddPage("Dics table is read-only: no DML operations allowed");
        }
        catch(Exception ex) {
            return showAddPage("You must fill out all fields to add this disc");
        }
    }

    /*
     * Deletes a Disc from the Disc database
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleDelete()
        throws HttpPresentationException, DiscRackPresentationException, AssertionDataObjectException {
        String discID = this.getComms().request.getParameter(DISC_ID);


/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run discRack_pres )
 * We need  to allow discRack_pres to be functional   
 */ 
try {
            
        Disc disc = ((DiscGenerator)DiscGeneratorFactory.getDiscGenerator("discRack.business.disc.DiscGeneratorImpl")).findDiscByID(discID);
  
        
                   
            String title = disc.getTitle();
            disc.delete();
            this.getSessionData().setUserMessage("The disc, " + title +
                                                 ", was deleted");
            
        
        }catch(NullPointerException e){
                    
        // Catch any possible database exception as well as the null pointer
        //  exception if the disc is not found and is null after findDiscByID
        }
        catch (AssertionDataObjectException ex) {
            this.getSessionData().setUserMessage("Read-only table!");
        }
        catch(Exception ex) {
            this.getSessionData().setUserMessage("Please choose a valid disc to delete");
        }
        // Redirect to the catalog page which will display the error message,
        //  if there was one set by the above exception
        throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
    }

    /**
     * Produce HTML for this PO, populated by the disc information
     * that the user wants to edit
     *
     * @param errorMsg the error messages
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject showEditPage(String errorMsg)
        throws HttpPresentationException, DiscRackPresentationException {
        String title = this.getComms().request.getParameter(TITLE_NAME);
        String artist = this.getComms().request.getParameter(ARTIST_NAME);
        String genre = this.getComms().request.getParameter(GENRE_NAME);
        String discID = this.getComms().request.getParameter(DISC_ID);
        // Instantiate the page object
        EditHTML page = (EditHTML)myComms.xmlcFactory.create(EditHTML.class);

        Disc disc = null;

/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run discRack_pres )
 * We need  to allow discRack_pres to be functional ,  response
 * will be default HTML page with message
 */ 
try {
        	
       disc = ((DiscGenerator)DiscGeneratorFactory.getDiscGenerator("discRack.business.disc.DiscGeneratorImpl")).findDiscByID(discID);
         
            
        }catch(NullPointerException e){
          page.setTextErrorText("You cannot edit disc when you are in DiscRack_pres");
           return page;
       
       // Catch any possible database exception in findDiscByID()
        } catch(DiscRackException ex) {
            this.getSessionData().setUserMessage("Please choose a valid disc to edit");
            throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
        }

        try {
            // If we received a valid discID then try to show the disc's contents,
            //  otherwise try to use the HTML input parameters
            page.getElementDiscID().setValue(disc.getHandle());

            if(null != title && title.length() != 0) {
                page.getElementTitle().setValue(title);
            } else {
                page.getElementTitle().setValue(disc.getTitle());
            }

            if(null != artist  && artist.length() != 0) {
                page.getElementArtist().setValue(artist);
            } else {
                page.getElementArtist().setValue(disc.getArtist());
            }

            if(null != genre && genre.length() != 0) {
                page.getElementGenre().setValue(genre);
            } else {
                page.getElementGenre().setValue(disc.getGenre());
            }

            if(null != this.getComms().request.getParameter(IS_LIKED)) {
                page.getElementLikeBox().setChecked(true);
            } else {
                page.getElementLikeBox().setChecked(disc.isLiked());
            }

            if(null == errorMsg) {
                page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
            } else {
                page.setTextErrorText(errorMsg);
            }
        }catch(NullPointerException ex) {
            return showAddPage(errorMsg);
      
        } catch(DiscRackException ex) {
            throw new DiscRackPresentationException("Error populating page for disc editing", ex);
        }

        page.getElementEventValue().setValue(EDIT_COMMAND);
        return page;
    }

    /**
     * Produce HTML for this PO
     *
     * @param errorMsg the error messages
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject showAddPage(String errorMsg)
        throws HttpPresentationException, DiscRackPresentationException {
        String title = this.getComms().request.getParameter(TITLE_NAME);
        String artist = this.getComms().request.getParameter(ARTIST_NAME);
        String genre = this.getComms().request.getParameter(GENRE_NAME);
        String discID = this.getComms().request.getParameter(DISC_ID);
        // Instantiate the page object
        
       
        EditHTML page = (EditHTML)myComms.xmlcFactory.create(EditHTML.class);
//        EditHTML page = new EditHTML();

        if(null != this.getComms().request.getParameter(TITLE_NAME)) {
            page.getElementTitle().setValue(this.getComms().request.getParameter(TITLE_NAME));
        }
        if(null != this.getComms().request.getParameter(ARTIST_NAME)) {
            page.getElementArtist().setValue(this.getComms().request.getParameter(ARTIST_NAME));
        }
        if(null != this.getComms().request.getParameter(GENRE_NAME)) {
            page.getElementGenre().setValue(this.getComms().request.getParameter(GENRE_NAME));
        }
        if(null != this.getComms().request.getParameter(IS_LIKED)) {
            page.getElementLikeBox().setChecked(true);
        } else {
            page.getElementLikeBox().setChecked(false);
        }

        if(null == errorMsg) {
            page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
        } else {
            page.setTextErrorText(errorMsg);
        }

        return page;
    }

    /**
     * Method to save a new or existing disc to the database
     *
     * @param disc the disc to be saved
     * @return html document
     * @exception HttpPresentationException
     */
    protected void saveDisc(Disc disc)
        throws HttpPresentationException, AssertionDataObjectException,NullPointerException,Exception{
       
        String title = this.getComms().request.getParameter(TITLE_NAME);
        String artist = this.getComms().request.getParameter(ARTIST_NAME);
        String genre = this.getComms().request.getParameter(GENRE_NAME);
            
    if (title.length() == 0 || artist.length() ==0 ||
            genre.length() == 0) {
            throw new Exception();
        }
    
    try {
            disc.setTitle(title);
            disc.setArtist(artist);
            disc.setGenre(genre);
    
           
            if(null != this.getComms().request.getParameter(IS_LIKED)) 
            {
                disc.setLiked(true);
            } else {
                disc.setLiked(false);
            }
      
            disc.setOwner(this.getUser());
            disc.save();
        
        }catch(NullPointerException ex) {
          throw new NullPointerException();
        }
        catch (AssertionDataObjectException ex) {
            throw new AssertionDataObjectException("Error adding disc: read-only table.", ex);
        }
        catch(Exception ex) {
            throw new DiscRackPresentationException("Error adding disc", ex);
        }
    }
}






