/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: Login.java,v 1.1 2004/08/16 09:39:21 slobodan Exp $
 */

package discRack.presentation.personMgmt;

import discRack.spec.*;
import discRack.presentation.BasePO;
import discRack.presentation.DiscRackPresentationException;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;

/**
 * Login.java handles the login functionality of the DiscRack app.
 *
 * @author
 * @version
 */
public class Login extends BasePO {

    /**
     * Constants
     */
    private static String SUBMIT_NAME = "submit";
    private static String LOGIN_NAME = "login";
    private static String PASSWORD_NAME = "password";
    private static String ERROR_NAME = "ERROR_NAME";

    /**
     * Superclass method override
     */
    public boolean loggedInUserRequired() {
        return false;
    }

    /**
     *  Default event. Just show the page.
     */
    public XMLObject handleDefault()
        throws HttpPresentationException {
        return showPage(null);
    }

    /**
     * Process login data
     *
     * @return wml document
     * @exception HttpPresentationException
     */
    public XMLObject handleLogin()
        throws HttpPresentationException {
        String login = this.getComms().request.getParameter(LOGIN_NAME);
        String password = this.getComms().request.getParameter(PASSWORD_NAME);
        
        
        Person user = null;

/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer when we run discRack_pres )
 * We need  to allow discRack_pres to be functional , to allow users who run discRack_pres response
 * will be default HTML page
 */ 

        try {
            
   user =((PersonGenerator)PersonGeneratorFactory.getPersonGenerator("discRack.business.person.PersonGeneratorImpl")).findPerson(login);
            
            
            
            if(null == user || !user.getPassword().equals(password)) {
                return showPage("Invalid username or password");
                // Show error message that user not found (bad username/password)
            } else {
                this.setUser(user);
                throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
            }
        }catch(NullPointerException ex){
                    
              throw new ClientPageRedirectException(getComms().request.getApplicationPath()+DISC_CATALOG_PAGE);
        
        } catch(DiscRackException ex) {
            this.writeDebugMsg("System error finding user: " + ex.getMessage());
            throw new DiscRackPresentationException("System error finding user", ex);
        }
    }

    /**
     * handle logout event
     *
     * @return html document
     * @exception HttpPresentationException
     */
    public XMLObject handleLogout()
        throws HttpPresentationException {
        this.removeUserFromSession();
        return (ExitHTML)myComms.xmlcFactory.create(ExitHTML.class);
//        return new ExitHTML();
    }

    /**
     * handle throw exception event.
     *
     * @return html document
     * @exception Exception
     */
    public XMLObject handleThrowException()
        throws Exception {
        throw new Exception("This is a test exception thrown from Login.java handleThrowException()");
    }

    /**
     * display page
     *
     * @param errorMsg, the error messages
     * @return html document
     */
    public XMLObject showPage(String errorMsg) {

	LoginHTML page = (LoginHTML)myComms.xmlcFactory.create(LoginHTML.class);

        if(null != errorMsg ||
           null != (errorMsg = this.getSessionData().getAndClearUserMessage())) {
            page.setTextErrorText(errorMsg);
        } else {
            page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
        }

        return page;
    }
}


