/* -----------------------------------------------------------------------------
 * Copyright (c) 2001, Low Kin Onn
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither name of the Scioworks Pte. Ltd. nor the names of its contributors
 * may beused to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -----------------------------------------------------------------------------
 */

package scioworks.imap.business.beans;

import java.io.*;
import java.util.Date;
import java.util.Vector;
import java.util.StringTokenizer;

import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;


import scioworks.imap.spec.*;


import scioworks.imap.business.util.MessagingUtilImpl;

import scioworks.imap.spec.beans.*;
import scioworks.imap.spec.ImapWebException;
import scioworks.imap.spec.ImapWebConstant;

public class IWMessageImpl extends PublicObject implements IWMessage {

  int fLineWidth;
  static final String fLineSeparator = "\n";

  public IWMessageImpl() {
    super();
    fLineWidth = 72;
  }

  public IWMessageImpl(int lineWidth) {
    super();
    this.fLineWidth = lineWidth;
  }

  /**
   * This method will extract the message body from a message
   */
  public String getMessageBody(Message mesg)
      throws ImapWebException {

    try {
      String msg_body = new String();
      Object o = mesg.getContent();

      //Keep on looping until we find a String part
      while (!(o instanceof String)){
        //we assume this is a multipart
        Multipart mp = (Multipart) o;
        o = mp.getBodyPart(0).getContent();
      }

      //if here that means we found a String part
      return (String) o;

    } catch (MessagingException e) {
      throw new ImapWebException("Error getting message body.", e);

    } catch (IOException e) {
      throw new ImapWebException("Error getting message body.", e);
    }

  }

/*
  private void processPart(Multipart forwardMultipart, Part part)
      throws MessagingException, IOException {

    //if this is a multipart, recursively call processPart
    if (part.isMimeType("multipart/*")){
      Multipart mp = ((Multipart)part.getContent());
      int count = mp.getCount();
      for (int i = 0; i < count; i++){
        processPart(forwardMultipart, mp.getBodyPart(i));
     }

    //if this is a rfc822 message, recursively call processPart
    } else if (part.isMimeType("message/rfc822")) {
      processPart(forwardMultipart, (Part) part.getContent());

    //this is a singular part that can be processed
    } else {
      Object obj = part.getContent();
      if (obj instanceof String) {
            MimeBodyPart mbp = new MimeBodyPart();
            mbp.setDisposition(MimeBodyPart.ATTACHMENT);
            mbp.setText((String) obj);
            forwardMultipart.addBodyPart(mbp);
      } else {
        forwardMultipart.addBodyPart((BodyPart) part);
      }
    }
  }
*/


  /**
   * Return the message given by folder and uid. Calling method is responsible
   *  for creating and closing the folder object.
   */
  public Message getMessage(Folder folder, long uid)
      throws ImapWebException {
    try {

      UIDFolder ufolder = (UIDFolder) folder;

      // open the folder for read and write
      folder.open(Folder.READ_WRITE);

      // get the message
      Message msg = ufolder.getMessageByUID(uid);

      // mark the msg as seen
      msg.setFlag(Flags.Flag.SEEN, true);

      return msg;

    } catch (MessagingException e) {
      throw new ImapWebException("Error getting message.", e);
    }
  }

  /**
   * Return a <b>copy</b> of the message given by folder name and uid.
   *  Closing of folder is handled internally by this method.
   */
  public Message getMessage(Store imapStore,
      String folderName, long uid) throws ImapWebException {
    try {

      Folder folder = imapStore.getFolder(folderName);

      UIDFolder ufolder = (UIDFolder) folder;

      // open the folder for read and write
      folder.open(Folder.READ_WRITE);

      // get the message
      MimeMessage msg = new MimeMessage((MimeMessage) ufolder.getMessageByUID(uid));

      // mark the msg as seen
      msg.setFlag(Flags.Flag.SEEN, true);

      folder.close(false);

      return msg;

    } catch (MessagingException e) {
      throw new ImapWebException("Error getting message.", e);
    }
  }

  public Message forwardMessage(Session imapSession, Store imapStore,
      String folderName, long uid, boolean isInline)
      throws ImapWebException {
    try {
      Folder folder = imapStore.getFolder(folderName);

      // open the folder for read
      folder.open(Folder.READ_ONLY);

      UIDFolder ufolder = (UIDFolder) folder;

      // get the message
      Message origMsg = ufolder.getMessageByUID(uid);

      // create a reply object
      MimeMessage forwardMsg = new MimeMessage(imapSession);

      if (origMsg.getSubject().startsWith("Fwd:")) {
        forwardMsg.setSubject(origMsg.getSubject());
      } else {
        forwardMsg.setSubject("Fwd: " + origMsg.getSubject());
      }

      if (isInline) {
        MessagingUtilImpl messagingUtilImpl = new MessagingUtilImpl();
   
        forwardMsg.setText(fLineSeparator +
                         "--- " + origMsg.getFrom()[0].toString() + " wrote:" +
                         fLineSeparator +
                           messagingUtilImpl.formatMsgBody(this.getMessageBody(origMsg), fLineWidth ));
      
      
      } else {
        forwardMsg.setText(fLineSeparator + "--- Forwarded message attached");
      }

      // close the folder
      folder.close(false);

      return forwardMsg;

    } catch (MessagingException e) {
      throw new ImapWebException("Error setting message forward.", e);
    }

  }

  public Message replyMessage(Session imapSession, Store imapStore,
      String folderName, long uid, boolean replyAll, boolean quoteOrig)
      throws ImapWebException {
    try {
      Folder folder = imapStore.getFolder(folderName);

      // open the folder for read
      folder.open(Folder.READ_ONLY);

      UIDFolder ufolder = (UIDFolder) folder;

      // get the message
      Message origMsg = ufolder.getMessageByUID(uid);

      // create a reply object
      Message replyMsg = origMsg.reply(replyAll);

      if (quoteOrig) {
      	 MessagingUtilImpl messagingUtilImpl = new MessagingUtilImpl();
        replyMsg.setText("--- " + origMsg.getFrom()[0].toString() + " wrote:" + fLineSeparator +
                         messagingUtilImpl.formatMsgBody(this.getMessageBody(origMsg), fLineWidth ));
      }

      // close the folder
      folder.close(false);

      return replyMsg;

    } catch (MessagingException e) {
      throw new ImapWebException("Error setting message reply.", e);
    }
  }


  /**
   * Creates and send a message as per the given parameter.
   * Does <b>not</b> handle attachements
   */
  public void sendMessage(Store imapStore, Session imapSession, URLName imapURL,
      String imapDomain,
      String to, String subject, String cc, String bcc, String message,
      Vector attachments, boolean isInline, boolean saveCopy,
      boolean isForwardAttach, String origFolderName, long origUid)
      throws ImapWebException {

    Message msg = new MimeMessage(imapSession);

    InternetAddress[] toAddrs = null;
    InternetAddress[] ccAddrs = null;
    InternetAddress[] bccAddrs = null;

    try {

	    if (!to.equals("")) {
        toAddrs = InternetAddress.parse(to, false);
        msg.setRecipients(Message.RecipientType.TO, toAddrs);
	    } else {
        throw new MessagingException("No \"To\" address specified");
	    }

	    if (!cc.equals("")) {
        ccAddrs = InternetAddress.parse(cc, false);
        msg.setRecipients(Message.RecipientType.CC, ccAddrs);
	    }

      if (!bcc.equals("")) {
        bccAddrs = InternetAddress.parse(bcc, false);
        msg.setRecipients(Message.RecipientType.BCC, bccAddrs);
      }

	    if (!subject.equals("")) {
        msg.setSubject(subject);
	    }

	    msg.setFrom(new InternetAddress(imapURL.getUsername() + "@" +
                                      imapDomain));

      // sent the sent date
      msg.setSentDate(new Date());

      if (message.equals("")) {
        message = " ";
      }

      if ((attachments.size() == 0) && !isForwardAttach) {
        // no attachments
        msg.setText(message);

      } else {

        // multiparts

        // create the Multipart for adding parts to it
        Multipart mp = new MimeMultipart();

        // create and fill the first message part
        MimeBodyPart mbp1 = new MimeBodyPart();
        mbp1.setDisposition(MimeBodyPart.INLINE);
        mbp1.setText(message);
        mp.addBodyPart(mbp1);

        FileRecord fr;
        for (int i=0; i<attachments.size(); i++) {
          fr = (FileRecord) attachments.elementAt(i);
          try {

            MimeBodyPart mbp2 = new MimeBodyPart();
            FileDataSource fds = new FileDataSource(fr.getTmpFilename());
            mbp2.setDataHandler(new DataHandler(fds));
            if (isInline) {
              mbp2.setDisposition(MimeBodyPart.INLINE);
            } else {
              mbp2.setDisposition(MimeBodyPart.ATTACHMENT);
            }
            mbp2.setFileName(fr.getFilename());

            mp.addBodyPart(mbp2);

          } catch (MessagingException e) {
            System.err.println("Error attaching file: "+fr.getTmpFilename());
            e.printStackTrace();
          }
        }

        if (isForwardAttach) {

          try {
            Folder origFolder = imapStore.getFolder(origFolderName);

            // open the folder for read
            origFolder.open(Folder.READ_ONLY);

            UIDFolder ufolder = (UIDFolder) origFolder;

            // create a copy of the message
            MimeMessage origMessage = new MimeMessage((MimeMessage) ufolder.getMessageByUID(origUid));

            // close the folder
            origFolder.close(false);

            BodyPart bodypart = new MimeBodyPart();
            bodypart.setContent(origMessage, "message/rfc822");
            mp.addBodyPart(bodypart);

//            processPart(mp, origMessage);

          } catch (MessagingException e) {
            System.err.println("Error attaching original message.");
            e.printStackTrace();
          }

        }

        // add the Multipart to the message
        msg.setContent(mp);

      }

	    Transport.send(msg);

      if (saveCopy) {
        Folder sentFolder = imapStore.getFolder(ImapWebConstant.singleton().folderSent());
        // check if sent folder exists
        if (!sentFolder.exists()) {
          // not exists, create a new one
          sentFolder.create(Folder.HOLDS_MESSAGES);
        }

        MimeMessage[] messages = new MimeMessage[1];
        messages[0] = (MimeMessage)msg;
        sentFolder.appendMessages(messages);
      }

    } catch (AddressException e) {
      throw new ImapWebException("Error creating email.", e);

    } catch (MessagingException e) {
      throw new ImapWebException("Error creating email.", e);
    }
  }

  public void deleteMessage(Store imapStore,
      String folderName, long[] uidArr) throws ImapWebException {

    try {
      // get the trash folder
      Folder trashFolder = imapStore.getFolder(ImapWebConstant.singleton().folderTrash());

      if (!trashFolder.exists()) {
        // trash folder not exist, create one
        trashFolder.create(Folder.HOLDS_MESSAGES);
      }

      // get the folder that contains the messages to be deleted
      Folder folder = imapStore.getFolder(folderName);

      UIDFolder ufolder = (UIDFolder) folder;

      // open the folder for read and write
      folder.open(Folder.READ_WRITE);

      // get the messages to be deleted
      Message[] messages = ufolder.getMessagesByUID(uidArr);

      // Check if this is the trash folder
      if (!folderName.equals(ImapWebConstant.singleton().folderTrash())) {
        // this not the trash folder, copy the messages to trash
        folder.copyMessages(messages, trashFolder);
      }

      // mark the messages for expunge
      for (int i=0; i<messages.length; i++) {
        messages[i].setFlag(Flags.Flag.DELETED, true);
      }

      // close the folder and expunge the deleted messages
      folder.close(true);

    } catch (MessagingException e) {
      throw new ImapWebException("Error deleting messages.", e);
    }
  }

  public void moveMessage(Store imapStore, String folderName,
      long[] uidArr, String targetFolderName) throws ImapWebException {

    try {
      // get the folder that contains the messages to be deleted
      Folder folder = imapStore.getFolder(folderName);

      UIDFolder ufolder = (UIDFolder) folder;

      // open the folder for read and write
      folder.open(Folder.READ_WRITE);

      // get the messages to be deleted
      Message[] messages = ufolder.getMessagesByUID(uidArr);

      // get the target folder
      Folder targetFolder = imapStore.getFolder(targetFolderName);

      if (!targetFolder.exists()) {
        // target folder not exist, create one
        targetFolder.create(Folder.HOLDS_MESSAGES);
      }

      // copy to the target folder
      folder.copyMessages(messages, targetFolder);

      // mark the messages for expunge
      for (int i=0; i<messages.length; i++) {
        messages[i].setFlag(Flags.Flag.DELETED, true);
      }

      // close the folder and expunge the moved messages
      folder.close(true);

    } catch (MessagingException e) {
      throw new ImapWebException("Error deleting messages.", e);
    }
  }


  public Part getMessagePart(Session imapSession, Store imapStore,
      String folderName, long uid, String partIdList) throws ImapWebException {
    try {

      Folder folder = imapStore.getFolder(folderName);

      UIDFolder ufolder = (UIDFolder) folder;

      // open the folder for read and write
      folder.open(Folder.READ_ONLY);

      // get the message
      MimeMessage msg = new MimeMessage((MimeMessage) ufolder.getMessageByUID(uid));

      folder.close(false);

      StringTokenizer strtok = new StringTokenizer(partIdList, ".");
      String partId;
      Part part = msg;
      while (strtok.hasMoreTokens()) {
        partId = strtok.nextToken();
        if (part.isMimeType("multipart/*")) {
          part = ((Multipart)part.getContent()).getBodyPart(Integer.parseInt(partId));
       } else if (part.isMimeType("message/rfc822")) {
          MimeMessage mm = (MimeMessage) part.getContent();
          part = ((Multipart)mm.getContent()).getBodyPart(Integer.parseInt(partId));
       }

      }

      return part;

    } catch (IOException e) {
      throw new ImapWebException("Error getting part.", e);

    } catch (MessagingException e) {
      throw new ImapWebException("Error getting part.", e);
    }
  }

  public Vector getFolderNames(Store imapStore, String currentFolder)
      throws ImapWebException {

    Vector res = new Vector();
    try {
      Folder[] folders = imapStore.getDefaultFolder().list();
      String fname;
      for (int i=0; i<folders.length; i++) {
        if (folders[i].getName().equals(currentFolder) ||
            folders[i].getName().equals(ImapWebConstant.singleton().folderTrash()) ||
            folders[i].getName().equals(ImapWebConstant.singleton().folderSent()) )
        {
          continue;
        } else {
          res.addElement(folders[i].getName());
        }
      }
    } catch (MessagingException e) {
      throw new ImapWebException("Error getting folder names.", e);
    }
    return res;
  }

  public void createFolder(Store imapStore, String name)
      throws ImapWebException {
    try {
      // get the current folder
      Folder folder = imapStore.getFolder(name);

      if (folder.exists()) {
        // target folder exists
        throw new ImapWebException("A folder with this name already exists.");

      } else {
        // target folder not exist, create one
        folder.create(Folder.HOLDS_MESSAGES);
      }
    } catch (MessagingException e) {
      throw new ImapWebException("Error creating new folder." , e);
    }
  }

  public void renameFolder(Store imapStore, String folderName, String name)
      throws ImapWebException {
    try {
      // get the current folder
      Folder folder = imapStore.getFolder(folderName);

      // get the target folder
      Folder targetFolder = imapStore.getFolder(name);

      if (!targetFolder.exists()) {
        // target folder not exist, create one
        targetFolder.create(Folder.HOLDS_MESSAGES);
      }

      // rename to the target foldr
      folder.renameTo(targetFolder);

    } catch (MessagingException e) {
      throw new ImapWebException("Error creating new folder." , e);
    }
  }

  public void removeFolder(Store imapStore, String folderName)
      throws ImapWebException {
    try {
      // get the current folder
      Folder folder = imapStore.getFolder(folderName);

      if (folder.getMessageCount() == 0) {
        // empty folder, can remove
        folder.delete(false);

      } else {
        // not empty, throw error
        throw new ImapWebException("Folders must be empty before it can be removed.");
      }

    } catch (MessagingException e) {
      throw new ImapWebException("Error creating new folder." , e);
    }
  }

}