package scioworks.imap.business.util;

import scioworks.imap.spec.util.MessagingUtil;
import java.util.*;

public class MessagingUtilImpl implements MessagingUtil{

  static final String fLineSeparator = "\n";

  /**
   * This method format a message body and append a "> "  character
   * in front of a new line. Length of a new line can be specified
   * in the parameter lineWidth
   */
  public static String formatMsgBody(String msgBody, int lineWidth){

    String arrow    = "> ";
    String line     = "";
    Vector lines1      = new Vector();
    Vector lines2      = new Vector();
    Vector returnLines = new Vector();
    StringBuffer replyString = new StringBuffer();

    int startIndex = 0;
    int endIndex ;
    int lineCapacity = lineWidth - arrow.length();

    //first split msgBody into lines delimited by "\n"
    //first we check the msgBody has at least one "\n" character
    //note the lines themselves do not contain a "\n" at the end of the line
    endIndex = msgBody.indexOf(fLineSeparator);
    if (endIndex != -1){
      while (endIndex != -1){
        line = msgBody.substring(startIndex, endIndex);
        lines1.addElement(line);
        startIndex = endIndex + 1;
        endIndex = msgBody.indexOf(fLineSeparator, startIndex);
      }
      if (startIndex < msgBody.length()){
        lines1.addElement(msgBody.substring(startIndex, msgBody.length() -1));
      }
    }
    else{
      lines1.addElement(msgBody);
    }

    // next split the lines by " "
    for(int i=0; i<lines1.size(); i++){
      lines2 = breakLine((String)lines1.elementAt(i), lineCapacity, " ") ;
      for(int j=0 ; j<lines2.size(); j++){
        returnLines.addElement((String)lines2.elementAt(j));
      }
    }

    for (int k=0; k<returnLines.size(); k++){
      replyString.append(arrow).append((String)returnLines.elementAt(k)).append(fLineSeparator);
    }

    return replyString.toString();

  }


  public static Vector breakLine(String inStr, int lineWidth, String delimiter){
    int startIndex = 0;
    int endIndex;
    String leftPart;
    String rightPart;
    Vector lines = new Vector();
    Vector rightStrings = new Vector();
    Vector leftStrings = new Vector();

    //check if inStr is > than lineWidth
    if (inStr.length() > lineWidth){

      endIndex = inStr.lastIndexOf(delimiter);
      if(endIndex != -1){        //there is at least one delimiter in the line
        leftPart = inStr.substring(startIndex, endIndex);
        if (inStr.length()-1 > endIndex){
          rightPart = inStr.substring(endIndex+1, inStr.length()-1);
        }
        else{
          rightPart = "";
        }
        //break the left part further if necessary
        if (leftPart.length()<lineWidth){
          leftStrings.addElement(leftPart);
        }
        else{
          leftStrings = breakLine(leftPart, lineWidth, delimiter);
        }
        //break the right part further if necessary
        if (rightPart.length()<lineWidth){
          rightStrings.addElement(rightPart);
        }
        else{
          rightStrings = breakLine(rightPart, lineWidth);
        }
      }
      else{                     //there is no delimiter in the line
        if(inStr.length()-1 > startIndex){
          rightPart = inStr.substring(startIndex, inStr.length()-1);
        }
        else{
          rightPart = "";
        }

        if (rightPart.length() < lineWidth){
          rightStrings.addElement(rightPart);
        }
        else{
          rightStrings = breakLine(rightPart, lineWidth);
        }
      }

      int i = 0;
      int j;
      for (i=0; i<leftStrings.size(); i++){
        lines.addElement((String)leftStrings.elementAt(i));
      }
      for (j=i; j<rightStrings.size(); j++){
        lines.addElement((String)rightStrings.elementAt(j));
      }
    }
    else{
      lines.addElement(inStr);
    }
    return lines;
  }


  /**
   * This procedure will break a string into equal string of length lineWidth
   */
  public static Vector breakLine(String inStr, int lineWidth){
    Vector lines = new Vector();
    int startIndex = 0;
    int endIndex = lineWidth ;

    while (endIndex < inStr.length()){
      lines.addElement(inStr.substring(startIndex, endIndex));
      startIndex = endIndex + 1;
      endIndex += lineWidth;
    }

    lines.addElement(inStr.substring(startIndex, inStr.length()-1));

    return(lines);
  }

  /**
   * Turns a null into " " so that DOM won't break
   */
  public String formatBodyText(String str) {
    if ((str == null) || (str.length() == 0)) {
      return " ";
    } else {
      return str;
    }
  }

  public String formatSubject(String str) {
    if ((str == null) || (str.length() == 0)) {
      return "[none]";
    } else {
      return str;
    }
  }

}