/* -----------------------------------------------------------------------------
 * Copyright (c) 2001, Low Kin Onn
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither name of the Scioworks Pte. Ltd. nor the names of its contributors
 * may beused to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -----------------------------------------------------------------------------
 */

package scioworks.imap.presentation.imapWeb;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.util.KeywordValueException;
import com.lutris.mime.*;
import java.io.*;

import scioworks.imap.presentation.base.*;
import scioworks.imap.spec.ImapWebException;
import scioworks.imap.spec.ImapWebConstant;

public class FileUpload extends BasePO {

  FileUploadSessionData fFileUploadSessionData = null;

  private String processUpload()
      throws HttpPresentationException, ImapWebException {

    if (fFileUploadSessionData.isExceedFileCount()) {
      return "Maximum file count reached.";
    }

    try {
      String contentType = super.getComms().request.getContentType();

      if (contentType == null) {
        contentType = "";
      }

      ContentHeader contentHdr = new ContentHeader("Content-Type: " + contentType);

      if (!contentHdr.getValue().toLowerCase().startsWith("multipart/")) {
        // not a multipart - do nothing and return
        return " ";
      }

      HttpPresentationInputStream input = super.getComms().request.getInputStream();

      MultipartMimeInput mime = new MultipartMimeInput(input, contentHdr);

      MultipartMimeInputStream in;

      while ((in = mime.nextPart()) != null) {

        String type;

        MimeHeader hdr = in.getHeader("Content-Type");

        if (hdr != null) {
          ContentHeader chdr = new ContentHeader(hdr);
          type = chdr.getValue();
        } else {
          type = "null";
        }

        String contentDisp = null;
        String contentName = null;
        String fileName = null;

        if ((hdr = in.getHeader("Content-Disposition")) != null) {
          ContentHeader chdr = new ContentHeader(hdr);
          contentDisp = chdr.getValue();
          contentName = chdr.getParameter("name");
          fileName = chdr.getParameter("filename").trim();
        }

        if (contentDisp == null) {
          contentDisp = "null";
        }

        if (contentName == null) {
          contentName = "null";
        }

        if (fileName == null) {
          fileName = "null";
        }

        int n, count=0;
        byte[] buffer = new byte[4096];

        if (fileName.equals("null")) {
          return "File not found.";

        } else {

        String actualFilename = fileName.substring(fileName.lastIndexOf(System.getProperty("file.separator"))+1);

        // create write to tmp file
        String tmpFilename = ImapWebConstant.singleton().fileTmpDir() +
                             System.currentTimeMillis() + actualFilename;

        FileOutputStream out = new FileOutputStream(tmpFilename);

        while ((n = in.read(buffer, 0, buffer.length)) > 0) {
          out.write(buffer, 0, n);
          count += n;
        }

        in.close();	  // Close this part (but not whole upload).
        out.close();  // Close output file.

        if (count == 0) {
          // not a valid upload
          return "Invalid file. Make sure the file exists.";
        }

        if (!fFileUploadSessionData.addFile(tmpFilename, actualFilename, count, type)) {
          return "You've exceeded the attachment limit.";
        }

        }

      } // while

      /** @todo : target url */
      return "File attached.";

    } catch (MimeException e) {
      throw new ImapWebException("Error uploading file.", e);

    } catch (FileNotFoundException e) {
      throw new ImapWebException("Error uploading file.", e);

    } catch (IOException e) {
      throw new ImapWebException("Error uploading file.", e);
    }

  }

  private String removeFile(FileUploadSessionData fileUploadSessionData)
      throws HttpPresentationException {

    // get input parameter
    int fFileId = super.getIntParameter(PARAM_fileid);

    String tmpFilename = fileUploadSessionData.getTmpFilename(fFileId);

    if (fileUploadSessionData.removeFile(fFileId)) {

      File file = new File(tmpFilename);
      if (!file.delete()) {
        System.out.println("Tmp file not deleted: " + tmpFilename);
      }

      return "Attachment removed";

    } else {
      return "Error removing attachment";
    }
  }

  public String handleDefault()
      throws HttpPresentationException {

    // get event parameter
    String fEvent  = super.getStringParameter(PARAM_event);

    String msg;

    try {
      // try get the session data
      Object obj = getComms().sessionData.get(FileUploadSessionData.SESSION_KEY);

      if (obj == null) {
        // no session data - first time
        fFileUploadSessionData =
            new FileUploadSessionData(ImapWebConstant.singleton().fileCount(),
                                      ImapWebConstant.singleton().fileTotalSize());
        // set the session data
        getComms().sessionData.set(FileUploadSessionData.SESSION_KEY, fFileUploadSessionData);

      } else {
        // get the existing session data
        fFileUploadSessionData = (FileUploadSessionData) obj;
      }

      if (fEvent.equals(EVENT_removeFile)) {
        msg = removeFile(fFileUploadSessionData);

      } else {
        // this is the default event , as we are processing a multi-part form
        msg = processUpload();
      }


      attachmentHTML page = (attachmentHTML)m_comms.xmlcFactory.create(attachmentHTML.class);

      // Get reference to the row to be modified
      HTMLTableRowElement fileListRow = page.getElementFileListRow();
      Node fileListTable = fileListRow.getParentNode();

      // set the message, if any
      if ((msg.length() == 0) || (msg == null)) {
        page.setTextMessage(" ");
      } else {
        page.setTextMessage(msg);
      }

      // set the max count and max file size allowed
      page.setTextFileMaxCount(Integer.toString(ImapWebConstant.singleton().fileCount()));
     
     
     
      page.setTextFileTotalSize(TextUtil.singleton().verboseFilesize(ImapWebConstant.singleton().fileTotalSize()));

      // get reference to the Remove file link
      HTMLAnchorElement fileRemoveLink = page.getElementFileRemoveLink();
      String currLink = fileRemoveLink.getHref() + "?" +
                        PARAM_event + "=" + EVENT_removeFile + "&" +
                        PARAM_fileid + "=" ;

      StringBuffer attachmentList = new StringBuffer();
      String verboseSize;
      for (int i=0; i<fFileUploadSessionData.getFileCount(); i++) {
        // set the name of the file
        page.setTextFilename(fFileUploadSessionData.getFilename(i));

        // set the size of the file
        verboseSize = TextUtil.singleton().verboseFilesize(fFileUploadSessionData.getFilesize(i));
        page.setTextFilesize(verboseSize);

        // set the remove link
        fileRemoveLink.setHref(currLink + i);

        // add to the attachment list
        attachmentList.append(fFileUploadSessionData.getFilename(i)).append(" ");
        attachmentList.append("(").append(verboseSize).append(");");

        fileListTable.appendChild(fileListRow.cloneNode(true));
      }
      fileListTable.removeChild(fileListRow);


      // prepare to set the attachment list
      if (attachmentList.length() == 0) {
        attachmentList.append("(None)");
      }

      HTMLBodyElement bodyEle = page.getElementPageBody();
      HTMLScriptElement attachScript = (HTMLScriptElement) page.createElement("SCRIPT");
      attachScript.appendChild(page.createTextNode("window.opener.SetAttachment('" +
                                                   attachmentList.toString() +
                                                   "');"));
      bodyEle.appendChild(attachScript);

      return page.toDocument();

    } catch (KeywordValueException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED,
                                 "Problem getting session data from session. " + e.getMessage(),
                                 "", "");

    } catch (ImapWebException e) {
      e.printStackTrace();
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(), "", "");

    }
  }

}