/* -----------------------------------------------------------------------------
 * Copyright (c) 2001, Low Kin Onn
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither name of the Scioworks Pte. Ltd. nor the names of its contributors
 * may beused to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -----------------------------------------------------------------------------
 */

package scioworks.imap.presentation.imapWeb;

import java.util.Vector;

import scioworks.imap.spec.beans.*;


public class FileUploadSessionData implements java.io.Serializable {

  public static String SESSION_KEY = "FileUploadSessionData";

  private Vector fFileList;
  private int    fFileMaxCount;
  private int    fTotalSize;

  public FileUploadSessionData(int fileMaxCount, int totalSize) {
    fFileMaxCount = fileMaxCount;
    fTotalSize = totalSize;
    fFileList = new Vector(fFileMaxCount);
  
  }

  private int getCurrentSize() {
    int size = 0;
    for (int i=0; i<fFileList.size(); i++) {
      size = size + ((FileRecord)fFileList.elementAt(i)).getSize();
    }
    return size;
  }

  public boolean isExceedFileCount() {
    if ((fFileList.size()+1) > fFileMaxCount) {
      return true;
    } else {
      return false;
    }
  }

  private boolean isExceedLimit(int size) {
    if ( ((fFileList.size()+1) > fFileMaxCount) ||
         ((getCurrentSize()+size) > fTotalSize) ) {
      return true;
    } else {
      return false;
    }
  }

  public Vector getFileList() {
    return new Vector(fFileList);
  }

  public boolean addFile(String tmpFilename, String filename, int size,
      String contentType) {
    if (!isExceedLimit(size)) {
       	
    fFileList.addElement(FileRecordFactory.getFileRecord("scioworks.imap.business.beans.FileRecordImpl",tmpFilename,filename,size,contentType));
    
      // ok to add
      return true;
    } else {
      // can't add anymore
      return false;
    }
  }

  public boolean removeFile(int index) {
    if (index >= fFileList.size()) {
      return false;
    } else {
      fFileList.remove(index);
      return true;
    }
  }

  public int getFileCount() {
    return fFileList.size();
  }

  public String getTmpFilename(int index) {
    return ((FileRecord)fFileList.elementAt(index)).getTmpFilename();
  }

  public String getFilename(int index) {
    return ((FileRecord)fFileList.elementAt(index)).getFilename();
  }

  public int getFilesize(int index) {
    return ((FileRecord)fFileList.elementAt(index)).getSize();
  }

  public String toString() {
    StringBuffer buf = new StringBuffer();
    FileRecord fr = null;
    for (int i=0; i<fFileList.size(); i++) {
      fr = (FileRecord)fFileList.elementAt(i);
      buf.append( fr.toString() );
    }
    return buf.toString();
  }
}