/* -----------------------------------------------------------------------------
 * Copyright (c) 2001, Low Kin Onn
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither name of the Scioworks Pte. Ltd. nor the names of its contributors
 * may beused to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -----------------------------------------------------------------------------
 */

package scioworks.imap.presentation.imapWeb;

import java.net.URLEncoder;
import java.util.Vector;

import javax.mail.*;
import javax.activation.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import com.lutris.appserver.server.httpPresentation.*;

import scioworks.imap.presentation.base.*;
import scioworks.imap.presentation.imapWeb.*;
import scioworks.imap.spec.ImapWebConstant;
import scioworks.imap.spec.beans.*;
import scioworks.imap.spec.ImapWebException;

public class folder_list extends BasePO {
  // Override login requirement
  protected boolean isLoginRequired() {
    return false;
  }

  public String handleDefault()
      throws HttpPresentationException {

    folder_listHTML page = (folder_listHTML)m_comms.xmlcFactory.create(folder_listHTML.class);


    try {

      // Get reference to the row to be modified
      HTMLTableRowElement folderListRow = page.getElementFolderListRow();
      Node folderListTable = folderListRow.getParentNode();

      // Get reference to the URL to be modified
      HTMLAnchorElement mailListLink = page.getElementMailListLink();
      /** @todo : complete the link */
      String currLink = mailListLink.getHref() + "?" +
                        PARAM_folder + "=";

      // Get and set inbox folder
  
      Folder folder = super.getImapWebSessionData().getImapStore().getFolder("INBOX");
    
      page.setTextInboxMessageCount(Integer.toString(folder.getMessageCount()));
      page.setTextInboxUnreadMessageCount(Integer.toString(folder.getUnreadMessageCount()));

      // Get and set Sent folder
      folder = super.getImapWebSessionData().getImapStore().getFolder(ImapWebConstant.singleton().folderSent());
      page.setTextSentName(ImapWebConstant.singleton().folderSent());
      page.setTextSentMessageCount(Integer.toString(folder.getMessageCount()));
      page.setTextSentUnreadMessageCount(Integer.toString(folder.getUnreadMessageCount()));
      page.getElementSentLink().setHref(currLink + URLEncoder.encode(ImapWebConstant.singleton().folderSent()));

      // Get and set Trash folder
      folder = super.getImapWebSessionData().getImapStore().getFolder(ImapWebConstant.singleton().folderTrash());
      page.setTextTrashName(ImapWebConstant.singleton().folderTrash());
      page.setTextTrashMessageCount(Integer.toString(folder.getMessageCount()));
      page.setTextTrashUnreadMessageCount(Integer.toString(folder.getUnreadMessageCount()));
      page.getElementTrashLink().setHref(currLink + URLEncoder.encode(ImapWebConstant.singleton().folderTrash()));

      // the rest of the folders
      HTMLElement msgCountEle = page.getElementMessageCount();
      HTMLElement unreadMsgCountEle = page.getElementUnreadMessageCount();

      msgCountEle.removeAttribute("id");
      unreadMsgCountEle.removeAttribute("id");

      Folder[] folders = super.getImapWebSessionData().getImapStore().getDefaultFolder().list();

      for (int i=0; i<folders.length; i++) {

        folder = folders[i];

        if (folder.getName().equalsIgnoreCase("INBOX")) {
          continue;
        } else if (folder.getName().equalsIgnoreCase(ImapWebConstant.singleton().folderSent())) {
          continue;
        } else if (folder.getName().equalsIgnoreCase(ImapWebConstant.singleton().folderTrash())) {
          continue;
        }

        page.setTextFolderName(folder.getName());
        page.setTextMessageCount(Integer.toString(folder.getMessageCount()));
        page.setTextUnreadMessageCount(Integer.toString(folder.getUnreadMessageCount()));

        mailListLink.setHref(currLink + URLEncoder.encode(folder.getName()));

        folderListTable.appendChild(folderListRow.cloneNode(true));
      }
      folderListTable.removeChild(folderListRow);


      // set the drop down list of target folders for Move
     IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
     
      try {
        Vector targetFolders = iwMsg.getFolderNames(super.getImapWebSessionData().getImapStore(),
                                                    "INBOX");
        super.processSelectList(page, page.getElementFolder(),
                                targetFolders, targetFolders, "-1");
      
  
      
      } catch (ImapWebException e) {
        return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(), "", "");
      }

    }catch(NullPointerException ex) {
    /*
     * The response will be default HTML page 
     * We need  to allow imapWeb_pres to be functional
     */ 
     return ((folder_listHTML)m_comms.xmlcFactory.create(folder_listHTML.class)).toDocument();
    
    } catch (MessagingException e) {
      e.printStackTrace();
      return super.showErrorPage("Error getting folders", e.getMessage(), "", "");
    }


    return page.toDocument();
  }
}