/* -----------------------------------------------------------------------------
 * Copyright (c) 2001, Low Kin Onn
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither name of the Scioworks Pte. Ltd. nor the names of its contributors
 * may beused to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -----------------------------------------------------------------------------
 */

package scioworks.imap.presentation.imapWeb;

import java.util.Vector;
import java.io.*;

import javax.mail.*;
import javax.mail.internet.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLCUtil;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.util.KeywordValueException;

import scioworks.imap.presentation.base.*;
import scioworks.imap.presentation.imapWeb.*;
import scioworks.imap.spec.beans.*;
import scioworks.imap.spec.ImapWebException;
import scioworks.imap.spec.ImapWebConstant;

public class mailAction extends BasePO {

  private String send() throws HttpPresentationException {
    // get input parameters
    String fTo      = super.getStringParameter(PARAM_to);
    String fSubject = super.getStringParameter(PARAM_subject);
    String fCc      = super.getStringParameter(PARAM_cc);
    String fBcc     = super.getStringParameter(PARAM_bcc);
    String fMessage = super.getStringParameter(PARAM_message);
    String[] fSaveCopy = super.getStringParameterArray(PARAM_savecopy);
    String fOrigFolder = super.getStringParameter(PARAM_origFolder);
    long   fOrigMsgid  = super.getLongParameter(PARAM_origMsgid);

    boolean saveCopy = (fSaveCopy.length != 0);

    boolean isForwardAttach;
    if (fOrigMsgid != -1) {
      isForwardAttach = true;
    } else {
      isForwardAttach = false;
    }

 IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
 
    try {

      Object obj = super.getComms().sessionData.get(FileUploadSessionData.SESSION_KEY);
      Vector attachments;
      if (obj != null) {
        attachments = ((FileUploadSessionData) obj).getFileList();

      } else {
        // create empty attachment vector
        attachments = new Vector();
      }

      iwMsg.sendMessage(super.getImapWebSessionData().getImapStore(),
                        super.getImapWebSessionData().getImapSession(),
                        super.getImapWebSessionData().getImapURL(),
                        ImapWebConstant.singleton().domain(),
                        fTo, fSubject, fCc, fBcc, fMessage,
                        attachments,
                        true,  // send as inline attachments for now
                        saveCopy,
                        isForwardAttach,
                        fOrigFolder,
                        fOrigMsgid);

      // remove the attachment for the session data
      if (obj != null) {
        // delete the tmp files
        String tmpFilename;
        for (int i=0; i<attachments.size(); i++) {
          tmpFilename = ((FileUploadSessionData)obj).getTmpFilename(i);
          File file = new File(tmpFilename);
          if (!file.delete()) {
            System.err.println("Cannot delete tmp file: "+ tmpFilename);
          }
        }
        super.getComms().sessionData.remove(FileUploadSessionData.SESSION_KEY);
      }

      /** @todo : set target url */
      return showCompletionPage(MSG_OPERATION_COMPLETED,
                                "Your email has been sent.", "", "");
     } catch(NullPointerException ex) {
   /*
    * The response will be default HTML page 
    * We need  to allow imapWeb_pres to be functional
    */
    return super.showCompletionPage(MSG_OPERATION_FAILED,
                                     "You cannot send massage while runing imapWeb_pres! That was a default HTML page",
                                     "", "");
    } catch (ImapWebException e) {
      // error, show message
      return showErrorPage(MSG_OPERATION_FAILED, e.getMessage(), "", "");

    } catch (KeywordValueException e) {
      return showErrorPage(MSG_OPERATION_FAILED, e.getMessage(), "", "");
    }
  }

  private String forward() throws HttpPresentationException {
    // get input parameters
    String fFolder  = super.getStringParameter(PARAM_folder);
    long   fMsgid   = super.getLongParameter(PARAM_msgid);
    String fForwardType  = super.getStringParameter(PARAM_forwardType);

    boolean isForwardInline;
    if (fForwardType.equalsIgnoreCase("INLINE")) {
      isForwardInline = true;
    } else {
      isForwardInline = false;
    }

    boolean quoteOrig = true;

IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
 
    try {
      Message forwardMsg = iwMsg.forwardMessage(super.getImapWebSessionData().getImapSession(),
                                     super.getImapWebSessionData().getImapStore(),
                                     fFolder, fMsgid, isForwardInline);

      composeHTML page = (composeHTML)m_comms.xmlcFactory.create(composeHTML.class);

      // reset the session data
      Object obj = getComms().sessionData.get(FileUploadSessionData.SESSION_KEY);
      if (obj != null) {
        obj = null;
      }

      if (isForwardInline) {
        ((HTMLInputElement)page.getElementOrigFolder()).setValue("-1");
        ((HTMLInputElement)page.getElementOrigMsgid()).setValue("-1");
      } else {
        ((HTMLInputElement)page.getElementOrigFolder()).setValue(fFolder);
        ((HTMLInputElement)page.getElementOrigMsgid()).setValue(Long.toString(fMsgid));
      }

      // set the 'to'
      page.getElementTo().setValue(InternetAddress.toString(forwardMsg.getAllRecipients()));
      // set the 'subject'
      page.getElementSubject().setValue(forwardMsg.getSubject());

      HTMLTextAreaElement messageEle = page.getElementMessage();

 
    
    String quote = iwMsg.getMessageBody(forwardMsg);
    
    

    
    
      if (XMLCUtil.findFirstText(messageEle) != null) {
        XMLCUtil.findFirstText(messageEle).setData(quote);
      } else {
        messageEle.appendChild(page.createTextNode(quote));
      }

      return page.toDocument();

     } catch(NullPointerException ex) {
   /*
    * The response will be default HTML page 
    * We need  to allow imapWeb_pres to be functional
    */
    return super.showCompletionPage(MSG_OPERATION_FAILED,
                                     "You cannot forward massage while runing imapWeb_pres! That was a default HTML page",
                                     "", "");
    } catch (MessagingException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(),
                                 "", "");

    } catch (ImapWebException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(),
                                 "", "");

    } catch (KeywordValueException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED,
                                 "Problem getting session data from session. " + e.getMessage(),
                                 "", "");
    }
  }

  private String reply(boolean replyAll) throws HttpPresentationException {
    // get input parameters
    String fFolder  = super.getStringParameter(PARAM_folder);
    long   fMsgid   = super.getLongParameter(PARAM_msgid);

    boolean quoteOrig = true;

  IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
 
    try {
      Message replyMsg = iwMsg.replyMessage(super.getImapWebSessionData().getImapSession(),
                                     super.getImapWebSessionData().getImapStore(),
                                     fFolder, fMsgid, replyAll, quoteOrig);

      composeHTML page = (composeHTML)m_comms.xmlcFactory.create(composeHTML.class);

      // reset the session data
      Object obj = getComms().sessionData.get(FileUploadSessionData.SESSION_KEY);
      if (obj != null) {
        obj = null;
      }

      // set the 'to'
      page.getElementTo().setValue(InternetAddress.toString(replyMsg.getAllRecipients()));
      // set the 'subject'
      page.getElementSubject().setValue(replyMsg.getSubject());

      // set the message box, if quoteOrig is true
      if (quoteOrig) {
        HTMLTextAreaElement messageEle = page.getElementMessage();


        String quote = iwMsg.getMessageBody(replyMsg);


        if (XMLCUtil.findFirstText(messageEle) != null) {
          XMLCUtil.findFirstText(messageEle).setData(quote);
        } else {
          messageEle.appendChild(page.createTextNode(quote));
        }
      }

      return page.toDocument();
     } catch(NullPointerException ex) {
   /*
    * The response will be default HTML page 
    * We need  to allow imapWeb_pres to be functional
    */
    return super.showCompletionPage(MSG_OPERATION_FAILED,
                                     "You cannot reply massage while runing imapWeb_pres! That was a default HTML page",
                                     "", "");
    
    } catch (MessagingException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(),
                                 "", "");

    } catch (ImapWebException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(),
                                 "", "");

    } catch (KeywordValueException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED,
                                 "Problem getting session data from session. " + e.getMessage(),
                                 "", "");
    }
  }

  private String delete() throws HttpPresentationException {
    // get input parameters
    String fFolder   = super.getStringParameter(PARAM_folder);
    long[] fMsgidArr = super.getLongParameterArray(PARAM_msgid);

    try {
      
   IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
    
      
      iwMsg.deleteMessage(super.getImapWebSessionData().getImapStore(),
                          fFolder, fMsgidArr);

      /** @todo : set target url */
      return super.showCompletionPage(MSG_OPERATION_COMPLETED,
                                     "The selected messages has been deleted.",
                                     "", "");
    } catch(NullPointerException ex) {
   /*
    * The response will be default HTML page 
    * We need  to allow imapWeb_pres to be functional
    */
    return super.showCompletionPage(MSG_OPERATION_FAILED,
                                     "You cannot delete massage while runing imapWeb_pres! That was a default HTML page",
                                     "", "");
    
    } catch (ImapWebException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(),
                                 "", "");
    }

  }

  private String move() throws HttpPresentationException {
    // get input parameters
    String fFolder   = super.getStringParameter(PARAM_folder);
    long[] fMsgidArr = super.getLongParameterArray(PARAM_msgid);
    String fTargetFolder = super.getStringParameter(PARAM_targetFolder);

    try {
 IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
 
      iwMsg.moveMessage(super.getImapWebSessionData().getImapStore(),
                        fFolder, fMsgidArr, fTargetFolder);

      /** @todo : set target url */
      return super.showCompletionPage(MSG_OPERATION_COMPLETED,
                                     "The selected messages has been moved.",
                                     "", "");
     } catch(NullPointerException ex) {
   /*
    * The response will be default HTML page 
    * We need  to allow imapWeb_pres to be functional
    */
    return super.showCompletionPage(MSG_OPERATION_FAILED,
                                     "You cannot move massage while runing imapWeb_pres! That was a default HTML page",
                                     "", "");
    } catch (ImapWebException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(),
                                 "", "");
    }
  }

  private String downloadAttachment() throws HttpPresentationException {
    // get input parameters
    String fFolder   = super.getStringParameter(PARAM_folder);
    long   fMsgid    = super.getLongParameter(PARAM_msgid);
    String fPart     = super.getStringParameter(PARAM_part);

    try {
       IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
   
      Part part = iwMsg.getMessagePart(super.getImapWebSessionData().getImapSession(),
                                       super.getImapWebSessionData().getImapStore(),
                                       fFolder, fMsgid, fPart);

      // get binary stream to retrieve BLOB data
      InputStream is = part.getInputStream();

      // Get the buffer size suggested by JDBC
      byte[] buffer = new byte[1024];
      int length = -1;

      this.getComms().response.setContentType(part.getContentType());
      this.getComms().response.setHeader("Content-disposition",
                                         "filename=" + part.getFileName());

      // Pipe the input stream to output stream
      while ((length = is.read(buffer)) != -1) {
        this.getComms().response.getOutputStream().write(buffer, 0, length);
      }

      // Close input stream
      is.close();

      // Flush the output
      this.getComms().response.flush();

      return "?";

     } catch(NullPointerException ex) {
   /*
    * The response will be default HTML page 
    * We need  to allow imapWeb_pres to be functional
    */
    return super.showCompletionPage(MSG_OPERATION_FAILED,
                                     "You cannot download attachment while runing imapWeb_pres! That was a default HTML page",
                                     "", "");
    } catch (IOException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(),
                                 "", "");
    } catch (MessagingException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(),
                                 "", "");

    } catch (ImapWebException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(),
                                 "", "");
    }
  }

  private String createFolder() throws HttpPresentationException {
    // get input parameter
    String fName   = super.getStringParameter(PARAM_name);

    try {
  IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
  
      iwMsg.createFolder(super.getImapWebSessionData().getImapStore(),
                         fName);

      /** @todo : set target url */
      return super.showCompletionPage(MSG_OPERATION_COMPLETED,
                                     "Folder created.","", "");
   } catch(NullPointerException ex) {
   /*
    * The response will be default HTML page 
    * We need  to allow imapWeb_pres to be functional
    */
    return super.showCompletionPage(MSG_OPERATION_FAILED,
                                     "You cannot create folder while runing imapWeb_pres! That was a default HTML page",
                                     "", "");                                
    } catch (ImapWebException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(),
                                 "", "");
    }
  }

  private String renameFolder() throws HttpPresentationException {
    // get input parameter
    String fFolder = super.getStringParameter(PARAM_folder);
    String fName = super.getStringParameter(PARAM_name);

    try {
  IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
  
      iwMsg.renameFolder(super.getImapWebSessionData().getImapStore(),
                         fFolder, fName);

      /** @todo : set target url */
      return super.showCompletionPage(MSG_OPERATION_COMPLETED,
                                     "Folder renamed.",
                                     "", "");
    } catch (ImapWebException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(),
                                 "", "");
    }
  }

  private String removeFolder() throws HttpPresentationException {
    // get input parameter
    String fFolder = super.getStringParameter(PARAM_folder);

    try {
   IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
    
      iwMsg.removeFolder(super.getImapWebSessionData().getImapStore(),
                         fFolder);

      /** @todo : set target url */
      return super.showCompletionPage(MSG_OPERATION_COMPLETED,
                                     "Folder removed.","", "");
     } catch(NullPointerException ex) {
   /*
    * The response will be default HTML page 
    * We need  to allow imapWeb_pres to be functional
    */
    return super.showCompletionPage(MSG_OPERATION_FAILED,
                                     "You cannot remove folder  while runing imapWeb_pres! That was a default HTML page",
                                     "", "");                             
    } catch (ImapWebException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(),
                                 "", "");
    }
  }

  public String handleDefault()
      throws HttpPresentationException {

    String event = getStringParameter(PARAM_event);
    String returnHTML = null;

    if (event.equals(EVENT_send)) {
      returnHTML = send();

    } else if (event.equals(EVENT_reply)) {
      returnHTML = reply(false);

    } else if (event.equals(EVENT_replyAll)) {
      returnHTML = reply(true);

    } else if (event.equals(EVENT_delete)) {
      returnHTML = delete();

    } else if (event.equals(EVENT_forward)) {
      returnHTML = forward();

    } else if (event.equals(EVENT_downloadAttachment)) {
      returnHTML = downloadAttachment();

    } else if (event.equals(EVENT_move)) {
      returnHTML = move();

    } else if (event.equals(EVENT_createFolder)) {
      returnHTML = createFolder();

    } else if (event.equals(EVENT_renameFolder)) {
      returnHTML = renameFolder();

    } else if (event.equals(EVENT_removeFolder)) {
      returnHTML = removeFolder();

    } else {
      return showErrorPage(ERROR_UNDEFINE_EVENT,
                           "Event " + event + " is undefined",
                           "", "");
    }

    return returnHTML;
  }

}