/* -----------------------------------------------------------------------------
 * Copyright (c) 2001, Low Kin Onn
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither name of the Scioworks Pte. Ltd. nor the names of its contributors
 * may beused to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -----------------------------------------------------------------------------
 */

package scioworks.imap.presentation.imapWeb;

import java.util.Vector;
import java.net.URLEncoder;

import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import com.lutris.appserver.server.httpPresentation.*;

import scioworks.imap.presentation.base.*;
import scioworks.imap.presentation.imapWeb.*;
import scioworks.imap.spec.beans.*;
import scioworks.imap.spec.ImapWebException;
import scioworks.imap.spec.util.*;

public class mail_list extends BasePO {

  public String handleDefault() throws HttpPresentationException {


    // get the input parameter
    String fFolder = super.getStringParameter(PARAM_folder);

    // set default folder to INBOX
    if (fFolder.equals("")) {
      fFolder = "INBOX";
    }

    mail_listHTML page = (mail_listHTML)m_comms.xmlcFactory.create(mail_listHTML.class);

    HTMLInputElement msgidEle   = page.getElementMsgid();
    HTMLElement fromEle    = page.getElementFrom();
    HTMLElement subjectEle = page.getElementSubject();
    HTMLElement dateEle    = page.getElementDate();
    HTMLElement sizeEle    = page.getElementSize();

    // Get reference to the row to be modified
    HTMLTableRowElement emailListRow = page.getElementEmailListRow();
    Node emailListTable = emailListRow.getParentNode();

    // Get reference to the URL to be modified
    HTMLAnchorElement emailLink = page.getElementEmailLink();
    String currLink = emailLink.getHref() + "?" +
                      PARAM_folder + "=" + URLEncoder.encode(fFolder) + "&" +
                      PARAM_msgid + "=";

    // set the drop down list of target folders for Move

    IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
   
    try {
      Vector targetFolders = iwMsg.getFolderNames(super.getImapWebSessionData().getImapStore(),
                                                  fFolder);
      targetFolders.insertElementAt("-- Choose Folder --", 0);
      super.processSelectList(page, page.getElementTargetFolder(),
                              targetFolders, targetFolders, "-1");
    
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run imapWeb_pres )
 * The response will be default HTML page 
 * We need  to allow imapWeb_pres to be functional
 */ 
    } catch(NullPointerException ex) {
    	return ((mail_listHTML)m_comms.xmlcFactory.create(mail_listHTML.class)).toDocument();
    } catch (ImapWebException e) {
      return super.showErrorPage(MSG_OPERATION_FAILED, e.getMessage(), "", "");
    }
    // Remove the id attribute
    msgidEle.removeAttribute("id");
    fromEle.removeAttribute("id");
    subjectEle.removeAttribute("id");
    dateEle.removeAttribute("id");
    sizeEle.removeAttribute("id");
    emailListRow.removeAttribute("id");
    emailLink.removeAttribute("id");

    // Set the folder attribute
    page.getElementFolder().setValue(fFolder);

    try {
      Folder folder = super.getImapWebSessionData().getImapStore().getFolder(fFolder);

      UIDFolder ufolder = (UIDFolder) folder;

      // open the folder for read
      folder.open(Folder.READ_ONLY);

      //get messages
      Message[] messages = folder.getMessages();

      long uid;
      String personalName;

      for (int i=0; i<messages.length; i++) {

        // get the uid
        uid = ufolder.getUID(messages[i]);

        // set the msgid
        msgidEle.setValue(Long.toString(uid));

        // display the from address
        personalName = ((InternetAddress)messages[i].getFrom()[0]).getPersonal();
        if (personalName == null) {
          page.setTextFrom((messages[i].getFrom()[0]).toString());
        } else {
          page.setTextFrom(personalName);
        }

        // display the date
        
  
 
        page.setTextDate(TextUtil.singleton().dateFormat(messages[i].getSentDate()));

        // display the size
        page.setTextSize(TextUtil.singleton().verboseFilesize(messages[i].getSize()));

        // display the subject
        MessagingUtil messagingUtil = MessagingUtilFactory.getMessagingUtil("scioworks.imap.business.util.MessagingUtilImpl");
             
        page.setTextSubject(messagingUtil.formatSubject(messages[i].getSubject()));

        // set the link
        emailLink.setHref(currLink+uid);

        emailListTable.appendChild(emailListRow.cloneNode(true));
      }

      // close the folder
      folder.close(false);

    } catch (MessagingException e) {
      super.showErrorPage("Error getting messages", e.getMessage(), "", "");

    } finally {
      emailListTable.removeChild(emailListRow);
    }

    return page.toDocument();
  }
}