/* -----------------------------------------------------------------------------
 * Copyright (c) 2001, Low Kin Onn
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither name of the Scioworks Pte. Ltd. nor the names of its contributors
 * may beused to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -----------------------------------------------------------------------------
 */

package scioworks.imap.presentation.imapWeb;

import java.io.*;
import java.net.URLEncoder;
import java.util.Vector;

import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import com.lutris.appserver.server.httpPresentation.*;

import scioworks.imap.presentation.base.*;
import scioworks.imap.presentation.imapWeb.*;
import scioworks.imap.spec.beans.*;
import scioworks.imap.spec.ImapWebException;
import scioworks.imap.spec.util.*;

public class view_mail extends BasePO {

  public String handleDefault() throws HttpPresentationException {

    String fFolder = super.getStringParameter(PARAM_folder);
    long   fMsgid  = super.getLongParameter(PARAM_msgid);

    try {

  
  
    IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
 
      Message msg = iwMsg.getMessage(super.getImapWebSessionData().getImapStore(),
                                     fFolder, fMsgid);
    // create the page
      view_mailHTML page = (view_mailHTML)m_comms.xmlcFactory.create(view_mailHTML.class);

     
      // set the hidden field values
      ((HTMLInputElement) page.getElementFolder1()).setValue(fFolder);
      ((HTMLInputElement) page.getElementMsgid1()).setValue(Long.toString(fMsgid));

      ((HTMLInputElement) page.getElementFolder2()).setValue(fFolder);
      ((HTMLInputElement) page.getElementMsgid2()).setValue(Long.toString(fMsgid));

      ((HTMLInputElement) page.getElementFolder3()).setValue(fFolder);
      ((HTMLInputElement) page.getElementMsgid3()).setValue(Long.toString(fMsgid));

      // set the drop down list of target folders for Move
      Vector targetFolders = iwMsg.getFolderNames(super.getImapWebSessionData().getImapStore(),
                                                  fFolder);
      targetFolders.insertElementAt("-- Choose Folder --", 0);
      super.processSelectList(page, page.getElementTargetFolder(),
                              targetFolders, targetFolders, "-1");

      page.getElementDate().removeAttribute("id");
      page.getElementTo().removeAttribute("id");
      page.getElementFrom().removeAttribute("id");
      page.getElementSubject().removeAttribute("id");
      page.getElementEmailText().removeAttribute("id");

      // get reference to the message row
      Node messageRow = page.getElementMessageRow();
      Node messageRowParent = messageRow.getParentNode();

      // set the email header and details
      page.setTextDate(msg.getSentDate().toString());
      page.setTextTo(InternetAddress.toString(msg.getAllRecipients()));
    
      MessagingUtil messagingUtil = MessagingUtilFactory.getMessagingUtil("scioworks.imap.business.util.MessagingUtilImpl");
  
      page.setTextSubject(messagingUtil.formatSubject(msg.getSubject()));
      page.setTextFrom(msg.getFrom()[0].toString());
      page.setTextEmailText(messagingUtil.formatBodyText(iwMsg.getMessageBody(msg)));

      messageRowParent.appendChild(messageRow.cloneNode(true));

      // set the attachments
      HTMLElement filenameEle = page.getElementFilename();
      HTMLAnchorElement downloadLinkEle = page.getElementDownloadLink();

      filenameEle.removeAttribute("id");
      downloadLinkEle.removeAttribute("id");

      // Get reference to the row to be modified
      HTMLTableRowElement attachmentListRow = page.getElementAttachmentListRow();
      Node attachmentListTable = attachmentListRow.getParentNode();

      String currLink = downloadLinkEle.getHref() + "?" +
                        PARAM_event + "=" + EVENT_downloadAttachment + "&" +
                        PARAM_folder + "=" + URLEncoder.encode(fFolder) + "&" +
                        PARAM_msgid + "=" + Long.toString(fMsgid) + "&" +
                        PARAM_part + "=" ;

      if (msg.isMimeType("multipart/*")) {
        Multipart mp = (Multipart)msg.getContent();
        for (int i=1; i<mp.getCount(); i++) {
          processEmail(page, mp.getBodyPart(i), currLink+Integer.toString(i));
        }
        attachmentListTable.removeChild(attachmentListRow);
      } else {
        super.removeElement(page, (Element)attachmentListTable);
      }

      messageRowParent.removeChild(messageRow);

      return page.toDocument();
   
   } catch(NullPointerException ex) {
   /*
    * The response will be default HTML page 
    * We need  to allow imapWeb_pres to be functional
    */ 
     return ((view_mailHTML)m_comms.xmlcFactory.create(view_mailHTML.class)).toDocument();
   
   
    } catch (ImapWebException e) {
      return super.showErrorPage("Error getting message", e.getMessage(), "", "");

    } catch (MessagingException e) {
      e.printStackTrace();
      return super.showErrorPage("Error getting messages", e.getMessage(), "", "");

    } catch (IOException e) {
      e.printStackTrace();
      return super.showErrorPage("Error getting message", e.getMessage(), "", "");
    }
  }

  public void processEmail(view_mailHTML page, Part part, String partId)
      throws HttpPresentationException, MessagingException, IOException, ImapWebException {

    if (part.isMimeType("multipart/*")) {

      Multipart mp = ((Multipart) part.getContent());
      int count = mp.getCount();
      for (int i = 1; i<mp.getCount(); i++) {
        processEmail(page, mp.getBodyPart(i), partId + "." + i);
      }

    } else if (part.isMimeType("message/rfc822")) {


      // get reference to the message row
      Node messageRow = page.getElementMessageRow();
      Node messageRowParent = messageRow.getParentNode();

      Message msg = (Message) part.getContent();
      page.setTextDate(msg.getSentDate().toString());
      page.setTextTo(InternetAddress.toString(msg.getAllRecipients()));
      MessagingUtil messagingUtil = MessagingUtilFactory.getMessagingUtil("scioworks.imap.business.util.MessagingUtilImpl");
  
      page.setTextSubject(messagingUtil.formatSubject(msg.getSubject()));
      page.setTextFrom(msg.getFrom()[0].toString());
     
 IWMessage iwMsg = IWMessageFactory.getIWMessage("scioworks.imap.business.beans.IWMessageImpl");
 
      page.setTextEmailText(messagingUtil.formatBodyText(iwMsg.getMessageBody(msg)));

      messageRowParent.appendChild(messageRow.cloneNode(true));

      if (msg.isMimeType("multipart/*")) {
        Multipart mp = (Multipart)msg.getContent();
        for (int i=1; i<mp.getCount(); i++) {
          processEmail(page, mp.getBodyPart(i), partId + "." + i);
        }
      }

    } else {
      // set the attachments
      HTMLElement filenameEle = page.getElementFilename();
      HTMLAnchorElement downloadLinkEle = page.getElementDownloadLink();

      // Get reference to the row to be modified
      HTMLTableRowElement attachmentListRow = page.getElementAttachmentListRow();
      Node attachmentListTable = attachmentListRow.getParentNode();

     page.setTextFilename(part.getFileName());
     downloadLinkEle.setHref(partId);

     attachmentListTable.appendChild(attachmentListRow.cloneNode(true));

    }
  }

}