/* -----------------------------------------------------------------------------
 * Copyright (c) 2001, Low Kin Onn
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither name of the Scioworks Pte. Ltd. nor the names of its contributors
 * may beused to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -----------------------------------------------------------------------------
 */

package scioworks.imap.spec;

import com.lutris.util.Config;
import com.lutris.util.ConfigException;
import com.lutris.appserver.server.Enhydra;

public class ImapWebConstant {

  private static ImapWebConstant fSingleton;

  private String fDomain;
  private String fSmtpHost;
  private String fImapHost;
  private int    fImapPort;

  private String fFolderSent;
  private String fFolderTrash;

  private String fFileTmpDir;
  private int    fFileCount;
  private int    fFileTotalSize;

  private ImapWebConstant() {

    Config config = Enhydra.getApplication().getConfig();

    try {
      fDomain   = config.getString("ImapWeb.Server.Domain");
      fSmtpHost = config.getString("ImapWeb.Server.SmtpHost");
      fImapHost = config.getString("ImapWeb.Server.ImapHost");
      fImapPort = config.getInt   ("ImapWeb.Server.ImapPort");

      fFolderSent  = config.getString("ImapWeb.Folder.FolderSent");
      fFolderTrash = config.getString("ImapWeb.Folder.FolderTrash");

      fFileTmpDir = config.getString("ImapWeb.Attachment.FileTmpDir");
      fFileCount  = config.getInt   ("ImapWeb.Attachment.FileCount");
      fFileTotalSize = config.getInt("ImapWeb.Attachment.FileTotalSize");

    } catch (ConfigException e) {
      e.printStackTrace();
    }

    // Currently values are hard-coded
    /*
    fDomain   = "scioworks.com";
    fSmtpHost = "gecko.scioworks.com";
    fImapHost = "gecko.scioworks.com";
    fImapPort = 143;

    fFolderSent = "Sent";
    fFolderTrash = "Trash";

    fFileTmpDir    = "c:\\temp\\";
    fFileCount     = 3;
    fFileTotalSize = 10 * 1024 * 1024;  // size in bytes
    */
  }

  public static synchronized ImapWebConstant singleton() {
    if (fSingleton == null) {
      fSingleton = new ImapWebConstant();
    }
    return fSingleton;
  }

  // Attributes retrieval methods goes here

  public String domain() {
    return fDomain;
  }

  public String smtpHost() {
    return fSmtpHost;
  }

  public String imapHost() {
    return fImapHost ;
  }

  public int imapPort() {
    return fImapPort;
  }

  public String folderSent() {
    return fFolderSent;
  }

  public String folderTrash() {
    return fFolderTrash;
  }

  public String fileTmpDir() {
    return fFileTmpDir;
  }

  public int fileCount() {
    return fFileCount;
  }

  public int fileTotalSize() {
    return fFileTotalSize;
  }
}

