/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ErrorHandler.java,v 1.1 2005/12/28 12:01:09 slobodan Exp $
 */

package jtaDiscRack.presentation;

import com.lutris.logging.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.*;
import java.io.*;

/**
 * Class to handle exceptions not caught anywhere else in the framework of
 * our application
 *
 */
public class ErrorHandler implements HttpPresentation {

	/**
	 * This implements the run method in HttpPresentation.
	 *
	 * @param comms HttpPresentationComms
	 * @exception HttpPresentationException
	 */
	public void run(HttpPresentationComms comms)
			throws HttpPresentationException {

		ErrorHTML errorPage = (ErrorHTML) comms.xmlcFactory
				.create(ErrorHTML.class);

		if (null != comms.exception) {
			StringWriter stringWriter = new StringWriter();
			comms.exception.printStackTrace(new PrintWriter(stringWriter));
			LogChannel logChannel = Enhydra.getLogChannel();
			int level = logChannel.getLevel("DEBUG");

			logChannel
					.write(level,
							"jtaDiscRack.presentation.ErrorHandler stack trace = ");
			logChannel.write(level, stringWriter.toString());
			logChannel.write(level,
					"jtaDiscRack.presentation.ErrorHandler caught an exception - "
							+ comms.exception.toString(), comms.exception);
			errorPage.setTextStackTrace(stringWriter.toString());
			String m = comms.exception.getMessage();
			if (null == m)
				m = "NULL message";
			errorPage.setTextErrorMessage(m);
		}

		comms.response.writeHTML(errorPage.toDocument());
	}
}
