/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: JtaDiscRackSessionData.java,v 1.1 2005/12/28 12:01:09 slobodan Exp $
 */

package jtaDiscRack.presentation;

import jtaDiscRack.spec.*;

/**
 * Object that will be held in session data. This should be the only object held
 * there. Methods should be called on this object to set and get data.
 */
public class JtaDiscRackSessionData implements java.io.Serializable {

	/**
	 * Hash key to save session data for the DiscRack app in the Session
	 */
	public static final String SESSION_KEY = "DiscRackSessionData";

	protected String userHandle = null;

	protected String userLogin = null;

	protected String userPassword = null;

	protected String userFirstName = null;

	protected String userLastName = null;

	protected String userMessage = null;

	/**
	 * Sets the person object
	 * 
	 * @param thePerson
	 *            the person object
	 */
	public void setUser(Person thePerson) {
		// this.myUser = thePerson;
		try {
			this.userHandle = thePerson.getHandle();
			this.userFirstName = thePerson.getFirstname();
			this.userLastName = thePerson.getLastname();
			this.userLogin = thePerson.getLogin();
			this.userPassword = thePerson.getPassword();
		} catch (JtaDiscRackException tdre) {
			// user nopt initialized
		}
	}

	/**
	 * Gets the person attribute
	 * 
	 * @return attValue
	 */
	
	public String getUserHandle() {
		return this.userHandle;
	}
	
	public String getUserFirstName() {
		return this.userFirstName;
	}
	
	public String getUserLastName() {
		return this.userLastName;
	}
	
	public String getUserLogin() {
		return this.userLogin;
	}
	
	public String getUserPassword() {
		return this.userPassword;
	}

	/**
	 * Method to remove the current user from the session
	 */
	public void removeUser() {
//		this.myUser = null;
		this.userHandle = null;
		this.userFirstName = null;
		this.userLastName = null;
		this.userLogin = null;
		this.userPassword = null;
	}

	/**
	 * Sets the message
	 * 
	 * @param msg
	 *            the message to be set
	 */
	public void setUserMessage(String msg) {
		this.userMessage = msg;
	}

	/**
	 * Retrieve the most recent user message and then clear it so no other app
	 * tries to use it.
	 */
	public String getAndClearUserMessage() {
		String msg = this.userMessage;
		this.userMessage = null;
		return msg;
	}
}
