/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DiscCatalog.java,v 1.1 2005/12/28 12:01:09 slobodan Exp $
 */

package jtaDiscRack.presentation.discMgmt;

import jtaDiscRack.spec.*;
import jtaDiscRack.presentation.BasePO;
import jtaDiscRack.presentation.JtaDiscRackPresentationException;

import com.lutris.appserver.server.httpPresentation.*;

import org.enhydra.xml.dom.DOMOps;
import org.enhydra.xml.xmlc.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

/**
 * This class populates discs and display on the page.
 * 
 */
public class DiscCatalog extends BasePO {

	/**
	 * Superclass method override
	 */
	public boolean loggedInUserRequired() {
		return true;
	}

	/**
	 * Default event. Just show the page.
	 * 
	 * @return html document
	 * @exception HttpPresentationException
	 */
	public XMLObject handleDefault()
			throws JtaDiscRackPresentationException {
		// Swap in our real run-time JavaScript to repalce the storyboard
		// JavaScript

		DiscCatalogHTML page = (DiscCatalogHTML) myComms.xmlcFactory
				.create(DiscCatalogHTML.class);

		HTMLScriptElement script = ((DiscCatalogScriptHTML) myComms.xmlcFactory
				.create(DiscCatalogScriptHTML.class)).getElementRealScript();
		DOMOps.replaceNode(script, page.getElementDummyScript());

		/*
		 * Catch Null pointer exception ( we canot make a instances of classes
		 * from business layer when we run in presentation mode ) We need
		 * to allow presentation mode to be functional , response will
		 * be default HTML page
		 */

		try {

			Disc[] discList = ((DiscGenerator) DiscGeneratorFactory
					.getDiscGenerator("jtaDiscRack.business.disc.DiscGeneratorImpl"))
					.findDiscsForPerson(this.getUserHandle());

			try {
				page.setTextOwner(this.getSessionData().getUserFirstName() + " "
						+ this.getSessionData().getUserLastName() + "'s Discs");
			} catch (Exception ex) {
				writeDebugMsg("Error getting user info: " + ex);
			}

			String errorMsg = this.getSessionData().getAndClearUserMessage();
			if (null == errorMsg) {
				page.getElementErrorText().getParentNode().removeChild(
						page.getElementErrorText());
			} else {
				page.setTextErrorText(errorMsg);
			}

			// Generate the person's Disc list and create the HTML template
			// references
			HTMLTableRowElement templateRow = page.getElementTemplateRow();
			HTMLOptionElement templateOption = page.getElementTemplateOption();
			Node discTable = templateRow.getParentNode();
			Node discSelect = templateOption.getParentNode();

			// Remove ids to prevent duplicates
			templateRow.removeAttribute("id");
			templateOption.removeAttribute("id");

			// Remove id attributes from the cells in the template row
			HTMLElement artistCellTemplate = page.getElementArtist();
			HTMLElement titleCellTemplate = page.getElementTitle();
			HTMLElement genreCellTemplate = page.getElementGenre();
			HTMLElement likeThisDisc = page.getElementLikeThisDisc();

			artistCellTemplate.removeAttribute("id");
			titleCellTemplate.removeAttribute("id");
			genreCellTemplate.removeAttribute("id");
			likeThisDisc.removeAttribute("id");

			// Remove the dummy storyboard text from the prototype HTML
			templateOption.removeChild(templateOption.getFirstChild());

			// Get collection of Discs and loop through collection
			// to add each disc as a row in the table.
			for (int numDiscs = 0; numDiscs < discList.length; numDiscs++) {
				Disc currentDisc = discList[numDiscs];
				// set text of new cells to values from string array
				page.setTextArtist(currentDisc.getArtist());
				page.setTextTitle(currentDisc.getTitle());
				page.setTextGenre(currentDisc.getGenre());

				if (currentDisc.isLiked()) {
					page.setTextLikeThisDisc("Yes");
				} else {
					page.setTextLikeThisDisc("Not");
				}

				// Add a deep clone of the row to the DOM
				Node clonedNode = templateRow.cloneNode(true);
				discTable.appendChild(clonedNode);
				// Alternative way to insert nodes below
				// insertBefore(newNode, oldNode);
				// discTable.insertBefore(clonedNode, templateRow);

				// Now populate the select list
				// This algorithm is obscure because options are not normal
				// HTML elements
				// First populate the option value (the disc database ID).
				// Then append a text child as the option
				// text, which is what is displayed as the text
				// in each row of the select box
				HTMLOptionElement clonedOption = (HTMLOptionElement) templateOption
						.cloneNode(true);
				clonedOption.setValue(currentDisc.getHandle());
				Node optionTextNode = clonedOption.getOwnerDocument()
						.createTextNode(
								currentDisc.getArtist() + ": "
										+ currentDisc.getTitle());
				clonedOption.appendChild(optionTextNode);
				// Do only a shallow copy of the option as we don't want the
				// text child
				// of the node option
				discSelect.appendChild(clonedOption);
				// Alternative way to insert nodes below
				// insertBefore(newNode, oldNode);
				// discSelect.insertBefore(clonedOption, templateOption);
			}
			
			// Finally remove the template row and template select option
			// from the page
			templateRow.getParentNode().removeChild(templateRow);
			templateOption.getParentNode().removeChild(templateOption);

		} catch (NullPointerException ex) {
			page.setTextErrorText("This is a default HTML page");
		} catch (Exception ex) {
			writeDebugMsg("Error populating disc catalog: " + ex);
			throw new JtaDiscRackPresentationException(
					"Error getting discs for user: ", ex);
		}

		// write out HTML
		return page;
	}
}
