package phoneList.business;

import java.util.Vector;
import java.sql.SQLException;
import phoneList.data.*;
import org.enhydra.dods.DODS;
import com.lutris.appserver.server.Enhydra;
import com.lutris.appserver.server.sql.DatabaseManager;
import com.lutris.appserver.server.sql.DBConnection;
import com.lutris.appserver.server.sql.DBQuery;
import com.lutris.appserver.server.sql.DBTransaction;
import com.lutris.appserver.server.sql.ObjectId;

import phoneList.spec.*;
/**
 * PhoneList defines a set of static methods that are used
 * to query and update phone information in the database.
 */
public class PhoneListImpl implements PhoneList {

    /**
     * Queries the database for all people/phones.
     *
     * @return Vector[0] contains the people's first names.
     *   Vector[1] contains the people's last names.
     *   Vector[2] contains the phone numbers.
     *   Vector[3] contains the people's identifiers.
     * @exception Exception all exceptions are thrown whenever an
     *   error occurs.
     */
    public Vector[] getLists() throws Exception {
        Vector[] results = new Vector[4];
        results[0] = new Vector();
        results[1] = new Vector();
        results[2] = new Vector();
        results[3] = new Vector();
 
        PersonQuery personQuery = new PersonQuery();
        PersonDO[] personDOs = personQuery.getDOArray();
        PersonDO   personX =null;
        if (personDOs.length>0){
	        for (int i=0 ; i<personDOs.length; i++){
			      results[0].addElement(personDOs[i].getFirstName());
			      results[1].addElement(personDOs[i].getLastName());
			      results[2].addElement(personDOs[i].getPhoneNumber());
			      results[3].addElement("" + personDOs[i].get_Handle());
	        }
	 	 }
        return results;
    }

    /**
     * Creates a new person and adds him/her to the database.
     * Demonstrates how to use a
     * transaction to insert data into the database.
     * 
     * @param firstName the person's first name.
     * @param lastName the person's last name.
     * @param phoneNumber the person's phone number.
     *
     * @exception Exception all exceptions are thrown.  A more
     *   robust application would handle them.....
     */
    public void addPerson(String firstName,
                                 String lastName,
                                 String phoneNumber)
    throws Exception {
        PersonDO newGuy = PersonDO.createVirgin();
        newGuy.setFirstName(firstName);
        newGuy.setLastName(lastName);
        newGuy.setPhoneNumber(phoneNumber);
        newGuy.save();
        newGuy.commit();
  }

    /**
     * Deletes a person from the database.  Demonstrates how to use a
     * transaction to delete data in the database.
     * 
     * @param id the person's id
     *
     * @exception Exception all exceptions are thrown.  A more
     *   robust application would handle them.....
     */
    public void deletePerson(String id) throws Exception {
       PersonDO personDO = getPersonDOById(id);
       if (personDO == null) {
            return;
       }
            // Multiple data objects could be update, deleted, inserted
            // here as a single transaction.....
			   personDO.delete();

  }

    /**
     * Modifies a person's profile.  Demonstrates how to use a
     * transaction to update data in the database.
     * 
     * @param id the person's id
     * @param newFirstName the person's new first name.
     * @param newLastName the person's new last name.
     * @param newPhoneNumber the person's new phone number.
     *
     * @exception Exception all exceptions are thrown.  A more
     *   robust application would handle them.....
     */
    public void modifyPerson(String id,
                                    String newFirstName,
                                    String newLastName,
                                    String newPhoneNumber) 
        throws Exception {
        PersonDO personDO = getPersonDOById(id);
        if (personDO == null) return;
        personDO.setFirstName(newFirstName);
        personDO.setLastName(newLastName);
        personDO.setPhoneNumber(newPhoneNumber);
            // Multiple data objects could be update, deleted, inserted
            // here as a single transaction.....
		  personDO.save();
		  personDO.commit();

    }

    /**
     * Queries a person from the database.
     *
     * @param id the person's id.
     * @exception Exception all exceptions are thrown.  A more
     *   robust applications would handle them.....
     */
    public Person getById(String id) throws Exception {
        PersonDO personDO = getPersonDOById(id);
        if (personDO == null) {
            return null;
        }
        return new PersonImpl(personDO);
    }
    
    /**
     * Queries a person from the database.
     *
     * @param id the person's id.
     * @exception Exception all exceptions are thrown.  A more
     *   robust applications would handle them.....
     */
     
     public static String[] getPersonData(String id) throws Exception {
        PersonDO personDO = getPersonDOById(id);
        if (personDO == null) {
            return null;
        }
        String personData[]=new String[3];
        personData[0]=personDO.getFirstName();
        personData[1]=personDO.getLastName();
        personData[2]=personDO.getPhoneNumber();
        return personData;
    }
    
    /**
     * Queries a person from the database.
     *
     * @param id the person's id.
     * @exception Exception all exceptions are thrown.  A more
     *   robust applications would handle them.....
     */
    private static PersonDO getPersonDOById(String id) throws Exception {
        ObjectId oid = new ObjectId(id);
        PersonQuery personQuery = new PersonQuery();
        personQuery.setQueryOId(oid);
		  PersonDO personX = personQuery.getNextDO();
        return personX ;
    }
    
} 

