package  phoneList.business;

import phoneList.spec.*;

import java.util.Vector;
import phoneList.business.axis.*;

/**
 * PhoneList defines a set of static methods that are used
 * to query and update phone information in the database.
 */
public class PhoneListImpl implements phoneList.spec.PhoneList {

  public static boolean configured = false;
  private static String server = "localhost";
  private static String port = "9000";
  private static String HTTP = "http://";
  private static char COLON = ':';
  private static String PHONEBOOK = "/phoneBook/PhoneBook";
  private static PhoneListService service = new PhoneListServiceLocator();
  private static phoneList.business.axis.PhoneList phoneBook;


  /**
   * Get current server
   * @return server
   */

  public String getServer(){
    return server;
  }

  /**
   * Get current port
   * @return port
   */

  public String getPort(){
   return port;
 }

  /**
  * Sets Server and port and
  * generates a new phoneList.business.axis.PhoneList accordingly
  *
  * @param srv the server name.
  * @param prt the server port.
  */

 public void setServerUrl(String srv, String prt) throws Exception {
   phoneBook = service.getPhoneBook(new java.net.URL(HTTP + srv +
       COLON + prt + PHONEBOOK));
   server = srv;
   port = prt;
 }


  /**
   * Queries the database for all people/phones.
   *
   * @return Vector[0] contains the people's first names.
   *   Vector[1] contains the people's last names.
   *   Vector[2] contains the phone numbers.
   *   Vector[3] contains the people's identifiers.
   * @exception Exception all exceptions are thrown whenever an
   *   error occurs.
   */

  public Vector[] getLists() throws Exception {
    Vector[] results = new Vector[4];
    Object[][] resultStrings = phoneBook.getLists();
    if (resultStrings.length==1){
	    //case of jonas - axis implementation
	    for (int j = 0; j < resultStrings[0].length; j++) {
	   	      results[j] = new Vector();
		      Vector temp = (Vector)resultStrings[0][j];
		      for (int k = 0; k < temp.size(); k++) {
		      	if (temp.elementAt(k)!=null)
		      		results[j].add( (String) temp.elementAt(k));
		      }
	    }
    } else {
	    for (int i = 0; i < resultStrings.length; i++) {
	      results[i] = new Vector();
	      for (int j = 0; j < resultStrings[i].length; j++) {
	        if (resultStrings[i][j]!=null)
	        	results[i].add( (String) resultStrings[i][j]);
			  }
	    }
    }
    return results;
  }

  /**
   * Creates a new person and adds him/her to the database.
   * Demonstrates how to use a
   * transaction to insert data into the database.
   *
   * @param firstName the person's first name.
   * @param lastName the person's last name.
   * @param phoneNumber the person's phone number.
   *
   * @exception Exception all exceptions are thrown.  A more
   *   robust application would handle them.....
   */

  public void addPerson(String firstName, String lastName,
                               String phoneNumber) throws Exception {
    phoneBook.addPerson(firstName, lastName, phoneNumber);
  }

  /**
   * Deletes a person from the database.  Demonstrates how to use a
   * transaction to delete data in the database.
   *
   * @param id the person's id
   *
   * @exception Exception all exceptions are thrown.  A more
   *   robust application would handle them.....
   */
  public void deletePerson(String id) throws Exception {
    phoneBook.deletePerson(id);
  }

  /**
   * Modifies a person's profile.  Demonstrates how to use a
   * transaction to update data in the database.
   *
   * @param id the person's id
   * @param newFirstName the person's new first name.
   * @param newLastName the person's new last name.
   * @param newPhoneNumber the person's new phone number.
   *
   * @exception Exception all exceptions are thrown.  A more
   *   robust application would handle them.....
   */
  public void modifyPerson(String id, String newFirstName,
                                  String newLastName,
                                  String newPhoneNumber) throws Exception {
    phoneBook.modifyPerson(id, newFirstName, newLastName, newPhoneNumber);
  }

  /**
   * Queries a person from the database.
   *
   * @param id the person's id.
   * @exception Exception all exceptions are thrown.  A more
   *   robust applications would handle them.....
   */
  public Person getById(String id) throws Exception {
    String[] personData = phoneBook.getPersonData(id);
    PersonImpl person = new PersonImpl(personData[0], personData[1], personData[2], id);
    return person;

  }
  
  /**
   *Is configarated
   */
  public boolean isConfigured(){
  	return configured;
  }
	
 /**
   *Sets the parameter configured to true
   */
  public void setConfigured(){
  	configured = true;
  }

}