/**
 * Title:        EJOSA - Enterprise Java Open Source Architecture
 *               My Piggy Bank Example
 * Description:  Business Object
 * Copyright:    Copyright (c) 2003 by B. Lofi Dewanto, T. Menzel
 * Company:      University of Muenster, HTWK Leipzig
 * @author  B. Lofi Dewanto, T. Menzel
 * @version 1.2
 */
package net.sourceforge.ejosa.piggybank.business.entity;

//import java.util.*;

import net.sourceforge.ejosa.piggybank.business.*;
import net.sourceforge.ejosa.piggybank.data.coin.*;
import net.sourceforge.ejosa.piggybank.spec.business.*;
import net.sourceforge.ejosa.piggybank.spec.*;


/**
 * The state/value object/data transfer object class.
 *
 * @author  B. Lofi Dewanto, T. Menzel
 * @version 1.2
 */
public class CoinImpl implements Coin {
    // Data object
    protected CoinDO myDO = null;
    
    // Primary Key
    public String id;

    // Object state
    public String date;
    public String amount;
    public String currency;

    /**
     * Constructor.
     */
    public CoinImpl() throws CoinException {
        try {
          this.myDO = CoinDO.createVirgin(); 
        } catch (Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinBusinessException("Exception in Coin():"+ ex.printStackTrace());
        } 
    }
    
    /**
     * Constructor.
     */
    public CoinImpl(CoinDO theCoin) throws CoinException {
        try {
          this.myDO = theCoin;
        } catch (Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in Coin(CoinDO):"+ ex.printStackTrace());
        }
    }
    
    /**
     * Constructor.
     */
    public CoinImpl(String id) throws CoinException {
        try {
          this.id = id;
        } catch (Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in Coin(String):"+ ex.printStackTrace());
        }
    }
    
    public String getHandle() throws CoinException {
        try {
          return this.myDO.getHandle();
        } catch (Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in getHandle():"+ ex.printStackTrace());
            return null;
        }
    }

    /**
     * Gets the Id of the coin.
     *
     * @return    the Id of the coin.
     * @exception CoinException.
     */
    public String getId() throws CoinException {
        try {
          id = myDO.getID();   
          return id;
        } catch (Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in getId():"+ ex.printStackTrace());
            return null;
        }
    }

    /**
     * Sets the Id of the coin.
     *
     * @param     the Id to be changed.
     * @exception CoinException.
     */
    public void setId(String id) throws CoinException {
        try {
          this.id = id;
          this.myDO.setID(id);
        } catch (Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in setId(String):"+ ex.printStackTrace());
        }
    }

    /**
     * Gets the date of insertion.
     *
     * @return    the date of insertion.
     * @exception CoinException.
     */
    public String getDate() throws CoinException {
        try {
          date = myDO.getDATE(); 
          return date;
        } catch (Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in getDate():"+ ex.printStackTrace());
            return null;
        }
    }

    /**
     * Sets the date of insertion.
     *
     * @param     the date to be changed.
     * @exception CoinException.
     */
    public void setDate(String inputDate) throws CoinException {
        try {
          date = inputDate;
          this.myDO.setDATE(inputDate);
        } catch (Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in setDate(String):"+ ex.printStackTrace());
        }
    }

    /**
     * Gets the amount of the coin.
     *
     * @return    the amount of the coin.
     * @exception CoinException.
     */
    public String getAmount() throws CoinException {
        try {
          amount = myDO.getAMOUNT(); 
          return amount;
        } catch (Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in getAmount():"+ ex.printStackTrace());
            return null;
        }
    }

    /**
     * Sets the amount of the coin.
     *
     * @param     the amount to be changed.
     * @exception CoinException.
     */
    public void setAmount(String amount) throws CoinException {
        try {
          this.amount = amount;
          this.myDO.setAMOUNT(amount);
        } catch (Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in setAmount(String):"+ ex.printStackTrace());
        }
    }

    /**
     * Gets the currency of the coin.
     *
     * @return    the currency of the coin.
     * @exception CoinException.
     */
    public String getCurrency() throws CoinException {
        try {
          currency = myDO.getCURENCY(); 
          return currency;
        } catch (Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in getCurrency():"+ ex.printStackTrace());
            return null;
        }
    }

    /**
     * Sets the currency of the coin.
     *
     * @param     the currency to be changed.
     * @exception CoinException.
     */
    public void setCurrency(String currency) throws CoinException {
        try {
          this.currency = currency;
          this.myDO.setCURENCY(currency);
        } catch (Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in setCurrency(String):"+ ex.printStackTrace());
        }
    }
    
    /**
     * Commits all changes to the database.
     *
     * @exception CoinException if an error occurs
     *   retrieving data (usually due to an underlying data layer
     *   error).
     */
    public void save() throws CoinException {
        try {
            this.myDO.commit();
        } catch(Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in save():"+ ex.printStackTrace());
        }
    }
    
    /**
     * Deletes the disc from the database.
     *
     * @exception CoinException if an error occurs
     *   deleting data (usually due to an underlying data layer
     *   error).
     */
    public void delete() throws CoinException {
        try {
            this.myDO.delete();
        } catch(Exception ex) {
            System.out.println("Exception: " + ex);
            //throw new CoinException("Exception in delete():"+ ex.printStackTrace());
        }
    }
}