/**
 * Title:        OpenUSS - Open Source University Support System
 *               My Piggy Bank Example
 * Description:  Enhydra Presentation Object
 * Copyright:    Copyright (c) 2003 by B. Lofi Dewanto, T. Menzel
 * Company:      University of Muenster, HTWK Leipzig
 * @author  B. Lofi Dewanto, T. Menzel
 * @version 1.1
 */
package net.sourceforge.ejosa.piggybank.presentation.enhydra;

import com.lutris.appserver.server.httpPresentation.*;

//import com.lutris.xml.xmlc.*;

import java.io.*;

import java.text.*;

import java.util.*;

import net.sourceforge.ejosa.piggybank.spec.*;
import net.sourceforge.ejosa.piggybank.spec.business.*;
import net.sourceforge.ejosa.piggybank.spec.system.*;

import org.openuss.presentation.enhydra.framework.*;

import org.openuss.utility.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;


/**
 * Save Page
 *
 * @author  B. Lofi Dewanto, T. Menzel
 * @version 1.1
 */
public class SavePage extends FoundationPO {
    // Form parameter
    protected String amount = null;

    // Page
    private SaveHTML mPage;

    /**
     * Default event. Just show the page.
     */
    public String handleDefault() throws HttpPresentationException {
        // Create the page
        mPage = (SaveHTML) this.getComms().xmlcFactory.create(SaveHTML.class);

        // Look for an error
        String errorMsg = this.getSessionData().getAndClearUserMessage();

        if (errorMsg == null) {
            // No error
            mPage.getElementTStatusbar().getParentNode()
                 .removeChild(mPage.getElementTStatusbar());
        } else {
            // Error, put it in the statusbar
            mPage.setTextTStatusbar(errorMsg);
        }

        // Work with the page
        return showLayout(mPage);
    }

    /**
     * Superclass method override.
     */
    public boolean loggedInUserRequired() {
        // This page doesn't need a login
        return false;
    }

    /**
     * Show the page, override the method.
     */
    public String showLayout(Object page) throws BasePOException {
        // Format the statusbar and treebar
        super.showLayout(mPage);


        // Save
        showSave();

        // Return the result
        return mPage.toDocument();
    }

    /**
     * Save.
     */
    private void showSave() throws FoundationPOException {
        // Get the form data
        // -----------------
        try {
            amount = "0";

            if (this.getComms().request.getParameter("amount") != null) {
                try {
                    int temp = Integer.parseInt(
                                       this.getComms().request.getParameter(
                                               "amount"));
                    amount = "" + temp;
                } catch (NumberFormatException nfe) {
                    //System.out.println("Error: NumberFormatException: " + nfe);
                }
            }
        } catch (Exception ex) {
            System.out.println("Form Reader Error: " + ex);
        }

        // Business access
        // !!!Only access to the specification!!!
        try {
            // Get the workflow object
            CoinManager coinManager = CoinManagerFactory.createCoinManager(
                                              "net.sourceforge.ejosa.piggybank.business.session.CoinManagerImpl");

            //System.out.println("Coin: create a coin manager!");
            // Create a value object / state object / data transfer object
            Coin coin = CoinFactory.createCoin(
                                "net.sourceforge.ejosa.piggybank.business.entity.CoinImpl");

            String now = DateFormat.getDateInstance(DateFormat.SHORT, 
                                                    Locale.ENGLISH)
                                   .format(new Date());

            coin.setDate(now);
            coin.setAmount(amount);
            coin.setCurrency("Dollar");

            coinManager.createCoin(coin);

            //System.out.println("Coin: create it!");
        } catch (Exception ex) {
            System.out.println("EJB Error: " + ex);
            ex.printStackTrace();
        }


        // Show...
        mPage.setTextVAmount(amount);
    }
}