/**
 * Title:        OpenUSS - Open Source University Support System
 *               My Piggy Bank Example
 * Description:  Enhydra Presentation Object
 * Copyright:    Copyright (c) 2003 by B. Lofi Dewanto, T. Menzel
 * Company:      University of Muenster, HTWK Leipzig
 * @author  B. Lofi Dewanto, T. Menzel
 * @version 1.1
 */
package net.sourceforge.ejosa.piggybank.presentation.enhydra;

import com.lutris.appserver.server.httpPresentation.*;

//import com.lutris.xml.xmlc.*;

import java.io.*;

import java.util.*;

import net.sourceforge.ejosa.piggybank.spec.*;
import net.sourceforge.ejosa.piggybank.spec.business.*;
import net.sourceforge.ejosa.piggybank.spec.system.*;

import org.openuss.presentation.enhydra.framework.*;

import org.openuss.utility.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;


/**
 * Smash Page
 *
 * @author  B. Lofi Dewanto, T. Menzel
 * @version 1.1
 */
public class SmashPage extends FoundationPO {
    // Page
    private SmashHTML mPage;

    /**
     * Default event. Just show the page.
     */
    public String handleDefault() throws HttpPresentationException {
        // Create the page
        mPage = (SmashHTML) this.getComms().xmlcFactory.create(SmashHTML.class);

        // Look for an error
        String errorMsg = this.getSessionData().getAndClearUserMessage();

        if (errorMsg == null) {
            // No error
            mPage.getElementTStatusbar().getParentNode()
                 .removeChild(mPage.getElementTStatusbar());
        } else {
            // Error, put it in the statusbar
            mPage.setTextTStatusbar(errorMsg);
        }

        // Work with the page
        return showLayout(mPage);
    }

    /**
     * Superclass method override.
     */
    public boolean loggedInUserRequired() {
        // This page doesn't need a login
        return false;
    }

    /**
     * Show the page, override the method.
     */
    public String showLayout(Object page) throws BasePOException {
        // Format the statusbar and treebar
        super.showLayout(mPage);


        // Show something
        showSmash();

        // Return the result
        return mPage.toDocument();
    }

    /**
     * Shows smash.
     */
    private void showSmash() throws FoundationPOException {
        int cur = 0;
        int sum = 0;

        // This method shows something
        HTMLTableRowElement templateRow = mPage.getElementTemplateRow();
        HTMLElement entryTemplate = mPage.getElementTEntry();
        HTMLElement dateTemplate = mPage.getElementTDate();
        HTMLElement amountTemplate = mPage.getElementTAmount();
        HTMLElement balanceTemplate = mPage.getElementTBalance();
        HTMLElement currencyTemplate = mPage.getElementTCurrency();

        templateRow.removeAttribute("id");
        entryTemplate.removeAttribute("id");
        dateTemplate.removeAttribute("id");
        amountTemplate.removeAttribute("id");
        balanceTemplate.removeAttribute("id");
        currencyTemplate.removeAttribute("id");

        Node table = templateRow.getParentNode();

        // Business access
        // !!!Only access to the specification!!!
        try {
            // Get the workflow object
            CoinManager coinManager = CoinManagerFactory.createCoinManager(
                                              "net.sourceforge.ejosa.piggybank.business.session.CoinManagerImpl");

            //System.out.println("Coin: create a coin helper with the home interface!");
            Vector coinList = coinManager.findAllCoins();
            Coin fc;
            for (int i = 0; i < coinList.size(); i++) {
                fc = (Coin) coinList.elementAt(i);
                //System.out.println("Coin number: " + i + " with coin amount " + fc.getAmount());
                mPage.setTextVEntry("" + (i + 1));
                mPage.setTextVDate(fc.getDate());
                cur = Integer.parseInt(fc.getAmount());
                sum = sum + cur;
                mPage.setTextVAmount("" + cur);
                mPage.setTextVBalance("" + sum);
                mPage.setTextVCurrency("Dollar");
                table.appendChild(templateRow.cloneNode(true));

                coinManager.removeCoin(fc.getId());
            }

            //System.out.println(coinList.size() + " coins found in the piggy bank!");
        } catch (Exception ex) {
            System.out.println("EJB Error: " + ex);
            ex.printStackTrace();
        }

        table.removeChild(templateRow);

        mPage.setTextVCash("" + sum);
    }
}