/**
 * Title:        OpenUSS - Open Source University Support System
 * Description:  Page Designer Factory for Presentation
 * Copyright:    Copyright (c) B. Lofi Dewanto
 * Company:      University of Muenster
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
package org.openuss.presentation.enhydra.framework.designer;

import com.lutris.appserver.server.httpPresentation.*;

import java.lang.reflect.*;

import java.util.*;

import org.enhydra.xml.xmlc.html.*;


/**
 * Page Designer Factory for the presentation objects in Enhydra.
 *
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
public class PageDesignerFactory {
    // Static variables
    private static String DESIGN_SUFFIX = "Designer";

    /**
     * Design the page with its respective designer.
     */
    public static void design(Object page, String fullClassName, Object po)
                       throws Exception {
        // The name of the designer class is
        // Page Name Class + Designer
        // Example:
        // - Page name        : LoginApplicationDetailOfAssistant.java
        // - Designer name    : LoginApplicationDetailOfAssistantDesigner.java
        String designFullClassName = new String(fullClassName.concat(
                                                        DESIGN_SUFFIX));

        // Create the page and design it
        Class objectClass = Class.forName(designFullClassName);

        // Get the constructor
        Class myPage = Class.forName("java.lang.Object");
        Class myPo = Class.forName("java.lang.Object");
        Class[] parameterTypes = new Class[2];
        parameterTypes[0] = myPage;
        parameterTypes[1] = myPo;

        Constructor objectConstructor = objectClass.getConstructor(
                                                parameterTypes);

        // Create the object with the constructor
        Object[] initArgs = new Object[2];
        initArgs[0] = page;
        initArgs[1] = po;

        BasePageDesigner designer = (BasePageDesigner) objectConstructor.newInstance(
                                            initArgs);

        // Design the page...
        try {
            designer.designPage();
        } catch (Exception ex) {
            // Error on design, just ignore
            System.out.println("Error on design the page: " + designer + 
                               " - " + ex);
        }
    }
}