/**
 * Title:        OpenUSS - Open Source University Support System
 * Description:  Page Translator Factory for Presentation
 * Copyright:    Copyright (c) B. Lofi Dewanto
 * Company:      University of Muenster
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
package org.openuss.presentation.enhydra.framework.multilanguage;

import com.lutris.appserver.server.httpPresentation.*;

import java.lang.reflect.*;

import java.util.*;

import org.enhydra.xml.xmlc.html.*;


/**
 * Page Translator Factory for the presentation objects in Enhydra.
 *
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
public class PageTranslatorFactory {
    // Static variables
    private static String TRANSLATION_SUFFIX = "Translator";

    /**
     * Translate the page with its respective translation.
     */
    public static HTMLObjectImpl createAndTranslate(HttpPresentationComms comms, 
                                                    Locale locale, 
                                                    String fullClassName)
                                             throws Exception {
        // The name of the translation class is
        // Page Name Class + Translator
        // Example:
        // - Page name        : LoginApplicationDetailOfAssistant.java
        // - Translation name : LoginApplicationDetailOfAssistantTranslator.java
        String translationFullClassName = new String(fullClassName.concat(
                                                             TRANSLATION_SUFFIX));

        // Create the page and translate it
        Class objectClass = Class.forName(translationFullClassName);

        // Get the constructor
        Class myHttpPresentationComms = Class.forName(
                                                "com.lutris.appserver.server.httpPresentation.HttpPresentationComms");
        Class myLocale = Class.forName("java.util.Locale");
        Class[] parameterTypes = new Class[2];
        parameterTypes[0] = myHttpPresentationComms;
        parameterTypes[1] = myLocale;

        Constructor objectConstructor = objectClass.getConstructor(
                                                parameterTypes);

        // Create the object with the constructor
        // Translate the page and return it
        Object[] initArgs = new Object[2];
        initArgs[0] = comms;
        initArgs[1] = locale;

        BasePageTranslator translator = (BasePageTranslator) objectConstructor.newInstance(
                                                initArgs);

        try {
            translator.translatePage();
        } catch (Exception ex) {
            // Error on translation, just ignore
            System.out.println("Error on translating the page: " + 
                               translator + " - " + ex);
        }

        return translator.getPage();
    }
}