/**
 * Title:        OpenUSS - Open Source University Support System
 * Description:  Layout Formatter
 * Copyright:    Copyright (c) B. Lofi Dewanto
 * Company:      University of Muenster
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
package org.openuss.presentation.layoutformatter;

import java.lang.reflect.*;


/**
 * The interface for layout formatter factory.
 *
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
public class LayoutFormatterFactory {
    /**
     * Gets the layout formatter.
     *
     * @return    the layout formatter implementation.
     */
    public static LayoutFormatter getLayoutFormatter(Object stateObject, 
                                                     String fullClassNameLayout)
                                              throws Exception {
        // Depends on the stateObject create the layout formatter
        Class objectClass = Class.forName(fullClassNameLayout);

        // Get the constructor
        Class myOwner = Class.forName("java.lang.Object");
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = myOwner;

        Constructor objectConstructor = objectClass.getConstructor(
                                                parameterTypes);

        // Create the object with the constructor
        Object[] initArgs = new Object[1];
        initArgs[0] = stateObject;

        LayoutFormatter layoutFormatter = (LayoutFormatter) objectConstructor.newInstance(
                                                  initArgs);

        return layoutFormatter;
    }
}