/**
 * Title:        EJOSA - Enterprise Java Open Source Architecture
 *               My Piggy Bank Example
 * Description:  Specification Object
 * Copyright:    Copyright (c) 2003 by B. Lofi Dewanto, T. Menzel
 * Company:      University of Muenster, HTWK Leipzig
 * @author  B. Lofi Dewanto, T. Menzel
 * @version 1.2
 */
package net.sourceforge.ejosa.piggybank.spec.business;

import net.sourceforge.ejosa.piggybank.spec.*;


/**
 * The business object interface.
 *
 * @author  B. Lofi Dewanto, T. Menzel
 * @version 1.2
 */
public interface Coin {
    /**
     * Gets the Id of the coin.
     *
     * @return    the Id of the coin.
     * @exception CoinException.
     */
    public String getId() throws CoinException;

    /**
     * Sets the Id of the coin.
     *
     * @param     the Id to be changed.
     * @exception CoinException.
     */
    public void setId(String id) throws CoinException;

    /**
     * Gets the date of insertion.
     *
     * @return    the date of insertion.
     * @exception CoinException.
     */
    public String getDate() throws CoinException;

    /**
     * Sets the date of insertion.
     *
     * @param     the date to be changed.
     * @exception CoinException.
     */
    public void setDate(String inputDate) throws CoinException;

    /**
     * Gets the amount of the coin.
     *
     * @return    the amount of the coin.
     * @exception CoinException.
     */
    public String getAmount() throws CoinException;

    /**
     * Sets the amount of the coin.
     *
     * @param     the amount to be changed.
     * @exception CoinException.
     */
    public void setAmount(String amount) throws CoinException;

    /**
     * Gets the currency of the coin.
     *
     * @return    the currency of the coin.
     * @exception CoinException.
     */
    public String getCurrency() throws CoinException;

    /**
     * Sets the currency of the coin.
     *
     * @param     the currency to be changed.
     * @exception CoinException.
     */
    public void setCurrency(String currency) throws CoinException;
}