/**
 * Title:        OpenUSS - Open Source University Support System
 * Description:  Utility for OpenUSS
 * Copyright:    Copyright (c) B. Lofi Dewanto
 * Company:      University of Muenster
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
package org.openuss.utility;

import java.io.*;

import java.rmi.RemoteException;

import java.sql.*;

import java.util.*;

//import javax.ejb.*;

import javax.naming.*;

import javax.rmi.*;

import javax.sql.*;

//import javax.transaction.UserTransaction;


/**
 * Factory for all connections use in OpenUSS.
 *
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
public class ConnectionFactory {
    // Context
    private static Context initialContext = null;

    /**
     * @return    the connection from the dataSource
     * @param     env. for Jdbc
     * @exception EJBException  Thrown by the method if the dataSource is not found
     *                          in the naming.
     * @exception SQLException may be thrown by dataSource.getConnection()
     */
    public static Connection getConnection(String envJdbc)
                                    throws Exception {//EJBException, SQLException
        // Do we need to create the initial context?
        if (initialContext == null) {
            // Create the initial context
            try {
                initialContext = new InitialContext();
            } catch (Exception e) {
                System.out.println("Error getting initial context from JNDI: " + 
                                   e.toString());
            }
        }

        // Get the datasource
        DataSource dataSource = null;

        try {
            // Finds DataSource from JNDI
            dataSource = (DataSource) initialContext.lookup(envJdbc);

            // Return the result
            return dataSource.getConnection();
        } catch (Exception e) {
            System.out.println("Cannot lookup data source  " + e.toString());
            throw new Exception("Cannot lookup data source ");//javax.ejb.EJBException
        }
    }
}