/**
 * Title:        OpenUSS - Open Source University Support System
 * Description:  Utility for OpenUSS
 * Copyright:    Copyright (c) B. Lofi Dewanto
 * Company:      University of Muenster
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
package org.openuss.utility;

import java.io.*;

import java.rmi.RemoteException;

import java.util.*;

//import javax.ejb.*;

import javax.naming.*;

import javax.rmi.*;

//import javax.transaction.UserTransaction;


/**
 * Factory for all home interfaces use in OpenUSS.
 *
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
public class HomeInterfaceFactory {
    // Context
    private static Context initialContext = null;

    /**
     * Get the home interface.
     */
    public static Object create(Class homeClass, String homeName) {
        // Get JNDI initial context
        // System.out.println("Getting an initial context from JNDI");
        // Check the initial context first
        if (initialContext == null) {
            // No initial context, create
            try {
                initialContext = new InitialContext();
            } catch (Exception e) {
                System.out.println("Error getting initial context from JNDI: " + 
                                   e.toString());
            }
        }

        // Connecting to Home thru JNDI
        // System.out.println("Connecting to the " + homeName);
        Object home = null;

        try {
            home = PortableRemoteObject.narrow(initialContext.lookup(homeName), 
                                               homeClass);
        } catch (Exception e) {
            System.out.println("Error getting home object: " + e.toString());
        }

        // give the home back
        return home;
    }
}