/*
 * Copyright (c) 2001 Matthew Feldt. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided the copyright notice above is
 * retained.
 *
 * THIS SOFTWARE IS PROVIDED ''AS IS'' AND WITHOUT ANY EXPRESSED OR
 * IMPLIED WARRANTIES.
 */

/**
 * DirectoryList.java
 *
 * Java Examples In A Nutshell Copyright (c) 2000 David Flanagan
 * Exercise 3-6:
 * Write a program named Uncompress that uncompresses files and directories
 * compressed by the Compress example in this chapter.
 *
 * @author Matthew Feldt <matthew@feldt.com>
 * @version 1.0, 02/08/2001 22:08
 */

/*
 * Some lines are changed for this application
 */
package org.openuss.utility;

import java.io.*;

import java.util.zip.*;


public class Uncompress {
    /**
     * Use gunzip the string.
     */
    public static void gunzipFile(String from, String to)
                           throws IllegalArgumentException, IOException {
        byte[] buffer = new byte[4096];
        int bytes_read;

        // check input file
        File fin = new File(from);

        if (!fin.exists()) {
            throw new IllegalArgumentException(from + " does not exist.");
        }

        if (!fin.canRead()) {
            throw new IllegalArgumentException(from + " read protected.");
        }

        // GZIPInputStream in = new GZIPInputStream(fileInputStream);
        GZIPInputStream in = new GZIPInputStream(new FileInputStream(from));

        File fout = new File(to);

        if (fout.exists()) {
            // don't overwrite existing files
            throw new IllegalArgumentException("File '" + to + 
                                               "' already exisits.");
        }

        FileOutputStream out = new FileOutputStream(to);

        while ((bytes_read = in.read(buffer)) != -1)

            // write file
            out.write(buffer, 0, bytes_read);

        in.close();
        out.close();
    }

    /**
     * Use unzip the string.
     */
    public static void unzipFile(String from, String to)
                          throws IllegalArgumentException, IOException {
        byte[] buffer = new byte[4096];
        int bytes_read;

        // check input file
        File fin = new File(from);

        if (!fin.exists()) {
            throw new IllegalArgumentException(from + " does not exist.");
        }

        if (!fin.canRead()) {
            throw new IllegalArgumentException(from + " read protected.");
        }

        ZipInputStream in = new ZipInputStream(new FileInputStream(from));
        ZipEntry entry;
        FileOutputStream out;

        while ((entry = in.getNextEntry()) != null) {
            String toName = to + File.separator + entry.getName();
            File fout = new File(toName);

            /*
             * This must be removed
             *
                                                       
            if (fout.exists()) { // don't overwrite existing files and continue
              System.err.println("File '" + toName + "' already exisits.");
              continue;
            }
                                                
            * This must be insert
            */
            if (entry.isDirectory()) {
                // create directory for directory entries
                if (!fout.mkdirs()) {
                    System.err.println("Unable to create directory: " + 
                                       toName);
                }
            } else {
                // write file
                out = new FileOutputStream(toName);

                while ((bytes_read = in.read(buffer)) != -1)
                    out.write(buffer, 0, bytes_read);

                out.close();
            }

            // System.out.println(toName);
        }

        in.close();
    }
}