package poker.business;

import java.util.Vector;
import java.sql.SQLException;
import poker.data.DODS_GameData.*;
import poker.data.DODS_Fortune.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.Logger;
import com.lutris.util.*;
//import dods.builder.sourceGenerators.Query.*;
import com.lutris.dods.builder.generator.query.*;
import poker.spec.*;
/**
 *
 * EnhyDraw!, beta 4, 5/21/99
 *
 * Copyright 1999, Larry Wolcot & Daryl Tempesta
 * ALL rights reserved. Not for commercial use
 * without written permission from both authors.
 *
 * This is the GameList object that does all of the database dirtywork.
 * The GameManager class started off just using a Vector to keep a list
 * of all active PokerGame classes. It's not the most direct way of doing
 * things, but by far the easiest way to illustrate how the DODS Objects
 * can easily be used within a standard application.
 *
 * In the GameManager, I just changed the gameList to extend this object
 * instead of a Vector. Now, we can simply divert some of the Vector's
 * methods and use it more like a standard database adapter. Otherwise,
 * if the useDB flag is set to false, we can just do a pass-through to
 * the Super Object (Vector).
 */
public class GameList extends Vector implements java.io.Serializable {

    private boolean useDB = false;

   /**
    * Public Constructor
    */
    public GameList(int n){
	    super(n);
    }

    /**
     * Private Constructor used by getCopy() method
     */
    private GameList(){
	    super(0);
    }

    /**
     * This method returns a copy of the gameList object.
     */
    protected GameList getCopy(){
      GameList newCopy = new GameList();
      newCopy = this;
      return newCopy;
    }


    /**
     * Here is where we determine how to store our data.. db or memory.
     */
    public void setUseDB(boolean useDB) throws Exception{
      try {
	  this.useDB = useDB;
      } catch(Exception e) {
          throw e;
      }
    }

   /**
    * Send to the Vector() or the DB depending on the useDB flag
    * This is where we hook into the DODS_GameData Object
    */
    public synchronized void addGame(PokerGame thisGame){
      if (useDB == false) {
          super.addElement(thisGame);
      } else {
          try {
              addGameToDB(thisGame);
          } catch (Exception e) {
        //There was a problem adding the game!  bummer..
          }
      }
    }

   /**
    * This is a private method that maps the PokerGame to
    * a GameDataDO Object so it can be inserted into the database.
    */
    public synchronized void addGameToDB(PokerGame newGame) throws Exception {
	//remap the PokerGame Object into a GameDataDO built by DODS.
    GameDataDO nullDO = null;
    GameDataDO newGameDataDO = mapToDO(nullDO, newGame);

    try {
        DBTransaction db =Enhydra.getDatabaseManager().createTransaction();

	    try {

		//Insert the newly mapped object into the DB
		db.insert(newGameDataDO);
		db.commit();
	    } catch (SQLException se) {

		//Something went wrong, so roll it back and pass it on!
		db.rollback();
	        throw se;
	    } finally {

		//This releases all resources allocated in last trans
	 	db.release();
	    }
	} catch (Exception e) {
	    throw e;
        }
    }

    /**
     * This private method takes a PokerGame object and maps it
     * into a DODS GameDataDO object. Ideally, they would be the same
     * but this is used to illustrate adding a DODS objects to
     * a completed application.
     * Hopefully, Jay will finish the "fake persistance" sooner
     * or later, then this will look archaic.
     */
    private synchronized GameDataDO mapToDO(
			GameDataDO newGameDataDO, PokerGame newGame){
	try {
	    newGameDataDO = GameDataDO.createVirgin();
	    newGameDataDO.setName(newGame.getName());
	    newGameDataDO.setPassword(newGame.getPassword());
            newGameDataDO.setEmail(newGame.getEmail());
            newGameDataDO.setCash(newGame.getCash());
            newGameDataDO.setLargestBet(newGame.getLargestBet());
            newGameDataDO.setSmallestBet(newGame.getSmallestBet());
            newGameDataDO.setTotalPlayed(newGame.getTotalPlayed());
            newGameDataDO.setTotalWon(newGame.getTotalWon());
            newGameDataDO.setDeck(newGame.getDeck());
	} catch (Exception e){
          System.out.print(e.getMessage());
          e.printStackTrace();
	   //This is a DODS sourceGenetaror exception
	}
	return newGameDataDO;
    }

    /**
     * This private method takes a GameDataDO object and maps it
     * into a PokerGame object. Ideally, they would be the same
     * but this is used to illustrate adding a DODS objects to
     * a completed application. Ditto from the method above
     */
    private synchronized PokerGame mapNewPokerGame(GameDataDO thisGame){
	PokerGameImpl newPokerGame = new PokerGameImpl("", "", "",0);

        try {
            newPokerGame.setName(thisGame.getName());
            newPokerGame.setPassword(thisGame.getPassword());
            newPokerGame.setEmail(thisGame.getEmail());
            newPokerGame.setCash(thisGame.getCash());
            newPokerGame.setLargestBet(thisGame.getLargestBet());
            newPokerGame.setSmallestBet(thisGame.getSmallestBet());
            newPokerGame.setTotalPlayed(thisGame.getTotalPlayed());
            newPokerGame.setTotalWon(thisGame.getTotalWon());
            newPokerGame.setDeck(thisGame.getDeck());
	    newPokerGame.setID(thisGame.getOId().toString());
        } catch (Exception e){
           //There really shouldn't be an exception
        }
        return newPokerGame;
    }

    /**
     * This method detremines the storage medium, then dishes
     * the removeGame call out to the proper method
     */
    public synchronized void removeGame(String thisName){
	if (this.useDB) {
	    try {
		removeDBGame(thisName);
	    } catch (Exception e) {
		//FIX: nothing done!
	    }
	} else {
	    removeMemoryGame(thisName);
	}
    }

    /**
     * This method removes thisGame's record from the DB
     */
    public synchronized void removeDBGame(String name) throws Exception{
        GameDataDO gameDataDO = getGameDataByName(name);
        try {
            DBTransaction db =
		Enhydra.getDatabaseManager().createTransaction();
            try {
                // Multiple data objects could be update, deleted,inserted
                // here as a single transaction.....
		//Thread.currentThread().dumpStack();
                db.delete(gameDataDO);
                db.commit();
            } catch (SQLException sqle) {
                db.rollback();
                throw sqle;
            } finally {
                // You must call this to free any resources (such as
                // connections) that were allocated by the transaction.
                db.release();
            }
        } catch (Exception e) {
            // You should deal with any exceptions here....
            throw e;
        }
    }

    /**
     * This method removes the thisGame object from the Vector
     */
    public void removeMemoryGame(String name){
        int marker = 0;
        PokerGame thatGame;

        //Lock gamelist before opps, for thread paranoia!
        synchronized (this){
	    //get the index equal to thisGame, then remove it
	    for (int i = 0; i < super.size(); i++){
               	thatGame = (PokerGame)super.elementAt(i);
	       	//if(thatGame.getName().equalsIgnoreCase(name)){
		if(thatGame.getName().equals(name)){
		   marker = i;
	        }
            }
        super.removeElementAt(marker);
	}
    }

    /**
     * This is a provate method that returns the GameDataDO
     * from the database where oId = id
     */
    private GameDataDO getGameDataByName(String name)
						throws Exception{
	GameDataDO gotGameDataDO = null;
	GameDataQuery dq = new GameDataQuery();
	dq.setQueryName(name);
        try {
            gotGameDataDO = (GameDataDO) dq.getNextDO();
        }
        finally {
            // Free up any resources
//            dq.reset();
        }
        return gotGameDataDO;
    }

    /**
     * This method ddtermines which storage medium is ised, then
     * doles the getGameByName request to the right place.
     */
    public PokerGame getGame(String name){
        PokerGame thisGame = null;

	if (useDB){
	    try {
	       GameDataDO thisDO = null;
	       thisDO = getGameDataByName(name);
	       thisGame = mapNewPokerGame(thisDO);

	    } catch (Exception e) {
		Enhydra.getLogChannel().write(Logger.DEBUG, "ERROR! " +
			e.toString());
	       //FIX: Still need to take care of some exceptions :)
            }
	} else {
	    thisGame = getGameInMemory(name);
	}

	return thisGame;

    }

    /**
     * This method ddtermines which storage medium is ised, then
     * doles the getIsNameUsed request to the right place.
     */
    public boolean getIsNameUsed(String name){
	boolean isUsed = true;

	if (this.useDB) {
	    isUsed = getIsNameUsedInDB(name);
	} else {
	    isUsed = getIsNameUsedInMemory(name);
	}

	return isUsed;
    }

    /**
     * This method ddtermines which storage medium is ised, then
     * doles the getTopTen method to the right place.
     */
    public Vector getTopTen(){
        Vector topTen = new Vector(0);
        if (this.useDB) {
            topTen = getTopTenInDB();
        } else {
            topTen = getTopTenInMemory();
        }

        return topTen;
    }

    /**
     * This method ddtermines which storage medium is ised, then
     * doles the authenticate request to the right place.
     */
    public boolean authenticate(String name, String pw){
      boolean auth = false;

      if (useDB) {
        auth = authenticateInDB(name, pw);
      }
      else {
        auth = authenticateInMemory(name, pw);
      }

      return auth;
    }

    /**
     * Check to see if this name has been used in the DB
     */
    public synchronized boolean getIsNameUsedInDB(String name){
        boolean isUsed = false;
//        GameDataDO thisGameDataDO = null;
	try {
            if (getGameDataByName(name) == null) {
	        isUsed = false;
	    }
            else {
	        isUsed = true;
	    }
	} catch (Exception e) { isUsed = false; }
        return isUsed;
    }

    /**
     * Check to see if this name has been used in Memory
     */
    public synchronized boolean getIsNameUsedInMemory(String name){
        boolean isUsed = false;
        PokerGame thisGame;

        //Get copy of gameList object so we dont have to
        // worry about locking the object for threadsafe opps
        GameList gameListCopy = this.getCopy();

        // I used the for-next so that I could use equalsIgnorecase
        // instead of Vector.contains()
        for (int i = 0; i < gameListCopy.size(); i++){
            thisGame = (PokerGame)gameListCopy.elementAt(i);
            //if (thisGame.getName().equalsIgnoreCase(name)){
            if (thisGame.getName().equals(name)){
                isUsed = true;
            }
        }
        return isUsed;
    }

    /**
     * Authenticate user in DB
     */
    public synchronized boolean authenticateInDB
        (String name, String pw) {
      boolean isAuth = false;
      try {
        GameDataDO thisGameDataDO = getGameDataByName(name);
        if (thisGameDataDO != null) {
          if (thisGameDataDO.getPassword().equalsIgnoreCase(pw)) {
            isAuth = true;
          }
        }
        else {
          isAuth = false;
        }
      }
      catch (Exception e) {
        e.printStackTrace(); // VR
        //Ooops! Not much excaption handeling in the app yet!
      }
      return isAuth;
    }

    /**
     * Authenticate user in memory
     */
    public synchronized boolean authenticateInMemory (String name, String pw){
        boolean isAuth = false;
        PokerGame thisGame;

        //Get copy of the gameList object so we dont have to
        //worry about locking it for threadsave opps
        GameList gameListCopy = this.getCopy();

        // I used the for-next so that I could use equalsIgnorecase
        // instead of Vector.contains()
        for (int i = 0; i < gameListCopy.size(); i++) {
            thisGame = (PokerGame)gameListCopy.elementAt(i);
            //if ((thisGame.getName().equalsIgnoreCase(name)) &
            //   (thisGame.getPassword().equalsIgnoreCase(pw))) {
            if ((thisGame.getName().equals(name)) &
               (thisGame.getPassword().equals(pw))) {

                  isAuth = true;
            }
        }
        return isAuth;
    }

    /**
     * Return the game that belongs to user name
     * from the Vector in memory
     */
    public synchronized PokerGame getGameInMemory(String name){
        PokerGameImpl thisGame = new PokerGameImpl(name , "", "",0);

        //Get copy of the gameList object so we dont have to
        //worry about locking it for threadsafe opps
        GameList gameListCopy = this.getCopy();
        synchronized(thisGame){
           //Yeah, I could have done this another way, but I didn't
           for (int i = 0; i < gameListCopy.size(); i++){
               thisGame = (PokerGameImpl)gameListCopy.elementAt(i);
               //if (thisGame.getName().equalsIgnoreCase(name)){
               if (thisGame.getName().equals(name)){
                   return thisGame;
               }
           }
        }
        return thisGame;
    }

   /**
    * return the top ten players ranked by cash won
    */
    public synchronized Vector getTopTenInMemory(){
        Vector topTen = new Vector(0);
        PokerGameImpl blankGame;
        for (int i = 0; i < 10; i++){
            blankGame = new PokerGameImpl("", "", "",0);
            topTen.addElement(blankGame);
        }
        PokerGame game;
        int rank[] = new int[10]; int Cash = 0;

        //get copy of gameList so we don't have to
        //worry about locking the object for threadsafe opps
        GameList gameListCopy = this.getCopy();

        //Really scary sort routine I came up with.
        for (int x = 0; x < 10; x++){
            for (int i = 0; i < gameListCopy.size(); i++){
                 game = (PokerGame)gameListCopy.elementAt(i);
                 if (x < 1){
                     if (game.getCash() > rank[x]){
                         rank[x] = game.getCash();
                         topTen.setElementAt(game, x);
                     }
                 } else {
                     if ((rank[x-1] >= game.getCash()) &
                        (!topTen.contains(game)) &
                        (rank[x] <= game.getCash())){
                          rank[x] = game.getCash();
                          topTen.setElementAt(game, x);
                     }
                 }
            }
        }
        return topTen;
    }

    /**
     * Check to see if the DO changes need to be updated to the DB
     */
    public synchronized void updateGame(PokerGame thisGame){
	if ( (this.useDB) & (thisGame.getIsDirty()) ) {
	    try {
	       GameDataDO gameDataDO =
	               getGameDataByName(thisGame.getName());
	       gameDataDO.setDeck(thisGame.getDeck());
               gameDataDO.setLargestBet(thisGame.getLargestBet());
               gameDataDO.setSmallestBet(thisGame.getSmallestBet());
               gameDataDO.setTotalPlayed(thisGame.getTotalPlayed());
               gameDataDO.setTotalWon(thisGame.getTotalWon());
               gameDataDO.setCash(thisGame.getCash());

//vr              DBTransaction db = Enhydra.getDatabaseManager().createTransaction();
               try {
//vr                   db.update(gameDataDO);
//vr                   db.commit();
                 gameDataDO.save(); //vr
               }
               catch (SQLException sqle) {
		   Enhydra.getLogChannel().write(Logger.INFO, "ERROR! " +
						sqle.toString());
//vr                  db.rollback();
//vr                  throw sqle;
               }
               finally {
//vr                   db.release();
               }
	    } catch (Exception e) {
                Enhydra.getLogChannel().write(Logger.DEBUG, "ERROR! " +
                        e.toString());
                //FIX: Still need to take care of some exceptions :)
	    }
	}
    }

    /**
     * This method returns the total players registered in the
     * DB or Vector
     */
    public int getCount(){
	int n = 0;
	if (!useDB){
	    n = this.size();
  	} else {
           GameDataQuery dq = new GameDataQuery();
           dq.getQueryBuilder().addEndClause( " order by Cash" );
           PokerGame tempGame = null;
           try {
               GameDataDO[] allGames = dq.getDOArray();
	       n = allGames.length;
           } catch (Exception e) {
               Enhydra.getLogChannel().write(Logger.INFO, "ERROR: " +
                                                          e.toString());
	   }

	}

	return n;
    }

    /**
     * This method queries the database for all games sorted
     * by score, then returns a vector with the top 10
     */
    public Vector getTopTenInDB(){
        GameDataQuery dq = new GameDataQuery();
        dq.getQueryBuilder().addEndClause( " order by Cash" );
	Vector topTen = new Vector(0);
	for (int i = 0; i < 10; i++){
	     topTen.addElement(new PokerGameImpl("","","",0));
        }

	PokerGame tempGame = null;
	try {
            GameDataDO[] allGames = dq.getDOArray();

	    int x = 0;
            for (int i = allGames.length -1; i > -1; i--){
		tempGame = mapNewPokerGame(allGames[i]);
		topTen.setElementAt(tempGame,x);
		x++;
		if (x == 10) { i = -1; }
            }

	} catch (Exception e) {
            Enhydra.getLogChannel().write(Logger.INFO, "ERROR: " +
							e.toString());
	    //FIX: Oops!  I really should handle this
	}

        return topTen;
    }

    /**
     * get rank of current dollars compared to the gameList
     */
    public synchronized int getRank(int cash){
        PokerGame game;
	GameList gameListCopy = this.getCopy();
        int rank = 0;
	if (!useDB){
           //opps to be done on copy, eliminates need for object lock
           for (int i = 0; i < gameListCopy.size(); i++){
                game = (PokerGame)gameListCopy.elementAt(i);
                if (game.getCash() > cash) { rank++; }
           }
	} else {
           GameDataQuery dq = new GameDataQuery();
           dq.getQueryBuilder().addEndClause( " order by Cash" );
           try {
               GameDataDO[] allGames = dq.getDOArray();
               for (int i = 0; i < allGames.length; i++){
                    if (allGames[i].getCash() > cash) { rank++; }
               }
           } catch (Exception e) {
               Enhydra.getLogChannel().write(Logger.INFO, "ERROR: " +
                                                          e.toString());
           }
	}

        return rank + 1;
    }


}
