package poker.business;

import java.util.Vector;
import java.sql.SQLException;
import poker.data.user.*;
import poker.data.game.*;
import poker.business.GameList;
import poker.spec.*;
/**
 *
 * EnhyDraw!, beta 4, 5/21/99
 *
 * Copyright 1999, Larry Wolcot & Daryl Tempesta
 * ALL rights reserved. Not for commercial use
 * without written permission from both authors.
 *
 * This class is a basic wrapper for the BDO's (like GameList)
 * It manages all of the active cardGames, PitBoss Statistics, etc.
 */
public class GameManagerImpl implements GameManager,java.io.Serializable {

    private PokerGame game;
    private GameList gameList = new GameList(0);
    private PitBossListImpl pitBossList = new PitBossListImpl();
    private String appName = "Poker";

/**
 * Set the useDB flag in the GameList object. If useDB is false,
 * the GameList object will use a standard Vector (in memory)
 * to hold all of the active PokerGame objects. if useDB is set to
 * true, it will mirror the Vector's elements into the database.
 */
  public void setUseDB(boolean useDB) throws Exception{
    try {
         gameList.setUseDB(useDB);
         //PitBoss persistance is not finished
         pitBossList.setUseDB(useDB);
    } catch (Exception e) {
         throw e;
    }
  }

/**
 * Total number of players sent to the poor house!
 */
  public String getTotalBankrupt(){
  //String ret = new Integer(totalBankrupt).toString();
  //return ret;
      int total = 0;
      total =  pitBossList.getPitBossVDO(appName).getTotalBankrupt();
      return new Integer(total).toString();
  }

  /**
   * Total dollars the house has won/lost
   */
  public String getHouseProfit(){
      int total = 0;
      total =  pitBossList.getPitBossVDO(appName).getTotalDollars();
      return new Integer(total).toString();
  }

  /**
   * Check the active gameList and return total number of players
   */
  public String getTotalPlayers(){
  String ret= new Integer(gameList.getCount()).toString();
  return ret;
  }

  /**
   * Total number hands dealt
   */
  public String getTotalHandsDealt(){
      int total = 0;
      total =  pitBossList.getPitBossVDO(appName).getTotalHandsDealt();
      return new Integer(total).toString();
  }

  /**
   * Total number of hands won by all players
   */
  public String getTotalHandsWon(){
      int total = 0;
      total =  pitBossList.getPitBossVDO(appName).getTotalHandsWon();
      return new Integer(total).toString();
  }

  /**
   * Total number of hands lost by all players
   */
  public String getTotalHandsLost(){
    int totalWon = 0;
    int totalPlayed = 0;
    totalWon =  pitBossList.getPitBossVDO(appName).getTotalHandsWon();
    totalPlayed =
    pitBossList.getPitBossVDO(appName).getTotalHandsDealt();
    return new Integer(totalPlayed - totalWon).toString();
  }

  /**
   * Incriment totalHandsWon by n
   */
  public synchronized void setTotalHandsWon(int n) {
    synchronized(pitBossList) {
       n = n + pitBossList.getPitBossVDO(appName).getTotalHandsWon();
       pitBossList.getPitBossVDO(appName).setTotalHandsWon(n);
      }
  }

  /**
   * Incriment totalBankrupt by n
   */
  public synchronized void setTotalBankrupt(int n){
      synchronized(pitBossList){
         n = n + pitBossList.getPitBossVDO(appName).getTotalBankrupt();
         pitBossList.getPitBossVDO(appName).setTotalBankrupt(n);
      }
  }

  /**
   * Incriment totalHandsDealt by n
   */
  public synchronized void setHandsDealt(int n){
      synchronized(pitBossList){
         n = n +pitBossList.getPitBossVDO(appName).getTotalHandsDealt();
         pitBossList.getPitBossVDO(appName).setTotalHandsDealt(n);
      }
  }

  /**
   * Incriment totalDollars (won) by n
   */
  public synchronized void setDollars(int n){
      synchronized(pitBossList){
         n = n + pitBossList.getPitBossVDO(appName).getTotalDollars();
         pitBossList.getPitBossVDO(appName).setTotalDollars(n);
      }
  }

  /**
   * same as setDollars
   */
  public synchronized void setTotalDollars(int n){
    setDollars(n);
  }

  /**
   * Return a (COPY)list of active games
   */
  public synchronized Vector getGameList(){
    GameList gameListCopy = this.gameList.getCopy();
    return gameListCopy;
  }

  /**
   * Return the size() of gameList
   */
  public synchronized int getCount(){
    return gameList.getCount();
  }

  /**
  * return the top ten players ranked by cash won
  */
  public synchronized Vector getTopTen(){
    Vector topTen = null;

    topTen = gameList.getTopTen();

    return topTen;

  }

  /**
   * Check to see if this name has been used
   */
  public synchronized boolean getIsNameUsed(String name){
    boolean isUsed = false;

    if (gameList.getIsNameUsed(name)) { isUsed = true; }

    return isUsed;
  }

  /**
   * Authenticate user against gameList object
   */
  public synchronized boolean authenticate(String name, String pw){
    boolean isAuth = false;

    if (gameList.authenticate(name, pw)) {
      isAuth = true;
    }
    return isAuth;
  }

  /**
   * Return the game that belongs to user name String
   */
  public synchronized PokerGame getGame(String name) {
    PokerGame thisGame = null;

    thisGame = gameList.getGame(name);

    return thisGame;
  }

  /**
   * Add a game to the gameList
   */
  public synchronized void addGame(PokerGame newGame){
  //lock gameList object for TOTAL threadsave opps
  //This is OK for less intense apps like this one,
      //but not the best way to go for high transaction
  //DB volumes
  synchronized(gameList){
   gameList.addGame(newGame);
  }
  }

  /**
   * get rank of current dollars compared to the gameList
   */
  public synchronized int getRank(int cash){
  int rank = gameList.getRank(cash);
  return rank;
  }

  /**
   * remove thisGame from the gameList
   */
  public synchronized void removeGame(String name){
      gameList.removeGame(name);
  }

  /**
   * Make sure the DB gets updated if this is a DODS object
   */
  public synchronized void updateGame(PokerGame thisGame){
      gameList.updateGame(thisGame);
  }

  }
