package poker.business;

import poker.data.user.*;
import poker.data.game.*;
import java.util.Vector;
import poker.spec.user.PlayerDO;
import poker.spec.*;

/**
 *
 * EnhyDraw!, beta 4, 5/21/99
 *
 * Copyright 1999, Larry Wolcot & Daryl Tempesta
 * ALL rights reserved. Not for commercial use
 * without written permission from both authors.
 *
 * This is the business object that encapsulates the
 * PlayerDO and PokerGameDO objects.
 * It really doesn't do much other than that!
 */
public class PokerGameImpl implements PokerGame,java.io.Serializable {

    private boolean isDirty = false;
    private PokerGameDO pokerGameDO;
    private PlayerDO playerDO;

    /*
     * Constructor that inits the DO's
     */
    public PokerGameImpl(){};
    
    public PokerGameImpl(String name,String pw,String id,int cash){
	    playerDO = new PlayerDOImpl(name, pw);
	    pokerGameDO = new PokerGameDO(id, cash);
    }

    /*
     * Return the PokerGameDO for this object
     */
    public PokerGameDO getPokerGameDO(){
	    return pokerGameDO;
    }

    /*
     * return the name from the PlayerDO
     */
    public String getName(){
	    return playerDO.getName();
    }

    /*
     * Check to see if this DO has been changed
     */
    public boolean getIsDirty(){
      return this.isDirty;
    }

    /*
     * return the email from the PlayerDO
     */
    public String getEmail(){
        return playerDO.getEmail();
    }

    /*
     * return the password from the PlayerDO
     */
    public String getPassword(){
	    return playerDO.getPassword();
    }

    /*
     * return the cash from the pokerGameDO
     */
    public int getCash(){
	    return pokerGameDO.getCash();
    }

    /*
     * return the getPlayRound from the pokerGameDO
     */
    public int getPlayRound(){
	    return pokerGameDO.getPlayRound();
    }


    /*
     * return the getID from the pokerGameDO
     */
    public String getID(){
	    return pokerGameDO.getID();
    }

    /*
     * return the current Bet from the pokerGameDO
     */
    public int getBet(){
	    return pokerGameDO.getBet();
    }

    /*
     * return the current version from the pokerGameDO
     */
    public int getVersion(){
        return pokerGameDO.getVersion();
    }

    /*
     * return the largest bet from the pokerGameDO
     */
    public int getLargestBet(){
	    return pokerGameDO.getLargestBet();
    }

    /*
     * return the smallest bet from the pokerGameDO
     */
    public int getSmallestBet(){
	    return pokerGameDO.getSmallestBet();
    }

    /*
     * return the total games played from the pokerGameDO
     */
    public int getTotalPlayed(){
	    return pokerGameDO.getTotalPlayed();
    }

    /*
     * return the total games won from the pokerGameDO
     */
    public int getTotalWon(){
	    return pokerGameDO.getTotalWon();
    }

    /*
     * return the Vector of cards in hand from the pokerGameDO
     */
    public Vector getCardsInHand(){
	    return pokerGameDO.getCardsInHand();
    }

    /*
     * return the Vector of cards already dealt from the pokerGameDO
     */
    public Vector getCardsUsed(){
	    return pokerGameDO.getCardsUsed();
    }

    /*
     * return the default bet from the pokerGameDO
     */
    public int getDefaultBet(){
	    return pokerGameDO.getDefaultBet();
    }

    /*
     * return the Vector of cards to drop from the pokerGameDO
     */
    public Vector getDropCards(){
	    return pokerGameDO.getDropCards();
    }

    /*
     * return the default deck image name from the pokerGameDO
     */
    public String getDeck(){
	    return pokerGameDO.getDeck();
    }

    /*
     * return the last error from the pokerGameDO
     * not in use, but too paranoid to remove
     */
    public String getLastError(){
	    return pokerGameDO.getLastError();
    }

    /*
     * return the PlayerDO from the pokerGameDO
     * Yeah, I know this ills the encapsulation rules, but hey!
     * it's only game!
     */
    public PlayerDO getPlayer(){ return playerDO; }

   /*
    * Set the cash in the pokerGameDO
    */
    public void setCash(int cash){
	    this.isDirty = true;
	    pokerGameDO.setCash(cash);
    }

   /*
    * Set the playround in the pokerGameDO
    */
    public void setPlayRound(int playRound){
	    this.isDirty = true;
	    pokerGameDO.setPlayRound(playRound);
    }

   /*
    * Set the last error in the pokerGameDO
    */
    public void setLastError(String lastError){
        this.isDirty = true;
	    pokerGameDO.setLastError(lastError);
    }

   /*
    * Set the ID in the pokerGameDO
    */
    public void setID(String id){
        this.isDirty = true;
	    pokerGameDO.setID(id);
    }

   /*
    * Set the name in the playerDO
    */
    public void setName(String name){
        this.isDirty = true;
	    playerDO.setName(name);
    }

   /*
    * Set the password in the playerDO
    */
    public void setPassword(String pw){
        this.isDirty = true;
        playerDO.setPassword(pw);
    }

   /*
    * Set the email in the playerDO
    */
    public void setEmail(String email){
        this.isDirty = true;
        playerDO.setEmail(email);
    }


   /*
    * Set the current bet in the pokerGameDO
    */
    public void setBet(int bet){
        this.isDirty = true;
	    pokerGameDO.setBet(bet);
    }

   /*
    * Set the current version in the pokerGameDO
    */
    public void setVersion(int version){
        this.isDirty = true;
        pokerGameDO.setVersion(version);
    }

   /*
    * Set the largest bet placed in the pokerGameDO
    */
    public void setLargestBet(int largestBet){
        this.isDirty = true;
	    pokerGameDO.setLargestBet(largestBet);
    }

   /*
    * Set the smallest bet in the pokerGameDO
    */
    public void setSmallestBet(int smallestBet){
        this.isDirty = true;
	    pokerGameDO.setSmallestBet(smallestBet);
    }

   /*
    * Set the total hands played in the pokerGameDO
    */
    public void setTotalPlayed(int totalPlayed){
        this.isDirty = true;
	    pokerGameDO.setTotalPlayed(totalPlayed);
    }

   /*
    * Set the total hands won in the pokerGameDO
    */
    public void setTotalWon(int totalWon){
        this.isDirty = true;
	    pokerGameDO.setTotalWon(totalWon);
    }

   /*
    * Set the current cards in hand in the pokerGameDO
    */
    public void setCardsInHand(Vector cardsInHand){
	    pokerGameDO.setCardsInHand(cardsInHand);
    }

  /*
   * Set the cards used in the pokerGameDO
   */
    public void setCardsUsed(Vector cardsUsed){
	    pokerGameDO.setCardsUsed(cardsUsed);
    }

   /*
    * Set the cards to be dropped in the pokerGameDO
    */
    public void setDropCards(Vector dropCards){
	    pokerGameDO.setDropCards(dropCards);
    }

   /*
    * Set the default deck image name in the pokerGameDO
    */
    public void setDeck(String deck){
        this.isDirty = true;
	    pokerGameDO.setDeck(deck);
    }

   /*
    * Set the default bet in the pokerGameDO
    */
    public void setDefaultBet(int defaultBet){
        this.isDirty = true;
	    pokerGameDO.setDefaultBet(defaultBet);
    }

}
