/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * /usr/local/apps/Poker/poker/data/DODS_Fortune/FortuneBDO.java
 *-----------------------------------------------------------------------------
 */

package poker.data.DODS_Fortune;

//import dods.builder.sourceGenerators.Query.DataObjectException;
import com.lutris.dods.builder.generator.query.*;

/**
 * FortuneBDO contains the same set and get methods as
 * the FortuneDO class.
 * Business Object (BO) classes typically need these set and get methods.
 * So by deriving a BO from a BDO, or by implementing a BO that
 * contains a BDO, the developer of the BO is spared some work.
 *
 * @author <AUTHOR>
 * @version $Revision: 1.1 $
 */
public class FortuneBDO implements java.io.Serializable {

    /**
     * The FortuneDO object upon which the set and get methods operate.
     * This member is protected so that classes derived from FortuneBDO
     * can access the underlying Data Object.
     */
    protected FortuneDO DO;

    /**
     * Like the class <CODE>FortuneDO</CODE>,
     * this class acts as a factory.
     * Business Object (BO) classes typically need these set and get methods.
     * So by deriving a BO from a BDO, or by implementing a BO that
     * contains one or more BDOs, the developer of the BO is spared some work.
     */
    public static FortuneBDO createVirgin() throws Exception {
	FortuneBDO bdo = new FortuneBDO ();
	bdo.DO = FortuneDO.createVirgin();
	return bdo;
    }

    /**
     * The createExisting method is used to create a <CODE>FortuneBDO</CODE>
     * from a <CODE>FortuneDO</CODE> that was returned by
     * the <CODE>FortuneQuery</CODE> class.
     */
    public static FortuneBDO createExisting( FortuneDO DO ) {
	FortuneBDO bdo = new FortuneBDO ();
	bdo.DO = DO;
	return bdo;
    }

    /**
     * The developer of a Business Object that derives from this class
     * can override the methods:<CODE>
     *     beforeAnyGet
     *     beforeAnySet
     *     afterAnySet
     * </CODE> to handle any general assertions or cleanup needed
     * for <CODE>get</CODE> and <CODE>set</CODE> methods.
     */
    protected void beforeAnyGet() {}
    protected void beforeAnySet() {}
    protected void afterAnySet() {}

   /**
    * Get Fortune of the FortuneDO
    *
    * @return Fortune of the FortuneDO
    */
   public String getFortune ()
   throws DataObjectException {
      return DO.getFortune ();
   }

   /**
    * Set Fortune of the FortuneDO
    *
    * @param Fortune of the FortuneDO
    */

   public void setFortune ( String Fortune )
   throws DataObjectException {
      DO.setFortune ( Fortune );
   }


   /**
    * Get FortuneNumber of the FortuneDO
    *
    * @return FortuneNumber of the FortuneDO
    */
   public java.math.BigDecimal getFortuneNumber ()
   throws DataObjectException {
      return DO.getFortuneNumber ();
   }

   /**
    * Set FortuneNumber of the FortuneDO
    *
    * @param FortuneNumber of the FortuneDO
    */

   public void setFortuneNumber ( java.math.BigDecimal FortuneNumber )
   throws DataObjectException {
      DO.setFortuneNumber ( FortuneNumber );
   }


   /**
    * Get HasBeenUsed of the FortuneDO
    *
    * @return HasBeenUsed of the FortuneDO
    */
   public boolean getHasBeenUsed ()
   throws DataObjectException {
      return DO.getHasBeenUsed ();
   }

   /**
    * Set HasBeenUsed of the FortuneDO
    *
    * @param HasBeenUsed of the FortuneDO
    */

   public void setHasBeenUsed ( boolean HasBeenUsed )
   throws DataObjectException {
      DO.setHasBeenUsed ( HasBeenUsed );
   }


}
