/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * /root/test/poker/data/DODS_GameData/GameDataBDO.java
 *-----------------------------------------------------------------------------
 */

package poker.data.DODS_GameData;

import com.lutris.dods.builder.generator.query.DataObjectException;

/**
 * GameDataBDO contains the same set and get methods as
 * the GameDataDO class.
 * Business Object (BO) classes typically need these set and get methods.
 * So by deriving a BO from a BDO, or by implementing a BO that
 * contains a BDO, the developer of the BO is spared some work.
 *
 * @author <AUTHOR>
 * @version $Revision: 1.1 $
 */
public class GameDataBDO implements java.io.Serializable {

    /**
     * The GameDataDO object upon which the set and get methods operate.
     * This member is protected so that classes derived from GameDataBDO
     * can access the underlying Data Object.
     */
    protected GameDataDO DO;

    /**
     * Like the class <CODE>GameDataDO</CODE>,
     * this class acts as a factory.
     * Business Object (BO) classes typically need these set and get methods.
     * So by deriving a BO from a BDO, or by implementing a BO that
     * contains one or more BDOs, the developer of the BO is spared some work.
     */
    public static GameDataBDO createVirgin() throws Exception {
	GameDataBDO bdo = new GameDataBDO ();
	bdo.DO = GameDataDO.createVirgin();
	return bdo;
    }

    /**
     * The createExisting method is used to create a <CODE>GameDataBDO</CODE>
     * from a <CODE>GameDataDO</CODE> that was returned by
     * the <CODE>GameDataQuery</CODE> class.
     */
    public static GameDataBDO createExisting( GameDataDO DO ) {
	GameDataBDO bdo = new GameDataBDO ();
	bdo.DO = DO;
	return bdo;
    }

    /**
     * The developer of a Business Object that derives from this class
     * can override the methods:<CODE>
     *     beforeAnyGet
     *     beforeAnySet
     *     afterAnySet
     * </CODE> to handle any general assertions or cleanup needed
     * for <CODE>get</CODE> and <CODE>set</CODE> methods.
     */
    protected void beforeAnyGet() {}
    protected void beforeAnySet() {}
    protected void afterAnySet() {}

   /**
    * Get Cash of the GameDataDO
    *
    * @return Cash of the GameDataDO
    */
   public int getCash ()
   throws DataObjectException {
      return DO.getCash ();
   }

   /**
    * Set Cash of the GameDataDO
    *
    * @param Cash of the GameDataDO
    */

   public void setCash ( int Cash )
   throws DataObjectException {
      DO.setCash ( Cash );
   }


   /**
    * Get Deck of the GameDataDO
    *
    * @return Deck of the GameDataDO
    */
   public String getDeck ()
   throws DataObjectException {
      return DO.getDeck ();
   }

   /**
    * Set Deck of the GameDataDO
    *
    * @param Deck of the GameDataDO
    */

   public void setDeck ( String Deck )
   throws DataObjectException {
      DO.setDeck ( Deck );
   }


   /**
    * Get Email of the GameDataDO
    *
    * @return Email of the GameDataDO
    */
   public String getEmail ()
   throws DataObjectException {
      return DO.getEmail ();
   }

   /**
    * Set Email of the GameDataDO
    *
    * @param Email of the GameDataDO
    */

   public void setEmail ( String Email )
   throws DataObjectException {
      DO.setEmail ( Email );
   }


   /**
    * Get LargestBet of the GameDataDO
    *
    * @return LargestBet of the GameDataDO
    */
   public int getLargestBet ()
   throws DataObjectException {
      return DO.getLargestBet ();
   }

   /**
    * Set LargestBet of the GameDataDO
    *
    * @param LargestBet of the GameDataDO
    */

   public void setLargestBet ( int LargestBet )
   throws DataObjectException {
      DO.setLargestBet ( LargestBet );
   }


   /**
    * Get Name of the GameDataDO
    *
    * @return Name of the GameDataDO
    */
   public String getName ()
   throws DataObjectException {
      return DO.getName ();
   }

   /**
    * Set Name of the GameDataDO
    *
    * @param Name of the GameDataDO
    */

   public void setName ( String Name )
   throws DataObjectException {
      DO.setName ( Name );
   }


   /**
    * Get Password of the GameDataDO
    *
    * @return Password of the GameDataDO
    */
   public String getPassword ()
   throws DataObjectException {
      return DO.getPassword ();
   }

   /**
    * Set Password of the GameDataDO
    *
    * @param Password of the GameDataDO
    */

   public void setPassword ( String Password )
   throws DataObjectException {
      DO.setPassword ( Password );
   }


   /**
    * Get SmallestBet of the GameDataDO
    *
    * @return SmallestBet of the GameDataDO
    */
   public int getSmallestBet ()
   throws DataObjectException {
      return DO.getSmallestBet ();
   }

   /**
    * Set SmallestBet of the GameDataDO
    *
    * @param SmallestBet of the GameDataDO
    */

   public void setSmallestBet ( int SmallestBet )
   throws DataObjectException {
      DO.setSmallestBet ( SmallestBet );
   }


   /**
    * Get TotalPlayed of the GameDataDO
    *
    * @return TotalPlayed of the GameDataDO
    */
   public int getTotalPlayed ()
   throws DataObjectException {
      return DO.getTotalPlayed ();
   }

   /**
    * Set TotalPlayed of the GameDataDO
    *
    * @param TotalPlayed of the GameDataDO
    */

   public void setTotalPlayed ( int TotalPlayed )
   throws DataObjectException {
      DO.setTotalPlayed ( TotalPlayed );
   }


   /**
    * Get TotalWon of the GameDataDO
    *
    * @return TotalWon of the GameDataDO
    */
   public int getTotalWon ()
   throws DataObjectException {
      return DO.getTotalWon ();
   }

   /**
    * Set TotalWon of the GameDataDO
    *
    * @param TotalWon of the GameDataDO
    */

   public void setTotalWon ( int TotalWon )
   throws DataObjectException {
      DO.setTotalWon ( TotalWon );
   }


}
