/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * /root/test/poker/data/DODS_PitBoss/PitBossBDO.java
 *-----------------------------------------------------------------------------
 */

package poker.data.DODS_PitBoss;

//import dods.builder.sourceGenerators.Query.DataObjectException;
import com.lutris.dods.builder.generator.query.*;

/**
 * PitBossBDO contains the same set and get methods as
 * the PitBossDO class.
 * Business Object (BO) classes typically need these set and get methods.
 * So by deriving a BO from a BDO, or by implementing a BO that
 * contains a BDO, the developer of the BO is spared some work.
 *
 * @author <AUTHOR>
 * @version $Revision: 1.1 $
 */
public class PitBossBDO implements java.io.Serializable {

    /**
     * The PitBossDO object upon which the set and get methods operate.
     * This member is protected so that classes derived from PitBossBDO
     * can access the underlying Data Object.
     */
    protected PitBossDO DO;

    /**
     * Like the class <CODE>PitBossDO</CODE>,
     * this class acts as a factory.
     * Business Object (BO) classes typically need these set and get methods.
     * So by deriving a BO from a BDO, or by implementing a BO that
     * contains one or more BDOs, the developer of the BO is spared some work.
     */
    public static PitBossBDO createVirgin() throws Exception {
	PitBossBDO bdo = new PitBossBDO ();
	bdo.DO = PitBossDO.createVirgin();
	return bdo;
    }

    /**
     * The createExisting method is used to create a <CODE>PitBossBDO</CODE>
     * from a <CODE>PitBossDO</CODE> that was returned by
     * the <CODE>PitBossQuery</CODE> class.
     */
    public static PitBossBDO createExisting( PitBossDO DO ) {
	PitBossBDO bdo = new PitBossBDO ();
	bdo.DO = DO;
	return bdo;
    }

    /**
     * The developer of a Business Object that derives from this class
     * can override the methods:<CODE>
     *     beforeAnyGet
     *     beforeAnySet
     *     afterAnySet
     * </CODE> to handle any general assertions or cleanup needed
     * for <CODE>get</CODE> and <CODE>set</CODE> methods.
     */
    protected void beforeAnyGet() {}
    protected void beforeAnySet() {}
    protected void afterAnySet() {}

   /**
    * Get GameName of the PitBossDO
    *
    * @return GameName of the PitBossDO
    */
   public String getGameName ()
   throws DataObjectException {
      return DO.getGameName ();
   }

   /**
    * Set GameName of the PitBossDO
    *
    * @param GameName of the PitBossDO
    */

   public void setGameName ( String GameName )
   throws DataObjectException {
      DO.setGameName ( GameName );
   }


   /**
    * Get TotalDollars of the PitBossDO
    *
    * @return TotalDollars of the PitBossDO
    */
   public int getTotalDollars ()
   throws DataObjectException {
      return DO.getTotalDollars ();
   }

   /**
    * Set TotalDollars of the PitBossDO
    *
    * @param TotalDollars of the PitBossDO
    */

   public void setTotalDollars ( int TotalDollars )
   throws DataObjectException {
      DO.setTotalDollars ( TotalDollars );
   }


   /**
    * Get TotalBankrupt of the PitBossDO
    *
    * @return TotalBankrupt of the PitBossDO
    */
   public int getTotalBankrupt ()
   throws DataObjectException {
      return DO.getTotalBankrupt ();
   }

   /**
    * Set TotalBankrupt of the PitBossDO
    *
    * @param TotalBankrupt of the PitBossDO
    */

   public void setTotalBankrupt ( int TotalBankrupt )
   throws DataObjectException {
      DO.setTotalBankrupt ( TotalBankrupt );
   }


   /**
    * Get TotalHandsDealt of the PitBossDO
    *
    * @return TotalHandsDealt of the PitBossDO
    */
   public int getTotalHandsDealt ()
   throws DataObjectException {
      return DO.getTotalHandsDealt ();
   }

   /**
    * Set TotalHandsDealt of the PitBossDO
    *
    * @param TotalHandsDealt of the PitBossDO
    */

   public void setTotalHandsDealt ( int TotalHandsDealt )
   throws DataObjectException {
      DO.setTotalHandsDealt ( TotalHandsDealt );
   }


   /**
    * Get TotalHandsWon of the PitBossDO
    *
    * @return TotalHandsWon of the PitBossDO
    */
   public int getTotalHandsWon ()
   throws DataObjectException {
      return DO.getTotalHandsWon ();
   }

   /**
    * Set TotalHandsWon of the PitBossDO
    *
    * @param TotalHandsWon of the PitBossDO
    */

   public void setTotalHandsWon ( int TotalHandsWon )
   throws DataObjectException {
      DO.setTotalHandsWon ( TotalHandsWon );
   }


}
