package poker.presentation.login;

import java.io.*;
import java.net.*;
import com.lutris.http.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.user.*;
import com.lutris.util.*;
import poker.spec.*;
import poker.Poker;

import poker.presentation.main.FirstDealHTML;

/**
 *   Presentation Object that processes the creation request and redirects
 *      to the appropriate page.
 *
 *
 * EnhyDraw!, beta 4, 5/21/99
 *
 * Copyright 1999, Larry Wolcot & Daryl Tempesta
 * ALL rights reserved. Not for commercial use
 * without written permission from both authors.
 *
 * @author Larry Wolcott
 * @version $Revision: 1.1 $
 */
public class CreationProcessorPresentation implements HttpPresentation{


    private final String homePage = "main/PlaceBetPresentation.po";
    private final String loginPage = "login/CreatePlayerPresentation.po";
    private GameManager gameManager;

  public void run(HttpPresentationComms comms) throws Exception{


    Poker application = (Poker)comms.application;
    gameManager = application.getGameManager();

    String username = comms.request.getParameter("username");
    String pw = comms.request.getParameter("pw");
    String email = comms.request.getParameter("email");
    String cashAsString = comms.request.getParameter("cash");

    String redirect = loginPage + "?" + "password=" + pw + "&" +
                                        "username=" + username + "&" +
                                        "cash=" + cashAsString;

    String deck = comms.request.getParameter("deck");
    String thisDeck = "deck1";
    if (deck.equals("2")) {
      thisDeck = "deck2";
    }
    else if (deck.equals("3")) {
      thisDeck = "deck3";
    }
    else if (deck.equals("4")) {
      thisDeck = "deck4";
    }
    else if (deck.equals("5")) {
      thisDeck = "deck5";
    }

    if( username==null || pw==null ) {
        comms.session.getSessionData().set("error",
                      "You must specify a username and password to login.");
        throw new ClientPageRedirectException(
                  comms.request.getAppFileURIPath(redirect));
    }

    int cash = 0;
    if(cashAsString != null && !cashAsString.equals("")) {
      try {
       cash = Integer.parseInt(cashAsString);
      }
      catch( NumberFormatException e ) {
        comms.session.getSessionData().set("error",
                                            "Invalid input in cash field!");
        throw new ClientPageRedirectException(
                  comms.request.getAppFileURIPath(redirect));
      }
    }

    if( cash < 0 || cash > 1000000 ) {
        comms.session.getSessionData().set("error",
                      "Cash amount can't be negative or greather than 1000000!");
        throw new ClientPageRedirectException(
                  comms.request.getAppFileURIPath(redirect));
    }

    boolean auth = true;

    String id = comms.session.getSessionKey();
/*
 *catch Null pointer exception ( we canot make a instances of classes from business layer when we run poker_pres )
 *We need  to allow poker_pres to be dynamic , so if the requested url is /poker_pres/..... we dont need to chack user
 */

try{
    if ((!username.equals("")) && (!pw.equals(""))){


       auth = gameManager.getIsNameUsed(username);

       if (auth)
         comms.session.getSessionData().set("error", "Name is already used!");
       else {

         PokerGame newGame = PokerGameFactory.getPokerGame("poker.business.PokerGameImpl",username, pw, id, cash);
         
         GameManager g = application.getGameManager();
         newGame.setDeck(thisDeck);
         newGame.setEmail(email);
         gameManager.addGame(newGame);
         comms.session.setUser((User)newGame.getPlayer());
         comms.session.getSessionData().set("game", newGame);
         comms.session.getSessionData().set("error", "");
         redirect = homePage;
       }
    }
    else
      comms.session.getSessionData().set("error", "Incorrect username or password!");

   
 
}catch (NullPointerException  e){
   throw new ClientPageRedirectException(
              comms.request.getAppFileURIPath(homePage));
}
 throw new ClientPageRedirectException(comms.request.getAppFileURIPath(redirect));	
}
}
