package poker.presentation.main;

import java.io.*;
import java.net.*;
import com.lutris.http.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.user.*;
import com.lutris.util.*;
import poker.spec.*;
import poker.Poker;

/**
 *   Presentation Object that processes the bet request and redirects
 *      to the appropriate page.
 *
 * EnhyDraw!, beta 4, 5/21/99
 *
 * Copyright 1999, Larry Wolcot & Daryl Tempesta
 * ALL rights reserved. Not for commercial use
 * without written permission from both authors.
 *
 */
public class BetProcessorPresentation implements HttpPresentation{

  static final String continuePage = "main/FirstDealPresentation.po";
  static final String errorPage = "main/PlaceBetPresentation.po";
  static final String debugArg = "debug";
  static final String helpArg = "help";



  public void run(HttpPresentationComms comms) throws Exception{
    String bet = comms.request.getParameter("bet");
    String def = comms.request.getParameter("default");
    String quit = comms.request.getParameter("QUIT");
    if (quit != null){
      if (quit.equals("QUIT")){
        throw new ClientPageRedirectException(
            comms.request.getAppFileURIPath("main/QuitPresentation.po"));
        }
      }
    if (def == null) {def = "empty";}
    PokerGame game;
    game = (PokerGame)comms.session.getSessionData().get("game");
    String redirect = "";
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer when we run poker_pres )
 * We need  to allow poker_pres to be dynamic 
 */   
try{
    
    try{
    game.setBet(new Integer(bet).intValue());
    }catch(NumberFormatException e){
    
       game.setLastError("Invalid bet value!!");
       throw new ClientPageRedirectException(
                comms.request.getAppFileURIPath(errorPage));
    }
    String pp = comms.request.getApplicationPath();
    if (!pp.endsWith("/"))
       pp += "/";

    HttpPresentationOutputStream out =comms.response.getOutputStream();
    boolean validBet = false;
    
    int thisBet = 0;
    if (bet != null){
       thisBet = new Integer(bet).intValue();
    }

    if (game.getPlayRound() > 1) {
      redirect = continuePage;
    } else {
        if ((thisBet > game.getCash()) || (thisBet == 0)){
       redirect = errorPage;
       game.setLastError("Invalid bet value!!");
       game.setPlayRound(0);
        } else {
           Poker mainApp;
           mainApp = (Poker)comms.session.getSessionData().get("app");
           GameManager g = mainApp.getGameManager();
     g.setTotalDollars(thisBet);
     g.setHandsDealt(1);
       redirect = continuePage;
       game.setBet(thisBet);
       game.setTotalPlayed(game.getTotalPlayed() + 1);
       game.setCash(game.getCash() - thisBet);
       if (def.equals("on")){
     game.setDefaultBet(thisBet);
       }
       g.updateGame(game);
        }
     }
}catch (NullPointerException  e){
	   throw new ClientPageRedirectException(
                comms.request.getAppFileURIPath(errorPage));

	}

   throw new ClientPageRedirectException(
                comms.request.getAppFileURIPath(redirect));
  }
}
