/*
 *  Poker
 *
 *  Enhydra super-servlet presentation object
 *
 */

package poker.presentation.main;

// Enhydra SuperServlet imports
import com.lutris.appserver.server.httpPresentation.HttpPresentation;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.util.KeywordValueException;

import poker.Poker;
import poker.spec.PokerGame;
import poker.spec.GameManager;


public class StatsPresentation implements HttpPresentation {


  public void run(HttpPresentationComms comms)
      throws HttpPresentationException, KeywordValueException {

    StatsHTML stats;
    stats = (StatsHTML)comms.xmlcFactory.create(StatsHTML.class);

    Poker mainApp = (Poker)comms.session.getSessionData().get("app");
    GameManager gameManager = mainApp.getGameManager();
    PokerGame game = (PokerGame)comms.session.getSessionData().get("game");

/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer when we run poker_pres )
 * We need  to allow poker_pres to be dynamic , so if the requested url is /poker_pres/..... the response
 * will be default HTML page
 */
try{
    int rank = gameManager.getRank(game.getCash());
    int count = gameManager.getCount();
    String ranking = Integer.toString(rank) + " of " + Integer.toString(count);

    stats.setTextName(game.getName());
    stats.setTextRank(ranking);
    stats.setTextCash(Integer.toString(game.getCash()));
    stats.setTextTotal(Integer.toString(game.getTotalPlayed()));
    stats.setTextLargest(Integer.toString(game.getLargestBet()));
    stats.setTextSmallest(Integer.toString(game.getSmallestBet()));
    stats.setTextWon(Integer.toString(game.getTotalWon()));
    stats.setTextLost(Integer.toString(game.getTotalPlayed() - game.getTotalWon()));

}catch (NullPointerException  e){}

    comms.response.writeDOM(stats);
  }

}
