package projectmanagement.business.customer;

import projectmanagement.business.ProjectManagementBusinessException;
import projectmanagement.data.customer.*;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.DataObjectException;

import projectmanagement.spec.customer.*;
/**
 * Represents a customer.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class CustomerImpl implements Customer {
   /**
    * The DO of the Customer.
    */
   protected CustomerDO myDO = null;

   /**
    * The public constructor.
    */
   public CustomerImpl() throws ProjectManagementBusinessException {
      try {
         this.myDO = CustomerDO.createVirgin();
      } catch(DatabaseManagerException ex) {
         throw new ProjectManagementBusinessException("Error creating empty customer", ex);
      } catch(ObjectIdException ex) {
         throw new ProjectManagementBusinessException("Error creating object ID for customer", ex);
      }
   }

   /** The public constructor
    *
    * @param theCustomer. The data object of the customer.
    */
   public CustomerImpl(CustomerDO theCustomer) {
      this.myDO = theCustomer;
   }

   /**
    * Gets the DO object for the customer
    *
    * @return the DO object.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data.
    */
   public CustomerDO getDO()
         throws ProjectManagementBusinessException {
      if (this.myDO!=null) {
         return this.myDO;
      } else {
         throw new ProjectManagementBusinessException("Error getting DO object for customer");
      }
   }

   /**
    * Gets the object id for the customer
    *
    * @return the object id.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getHandle()
         throws ProjectManagementBusinessException {
      try {
         return this.myDO.getHandle();
      } catch(DatabaseManagerException ex) {
         throw new ProjectManagementBusinessException("Error getting handle for customer", ex);
      }
   }

   /**
    * Sets the company name for the customer.
    *
    * @param the company name.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setCompanyName(String companyName)
      throws ProjectManagementBusinessException {
      try {
         myDO.setCompanyName(companyName);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting company name", ex);
      }
   }

   /**
    * Gets the company name for the customer
    *
    * @return the company name.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getCompanyName ()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getCompanyName();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting company name", ex);
      }
   }

   /**
    * Sets the contact name for the customer.
    *
    * @param the contact name.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setContactName(String contactName)
      throws ProjectManagementBusinessException {
      try {
         myDO.setContactName(contactName);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting contact name", ex);
      }
   }

   /**
    * Gets the contact name for the customer
    *
    * @return the contact name.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getContactName ()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getContactName();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting contact name", ex);
      }
   }

   /**
    * Sets the contact title.
    *
    * @param the contact title.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setContactTitle(String contactTitle)
      throws ProjectManagementBusinessException {
      try {
         myDO.setContactTitle(contactTitle);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting contact title", ex);
      }
   }

   /**
    * Gets the contact title
    *
    * @return the contact title.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getContactTitle()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getContactTitle();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting contact title", ex);
      }
   }

   /**
    * Sets the address for the customer.
    *
    * @param the address.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setAddress(String address)
      throws ProjectManagementBusinessException {
      try {
         myDO.setAddress(address);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting address", ex);
      }
   }

   /**
    * Gets the address.
    *
    * @return the address.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getAddress()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getAddress();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting address", ex);
      }
   }

   /**
    * Sets the customer's city.
    *
    * @param the customer's city.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setCity(String city)
      throws ProjectManagementBusinessException {
      try {
         myDO.setCity(city);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting city", ex);
      }
   }

   /**
    * Gets the customer city.
    *
    * @return the customer city.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getCity()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getCity();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting customer city", ex);
      }
   }

   /**
    * Sets the customer's region.
    *
    * @param the customer's region.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setRegion(String region)
      throws ProjectManagementBusinessException {
      try {
         myDO.setRegion(region);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting region", ex);
      }
   }

   /**
    * Gets the customer's region.
    *
    * @return the customer region.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getRegion()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getRegion();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting customer region", ex);
      }
   }

   /**
    * Sets the customer's postal code.
    *
    * @param the customer's postal code.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setPostalCode(String postalCode)
      throws ProjectManagementBusinessException {
      try {
         myDO.setPostalCode(postalCode);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting postal code", ex);
      }
   }

   /**
    * Gets the customer's postal code.
    *
    * @return the customer postal code.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getPostalCode()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getPostalCode();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting customer postal code", ex);
      }
   }

   /**
    * Sets the customer's country.
    *
    * @param the customer's country.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setCountry(String country)
      throws ProjectManagementBusinessException {
      try {
         myDO.setCountry(country);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting country", ex);
      }
   }

   /**
    * Gets the customer's country.
    *
    * @return the customer country.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getCountry()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getCountry();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting customer country", ex);
      }
   }

   /**
    * Sets the customer's phone.
    *
    * @param the customer's phone.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setPhone(String phone)
      throws ProjectManagementBusinessException {
      try {
         myDO.setPhone(phone);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting phone", ex);
      }
   }

   /**
    * Gets the customer's phone.
    *
    * @return the customer phone.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getPhone()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getPhone();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting customer phone", ex);
      }
   }

   /**
    * Sets the customer's fax.
    *
    * @param the customer's fax.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setFax(String fax)
      throws ProjectManagementBusinessException {
      try {
         myDO.setFax(fax);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting fax", ex);
      }
   }

   /**
    * Gets the customer's fax.
    *
    * @return the customer fax.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getFax()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getFax();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting customer fax", ex);
      }
   }

   /**
    * Sets the customer's email.
    *
    * @param the customer's email.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setEmail(String email)
      throws ProjectManagementBusinessException {
      try {
         myDO.setEmail(email);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting email", ex);
      }
   }

   /**
    * Gets the customer's email.
    *
    * @return the customer email.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getEmail()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getEmail();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting customer email", ex);
      }
   }

   /**
    * Sets the customer's notes.
    *
    * @param the customer's notes.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setNotes(String notes)
      throws ProjectManagementBusinessException {
      try {
         myDO.setNotes(notes);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting notes", ex);
      }
   }

   /**
    * Gets the customer's notes.
    *
    * @return the customer notes.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getNotes()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getNotes();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting customer notes", ex);
      }
   }

   /**
    * Commits all changes to the database.
    *
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public void save() throws ProjectManagementBusinessException {
      try {
         this.myDO.save();
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Error saving customer", ex);
      }
   }

   /**
    * Deletes the customer from database.
    *
    * @exception ProjectManagementBusinessException if an error occurs
    *   deleting data (usually due to an underlying data layer
    *   error).
    */
   public void delete() throws ProjectManagementBusinessException {
      try {
         this.myDO.delete();
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Error deleting customer", ex);
      }
   }

}
