package projectmanagement.business.customer;

import projectmanagement.business.ProjectManagementBusinessException;
import projectmanagement.data.customer.*;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectId;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.*;

import projectmanagement.spec.customer.*;
/**
 * Used to find the instance of Customer.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class CustomerManagerImpl implements CustomerManager {

   /**
    * The getAllCustomers method performs a database query to
    * return all <CODE>Customer</CODE> objects representing the
    * row in the <CODE>Customers</CODE> table.
    * @return all the customers, or null if there are no any.
    * @exception ProjectManagementBusinessException
    *    if there is a problem retrieving customer information.
    */
    public Customer[] getAllCustomers()
         throws ProjectManagementBusinessException {
      try {
         CustomerQuery query = new CustomerQuery();

         CustomerDO[] foundCustomers = query.getDOArray();
         if(foundCustomers.length != 0) {
            CustomerImpl[] cs=new CustomerImpl[foundCustomers.length];
            for (int i=0; i<foundCustomers.length; i++) {
               cs[i]=new CustomerImpl(foundCustomers[i]);
            }
            return cs;
         } else {
            return null;
         }
      } catch(NonUniqueQueryException ex) {
         Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique customer found in database: "+ex.getMessage());
         throw new ProjectManagementBusinessException("Non unique customer found");
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Database error retrieving customers: ", ex);
      } /*catch(QueryException ex) {
         throw new ProjectManagementBusinessException("Query exception retrieving customers: ", ex);
      }*/
   }

   /**
    * The findCustomerByID method performs a database query to
    * return a <CODE>Customer</CODE> object
    * representing the row in the <CODE>customer</CODE> table
    * that matches the object id.
    *
    * @param id, the object id of the customer table.
    * @return
    *    the customer. null if there isn't a customer associated
    *    the id
    * @exception ProjectManagementBusinessException
    *    if there is a problem retrieving customer information.
    */
   public Customer findCustomerByID(String id)
         throws ProjectManagementBusinessException {
      CustomerImpl theCustomer = null;

      try {
         CustomerQuery query = new CustomerQuery();
         //set query
         query.setQueryOId(new ObjectId(id));
         // Throw an exception if more than one user by this name is found
         query.requireUniqueInstance();
         CustomerDO theCustomerDO = query.getNextDO();
         theCustomer = new CustomerImpl(theCustomerDO);
         return theCustomer;
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Exception in findCustomerByID()", ex);
      }
   }
   
   public Customer getCustomer()
         throws ProjectManagementBusinessException{
    return new CustomerImpl();
   }
}

