package projectmanagement.business.employee;

import projectmanagement.business.ProjectManagementBusinessException;
import projectmanagement.data.employee.*;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.DataObjectException;

import projectmanagement.spec.employee.*;

import java.sql.Date;

/**
 * Represents an employee.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class EmployeeImpl implements Employee,java.io.Serializable {
   /**
    * The DO of the Employee.
    */
   protected EmployeeDO myDO = null;

   /**
    * The public constructor.
    */
   public EmployeeImpl() throws ProjectManagementBusinessException {
      try {
         this.myDO = EmployeeDO.createVirgin();
      } catch(DatabaseManagerException ex) {
         throw new ProjectManagementBusinessException("Error creating empty employee", ex);
      } catch(ObjectIdException ex) {
         throw new ProjectManagementBusinessException("Error creating object ID for employee", ex);
      }
   }

   /** The public constructor
    *
    * @param theEmployee. The data object of the employee.
    */
   public EmployeeImpl(EmployeeDO theEmployee) {
      this.myDO = theEmployee;
   }

   /**
    * Gets the DO object for the employee
    *
    * @return the DO object.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data.
    */
   public EmployeeDO getDO()
         throws ProjectManagementBusinessException {
      if (this.myDO!=null) {
         return this.myDO;
      } else {
         throw new ProjectManagementBusinessException("Error getting DO object for employee");
      }
   }

   /**
    * Gets the object id for the employee
    *
    * @return the object id.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getHandle()
      throws ProjectManagementBusinessException {
      try {
         return this.myDO.getHandle();
      } catch(DatabaseManagerException ex) {
         throw new ProjectManagementBusinessException("Error getting handle for employee", ex);
      }
   }

   public int getAuthLevel () {
      try {
         boolean isAdmin=getIsAdmin();
         if (isAdmin) {
            return 2;
         } else {
            return 1;
         }
      } catch (Exception ex) {
         return 0;
      }
   }

   /**
    * Sets the login name for the employee.
    *
    * @param the login name.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setLogin(String login)
      throws ProjectManagementBusinessException {
      try {
         myDO.setLogin(login);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting user's login name", ex);
      }
   }

   /**
    * Gets the login name for the employee
    *
    * @return the login name.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getLogin()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getLogin();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting user's login name", ex);
      }
   }

   /**
    * Sets the password for the employee.
    *
    * @param the password.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setPassword(String password)
      throws ProjectManagementBusinessException {
      try {
         myDO.setPassword(password);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting user's password", ex);
      }
   }

   /**
    * Gets the password for the employee
    *
    * @return the password.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getPassword()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getPassword();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting user's password", ex);
      }
   }

   /**
    * Sets the firstname for the employee.
    *
    * @param the firstname.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setFirstName(String firstname)
      throws ProjectManagementBusinessException {
      try {
         myDO.setFirstName(firstname);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting user's first name", ex);
      }
   }

   /**
    * Gets the firstname for the employee
    *
    * @return the firstname.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getFirstName()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getFirstName();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting user's first name", ex);
      }
   }

   /**
    * Sets the lastname for the employee.
    *
    * @param the lastname.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setLastName(String lastname)
      throws ProjectManagementBusinessException {
      try {
         myDO.setLastName(lastname);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting user's last name", ex);
      }
   }

   /**
    * Gets the lastname for the employee
    *
    * @return the lastname.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getLastName()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getLastName();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting user's last name", ex);
      }
   }

   /**
    * Sets the title for the employee.
    *
    * @param the title.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setTitle(String title)
      throws ProjectManagementBusinessException {
      try {
         myDO.setTitle(title);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting user's title", ex);
      }
   }

   /**
    * Gets the title for the employee
    *
    * @return the title.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getTitle()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getTitle();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting user's title", ex);
      }
   }

   /**
    * Sets the title of courtesy for the employee.
    *
    * @param the title of courtesy.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setTitleOfCourtesy(String titleOfCourtesy)
      throws ProjectManagementBusinessException {
      try {
         myDO.setTitleOfCourtesy(titleOfCourtesy);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting user's title of courtesy", ex);
      }
   }

   /**
    * Gets the title of courtesy for the employee
    *
    * @return the title of courtesy.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getTitleOfCourtesy()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getTitleOfCourtesy();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting user's title of courtesy", ex);
      }
   }

   /**
    * Sets the birth date for the employee.
    *
    * @param the birth date.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setBirthDate(Date birthDate)
      throws ProjectManagementBusinessException {
      try {
         myDO.setBirthDate(birthDate);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting user's birth date", ex);
      }
   }

   /**
    * Gets the birth date for the employee
    *
    * @return the birth date.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public Date getBirthDate()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getBirthDate();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting user's birth date", ex);
      }
   }

   /**
    * Sets the hire date for the employee.
    *
    * @param the hire date.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setHireDate(Date hireDate)
      throws ProjectManagementBusinessException {
      try {
         myDO.setHireDate(hireDate);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting user's hire date", ex);
      }
   }

   /**
    * Gets the hire date for the employee
    *
    * @return the hire date.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public Date getHireDate()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getHireDate();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting user's hire date", ex);
      }
   }

   /**
    * Sets the address for the customer.
    *
    * @param the address.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setAddress(String address)
      throws ProjectManagementBusinessException {
      try {
         myDO.setAddress(address);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting address", ex);
      }
   }

   /**
    * Gets the address.
    *
    * @return the address.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getAddress()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getAddress();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting address", ex);
      }
   }

   /**
    * Sets the employee's city.
    *
    * @param the employee's city.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setCity(String city)
      throws ProjectManagementBusinessException {
      try {
         myDO.setCity(city);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting city", ex);
      }
   }

   /**
    * Gets the employee city.
    *
    * @return the employee city.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getCity()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getCity();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting employee city", ex);
      }
   }

   /**
    * Sets the employee's region.
    *
    * @param the employee's region.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setRegion(String region)
      throws ProjectManagementBusinessException {
      try {
         myDO.setRegion(region);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting region", ex);
      }
   }

   /**
    * Gets the employee's region.
    *
    * @return the employee region.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getRegion()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getRegion();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting employee region", ex);
      }
   }

   /**
    * Sets the employee's postal code.
    *
    * @param the employee's postal code.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setPostalCode(String postalCode)
      throws ProjectManagementBusinessException {
      try {
         myDO.setPostalCode(postalCode);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting postal code", ex);
      }
   }

   /**
    * Gets the employee's postal code.
    *
    * @return the employee postal code.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getPostalCode()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getPostalCode();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting employee postal code", ex);
      }
   }

   /**
    * Sets the employee's country.
    *
    * @param the employee's country.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setCountry(String country)
      throws ProjectManagementBusinessException {
      try {
         myDO.setCountry(country);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting country", ex);
      }
   }

   /**
    * Gets the employee's country.
    *
    * @return the employee country.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getCountry()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getCountry();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting employee country", ex);
      }
   }

   /**
    * Sets the employee's home phone.
    *
    * @param the employee's home phone.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setHomePhone(String homePhone)
      throws ProjectManagementBusinessException {
      try {
         myDO.setHomePhone(homePhone);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting home phone", ex);
      }
   }

   /**
    * Gets the employee's home phone.
    *
    * @return the employee home phone.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getHomePhone()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getHomePhone();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting employee home phone", ex);
      }
   }

   /**
    * Sets the employee's mobile phone.
    *
    * @param the employee's mobile phone.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setMobilePhone(String mobilePhone)
      throws ProjectManagementBusinessException {
      try {
         myDO.setMobilePhone(mobilePhone);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting mobile phone", ex);
      }
   }

   /**
    * Gets the employee's mobile phone.
    *
    * @return the employee mobile phone.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getMobilePhone()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getMobilePhone();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting employee mobile phone", ex);
      }
   }

   /**
    * Sets the employee's email.
    *
    * @param the employee's email.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setEmail(String email)
      throws ProjectManagementBusinessException {
      try {
         myDO.setEmail(email);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting email", ex);
      }
   }

   /**
    * Gets the employee's email.
    *
    * @return the employee email.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getEmail()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getEmail();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting employee email", ex);
      }
   }

   /**
    * Sets the employee's notes.
    *
    * @param the employee's notes.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setNotes(String notes)
      throws ProjectManagementBusinessException {
      try {
         myDO.setNotes(notes);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting notes", ex);
      }
   }

   /**
    * Gets the employee's notes.
    *
    * @return the employee notes.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getNotes()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getNotes();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting employee notes", ex);
      }
   }

   /**
    * Sets the flag that indicates if employee is admin.
    *
    * @param isAdmin Indication if employee is admin.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setIsAdmin(boolean isAdmin)
      throws ProjectManagementBusinessException {
      try {
         myDO.setIsAdmin(isAdmin);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting administration flag", ex);
      }
   }

   /**
    * Returns true if the employee is admin.
    *
    * @return true if the employee is admin.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public boolean getIsAdmin()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getIsAdmin();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting indication if th user is admin", ex);
      }
   }

   /**
    * Commits all changes to the database.
    *
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public void save() throws ProjectManagementBusinessException {
      try {
         this.myDO.commit();
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Error saving employee", ex);
      }
   }

   /**
    * Deletes the employee from database.
    *
    * @exception ProjectManagementBusinessException if an error occurs
    *   deleting data (usually due to an underlying data layer
    *   error).
    */
   public void delete() throws ProjectManagementBusinessException {
      try {
         this.myDO.delete();
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Error deleting employee", ex);
      }
   }

}
