package projectmanagement.presentation;

import projectmanagement.spec.employee.*;
import projectmanagement.spec.*;
import projectmanagement.ProjectManagement;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.XMLObject;
import projectmanagement.spec.ProjectManagementException;
import projectmanagement.presentation.ProjectManagementPresentationException;

/**
 * Login.java handles the login functionality of the ProjectManagement app.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class Login extends BasePO {

   /**
    * Constants
    */
   private static String LOGIN_NAME = "login";
   private static String PASSWORD_NAME = "password";
   private static String INDEX_PAGE = "Index.po";

   /**
    * Superclass method override. Returns 0.
    */
   protected int getRequiredAuthLevel() {
      return 0;
   }

   /**
    *  Default event. Just show the page.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {
      return showPage(null);
   }

   /**
    * Process login data
    *
    * @return wml document
    * @exception HttpPresentationException
    */
   public XMLObject handleLogin()
         throws HttpPresentationException {
      String login = this.getComms().request.getParameter(LOGIN_NAME);
      String password = this.getComms().request.getParameter(PASSWORD_NAME);
      Employee user = null;

      ProjectManagement pm=getApplication();
      boolean isAdmin=false;
      if (pm.getAdminUsername().equals(login) && pm.getAdminPassword().equals(password)) {
         isAdmin=true;
      }
      this.getSessionData().setAdmin(isAdmin);
      try {
     
     EmployeeManager employeeManager = EmployeeManagerFactory.getEmployeeManager("projectmanagement.business.employee.EmployeeManagerImpl");
     user = employeeManager.findEmployee(login);
     
         if(!isAdmin && (null == user || !user.getPassword().equals(password))) {
            return showPage("Invalid username or password");
            // Show error message that user not found (bad username/password)
         } else {
            this.setUser(user);
            throw new ClientPageRedirectException(INDEX_PAGE);
         }
/*
 * We need  to allow ProjectManagement_pres to be functional , we must allow user to acsses next page if he run ProjectManagement_pres
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * 
 */
      }catch(NullPointerException e){
      throw new ClientPageRedirectException(INDEX_PAGE);
            
      } catch(ProjectManagementException ex) {
         this.writeDebugMsg("System error finding user: " + ex.getMessage());
         throw new ProjectManagementPresentationException("System error finding user", ex);
       
    }
   }

   /**
    * display page
    *
    * @param errorMsg, the error messages
    * @return html document
    */
   public XMLObject showPage(String errorMsg) {

	   LoginHTML page = new LoginHTML();

      if(null != errorMsg ||
            null != (errorMsg = this.getSessionData().getAndClearUserMessage())) {
         page.setTextLblErrorText(errorMsg);
      } else {
         page.getElementLblErrorText().getParentNode().removeChild(page.getElementLblErrorText());
      }

      return page;
   }
}


