package projectmanagement.presentation;

import projectmanagement.spec.employee.Employee;

/**
 * Object that will be held in session data.  This should
 * be the only object held there.  Methods should be called
 * on this object to set and get data.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class ProjectManagementSessionData implements java.io.Serializable {

   /**
    * Hash key to save session data for the ProjectManagement app in the Session
    */
   public static final String SESSION_KEY = "ProjectManagementSessionData";

   protected Employee myUser = null;
   protected String userMessage = null;
   protected boolean isAdmin=false;

   /**
    * Sets the employee object
    *
    * @param theEmployee the employee object
    */
   public void setUser(Employee theEmployee) {
      this.myUser = theEmployee;
   }

   /**
    * Gets the employee object
    *
    * @return employee
    */
   public Employee getUser() {
      return this.myUser;
   }

   /**
    * Method to remove the current user from the session
    */
   public void removeUser() {
      this.myUser = null;
   }

   /**
    * Sets the message
    *
    * @param msg the message to be set
    */
   public void setUserMessage(String msg) {
      this.userMessage = msg;
   }

   public void setAdmin (boolean isAdmin) {
      this.isAdmin=isAdmin;
   }

   public int getAuthLevel() {
      if (isAdmin) {
         return 2;
      } else if (myUser!=null) {
         return myUser.getAuthLevel();
      } else {
         return 0;
      }
   }

   /**
    * Retrieve the most recent user message and then clear it so no
    * other app tries to use it.
    */
   public String getAndClearUserMessage() {
      String msg = this.userMessage;
      this.userMessage = null;
      return msg;
   }
}
