package projectmanagement.presentation.customers;

import projectmanagement.presentation.*;
import projectmanagement.spec.customer.*;
import com.lutris.appserver.server.httpPresentation.*;
import org.enhydra.xml.xmlc.XMLObject;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

/**
 * Shows a list of customers to administer.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class Administering extends BasePO {

   /**
    * Superclass method override. Returns 2.
    */
   protected int getRequiredAuthLevel() {
      return 2;
   }

   /**
    *  Default event. Just show the page.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {

	   AdministeringHTML page = new AdministeringHTML();

      String errorMsg = this.getSessionData().getAndClearUserMessage();
      /*if(null != errorMsg) {
         page.setTextLblErrorText(errorMsg);
      } else {
         page.getElementLblErrorText().getParentNode().removeChild(page.getElementLblErrorText());
      }*/

    
      try {
       
     CustomerManager customerManager = CustomerManagerFactory.getCustomerManager("projectmanagement.business.customer.CustomerManagerImpl");
     Customer[] customers = customerManager.getAllCustomers();

  
    // fetches the table element from HTML document
      HTMLTableElement table = page.getElementTblCustomers();

  
      if (customers!=null) {
            for (int i=0; i<customers.length; i++) {
               // creating row in HTML table which describes table properties
               HTMLTableRowElement htmlRE=createNewRow(page,customers[i]);
               // appending row at the end of table
               table.appendChild(htmlRE);
            }
         }
      //Remove template selection row from table
      table.removeChild(page.getElementTemplateRow());
      
      
      /*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page 
 */ 
      } catch(NullPointerException ex) {
      } catch(Exception ex) {
         this.writeDebugMsg("Error populating list of customers: " + ex);
         throw new ProjectManagementPresentationException("Error getting list of customers: ", ex);
      }

      

      // write out HTML
      return page;
   }

   /**
   * Creates the new row of HTML table element, based on given parameters.
   */
   private HTMLTableRowElement createNewRow(AdministeringHTML administeringHTML,
         Customer customer) {

      String customerID="";

      try {
         customerID=customer.getHandle();
         administeringHTML.setTextTxtCompanyName(customer.getCompanyName());
         administeringHTML.setTextTxtContactName(customer.getContactName());
         administeringHTML.setTextTxtPhone(customer.getPhone());
         administeringHTML.setTextTxtEmail(customer.getEmail());
      } catch (Exception ex) {}

      // image to get details on customer
      HTMLImageElement detailsImg=administeringHTML.getElementImgDetails();
      detailsImg.setName(customerID);

      // image to modify customer information
      HTMLImageElement modifyImg=administeringHTML.getElementImgModify();
      modifyImg.setName(customerID);

      // image to delete customer
      HTMLImageElement deleteImg=administeringHTML.getElementImgDelete();
      deleteImg.setName(customerID);

      // image to show customer projects
      HTMLImageElement projectImg=administeringHTML.getElementImgProject();
      projectImg.setName(customerID);

      HTMLTableRowElement row=(HTMLTableRowElement)administeringHTML.
         getElementTemplateRow().cloneNode(true);

      return row;
   }

}

