package projectmanagement.presentation.customers;

import projectmanagement.presentation.*;
import projectmanagement.spec.*;
import projectmanagement.spec.customer.*;
import com.lutris.appserver.server.httpPresentation.*;
import org.enhydra.xml.xmlc.XMLObject;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

/**
 * Manages all actions on customers.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class Edit extends BasePO {

   /**
    * Constants
    */
   private static String COMPANY_NAME = "txtCompanyName";
   private static String CONTACT_NAME = "txtContactName";
   private static String CONTACT_TITLE = "txtContactTitle";
   private static String ADDRESS = "txtAddress";
   private static String CITY = "txtCity";
   private static String REGION = "txtRegion";
   private static String POSTAL_CODE = "txtPostalCode";
   private static String COUNTRY = "txtCountry";
   private static String PHONE = "txtPhone";
   private static String FAX = "txtFax";
   private static String EMAIL = "txtEmail";
   private static String NOTES = "txtNotes";

   private static String CUSTOMER_ID = "customerID";
   private static String CONTEXT_PAGE = "Context.po";
   private static String ADD = "add";
   private static String DELETE = "delete";
   private static String MODIFY = "modify";
   private static String DETAILS = "showDetailsPage";
   private static String EVENT = "event";

   /**
    * Superclass method override. Returns 1.
    */
   protected int getRequiredAuthLevel() {
      String event="";
      try {
         event=this.getComms().request.getParameter(EVENT);
      } catch (Exception ex) {}

      if (event!=null && event.equalsIgnoreCase(DETAILS)) {
         return 1;
      } else {
         return 2;
      }
   }

   /**
    *  Default event. Just show the page for editing.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {
      return showModifyPage(null,false);
   }

   /**
    * handle show add customer page event.
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleShowAddPage()
      throws HttpPresentationException {
      return showAddPage(null);
   }

   /**
    * handle show details customer page event.
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleShowDetailsPage ()
         throws HttpPresentationException {
      return showModifyPage(null,true);
   }

   /*
    * Modifies an existing customer
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleModify()
         throws HttpPresentationException {
      String customerID = this.getComms().request.getParameter(CUSTOMER_ID);
      Customer customer=null;

      // Try to get the customer by its ID
      try {
       
      CustomerManager customerManager = CustomerManagerFactory.getCustomerManager("projectmanagement.business.customer.CustomerManagerImpl");
      customer = customerManager.findCustomerByID(customerID);
    
      } catch(Exception ex) {
         this.getSessionData().setUserMessage("Please choose a valid customer to edit");
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      }

      try {
         saveCustomer(customer);
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      } catch(Exception ex) {
         return showModifyPage("To modify this customer, you must fill out fields properly!",false);
      }
   }

   /*
    * Adds a customer to the customer database
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleAdd()
         throws HttpPresentationException {
      try {
          CustomerManager customerManager = CustomerManagerFactory.getCustomerManager("projectmanagement.business.customer.CustomerManagerImpl");
           Customer customer = customerManager.getCustomer();   
   
         saveCustomer(customer);
         throw new ClientPageRedirectException(CONTEXT_PAGE);
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page 
 */ 
 
      } catch(NullPointerException ex) {
          return showAddPage("You cannot add customer while runing ProjectManagement_pres");
      } catch(Exception ex) {
         return showAddPage("To add this customer, you must fill out fields properly!");
      }
   }

   /*
    * Deletes a Customer from the Customer database
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleDelete()
         throws HttpPresentationException, ProjectManagementPresentationException {
      String customerID = this.getComms().request.getParameter(CUSTOMER_ID);

      try {
CustomerManager customerManager = CustomerManagerFactory.getCustomerManager("projectmanagement.business.customer.CustomerManagerImpl");
      Customer customer = customerManager.findCustomerByID(customerID);
      
      
         String companyName = customer.getCompanyName();
         customer.delete();
         this.getSessionData().setUserMessage("The customer, " + companyName + ", was deleted");
         // Catch any possible database exception as well as the null pointer
         //  exception if the customer is not found and is null after findCustomerByID
      } catch(Exception ex) {
         this.getSessionData().setUserMessage("Please choose a valid customer to delete");
      }
      // Redirect to the catalog page which will display the error message,
      //  if there was one set by the above exception
      throw new ClientPageRedirectException(CONTEXT_PAGE);
   }

   /**
    * Produce HTML for this PO, populated by the customer information
    * that the user wants to edit
    *
    * @param errorMsg, the error messages
    * @param disabled, if controls are disabled
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject showModifyPage(String errorMsg,boolean disabled)
         throws HttpPresentationException, ProjectManagementPresentationException {
      String companyName = this.getComms().request.getParameter(COMPANY_NAME);
      String contactName = this.getComms().request.getParameter(CONTACT_NAME);
      String contactTitle = this.getComms().request.getParameter(CONTACT_TITLE);
      String address = this.getComms().request.getParameter(ADDRESS);
      String city = this.getComms().request.getParameter(CITY);
      String region = this.getComms().request.getParameter(REGION);
      String postalCode = this.getComms().request.getParameter(POSTAL_CODE);
      String country = this.getComms().request.getParameter(COUNTRY);
      String phone = this.getComms().request.getParameter(PHONE);
      String fax = this.getComms().request.getParameter(FAX);
      String email = this.getComms().request.getParameter(EMAIL);
      String notes = this.getComms().request.getParameter(NOTES);

      String customerID = this.getComms().request.getParameter(CUSTOMER_ID);
      // Instantiate the page object
      EditHTML page = new EditHTML();

      Customer customer = null;

      try {
    CustomerManager customerManager = CustomerManagerFactory.getCustomerManager("projectmanagement.business.customer.CustomerManagerImpl");
       customer = customerManager.findCustomerByID(customerID);
  /*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page with message
 */ 
 
      } catch(NullPointerException ex) {
           page.setTextErrorText("This is a default HTML page");
           return page;
         // Catch any possible database exception in findCustomerByID()
      } catch(ProjectManagementException ex) {
         this.getSessionData().setUserMessage("Please choose a valid customer to edit");
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      }

      try {
         // If we received a valid customerID then try to show the customer's contents,
         // otherwise try to use the HTML input parameters
         page.getElementCustomerID().setValue(customer.getHandle());

         HTMLInputElement el=page.getElementTxtCompanyName();
         el.setDisabled(disabled);
         if (null != companyName && companyName.length() != 0) {
            el.setValue(companyName);
         } else {
            el.setValue(customer.getCompanyName());
         }

         el=page.getElementTxtContactName();
         el.setDisabled(disabled);
         if (null != contactName && contactName.length() != 0) {
            el.setValue(contactName);
         } else {
            el.setValue(customer.getContactName());
         }

         el=page.getElementTxtContactTitle();
         el.setDisabled(disabled);
         if (null != contactTitle && contactTitle.length() != 0) {
            el.setValue(contactTitle);
         } else {
            el.setValue(customer.getContactTitle());
         }

         el=page.getElementTxtAddress();
         el.setDisabled(disabled);
         if (null != address && address.length() != 0) {
            el.setValue(address);
         } else {
            el.setValue(customer.getAddress());
         }

         el=page.getElementTxtCity();
         el.setDisabled(disabled);
         if (null != city && city.length() != 0) {
            el.setValue(city);
         } else {
            el.setValue(customer.getCity());
         }

         el=page.getElementTxtRegion();
         el.setDisabled(disabled);
         if (null != region && region.length() != 0) {
            el.setValue(region);
         } else {
            el.setValue(customer.getRegion());
         }

         el=page.getElementTxtPostalCode();
         el.setDisabled(disabled);
         if (null != postalCode && postalCode.length() != 0) {
            el.setValue(postalCode);
         } else {
            el.setValue(customer.getPostalCode());
         }

         el=page.getElementTxtCountry();
         el.setDisabled(disabled);
         if (null != country && country.length() != 0) {
            el.setValue(country);
         } else {
            el.setValue(customer.getCountry());
         }

         el=page.getElementTxtPhone();
         el.setDisabled(disabled);
         if (null != phone && phone.length() != 0) {
            el.setValue(phone);
         } else {
            el.setValue(customer.getPhone());
         }

         el=page.getElementTxtFax();
         el.setDisabled(disabled);
         if (null != fax && fax.length() != 0) {
            el.setValue(fax);
         } else {
            el.setValue(customer.getFax());
         }

         el=page.getElementTxtEmail();
         el.setDisabled(disabled);
         if (null != email && email.length() != 0) {
            el.setValue(email);
         } else {
            el.setValue(customer.getEmail());
         }

         //HTMLTextAreaElement tael=page.getElementTxtNotes();
         el=page.getElementTxtNotes();
         el.setDisabled(disabled);
         if (null != notes && notes.length() != 0) {
            el.setValue(notes);
         } else {
            el.setValue(customer.getNotes());
         }

         el=page.getElementBtnSave();
         el.setDisabled(disabled);

         if (null == errorMsg) {
            page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
         } else {
            page.setTextErrorText(errorMsg);
         }
      } catch(ProjectManagementException ex) {
         throw new ProjectManagementPresentationException("Error populating page for customer editing", ex);
      }

      page.getElementEvent().setValue(MODIFY);
      return page;
   }

   /**
    * Produce HTML for this PO
    *
    * @param errorMsg, the error messages
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject showAddPage(String errorMsg)
         throws HttpPresentationException, ProjectManagementPresentationException {

      String companyName = this.getComms().request.getParameter(COMPANY_NAME);
      String contactName = this.getComms().request.getParameter(CONTACT_NAME);
      String contactTitle = this.getComms().request.getParameter(CONTACT_TITLE);
      String address = this.getComms().request.getParameter(ADDRESS);
      String city = this.getComms().request.getParameter(CITY);
      String region = this.getComms().request.getParameter(REGION);
      String postalCode = this.getComms().request.getParameter(POSTAL_CODE);
      String country = this.getComms().request.getParameter(COUNTRY);
      String phone = this.getComms().request.getParameter(PHONE);
      String fax = this.getComms().request.getParameter(FAX);
      String email = this.getComms().request.getParameter(EMAIL);
      String notes = this.getComms().request.getParameter(NOTES);

      // Instantiate the page object
      EditHTML page = new EditHTML();

      HTMLInputElement el=page.getElementTxtCompanyName();
      if (null != companyName) {
         el.setValue(companyName);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtContactName();
      if (null != contactName) {
         el.setValue(contactName);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtContactTitle();
      if (null != contactTitle) {
         el.setValue(contactTitle);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtAddress();
      if (null != address) {
         el.setValue(address);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtCity();
      if (null != city) {
         el.setValue(city);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtRegion();
      if (null != region) {
         el.setValue(region);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtPostalCode();
      if (null != postalCode) {
         el.setValue(postalCode);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtCountry();
      if (null != country) {
         el.setValue(country);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtPhone();
      if (null != phone) {
         el.setValue(phone);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtFax();
      if (null != fax) {
         el.setValue(fax);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtEmail();
      if (null != email) {
         el.setValue(email);
      } else {
         el.setValue("");
      }

      //HTMLTextAreaElement tael=page.getElementTxtNotes();
      el=page.getElementTxtNotes();
      if (null != notes) {
         el.setValue(notes);
      } else {
         el.setValue("");
      }

      if (null == errorMsg) {
         page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
      } else {
         page.setTextErrorText(errorMsg);
      }

      return page;
   }

   /**
    * Method to save a new or existing customer to the database
    *
    * @param customer, the customer to be saved
    * @return html document
    * @exception HttpPresentationException
    */
   protected void saveCustomer(Customer theCustomer)
         throws HttpPresentationException {
      String companyName = this.getComms().request.getParameter(COMPANY_NAME);
      String contactName = this.getComms().request.getParameter(CONTACT_NAME);
      String contactTitle = this.getComms().request.getParameter(CONTACT_TITLE);
      String address = this.getComms().request.getParameter(ADDRESS);
      String city = this.getComms().request.getParameter(CITY);
      String region = this.getComms().request.getParameter(REGION);
      String postalCode = this.getComms().request.getParameter(POSTAL_CODE);
      String country = this.getComms().request.getParameter(COUNTRY);
      String phone = this.getComms().request.getParameter(PHONE);
      String fax = this.getComms().request.getParameter(FAX);
      String email = this.getComms().request.getParameter(EMAIL);
      String notes = this.getComms().request.getParameter(NOTES);

      if (isNullField(companyName) || isNullField(contactName)) {
         throw new ProjectManagementPresentationException("Error adding customer",new Exception());
      }

      try {
         theCustomer.setCompanyName(companyName);
         theCustomer.setContactName(contactName);
         theCustomer.setContactTitle(contactTitle);
         theCustomer.setAddress(address);
         theCustomer.setCity(city);
         theCustomer.setRegion(region);
         theCustomer.setPostalCode(postalCode);
         theCustomer.setCountry(country);
         theCustomer.setPhone(phone);
         theCustomer.setFax(fax);
         theCustomer.setEmail(email);
         theCustomer.setNotes(notes);

         theCustomer.save();
      } catch(Exception ex) {
         throw new ProjectManagementPresentationException("Error adding customer", ex);
      }
   }

}

