package projectmanagement.presentation.employees;

import projectmanagement.*;
import projectmanagement.presentation.*;

import projectmanagement.spec.employee.*;
import projectmanagement.spec.*;
import com.lutris.appserver.server.httpPresentation.*;
import org.enhydra.xml.xmlc.XMLObject;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import java.sql.Date;
import java.util.*;

/**
 * Manages all actions on employees.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class Edit extends BasePO {

   /**
    * Constants
    */
   private static String FIRST_NAME = "txtFirstName";
   private static String LAST_NAME = "txtLastName";
   private static String LOGIN = "txtLogin";
   private static String PASSWORD = "txtPassword";
   private static String TITLE = "txtTitle";
   private static String TITLE_OF_COURTESY = "txtTitleOfCourtesy";
   private static String BIRTH_DATE_YYYY = "txtBirthDateYYYY";
   private static String BIRTH_DATE_MM = "txtBirthDateMM";
   private static String BIRTH_DATE_DD = "txtBirthDateDD";
   private static String HIRE_DATE_YYYY = "txtHireDateYYYY";
   private static String HIRE_DATE_MM = "txtHireDateMM";
   private static String HIRE_DATE_DD = "txtHireDateDD";
   private static String ADDRESS = "txtAddress";
   private static String CITY = "txtCity";
   private static String REGION = "txtRegion";
   private static String POSTAL_CODE = "txtPostalCode";
   private static String COUNTRY = "txtCountry";
   private static String HOME_PHONE = "txtHomePhone";
   private static String MOBILE_PHONE = "txtMobilePhone";
   private static String EMAIL = "txtEmail";
   private static String NOTES = "txtNotes";
   private static String IS_ADMIN = "cbxIsAdmin";

   private static String EMPLOYEE_ID = "employeeID";
   private static String CONTEXT_PAGE = "Context.po";
   private static String ADD = "add";
   private static String DELETE = "delete";
   private static String MODIFY = "modify";
   private static String DETAILS = "showDetailsPage";
   private static String EVENT = "event";

   /**
    * Superclass method override. Returns 1.
    */
   protected int getRequiredAuthLevel() {
      String event="";
      try {
         event=this.getComms().request.getParameter(EVENT);
      } catch (Exception ex) {}

      if (event!=null && event.equalsIgnoreCase(DETAILS)) {
         return 1;
      } else {
         return 2;
      }
   }

   /**
    *  Default event. Just show the page for editing.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {
      return showModifyPage(null,false);
   }

   /**
    * handle show add employee page event.
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleShowAddPage()
      throws HttpPresentationException {
      return showAddPage(null);
   }

   /**
    * handle show details employee page event.
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleShowDetailsPage ()
         throws HttpPresentationException {
      return showModifyPage(null,true);
   }

   /*
    * Modifies an existing employee
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleModify()
         throws HttpPresentationException {
      String employeeID = this.getComms().request.getParameter(EMPLOYEE_ID);
      Employee employee=null;

      // Try to get the employee by its ID
      try {
  
    EmployeeManager employeeManager = EmployeeManagerFactory.getEmployeeManager("projectmanagement.business.employee.EmployeeManagerImpl");
    employee = employeeManager.findEmployeeByID(employeeID);
   
      } catch(Exception ex) {
         this.getSessionData().setUserMessage("Please choose a valid employee to edit");
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      }

      try {
         // check if the username already exists
         if (!isExistingLogin(employee)) {
            saveEmployee(employee);
            throw new ClientPageRedirectException(CONTEXT_PAGE);
         } else {
            return showModifyPage("The login you've entered already exists !",false);
         }
      } catch(Exception ex) {
         return showModifyPage("To modify this employee, you must fill out fields properly!",false);
      }
   }

   /*
    * Adds a employee to the employee database
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleAdd()
         throws HttpPresentationException {
      try {
   
         EmployeeManager employeeManager = EmployeeManagerFactory.getEmployeeManager("projectmanagement.business.employee.EmployeeManagerImpl");
         Employee employee = employeeManager.getEmployee();
      
         // check if the username already exists
         if (!isExistingLogin(employee)) {
            saveEmployee(employee);
            throw new ClientPageRedirectException(CONTEXT_PAGE);
         } else {
            return showAddPage("The login you've entered already exists !");
         }
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page 
 */ 
     } catch(NullPointerException ex) {
          return showAddPage("You cannot add employee while runing ProjectManagement_pres");
     } catch(Exception ex) {
         return showAddPage("To add this employee, you must fill out fields properly!");
      }
   }

   /*
    * Deletes a Employee from the Employee database
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleDelete()
         throws HttpPresentationException, ProjectManagementPresentationException {
      String employeeID = this.getComms().request.getParameter(EMPLOYEE_ID);

      try {
    
     EmployeeManager employeeManager = EmployeeManagerFactory.getEmployeeManager("projectmanagement.business.employee.EmployeeManagerImpl");
     Employee employee = employeeManager.findEmployeeByID(employeeID);
     
         String fullName = employee.getFirstName() + " " + employee.getLastName();
         employee.delete();
         this.getSessionData().setUserMessage("The employee, " + fullName + ", was deleted");
         // Catch any possible database exception as well as the null pointer
         //  exception if the employee is not found and is null after findEmployeeByID
      } catch(Exception ex) {
         this.getSessionData().setUserMessage("Please choose a valid employee to delete");
      }
      // Redirect to the catalog page which will display the error message,
      //  if there was one set by the above exception
      throw new ClientPageRedirectException(CONTEXT_PAGE);
   }

   /**
    * Produce HTML for this PO, populated by the employee information
    * that the user wants to edit
    *
    * @param errorMsg, the error messages
    * @param disabled, if controls are disabled
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject showModifyPage(String errorMsg,boolean disabled)
         throws HttpPresentationException, ProjectManagementPresentationException {

      String firstName = this.getComms().request.getParameter(FIRST_NAME);
      String lastName = this.getComms().request.getParameter(LAST_NAME);
      String login = this.getComms().request.getParameter(LOGIN);
      String password = this.getComms().request.getParameter(PASSWORD);
      String title = this.getComms().request.getParameter(TITLE);
      String titleOfCourtesy = this.getComms().request.getParameter(TITLE_OF_COURTESY);
      String birthDateYYYY = this.getComms().request.getParameter(BIRTH_DATE_YYYY);
      String birthDateMM = this.getComms().request.getParameter(BIRTH_DATE_MM);
      String birthDateDD = this.getComms().request.getParameter(BIRTH_DATE_DD);
      String hireDateYYYY = this.getComms().request.getParameter(HIRE_DATE_YYYY);
      String hireDateMM = this.getComms().request.getParameter(HIRE_DATE_MM);
      String hireDateDD = this.getComms().request.getParameter(HIRE_DATE_DD);
      String address = this.getComms().request.getParameter(ADDRESS);
      String city = this.getComms().request.getParameter(CITY);
      String region = this.getComms().request.getParameter(REGION);
      String postalCode = this.getComms().request.getParameter(POSTAL_CODE);
      String country = this.getComms().request.getParameter(COUNTRY);
      String homePhone = this.getComms().request.getParameter(HOME_PHONE);
      String mobilePhone = this.getComms().request.getParameter(MOBILE_PHONE);
      String email = this.getComms().request.getParameter(EMAIL);
      String notes = this.getComms().request.getParameter(NOTES);
      String isAdmin = this.getComms().request.getParameter(IS_ADMIN);

      String employeeID = this.getComms().request.getParameter(EMPLOYEE_ID);
      // Instantiate the page object
      EditHTML page = new EditHTML();

      Employee employee = null;

      try {
  
       EmployeeManager employeeManager = EmployeeManagerFactory.getEmployeeManager("projectmanagement.business.employee.EmployeeManagerImpl");
       employee = employeeManager.findEmployeeByID(employeeID);
           
      
  /*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page with message
 */ 
       } catch(NullPointerException ex) {
           page.setTextErrorText("This is a default HTML page");
           return page;
        // Catch any possible database exception in findEmployeeByID()
      } catch(ProjectManagementException ex) {
         this.getSessionData().setUserMessage("Please choose a valid employee to edit");
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      }

      try {
         // If we received a valid employeeID then try to show the employee's contents,
         // otherwise try to use the HTML input parameters
         page.getElementEmployeeID().setValue(employee.getHandle());

         HTMLInputElement el=page.getElementTxtFirstName();
         el.setDisabled(disabled);
         if (null != firstName && firstName.length() != 0) {
            el.setValue(firstName);
         } else {
            el.setValue(employee.getFirstName());
         }

         el=page.getElementTxtLastName();
         el.setDisabled(disabled);
         if (null != lastName && lastName.length() != 0) {
            el.setValue(lastName);
         } else {
            el.setValue(employee.getLastName());
         }

         el=page.getElementTxtLogin();
         el.setDisabled(disabled);
         if (!disabled) {
            if (null != login && login.length() != 0) {
               el.setValue(login);
            } else {
               el.setValue(employee.getLogin());
            }
         } else {
            el.getParentNode().getParentNode().getParentNode().removeChild(el.getParentNode().getParentNode());
         }

         el=page.getElementTxtPassword();
         el.setDisabled(disabled);
         if (!disabled) {
            if (null != password && password.length() != 0) {
               el.setValue(password);
            } else {
               el.setValue(employee.getPassword());
            }
         } else {
            el.getParentNode().getParentNode().getParentNode().removeChild(el.getParentNode().getParentNode());
         }


         el=page.getElementTxtTitle();
         el.setDisabled(disabled);
         if (null != title && title.length() != 0) {
            el.setValue(title);
         } else {
            el.setValue(employee.getTitle());
         }

         el=page.getElementTxtTitleOfCourtesy();
         el.setDisabled(disabled);
         if (null != titleOfCourtesy && titleOfCourtesy.length() != 0) {
            el.setValue(titleOfCourtesy);
         } else {
            el.setValue(employee.getTitleOfCourtesy());
         }

         el=page.getElementTxtBirthDateYYYY();
         el.setDisabled(disabled);
         if (null != birthDateYYYY && birthDateYYYY.length() != 0) {
            el.setValue(birthDateYYYY);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(employee.getBirthDate());
            el.setValue(String.valueOf(cal.get(Calendar.YEAR)));
         }

         el=page.getElementTxtBirthDateMM();
         el.setDisabled(disabled);
         if (null != birthDateMM && birthDateMM.length() != 0) {
            el.setValue(birthDateMM);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(employee.getBirthDate());
            el.setValue(String.valueOf(cal.get(Calendar.MONTH)+1));
         }

         el=page.getElementTxtBirthDateDD();
         el.setDisabled(disabled);
         if (null != birthDateDD && birthDateDD.length() != 0) {
            el.setValue(birthDateDD);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(employee.getBirthDate());
            el.setValue(String.valueOf(cal.get(Calendar.DAY_OF_MONTH)));
         }

         el=page.getElementTxtHireDateYYYY();
         el.setDisabled(disabled);
         if (null != hireDateYYYY && hireDateYYYY.length() != 0) {
            el.setValue(hireDateYYYY);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(employee.getHireDate());
            el.setValue(String.valueOf(cal.get(Calendar.YEAR)));
         }

         el=page.getElementTxtHireDateMM();
         el.setDisabled(disabled);
         if (null != hireDateMM && hireDateMM.length() != 0) {
            el.setValue(hireDateMM);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(employee.getHireDate());
            el.setValue(String.valueOf(cal.get(Calendar.MONTH)+1));
         }

         el=page.getElementTxtHireDateDD();
         el.setDisabled(disabled);
         if (null != hireDateDD && hireDateDD.length() != 0) {
            el.setValue(hireDateDD);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(employee.getHireDate());
            el.setValue(String.valueOf(cal.get(Calendar.DAY_OF_MONTH)));
         }

         el=page.getElementTxtAddress();
         el.setDisabled(disabled);
         if (null != address && address.length() != 0) {
            el.setValue(address);
         } else {
            el.setValue(employee.getAddress());
         }

         el=page.getElementTxtCity();
         el.setDisabled(disabled);
         if (null != city && city.length() != 0) {
            el.setValue(city);
         } else {
            el.setValue(employee.getCity());
         }

         el=page.getElementTxtRegion();
         el.setDisabled(disabled);
         if (null != region && region.length() != 0) {
            el.setValue(region);
         } else {
            el.setValue(employee.getRegion());
         }

         el=page.getElementTxtPostalCode();
         el.setDisabled(disabled);
         if (null != postalCode && postalCode.length() != 0) {
            el.setValue(postalCode);
         } else {
            el.setValue(employee.getPostalCode());
         }

         el=page.getElementTxtCountry();
         el.setDisabled(disabled);
         if (null != country && country.length() != 0) {
            el.setValue(country);
         } else {
            el.setValue(employee.getCountry());
         }

         el=page.getElementTxtHomePhone();
         el.setDisabled(disabled);
         if (null != homePhone && homePhone.length() != 0) {
            el.setValue(homePhone);
         } else {
            el.setValue(employee.getHomePhone());
         }

         el=page.getElementTxtMobilePhone();
         el.setDisabled(disabled);
         if (null != mobilePhone && mobilePhone.length() != 0) {
            el.setValue(mobilePhone);
         } else {
            el.setValue(employee.getMobilePhone());
         }

         el=page.getElementTxtEmail();
         el.setDisabled(disabled);
         if (null != email && email.length() != 0) {
            el.setValue(email);
         } else {
            el.setValue(employee.getEmail());
         }

         //HTMLTextAreaElement tael=page.getElementTxtNotes();
         el=page.getElementTxtNotes();
         el.setDisabled(disabled);
         if (null != notes && notes.length() != 0) {
            el.setValue(notes);
         } else {
            el.setValue(employee.getNotes());
         }

         el=page.getElementCbxIsAdmin();
         el.setDisabled(disabled);
         if (null != isAdmin) {
            el.setChecked(true);
         } else {
            el.setChecked(false);
         }

         el=page.getElementBtnSave();
         el.setDisabled(disabled);

         if (null == errorMsg) {
            page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
         } else {
            page.setTextErrorText(errorMsg);
         }
      } catch(ProjectManagementException ex) {
         throw new ProjectManagementPresentationException("Error populating page for employee editing", ex);
      }

      page.getElementEvent().setValue(MODIFY);
      return page;
   }

   /**
    * Produce HTML for this PO
    *
    * @param errorMsg, the error messages
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject showAddPage(String errorMsg)
         throws HttpPresentationException, ProjectManagementPresentationException {
	
      String firstName = this.getComms().request.getParameter(FIRST_NAME);
      String lastName = this.getComms().request.getParameter(LAST_NAME);
      String login = this.getComms().request.getParameter(LOGIN);
      String password = this.getComms().request.getParameter(PASSWORD);
      String title = this.getComms().request.getParameter(TITLE);
      String titleOfCourtesy = this.getComms().request.getParameter(TITLE_OF_COURTESY);
      String birthDateYYYY = this.getComms().request.getParameter(BIRTH_DATE_YYYY);
      String birthDateMM = this.getComms().request.getParameter(BIRTH_DATE_MM);
      String birthDateDD = this.getComms().request.getParameter(BIRTH_DATE_DD);
      String hireDateYYYY = this.getComms().request.getParameter(HIRE_DATE_YYYY);
      String hireDateMM = this.getComms().request.getParameter(HIRE_DATE_MM);
      String hireDateDD = this.getComms().request.getParameter(HIRE_DATE_DD);
      String address = this.getComms().request.getParameter(ADDRESS);
      String city = this.getComms().request.getParameter(CITY);
      String region = this.getComms().request.getParameter(REGION);
      String postalCode = this.getComms().request.getParameter(POSTAL_CODE);
      String country = this.getComms().request.getParameter(COUNTRY);
      String homePhone = this.getComms().request.getParameter(HOME_PHONE);
      String mobilePhone = this.getComms().request.getParameter(MOBILE_PHONE);
      String email = this.getComms().request.getParameter(EMAIL);
      String notes = this.getComms().request.getParameter(NOTES);
      String isAdmin = this.getComms().request.getParameter(IS_ADMIN);

      // Instantiate the page object
      EditHTML page = new EditHTML();

      HTMLInputElement el=page.getElementTxtFirstName();
      if (null != firstName) {
         el.setValue(firstName);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtLastName();
      if (null != lastName) {
         el.setValue(lastName);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtLogin();
      if (null != login) {
         el.setValue(login);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtPassword();
      if (null != password) {
         el.setValue(password);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtTitle();
      if (null != title) {
         el.setValue(title);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtTitleOfCourtesy();
      if (null != titleOfCourtesy) {
         el.setValue(titleOfCourtesy);
      } else {
         el.setValue("");
      }


      el=page.getElementTxtBirthDateYYYY();
      if (null != birthDateYYYY) {
         el.setValue(birthDateYYYY);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtBirthDateMM();
      if (null != birthDateMM) {
         el.setValue(birthDateMM);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtBirthDateDD();
      if (null != birthDateDD) {
         el.setValue(birthDateDD);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtHireDateYYYY();
      if (null != hireDateYYYY) {
         el.setValue(hireDateYYYY);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtHireDateMM();
      if (null != hireDateMM) {
         el.setValue(hireDateMM);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtHireDateDD();
      if (null != hireDateDD) {
         el.setValue(hireDateDD);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtAddress();
      if (null != address) {
         el.setValue(address);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtCity();
      if (null != city) {
         el.setValue(city);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtRegion();
      if (null != region) {
         el.setValue(region);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtPostalCode();
      if (null != postalCode) {
         el.setValue(postalCode);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtCountry();
      if (null != country) {
         el.setValue(country);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtHomePhone();
      if (null != homePhone) {
         el.setValue(homePhone);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtMobilePhone();
      if (null != mobilePhone) {
         el.setValue(mobilePhone);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtEmail();
      if (null != email) {
         el.setValue(email);
      } else {
         el.setValue("");
      }

      //HTMLTextAreaElement tael=page.getElementTxtNotes();
      el=page.getElementTxtNotes();
      if (null != notes) {
         el.setValue(notes);
      } else {
         el.setValue("");
      }

      el=page.getElementCbxIsAdmin();
      if (null != isAdmin) {
         el.setChecked(true);
      } else {
         el.setChecked(false);
      }


      if (null == errorMsg) {
         page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
      } else {
         page.setTextErrorText(errorMsg);
      }

      return page;
   }

   /**
    * Method to save a new or existing employee to the database
    *
    * @param employee, the employee to be saved
    * @return html document
    * @exception HttpPresentationException
    */
   protected void saveEmployee(Employee theEmployee)
         throws HttpPresentationException {

      String firstName = this.getComms().request.getParameter(FIRST_NAME);
      String lastName = this.getComms().request.getParameter(LAST_NAME);
      String login = this.getComms().request.getParameter(LOGIN);
      String password = this.getComms().request.getParameter(PASSWORD);
      String title = this.getComms().request.getParameter(TITLE);
      String titleOfCourtesy = this.getComms().request.getParameter(TITLE_OF_COURTESY);
      String birthDateYYYY = this.getComms().request.getParameter(BIRTH_DATE_YYYY);
      String birthDateMM = this.getComms().request.getParameter(BIRTH_DATE_MM);
      String birthDateDD = this.getComms().request.getParameter(BIRTH_DATE_DD);
      String hireDateYYYY = this.getComms().request.getParameter(HIRE_DATE_YYYY);
      String hireDateMM = this.getComms().request.getParameter(HIRE_DATE_MM);
      String hireDateDD = this.getComms().request.getParameter(HIRE_DATE_DD);
      String address = this.getComms().request.getParameter(ADDRESS);
      String city = this.getComms().request.getParameter(CITY);
      String region = this.getComms().request.getParameter(REGION);
      String postalCode = this.getComms().request.getParameter(POSTAL_CODE);
      String country = this.getComms().request.getParameter(COUNTRY);
      String homePhone = this.getComms().request.getParameter(HOME_PHONE);
      String mobilePhone = this.getComms().request.getParameter(MOBILE_PHONE);
      String email = this.getComms().request.getParameter(EMAIL);
      String notes = this.getComms().request.getParameter(NOTES);
      String isAdmin = this.getComms().request.getParameter(IS_ADMIN);

      if (isNullField(firstName) || isNullField(lastName) || isNullField(login)
          || isNullField(password) || !areDateFieldsValid(birthDateYYYY,birthDateMM,birthDateDD) ||
          !areDateFieldsValid(hireDateYYYY,hireDateMM,hireDateDD))  {
         throw new ProjectManagementPresentationException("Error adding employee",new Exception());
      }

      try {
         String birthDate=birthDateYYYY+"-"+birthDateMM+"-"+birthDateDD;
         String hireDate=hireDateYYYY+"-"+hireDateMM+"-"+hireDateDD;
         theEmployee.setFirstName(firstName);
         theEmployee.setLastName(lastName);
         theEmployee.setLogin(login);
         theEmployee.setPassword(password);
         theEmployee.setTitle(title);
         theEmployee.setTitleOfCourtesy(titleOfCourtesy);
         theEmployee.setBirthDate(Date.valueOf(birthDate));
         theEmployee.setHireDate(Date.valueOf(hireDate));
         theEmployee.setAddress(address);
         theEmployee.setCity(city);
         theEmployee.setRegion(region);
         theEmployee.setPostalCode(postalCode);
         theEmployee.setCountry(country);
         theEmployee.setHomePhone(homePhone);
         theEmployee.setMobilePhone(mobilePhone);
         theEmployee.setEmail(email);
         theEmployee.setNotes(notes);
         theEmployee.setIsAdmin(isAdmin!=null);
         theEmployee.save();
      } catch(Exception ex) {
         throw new ProjectManagementPresentationException("Error adding employee", ex);
      }
   }

   private boolean isExistingLogin (Employee employee) {
      boolean isExistingLogin=false;
      String login = null;
      try {
         login=this.getComms().request.getParameter(LOGIN).trim();
      } catch (Exception exc) {}
      if (!isNullField(login)) {
         ProjectManagement pm=getApplication();
         if (pm.getAdminUsername().equals(login)) {
            isExistingLogin=true;
         } else {
            try {
      
  EmployeeManager employeeManager = EmployeeManagerFactory.getEmployeeManager("projectmanagement.business.employee.EmployeeManagerImpl");
  Employee user = employeeManager.findEmployee(login);
               if (user!=null && (employee==null || !user.getHandle().equals(employee.getHandle()))) {
                  isExistingLogin=true;
               }
            } catch(ProjectManagementException ex) {
            }
         }
      }
      return isExistingLogin;
   }

}

