package projectmanagement.presentation.payrates;

import projectmanagement.presentation.*;
import projectmanagement.spec.timewage.*;
import projectmanagement.spec.project.*;

import com.lutris.appserver.server.httpPresentation.*;
import org.enhydra.xml.xmlc.XMLObject;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

/**
 * Shows a list of pay rates to administer.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class Administering extends BasePO {

   /**
    * Constants
    */
   private static String EMPLOYEE = "employee";
   private static String PROJECT = "project";


   /**
    * Superclass method override. Returns 2.
    */
   protected int getRequiredAuthLevel() {
      return 2;
   }

   /**
    *  Default event. Just show the page.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {

	   AdministeringHTML page = new AdministeringHTML();
  try {
      String employee = this.getComms().request.getParameter(EMPLOYEE);
      if (employee!=null && employee.length()==0) {
         employee=null;
      }
      String project = this.getComms().request.getParameter(PROJECT);
      if (project!=null && project.length()==0) {
         project=null;
      }

      String errorMsg = this.getSessionData().getAndClearUserMessage();
      /*if(null != errorMsg) {
         page.setTextLblErrorText(errorMsg);
      } else {
         page.getElementLblErrorText().getParentNode().removeChild(page.getElementLblErrorText());
      }*/
   
    PayRateManager payRateManager = PayRateManagerFactory.
          getPayRateManager("projectmanagement.business.timewage.PayRateManagerImpl");
          PayRate[] payRates=payRateManager.getAllPayRatesForEmployeeProjectPair(employee,project);
  
       // fetches the table element from HTML document
      HTMLTableElement table = page.getElementTblPayRates();
     
         if (payRates!=null) {
            for (int i=0; i<payRates.length; i++) {
               // creating row in HTML table which describes table properties
               HTMLTableRowElement htmlRE=createNewRow(page,payRates[i]);
               // appending row at the end of table
               table.appendChild(htmlRE);
            }
         }
      
      //Remove template selection row from table
      table.removeChild(page.getElementTemplateRow());
      
/*  Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page 
 */ 
      } catch(NullPointerException ex) {
           
      } catch(Exception ex) {
         this.writeDebugMsg("Error populating list of pay rates: " + ex);
         throw new ProjectManagementPresentationException("Error getting list of pay rates: ", ex);
      }


      // write out HTML
      return page;
   }

   /**
   * Creates the new row of HTML table element, based on given parameters.
   */
   private HTMLTableRowElement createNewRow(AdministeringHTML administeringHTML,
         PayRate payRate) {

      String payRateID="";

      try {
         payRateID=payRate.getHandle();
         administeringHTML.setTextTxtEmployee(payRate.getEmployee().getFirstName()+" "+
            payRate.getEmployee().getLastName());
         administeringHTML.setTextTxtProject(payRate.getProject().getName());
         administeringHTML.setTextTxtPayRate(String.valueOf(payRate.getRate()));
         administeringHTML.setTextTxtValidFrom(payRate.getValidFrom().toString());
      } catch (Exception ex) {}

      // image to get details on pay rate
      HTMLImageElement detailsImg=administeringHTML.getElementImgDetails();
      detailsImg.setName(payRateID);

      // image to modify pay rate information
      HTMLImageElement modifyImg=administeringHTML.getElementImgModify();
      modifyImg.setName(payRateID);

      // image to delete pay rate
      HTMLImageElement deleteImg=administeringHTML.getElementImgDelete();
      deleteImg.setName(payRateID);

      HTMLTableRowElement row=(HTMLTableRowElement)administeringHTML.
         getElementTemplateRow().cloneNode(true);

      return row;
   }

}

