/*
 *  projectManagement
 *
 *  Enhydra super-servlet presentation object
 *
 */

package projectmanagement.presentation.payrates;

import projectmanagement.spec.employee.*;
import projectmanagement.spec.project.*;
import projectmanagement.presentation.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

// Enhydra SuperServlet imports
import com.lutris.appserver.server.httpPresentation.HttpPresentation;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;

// Standard imports
import java.io.IOException;

/**
* Generates the blank HTML page.
*/
public class Controls implements HttpPresentation {

   /**
    * Constants
    */
   private static String EMPLOYEE = "employee";
   private static String PROJECT = "project";

   public void run(HttpPresentationComms comms) throws HttpPresentationException, IOException {
      ControlsHTML page=new ControlsHTML();

      String employee = comms.request.getParameter(EMPLOYEE);
      HTMLSelectElement sel=page.getElementCboEmployee();
      fillEmployeeSelection(page,sel,employee);

      String project = comms.request.getParameter(PROJECT);
      sel=page.getElementCboProject();
      fillProjectSelection(page,sel,project);

      comms.response.writeDOM(page);
   }

   private void fillEmployeeSelection (ControlsHTML page,HTMLSelectElement cboEmployees,
         String selectedEmployeeID) throws ProjectManagementPresentationException {
      HTMLOptionElement optEmployee = page.getElementOptEmployee();
    
 try {
     EmployeeManager employeeManager = EmployeeManagerFactory.getEmployeeManager("projectmanagement.business.employee.EmployeeManagerImpl");
     Employee[] employees=employeeManager.getAllEmployees();
 
      // Remove the dummy storyboard text from the prototype HTML
      optEmployee.removeChild(optEmployee.getFirstChild());

    
         // set the text to select all employees - default option
         HTMLOptionElement clonedOption = (HTMLOptionElement)optEmployee.cloneNode(true);
         clonedOption.setValue("");
         String all="All";
         Node optionTextNode = clonedOption.getOwnerDocument().
            createTextNode(all);
         clonedOption.appendChild(optionTextNode);
          clonedOption.setAttribute("selected","selected");

         // Do only a shallow copy of the option as we don't want the text child
         // of the node option
         cboEmployees.appendChild(clonedOption);

        
 
       // set all employees
         if (employees!=null) {
            for (int i=0; i<employees.length; i++) {
               Employee e=employees[i];
               // Now populate the combo with employees
               // This algorithm is obscure because options
               // are not normal HTML elements
               // First populate the option value (the employee database ID).
               //  Then append a text child as the option
               // text, which is what is displayed as the text
               // in each row of the select box
               clonedOption = (HTMLOptionElement)optEmployee.cloneNode(true);
               clonedOption.setValue(e.getHandle());
               optionTextNode = clonedOption.getOwnerDocument().
                   createTextNode(e.getFirstName()+" "+e.getLastName());
               clonedOption.appendChild(optionTextNode);
               if (selectedEmployeeID!=null && e.getHandle().equals(selectedEmployeeID)) {
                  clonedOption.setAttribute("selected","selected");

               }
               // Do only a shallow copy of the option as we don't want the text child
               // of the node option
               cboEmployees.appendChild(clonedOption);
            }
         }
         cboEmployees.removeChild(optEmployee);
      } catch(NullPointerException ex) {
      } catch (Exception ex) {
         //this.writeDebugMsg("Error populating list of employees: " + ex);
         throw new ProjectManagementPresentationException("Error populating employee list: ", ex);
      }

      
   }

   private void fillProjectSelection (ControlsHTML page,HTMLSelectElement cboProjects,
         String selectedProjectID) throws ProjectManagementPresentationException {
      HTMLOptionElement optProject = page.getElementOptProject();
   

  try {

        ProjectManager projectManager = ProjectManagerFactory.getProjectManager("projectmanagement.business.project.ProjectManagerImpl");
          Project[] projects=projectManager.getAllProjects();
   
       // Remove the dummy storyboard text from the prototype HTML
      optProject.removeChild(optProject.getFirstChild());

         // set the text to select all projects - default option
         HTMLOptionElement clonedOption = (HTMLOptionElement)optProject.cloneNode(true);
         clonedOption.setValue("");
         String all="All";
         Node optionTextNode = clonedOption.getOwnerDocument().
            createTextNode(all);
         clonedOption.appendChild(optionTextNode);
         clonedOption.setAttribute("selected","selected");

         // Do only a shallow copy of the option as we don't want the text child
         // of the node option
         cboProjects.appendChild(clonedOption);

         // set all projects
      
        
         if (projects!=null) {
            for (int i=0; i<projects.length; i++) {
               Project p=projects[i];
               // Now populate the combo with projects
               // This algorithm is obscure because options
               // are not normal HTML elements
               // First populate the option value (the project database ID).
               //  Then append a text child as the option
               // text, which is what is displayed as the text
               // in each row of the select box
               clonedOption = (HTMLOptionElement)optProject.cloneNode(true);
               clonedOption.setValue(p.getHandle());
               optionTextNode = clonedOption.getOwnerDocument().
                   createTextNode(p.getName());
               clonedOption.appendChild(optionTextNode);
               if (selectedProjectID!=null && p.getHandle().equals(selectedProjectID)) {
               clonedOption.setAttribute("selected","selected");

               }
               // Do only a shallow copy of the option as we don't want the text child
               // of the node option
               cboProjects.appendChild(clonedOption);
            }
         }
         
      cboProjects.removeChild(optProject);
      
      } catch(NullPointerException ex) {      
      } catch (Exception ex) {
         //this.writeDebugMsg("Error populating list of projects: " + ex);
         throw new ProjectManagementPresentationException("Error populating project list: ", ex);
      }

   }

}
