package projectmanagement.presentation.payrates;

import projectmanagement.presentation.*;
import projectmanagement.spec.*;
import projectmanagement.spec.timewage.*;
import projectmanagement.spec.project.*;
import projectmanagement.spec.employee.*;
import com.lutris.appserver.server.httpPresentation.*;
import org.enhydra.xml.xmlc.XMLObject;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import java.sql.Date;
import java.util.*;

/**
 * Manages all actions on pay rates.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class Edit extends BasePO {

   /**
    * Constants
    */
   private static String EMPLOYEE = "cboEmployee";
   private static String PROJECT = "cboProject";
   private static String PAY_RATE = "txtPayRate";
   private static String VALID_FROM_YYYY = "txtValidFromYYYY";
   private static String VALID_FROM_MM = "txtValidFromMM";
   private static String VALID_FROM_DD = "txtValidFromDD";

   private static String PAY_RATE_ID = "payRateID";
   private static String CONTEXT_PAGE = "Context.po";
   private static String ADD = "add";
   private static String DELETE = "delete";
   private static String MODIFY = "modify";
   private static String DETAILS = "showDetailsPage";
   private static String EVENT = "event";

   /**
    * Superclass method override. Returns 2.
    */
   protected int getRequiredAuthLevel() {
      return 2;
   }

   /**
    *  Default event. Just show the page for editing.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {
      return showModifyPage(null,false);
   }

   /**
    * handle show add pay rate page event.
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleShowAddPage()
      throws HttpPresentationException {
      return showAddPage(null);
   }

   /**
    * handle show details pay rate page event.
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleShowDetailsPage ()
         throws HttpPresentationException {
      return showModifyPage(null,true);
   }

   /*
    * Modifies an existing pay rate
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleModify()
         throws HttpPresentationException {
      String payRateID = this.getComms().request.getParameter(PAY_RATE_ID);
      PayRate payRate=null;
   
      // Try to get the pay rate by its ID
      try {
       PayRateManager payRateManager = PayRateManagerFactory.
          getPayRateManager("projectmanagement.business.timewage.PayRateManagerImpl");
         payRate = payRateManager.findPayRateByID(payRateID);
     
      } catch(Exception ex) {
         this.getSessionData().setUserMessage("Please choose a valid pay rate to edit");
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      }

      try {
         savePayRate(payRate);
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      } catch(Exception ex) {
         return showModifyPage("To modify this pay rate, you must fill out fields properly!",false);
      }
   }

   /*
    * Adds a pay rate to the pay rate database
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleAdd()
         throws HttpPresentationException {
      try {  
      PayRateManager payRateManager = PayRateManagerFactory.
          getPayRateManager("projectmanagement.business.timewage.PayRateManagerImpl");
          PayRate payRate = payRateManager.getPayRate();
          
         savePayRate(payRate);
         throw new ClientPageRedirectException(CONTEXT_PAGE);
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page 
 */ 
 
     } catch(NullPointerException ex) {
          return showAddPage("You cannot add pay rate while runing ProjectManagement_pres");
      } catch(Exception ex) {
         return showAddPage("To add this pay rate, you must fill out fields properly!");
      }
   }

   /*
    * Deletes a PayRate from the PayRate database
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleDelete()
         throws HttpPresentationException, ProjectManagementPresentationException {
      String payRateID = this.getComms().request.getParameter(PAY_RATE_ID);

      try {
       PayRateManager payRateManager = PayRateManagerFactory.
          getPayRateManager("projectmanagement.business.timewage.PayRateManagerImpl");
    PayRate payRate = payRateManager.findPayRateByID(payRateID);
        
         String payRateName = "["+payRate.getEmployee().getFirstName()+" "+
            payRate.getEmployee().getLastName()+","+payRate.getProject().getName()+"]";
         payRate.delete();
         this.getSessionData().setUserMessage("The pay rate, " + payRateName + ", was deleted");
         // Catch any possible database exception as well as the null pointer
         //  exception if the pay rate is not found and is null after findPayRateByID
      } catch(Exception ex) {
         this.getSessionData().setUserMessage("Please choose a valid pay rate to delete");
      }
      // Redirect to the catalog page which will display the error message,
      //  if there was one set by the above exception
      throw new ClientPageRedirectException(CONTEXT_PAGE);
   }

   /**
    * Produce HTML for this PO, populated by the pay rate information
    * that the user wants to edit
    *
    * @param errorMsg, the error messages
    * @param disabled, if controls are disabled
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject showModifyPage(String errorMsg,boolean disabled)
         throws HttpPresentationException, ProjectManagementPresentationException {
       
      String employee = this.getComms().request.getParameter(EMPLOYEE);
      String project = this.getComms().request.getParameter(PROJECT);
      String rate = this.getComms().request.getParameter(PAY_RATE);
      String validFromYYYY = this.getComms().request.getParameter(VALID_FROM_YYYY);
      String validFromMM = this.getComms().request.getParameter(VALID_FROM_MM);
      String validFromDD = this.getComms().request.getParameter(VALID_FROM_DD);

      String payRateID = this.getComms().request.getParameter(PAY_RATE_ID);
      // Instantiate the page object
      EditHTML page = new EditHTML();

      PayRate payRate = null;

      try {
    PayRateManager payRateManager = PayRateManagerFactory.
          getPayRateManager("projectmanagement.business.timewage.PayRateManagerImpl");
       payRate = payRateManager.findPayRateByID(payRateID);
      
  /*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page 
 */ 
 
      } catch(NullPointerException ex) {
           page.setTextErrorText("This is a default HTML page");
           return page;
         // Catch any possible database exception in findPayRateByID()
      } catch(ProjectManagementException ex) {
         this.getSessionData().setUserMessage("Please choose a valid pay rate to edit");
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      }

      try {
         // If we received a valid payRateID then try to show the pay rate's contents,
         // otherwise try to use the HTML input parameters
         page.getElementPayRateID().setValue(payRate.getHandle());

         HTMLSelectElement sel=page.getElementCboEmployee();
         if (null == employee || employee.length() == 0) {
            employee=payRate.getEmployee().getHandle();
         }
         fillEmployeeSelection(page,sel,employee);
         sel.setDisabled(disabled);

         sel=page.getElementCboProject();
         if (null == project || project.length() == 0) {
            project=payRate.getProject().getHandle();
         }
         fillProjectSelection(page,sel,project);
         sel.setDisabled(disabled);

         HTMLInputElement el=page.getElementTxtPayRate();
         el.setDisabled(disabled);
         if (null != rate && rate.length() != 0) {
            el.setValue(rate);
         } else {
            el.setValue(String.valueOf(payRate.getRate()));
         }

         el=page.getElementTxtValidFromYYYY();
         el.setDisabled(disabled);
         if (null != validFromYYYY && validFromYYYY.length() != 0) {
            el.setValue(validFromYYYY);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(payRate.getValidFrom());
            el.setValue(String.valueOf(cal.get(Calendar.YEAR)));
         }

         el=page.getElementTxtValidFromMM();
         el.setDisabled(disabled);
         if (null != validFromMM && validFromMM.length() != 0) {
            el.setValue(validFromMM);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(payRate.getValidFrom());
            el.setValue(String.valueOf(cal.get(Calendar.MONTH)+1));
         }

         el=page.getElementTxtValidFromDD();
         el.setDisabled(disabled);
         if (null != validFromDD && validFromDD.length() != 0) {
            el.setValue(validFromDD);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(payRate.getValidFrom());
            el.setValue(String.valueOf(cal.get(Calendar.DAY_OF_MONTH)));
         }

         el=page.getElementBtnSave();
         el.setDisabled(disabled);

         if (null == errorMsg) {
            page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
         } else {
            page.setTextErrorText(errorMsg);
         }
      } catch(ProjectManagementException ex) {
         throw new ProjectManagementPresentationException("Error populating page for pay rate editing", ex);
      }

      page.getElementEvent().setValue(MODIFY);
      return page;
   }

   /**
    * Produce HTML for this PO
    *
    * @param errorMsg, the error messages
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject showAddPage(String errorMsg)
         throws HttpPresentationException, ProjectManagementPresentationException {

      String employee = this.getComms().request.getParameter(EMPLOYEE);
      String project = this.getComms().request.getParameter(PROJECT);
      String rate = this.getComms().request.getParameter(PAY_RATE);
      String validFromYYYY = this.getComms().request.getParameter(VALID_FROM_YYYY);
      String validFromMM = this.getComms().request.getParameter(VALID_FROM_MM);
      String validFromDD = this.getComms().request.getParameter(VALID_FROM_DD);

      // Instantiate the page object
      EditHTML page = new EditHTML();

      HTMLSelectElement sel=page.getElementCboEmployee();
      fillEmployeeSelection(page,sel,employee);

      sel=page.getElementCboProject();
      fillProjectSelection(page,sel,project);

      HTMLInputElement el=page.getElementTxtPayRate();
      if (null != rate) {
         el.setValue(rate);
      } else {
         el.setValue("");
      }

      Calendar cal=new GregorianCalendar();
      el=page.getElementTxtValidFromYYYY();
      if (null != validFromYYYY) {
         el.setValue(validFromYYYY);
      } else {
         el.setValue(String.valueOf(cal.get(Calendar.YEAR)));
      }

      el=page.getElementTxtValidFromMM();
      if (null != validFromMM) {
         el.setValue(validFromMM);
      } else {
         el.setValue(String.valueOf(cal.get(Calendar.MONTH)+1));
      }

      el=page.getElementTxtValidFromDD();
      if (null != validFromDD) {
         el.setValue(validFromDD);
      } else {
         el.setValue(String.valueOf(cal.get(Calendar.DAY_OF_MONTH)));
      }

      if (null == errorMsg) {
         page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
      } else {
         page.setTextErrorText(errorMsg);
      }

      return page;
   }

   /**
    * Method to save a new or existing pay rate to the database
    *
    * @param payRate, the pay rate to be saved
    * @return html document
    * @exception HttpPresentationException
    */
   protected void savePayRate(PayRate thePayRate)
         throws HttpPresentationException {

      String employee = this.getComms().request.getParameter(EMPLOYEE);
      String project = this.getComms().request.getParameter(PROJECT);
      String rate = this.getComms().request.getParameter(PAY_RATE);
      String validFromYYYY = this.getComms().request.getParameter(VALID_FROM_YYYY);
      String validFromMM = this.getComms().request.getParameter(VALID_FROM_MM);
      String validFromDD = this.getComms().request.getParameter(VALID_FROM_DD);

      if (isNullField(employee) || isNullField(project) || isNullField(rate) ||
          !areDateFieldsValid(validFromYYYY,validFromMM,validFromDD)) {
         throw new ProjectManagementPresentationException("Error adding pay rate",new Exception());
      }

      try {
         String validFrom=validFromYYYY+"-"+validFromMM+"-"+validFromDD;
EmployeeManager employeeManager = EmployeeManagerFactory.getEmployeeManager("projectmanagement.business.employee.EmployeeManagerImpl");
         thePayRate.setEmployee(employeeManager.findEmployeeByID(employee));
ProjectManager projectManager = ProjectManagerFactory.getProjectManager("projectmanagement.business.project.ProjectManagerImpl");
         thePayRate.setProject(projectManager.findProjectByID(project));
         thePayRate.setRate(Double.parseDouble(rate));
         thePayRate.setValidFrom(Date.valueOf(validFrom));

         thePayRate.save();
      } catch(Exception ex) {
         throw new ProjectManagementPresentationException("Error adding pay rate", ex);
      }
   }

   private void fillEmployeeSelection (EditHTML page,HTMLSelectElement cboEmployees,
         String selectedEmployeeID) throws ProjectManagementPresentationException {
      //
      HTMLOptionElement optEmployee = page.getElementOptEmployee();
      
      try {
 EmployeeManager employeeManager = EmployeeManagerFactory.getEmployeeManager("projectmanagement.business.employee.EmployeeManagerImpl");
Employee[] employees=employeeManager.getAllEmployees();


     // Remove the dummy storyboard text from the prototype HTML
      optEmployee.removeChild(optEmployee.getFirstChild());
       
         if (employees!=null) {
            for (int i=0; i<employees.length; i++) {
               Employee e=employees[i];
               // Now populate the combo with employees
               // This algorithm is obscure because options
               // are not normal HTML elements
               // First populate the option value (the employee database ID).
               //  Then append a text child as the option
               // text, which is what is displayed as the text
               // in each row of the select box
               HTMLOptionElement clonedOption = (HTMLOptionElement)optEmployee.cloneNode(true);
               clonedOption.setValue(e.getHandle());
               Node optionTextNode = clonedOption.getOwnerDocument().
                   createTextNode(e.getFirstName()+" "+e.getLastName());
               clonedOption.appendChild(optionTextNode);
               if (e.getHandle().equals(selectedEmployeeID)) {
                  clonedOption.setAttribute("selected","selected");
               }
               // Do only a shallow copy of the option as we don't want the text child
               // of the node option
               cboEmployees.appendChild(clonedOption);
               // Alternative way to insert nodes below
               // insertBefore(newNode, oldNode);
               // discSelect.insertBefore(clonedOption, templateOption);
            }
         }
      cboEmployees.removeChild(optEmployee);
  
      } catch(NullPointerException ex) {
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of employees: " + ex);
         throw new ProjectManagementPresentationException("Error populating employee list: ", ex);
      }

      
   }

   private void fillProjectSelection (EditHTML page,HTMLSelectElement cboProjects,
         String selectedProjectID) throws ProjectManagementPresentationException {
      //
      HTMLOptionElement optProject = page.getElementOptProject();
      
      try {
      ProjectManager projectManager = ProjectManagerFactory.getProjectManager("projectmanagement.business.project.ProjectManagerImpl");
      Project[] projects=projectManager.getAllProjects();
     // Remove the dummy storyboard text from the prototype HTML
      optProject.removeChild(optProject.getFirstChild());

     
         if (projects!=null) {
            for (int i=0; i<projects.length; i++) {
               Project p=projects[i];
               // Now populate the combo with projects
               // This algorithm is obscure because options
               // are not normal HTML elements
               // First populate the option value (the project database ID).
               //  Then append a text child as the option
               // text, which is what is displayed as the text
               // in each row of the select box
               HTMLOptionElement clonedOption = (HTMLOptionElement)optProject.cloneNode(true);
               clonedOption.setValue(p.getHandle());
               Node optionTextNode = clonedOption.getOwnerDocument().
                   createTextNode(p.getName());
               clonedOption.appendChild(optionTextNode);
               if (p.getHandle().equals(selectedProjectID)) {
                  clonedOption.setAttribute("selected","selected");
               }
               // Do only a shallow copy of the option as we don't want the text child
               // of the node option
               cboProjects.appendChild(clonedOption);
               // Alternative way to insert nodes below
               // insertBefore(newNode, oldNode);
               // discSelect.insertBefore(clonedOption, templateOption);
            }
         } cboProjects.removeChild(optProject);
      } catch(NullPointerException ex) {
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of projects: " + ex);
         throw new ProjectManagementPresentationException("Error populating project list: ", ex);
      }

     
   }

}

