package projectmanagement.presentation.projects;

import projectmanagement.presentation.*;
import projectmanagement.spec.project.*;
import com.lutris.appserver.server.httpPresentation.*;
import org.enhydra.xml.xmlc.XMLObject;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

/**
 * Shows a list of projects to administer.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class Administering extends BasePO {

   /**
    * Constants
    */
   private static String CUSTOMER = "customer";

   /**
    * Superclass method override. Returns 2.
    */
   protected int getRequiredAuthLevel() {
      return 2;
   }

   /**
    *  Default event. Just show the page.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {

	   AdministeringHTML page = new AdministeringHTML();

      String customer = this.getComms().request.getParameter(CUSTOMER);
      if (customer!=null && customer.length()==0) {
         customer=null;
      }

      String errorMsg = this.getSessionData().getAndClearUserMessage();
      /*if(null != errorMsg) {
         page.setTextLblErrorText(errorMsg);
      } else {
         page.getElementLblErrorText().getParentNode().removeChild(page.getElementLblErrorText());
      }*/

     
      try {
ProjectManager projectManager = ProjectManagerFactory.getProjectManager("projectmanagement.business.project.ProjectManagerImpl");
 Project[] projects=projectManager.getAllProjectsForCustomer(customer);
     
     // fetches the table element from HTML document
      HTMLTableElement table = page.getElementTblProjects();

       
         if (projects!=null) {
            for (int i=0; i<projects.length; i++) {
               // creating row in HTML table which describes table properties
               HTMLTableRowElement htmlRE=createNewRow(page,projects[i]);
               // appending row at the end of table
               table.appendChild(htmlRE);
            }
         }
      //Remove template selection row from table
      table.removeChild(page.getElementTemplateRow());
      
/*  Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page 
 */ 
      } catch(NullPointerException ex) {
       
      } catch(Exception ex) {
         this.writeDebugMsg("Error populating list of projects: " + ex);
         throw new ProjectManagementPresentationException("Error getting list of projects: ", ex);
      }

     

      // write out HTML
      return page;
   }

   /**
   * Creates the new row of HTML table element, based on given parameters.
   */
   private HTMLTableRowElement createNewRow(AdministeringHTML administeringHTML,
         Project project) {

      String projectID="";

      try {
         projectID=project.getHandle();
         administeringHTML.setTextTxtCustomer(project.getCustomer().getCompanyName());
         administeringHTML.setTextTxtName(project.getName());
         administeringHTML.setTextTxtMoneyPerHour(String.valueOf(project.getMoneyPerHour()));
      } catch (Exception ex) {}

      // image to get details on project
      HTMLImageElement detailsImg=administeringHTML.getElementImgDetails();
      detailsImg.setName(projectID);

      // image to modify project information
      HTMLImageElement modifyImg=administeringHTML.getElementImgModify();
      modifyImg.setName(projectID);

      // image to delete project
      HTMLImageElement deleteImg=administeringHTML.getElementImgDelete();
      deleteImg.setName(projectID);

      HTMLTableRowElement row=(HTMLTableRowElement)administeringHTML.
         getElementTemplateRow().cloneNode(true);

      return row;
   }

}

