/*
 *  projectManagement
 *
 *  Enhydra super-servlet presentation object
 *
 */

package projectmanagement.presentation.projects;

import projectmanagement.presentation.*;
import projectmanagement.spec.customer.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

// Enhydra SuperServlet imports
import com.lutris.appserver.server.httpPresentation.HttpPresentation;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;

// Standard imports
import java.io.IOException;

/**
* Generates the blank HTML page.
*/
public class Controls implements HttpPresentation {

   /**
    * Constants
    */
   private static String CUSTOMER = "customer";

   public void run(HttpPresentationComms comms) throws HttpPresentationException, IOException {
      ControlsHTML page=new ControlsHTML();

      String customer = comms.request.getParameter(CUSTOMER);
      HTMLSelectElement sel=page.getElementCboCustomers();
      fillSelection(page,sel,customer);

      comms.response.writeDOM(page);
   }

   private void fillSelection (ControlsHTML page,HTMLSelectElement cboCustomers,
         String selectedCustomerID) throws ProjectManagementPresentationException {
      HTMLOptionElement optCustomer = page.getElementOptCustomer();
 
  try {
  	
     CustomerManager customerManager = CustomerManagerFactory.getCustomerManager("projectmanagement.business.customer.CustomerManagerImpl");
     Customer[] customers = customerManager.getAllCustomers();
  	
      // Remove the dummy storyboard text from the prototype HTML
      optCustomer.removeChild(optCustomer.getFirstChild());

     
         // set the text to select all customers - default option
         HTMLOptionElement clonedOption = (HTMLOptionElement)optCustomer.cloneNode(true);
         clonedOption.setValue("");
         String all="All";
         Node optionTextNode = clonedOption.getOwnerDocument().
            createTextNode(all);
         clonedOption.appendChild(optionTextNode);
          clonedOption.setAttribute("selected","selected");
         // Do only a shallow copy of the option as we don't want the text child
         // of the node option
         cboCustomers.appendChild(clonedOption);

        
         
         // set all customers
        
         if (customers!=null) {
            for (int i=0; i<customers.length; i++) {
               Customer c=customers[i];
               // Now populate the combo with customers
               // This algorithm is obscure because options
               // are not normal HTML elements
               // First populate the option value (the customer database ID).
               //  Then append a text child as the option
               // text, which is what is displayed as the text
               // in each row of the select box
               clonedOption = (HTMLOptionElement)optCustomer.cloneNode(true);
               clonedOption.setValue(c.getHandle());
               optionTextNode = clonedOption.getOwnerDocument().
                   createTextNode(c.getCompanyName());
               clonedOption.appendChild(optionTextNode);
               if (selectedCustomerID!=null && c.getHandle().equals(selectedCustomerID)) {
                   clonedOption.setAttribute("selected","selected");
               }
               // Do only a shallow copy of the option as we don't want the text child
               // of the node option
               cboCustomers.appendChild(clonedOption);
            }
         }
          cboCustomers.removeChild(optCustomer);
         
/*  Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page 
 */ 
      } catch(NullPointerException ex) {
      
      } catch (Exception ex) {
         //this.writeDebugMsg("Error populating list of customers: " + ex);
         throw new ProjectManagementPresentationException("Error populating customer list: ", ex);
      }

     
   }

}
