package projectmanagement.presentation.projects;

import projectmanagement.presentation.*;
import projectmanagement.spec.*;

import projectmanagement.spec.project.*;
import projectmanagement.spec.customer.*;
import com.lutris.appserver.server.httpPresentation.*;
import org.enhydra.xml.xmlc.XMLObject;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

/**
 * Manages all actions on projects.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class Edit extends BasePO {

   /**
    * Constants
    */
   private static String CUSTOMER = "cboCustomer";
   private static String NAME = "txtName";
   private static String MONEY_PER_HOUR = "txtMoneyPerHour";

   private static String PROJECT_ID = "projectID";
   private static String CONTEXT_PAGE = "Context.po";
   private static String ADD = "add";
   private static String DELETE = "delete";
   private static String MODIFY = "modify";
   private static String DETAILS = "showDetailsPage";
   private static String EVENT = "event";

   /**
    * Superclass method override. Returns 1.
    */
   protected int getRequiredAuthLevel() {
      /*String event="";
      try {
         event=this.getComms().request.getParameter(EVENT);
      } catch (Exception ex) {}

      if (event!=null && event.equalsIgnoreCase(DETAILS)) {
         return 1;
      } else {
         return 2;
      }*/
      return 2;
   }

   /**
    *  Default event. Just show the page for editing.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {
      return showModifyPage(null,false);
   }

   /**
    * handle show add project page event.
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleShowAddPage()
      throws HttpPresentationException {
      return showAddPage(null);
   }

   /**
    * handle show details project page event.
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleShowDetailsPage ()
         throws HttpPresentationException {
      return showModifyPage(null,true);
   }

   /*
    * Modifies an existing project
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleModify()
         throws HttpPresentationException {
      String projectID = this.getComms().request.getParameter(PROJECT_ID);
      Project project=null;

      // Try to get the project by its ID
      try {
ProjectManager projectManager = ProjectManagerFactory.getProjectManager("projectmanagement.business.project.ProjectManagerImpl");
     project = projectManager.findProjectByID(projectID);
    
      } catch(Exception ex) {
         this.getSessionData().setUserMessage("Please choose a valid project to edit");
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      }

      try {
         saveProject(project);
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      } catch(Exception ex) {
         return showModifyPage("To add this project, you must fill out fields properly!",false);
      }
   }

   /*
    * Adds a project to the project database
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleAdd()
         throws HttpPresentationException {
      try {
ProjectManager projectManager = ProjectManagerFactory.getProjectManager("projectmanagement.business.project.ProjectManagerImpl");
       Project project = projectManager.getProject();
       
         saveProject(project);
         throw new ClientPageRedirectException(CONTEXT_PAGE);
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page with message
 */ 
 
      } catch(NullPointerException ex) {
          return showAddPage("You cannot add project while runing ProjectManagement_pres");
      } catch(Exception ex) {
         return showAddPage("To add this project, you must fill out fields properly!");
      }
   }

   /*
    * Deletes a Project from the Project database
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleDelete()
         throws HttpPresentationException, ProjectManagementPresentationException {
      String projectID = this.getComms().request.getParameter(PROJECT_ID);

      try {
ProjectManager projectManager = ProjectManagerFactory.getProjectManager("projectmanagement.business.project.ProjectManagerImpl");
        Project project = projectManager.findProjectByID(projectID);
       
         String projectName = project.getName();
         project.delete();
         this.getSessionData().setUserMessage("The project, " + projectName + ", was deleted");
         // Catch any possible database exception as well as the null pointer
         //  exception if the project is not found and is null after findProjectByID
      } catch(Exception ex) {
         this.getSessionData().setUserMessage("Please choose a valid project to delete");
      }
      // Redirect to the catalog page which will display the error message,
      //  if there was one set by the above exception
      throw new ClientPageRedirectException(CONTEXT_PAGE);
   }

   /**
    * Produce HTML for this PO, populated by the project information
    * that the user wants to edit
    *
    * @param errorMsg, the error messages
    * @param disabled, if controls are disabled
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject showModifyPage(String errorMsg,boolean disabled)
         throws HttpPresentationException, ProjectManagementPresentationException {

      String customer = this.getComms().request.getParameter(CUSTOMER);
      String name = this.getComms().request.getParameter(NAME);
      String moneyPerHour = this.getComms().request.getParameter(MONEY_PER_HOUR);

      String projectID = this.getComms().request.getParameter(PROJECT_ID);
      // Instantiate the page object
      EditHTML page = new EditHTML();

      Project project = null;

      try {
   ProjectManager projectManager = ProjectManagerFactory.getProjectManager("projectmanagement.business.project.ProjectManagerImpl");
   project = projectManager.findProjectByID(projectID);
  /*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page with message
 */ 
 
      } catch(NullPointerException ex) {
           page.setTextErrorText("This is a default HTML page");
           return page;
         // Catch any possible database exception in findProjectByID()
      } catch(ProjectManagementException ex) {
         this.getSessionData().setUserMessage("Please choose a valid project to edit");
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      }

      try {
         // If we received a valid projectID then try to show the project's contents,
         // otherwise try to use the HTML input parameters
         page.getElementProjectID().setValue(project.getHandle());

         HTMLSelectElement sel=page.getElementCboCustomer();

         if (null == customer || customer.length() == 0) {
            customer=project.getCustomer().getHandle();
         }

         fillSelection(page,sel,customer);
         sel.setDisabled(disabled);

         HTMLInputElement el=page.getElementTxtName();
         el.setDisabled(disabled);
         if (null != name && name.length() != 0) {
            el.setValue(name);
         } else {
            el.setValue(project.getName());
         }

         el=page.getElementTxtMoneyPerHour();
         el.setDisabled(disabled);
         if (null != moneyPerHour && moneyPerHour.length() != 0) {
            el.setValue(moneyPerHour);
         } else {
            el.setValue(String.valueOf(project.getMoneyPerHour()));
         }

         el=page.getElementBtnSave();
         el.setDisabled(disabled);

         if (null == errorMsg) {
            page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
         } else {
            page.setTextErrorText(errorMsg);
         }
      } catch(ProjectManagementException ex) {
         throw new ProjectManagementPresentationException("Error populating page for project editing", ex);
      }

      page.getElementEvent().setValue(MODIFY);
      return page;
   }

   /**
    * Produce HTML for this PO
    *
    * @param errorMsg, the error messages
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject showAddPage(String errorMsg)
         throws HttpPresentationException, ProjectManagementPresentationException {

      String customer = this.getComms().request.getParameter(CUSTOMER);
      String name = this.getComms().request.getParameter(NAME);
      String moneyPerHour = this.getComms().request.getParameter(MONEY_PER_HOUR);

      // Instantiate the page object
      EditHTML page = new EditHTML();

      HTMLSelectElement sel=page.getElementCboCustomer();

      fillSelection(page,sel,customer);

      HTMLInputElement el=page.getElementTxtName();
      if (null != name) {
         el.setValue(name);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtMoneyPerHour();
      if (null != moneyPerHour) {
         el.setValue(moneyPerHour);
      } else {
         el.setValue("");
      }

      if (null == errorMsg) {
         page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
      } else {
         page.setTextErrorText(errorMsg);
      }

      return page;
   }

   /**
    * Method to save a new or existing project to the database
    *
    * @param project, the project to be saved
    * @return html document
    * @exception HttpPresentationException
    */
   protected void saveProject(Project theProject)
         throws HttpPresentationException {

      String customer = this.getComms().request.getParameter(CUSTOMER);
      String name = this.getComms().request.getParameter(NAME);
      String moneyPerHour = this.getComms().request.getParameter(MONEY_PER_HOUR);

      if (isNullField(customer) || isNullField(name) || isNullField(moneyPerHour)) {
         throw new ProjectManagementPresentationException("Error adding project",new Exception());
      }

      try {
      	CustomerManager customerManager = CustomerManagerFactory.getCustomerManager("projectmanagement.business.customer.CustomerManagerImpl");

      	
         theProject.setCustomer(customerManager.findCustomerByID(customer));
         theProject.setName(name);
         theProject.setMoneyPerHour(Double.parseDouble(moneyPerHour));

         theProject.save();
      } catch(Exception ex) {
         throw new ProjectManagementPresentationException("Error adding project", ex);
      }
   }

   private void fillSelection (EditHTML page,HTMLSelectElement cboCustomers,
         String selectedCustomerID) throws ProjectManagementPresentationException {
      //
      HTMLOptionElement optCustomer = page.getElementOptCustomer();
      
      try {
       
       CustomerManager customerManager = CustomerManagerFactory.getCustomerManager("projectmanagement.business.customer.CustomerManagerImpl");
     Customer[] customers = customerManager.getAllCustomers();

       // Remove the dummy storyboard text from the prototype HTML
      optCustomer.removeChild(optCustomer.getFirstChild());

         if (customers!=null) {
            for (int i=0; i<customers.length; i++) {
               Customer c=customers[i];
               // Now populate the combo with customers
               // This algorithm is obscure because options
               // are not normal HTML elements
               // First populate the option value (the customer database ID).
               //  Then append a text child as the option
               // text, which is what is displayed as the text
               // in each row of the select box
               HTMLOptionElement clonedOption = (HTMLOptionElement)optCustomer.cloneNode(true);
               clonedOption.setValue(c.getHandle());
               Node optionTextNode = clonedOption.getOwnerDocument().
                   createTextNode(c.getCompanyName());
               clonedOption.appendChild(optionTextNode);
               if (c.getHandle().equals(selectedCustomerID)) {
                  clonedOption.setAttribute("selected","selected");
               }
               // Do only a shallow copy of the option as we don't want the text child
               // of the node option
               cboCustomers.appendChild(clonedOption);
               // Alternative way to insert nodes below
               // insertBefore(newNode, oldNode);
               // discSelect.insertBefore(clonedOption, templateOption);
            }
         }
         cboCustomers.removeChild(optCustomer);
      } catch(NullPointerException ex) {
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of customers: " + ex);
         throw new ProjectManagementPresentationException("Error populating customer list: ", ex);
      }

      
   }
}

