package projectmanagement.presentation.worksheets;

import projectmanagement.presentation.*;
import projectmanagement.spec.timewage.*;
import projectmanagement.spec.employee.*;
import projectmanagement.spec.project.*;
import com.lutris.appserver.server.httpPresentation.*;
import org.enhydra.xml.xmlc.XMLObject;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import java.util.Calendar;
import java.sql.Date;
import java.sql.Time;
import java.lang.*;
import java.util.*;

/**
 * Shows a list of work sheets to administer.
 *
 * @author Sasa Bojanic, Nenad Stefanovic
 * @version 1.0
 */
public class Administering extends BasePO {

   private static String EMPLOYEE = "employee";
   private static String PROJECT = "project";
   private static String YEAR = "year";
   private static String MONTH = "month";

   private static String IS_PERSONAL = "isPersonal";

   /**
    * Superclass method override. Returns 1 or 2, depending on action.
    */
   protected int getRequiredAuthLevel() {
      String isPersonal="";
      try {
         isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      } catch (Exception ex) {}

      if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
         return 1;
      } else {
         return 2;
      }
   }

   /**
    *  Default event. Just show the page.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {

      AdministeringHTML page = new AdministeringHTML();

      String employee = this.getComms().request.getParameter(EMPLOYEE);
      if (employee!=null && employee.length()==0) {
         employee=null;
      }
      String project = this.getComms().request.getParameter(PROJECT);
      if (project!=null && project.length()==0) {
         project=null;
      }
      String year = this.getComms().request.getParameter(YEAR);
      if (year!=null && year.length()==0) {
         year=null;
      }
      String month = this.getComms().request.getParameter(MONTH);
      if (month!=null && month.length()==0) {
         month=null;
      }

      // fetches the table element from HTML document
      HTMLTableElement table = page.getElementTblWorkSheets();

      try {
         WorkSheet[] workSheets=null;
        
         WorkSheetManager workSheetManager = WorkSheetManagerFactory.
               getWorkSheetManager("projectmanagement.business.timewage.WorkSheetManagerImpl"); 
        
         if (year==null) {
               
        workSheets=workSheetManager.getAllWorkSheetsForEmployeeProjectPair(employee,project);
         } else {
            Calendar c1=Calendar.getInstance();
            Calendar c2=Calendar.getInstance();
            if (month==null) {
               c1.set(Integer.parseInt(year),1,1);
               c2.set(Integer.parseInt(year),12,31);
            } else {
               c1.set(Integer.parseInt(year),Integer.parseInt(month)-1,1);
               c2.set( Integer.parseInt(year),Integer.parseInt(month)-1,c1.getActualMaximum(Calendar.DAY_OF_MONTH) );
            }
         workSheets=workSheetManager.getAllWorkSheetsForEmployeeProjectPairFromDate1ToDate2(
                  employee,project,new Date(c1.getTimeInMillis()),new Date(c2.getTimeInMillis()));
         }


         double sumOfHours=0;
         double sumOfMoney=0;
         if (workSheets!=null) {
            HoursAndMoney ham=new HoursAndMoney();
            for (int i=0; i<workSheets.length; i++) {
               // creating row in HTML table which describes table properties
               HTMLTableRowElement htmlRE=createNewRow(page,workSheets[i],ham);
               // appending row at the end of table
               table.appendChild(htmlRE);
            }
            sumOfHours=ham.getHours();
            sumOfMoney=ham.getMoney();
            sumOfMoney=(double)(Math.round(sumOfMoney*100))/100;
            sumOfHours=(double)(Math.round(sumOfHours*100))/100;
         }
         page.setTextTxtSumOfHours(String.valueOf(sumOfHours));
         page.setTextTxtSumOfMoney(String.valueOf(sumOfMoney));
         //to remove back button
            page.getElementBackElement().getParentNode().removeChild( page.getElementBackElement() );
     
       //Remove template selection row from table
      table.removeChild(page.getElementTemplateRow());

      String errorMsg = this.getSessionData().getAndClearUserMessage();
      /*if(null != errorMsg) {
         page.setTextLblErrorText(errorMsg);
      } else {
         page.getElementLblErrorText().getParentNode().removeChild(page.getElementLblErrorText());
      }*/

      String isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
         HTMLInputElement ip=page.getElementIsPersonal();
         ip.setValue("true");
        }
      } catch(NullPointerException ex) {   
      } catch(Exception ex) {
         this.writeDebugMsg("Error populating list of work sheets: " + ex);
         throw new ProjectManagementPresentationException("Error getting list of work sheets: ", ex);
      }

    
     
      // write out HTML
      return page;
   }

   /**
   * Creates the new row of HTML table element, based on given parameters.
   */
   private HTMLTableRowElement createNewRow(AdministeringHTML administeringHTML,
         WorkSheet workSheet,HoursAndMoney ham) {

      String workSheetID="";

      try {
         workSheetID=workSheet.getHandle();
         PayRate pr=workSheet.getPayRate();
         Employee emp=pr.getEmployee();
         Project prj=pr.getProject();
         Time timeStarted=workSheet.getTimeStarted();
         Time timeFinished=workSheet.getTimeFinished();
         administeringHTML.setTextTxtEmployee(emp.getFirstName()+" "+
            emp.getLastName());
         administeringHTML.setTextTxtProject(prj.getName());
         Calendar cal=new GregorianCalendar();
         cal.setTime(workSheet.getWorkingDate());
         String wd=String.valueOf(cal.get(Calendar.YEAR))+"-"+
               String.valueOf(cal.get(Calendar.MONTH)+1)+"-"+
               String.valueOf(cal.get(Calendar.DAY_OF_MONTH));
         administeringHTML.setTextTxtWorkingDate(wd);
         cal.setTime(timeStarted);
         String tst=String.valueOf(cal.get(Calendar.HOUR_OF_DAY))+":"+
               String.valueOf(cal.get(Calendar.MINUTE));
         administeringHTML.setTextTxtTimeStarted(tst);
         cal.setTime(timeFinished);
         String tfn=String.valueOf(cal.get(Calendar.HOUR_OF_DAY))+":"+
               String.valueOf(cal.get(Calendar.MINUTE));
         administeringHTML.setTextTxtTimeFinished(tfn);
         double mny=(double)(Math.round(
               ham.calculate(calculateTimeDifferenceInHours(timeStarted,timeFinished),
               prj.getMoneyPerHour(),pr.getRate())*100))/100;
         administeringHTML.setTextTxtMoney(String.valueOf(mny));
      } catch (Exception ex) {}

      // image to get details on work sheet
      HTMLImageElement detailsImg=administeringHTML.getElementImgDetails();
      detailsImg.setName(workSheetID);

      // image to modify work sheet information
      HTMLImageElement modifyImg=administeringHTML.getElementImgModify();
      modifyImg.setName(workSheetID);

      // image to delete work sheet
      HTMLImageElement deleteImg=administeringHTML.getElementImgDelete();
      deleteImg.setName(workSheetID);

      HTMLTableRowElement row=(HTMLTableRowElement)administeringHTML.
         getElementTemplateRow().cloneNode(true);

      return row;
   }

   private double calculateTimeDifferenceInHours (Time timeStarted,
         Time timeFinished) {
      long ts=timeStarted.getTime();
      long tf=timeFinished.getTime();
      long diffInMilliseconds=tf-ts;
      double diffInHours=diffInMilliseconds/3600000.0;
      return diffInHours;
   }

   class HoursAndMoney {
      private double hours=0;
      private double money=0;

      public double calculate (double hrs,double moneyPerHour,double payRate) {
         hours+=hrs;
         double dm=hrs*moneyPerHour*payRate;
         money+=dm;
         return dm;
      }

      public double getHours () {
         return hours;
      }

      public double getMoney () {
         return money;
      }
   }

}

