package projectmanagement.presentation.worksheets;

import projectmanagement.presentation.*;
import projectmanagement.spec.timewage.*;
import projectmanagement.spec.employee.*;
import projectmanagement.spec.project.*;
import com.lutris.appserver.server.httpPresentation.*;
import org.enhydra.xml.xmlc.XMLObject;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.sql.Date;
import java.sql.Time;
import java.lang.*;

/**
 * Shows a list of work sheets to administer.
 *
 * @author Sasa Bojanic, Nenad Stefanovic
 * @version 1.0
 */
public class AdministeringSearch extends BasePO {

   private static String EMPLOYEE = "employee";
   private static String PROJECT = "project";
   private static String FROMYEAR = "fromYear";
   private static String FROMMONTH = "fromMonth";
   private static String FROMDAY = "fromDay";
   private static String TOYEAR = "toYear";
   private static String TOMONTH = "toMonth";
   private static String TODAY = "toDay";

   private static String IS_PERSONAL = "isPersonal";

   /**
    * Superclass method override. Returns 1 or 2, depending on action.
    */
   protected int getRequiredAuthLevel() {
      String isPersonal="";
      try {
         isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      } catch (Exception ex) {}

      if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
         return 1;
      } else {
         return 2;
      }
   }

   /**
    *  Default event. Just show the page.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException,ClientPageRedirectException {

     AdministeringHTML page = new AdministeringHTML();
      String employee=null;
      String project=null;
      boolean multiple = false;
        //for advanced search
      java.util.ArrayList employeeIDs=new java.util.ArrayList();;
      int index=0;
      String employeeID=EMPLOYEE+String.valueOf("0");

      while ( this.getComms().request.getParameter(employeeID) != null ) {
        employeeIDs.add( this.getComms().request.getParameter(employeeID) );
        index++;
        employeeID = EMPLOYEE+String.valueOf(index);
      }

        //checking if there are multiple selection or is ALL selected
     try{   
      if( employeeIDs.size() == 1 || employeeIDs.get(0).toString().equals("") ) {
        employee = employeeIDs.get(0).toString();
        if (employee!=null && employee.length()==0) {
             employee=null;
        }
      } else {
        multiple = true;
      }
     }catch(IndexOutOfBoundsException e){}
      java.util.ArrayList projectIDs=new java.util.ArrayList();
      index=0;
      String projectID=PROJECT+String.valueOf("0");
      while ( this.getComms().request.getParameter(projectID) != null ) {
       
        projectIDs.add( this.getComms().request.getParameter(projectID) );
        index++;
        projectID = PROJECT+String.valueOf(index);
       
      }
        //checking if there are multiple selection or is ALL selected
      if( projectIDs.size() == 1 || projectIDs.get(0).toString().equals("")) {
        project = projectIDs.get(0).toString();
        if (project!=null && project.length()==0) {
             project=null;
        }
      } else {
        multiple = true;
      }


      String fromYear = this.getComms().request.getParameter(FROMYEAR);
      if (fromYear!=null && fromYear.length()==0) {
         fromYear=null;
      }
      String fromMonth = this.getComms().request.getParameter(FROMMONTH);
      if (fromMonth!=null && fromMonth.length()==0) {
         fromMonth=null;
      }
      String fromDay = this.getComms().request.getParameter(FROMDAY);
      if (fromDay!=null && fromDay.length()==0) {
         fromDay=null;
      }

      String toYear = this.getComms().request.getParameter(TOYEAR);
      if ( toYear != null && toYear.length()==0) {
         toYear=null;
      }
      String toMonth = this.getComms().request.getParameter(TOMONTH);
      if (toMonth!=null && toMonth.length()==0) {
         toMonth=null;
      }
      String toDay = this.getComms().request.getParameter(TODAY);
      if (toDay!=null && toDay.length()==0) {
         toDay=null;
      }
      boolean checkDates = true;
      String params = "";
      if ( !checkDate( fromYear , fromMonth , fromDay ) ) {
         params += "?checkFromDate=false";
         checkDates = false;
      }
      if ( !checkDate( toYear , toMonth , toDay ) ) {
         if ( !checkDates ) {
          params += "&checkToDate=false";
         } else {
          params += "?checkToDate=false";
        }
         checkDates = false;
      }
      if ( !checkDates ) {
        throw new ClientPageRedirectException("Search.po"+params);
      }

      HTMLFormElement form = page.getElementForm();
      form.setAttribute("target","_self");


      // fetches the table element from HTML document
      HTMLTableElement table = page.getElementTblWorkSheets();
      try {
         WorkSheet[] workSheets=null;
         WorkSheetManager workSheetManager = WorkSheetManagerFactory.
               getWorkSheetManager("projectmanagement.business.timewage.WorkSheetManagerImpl");        
      
    
         Calendar c1=Calendar.getInstance();
         Calendar c2=Calendar.getInstance();

         if( fromYear!=null && fromMonth!=null && fromDay!=null && toYear!=null && toMonth!=null && toDay!=null ) {

             c1.set( Integer.parseInt(fromYear),Integer.parseInt(fromMonth)-1,Integer.parseInt(fromDay) );
             c2.set( Integer.parseInt(toYear),Integer.parseInt(toMonth)-1,Integer.parseInt(toDay) );

             if ( !multiple )  {
                workSheets=workSheetManager.
                   getAllWorkSheetsForEmployeeProjectPairFromDate1ToDate2(
                    employee,project,new Date(c1.getTimeInMillis()),new Date(c2.getTimeInMillis()));
             } else {
                workSheets=workSheetManager.
                   getAllWorksheetsForEmployeeArrayAndProjectArrayBetweenDates(
                    employeeIDs,projectIDs,new Date(c1.getTimeInMillis()),new Date(c2.getTimeInMillis()));
             }
         }
         double sumOfHours=0;
         double sumOfMoney=0;
         if (workSheets!=null) {
            HoursAndMoney ham=new HoursAndMoney();
            for (int i=0; i<workSheets.length; i++) {
               // creating row in HTML table which describes table properties
               HTMLTableRowElement htmlRE=createNewRow(page,workSheets[i],ham);
               // appending row at the end of table
               table.appendChild(htmlRE);
            }
            sumOfHours=ham.getHours();
            sumOfMoney=ham.getMoney();
            sumOfMoney=(double)(Math.round(sumOfMoney*100))/100;
            sumOfHours=(double)(Math.round(sumOfHours*100))/100;
         }
         page.setTextTxtSumOfHours(String.valueOf(sumOfHours));
         page.setTextTxtSumOfMoney(String.valueOf(sumOfMoney));
 
 /* Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page 
 */ 
 
     } catch(NullPointerException ex) {
     	 return page;  
     } catch(Exception ex) {
         this.writeDebugMsg("Error populating list of work sheets: " + ex);
         throw new ProjectManagementPresentationException("Error getting list of work sheets: ", ex);
      }

      //Remove template selection row from table
      table.removeChild(page.getElementTemplateRow());

      String errorMsg = this.getSessionData().getAndClearUserMessage();
      /*if(null != errorMsg) {
         page.setTextLblErrorText(errorMsg);
      } else {
         page.getElementLblErrorText().getParentNode().removeChild(page.getElementLblErrorText());
      }*/

      String isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
         HTMLInputElement ip=page.getElementIsPersonal();
         ip.setValue("true");
      }
      // write out HTML
      return page;
   }
   /**
   * Check if given date is OK
   */
    private boolean checkDate(String year, String month, String day) {
        GregorianCalendar cal=new GregorianCalendar();
        boolean retVal=true;
        cal.setLenient(false);
        cal.set( Integer.parseInt(year),Integer.parseInt(month)-1,1 );
        if ( cal.getActualMaximum(Calendar.DAY_OF_MONTH) < Integer.parseInt(day) ) {
            retVal = false;
        } else if( cal.isLeapYear( Integer.parseInt(year) ) && Integer.parseInt(month) == 1 && Integer.parseInt(day) == 29 ) {
            retVal = true;
        } else {
            retVal = true;
        }
        return retVal;
    }

   /**
   * Creates the new row of HTML table element, based on given parameters.
   */
   private HTMLTableRowElement createNewRow(AdministeringHTML administeringHTML,
         WorkSheet workSheet,HoursAndMoney ham) {

      String workSheetID="";

      try {
         workSheetID=workSheet.getHandle();
         PayRate pr=workSheet.getPayRate();
         Employee emp=pr.getEmployee();
         Project prj=pr.getProject();
         Time timeStarted=workSheet.getTimeStarted();
         Time timeFinished=workSheet.getTimeFinished();
         administeringHTML.setTextTxtEmployee(emp.getFirstName()+" "+
            emp.getLastName());
         administeringHTML.setTextTxtProject(prj.getName());
         Calendar cal=new GregorianCalendar();
         cal.setTime(workSheet.getWorkingDate());
         String wd=String.valueOf(cal.get(Calendar.YEAR))+"-"+
               String.valueOf(cal.get(Calendar.MONTH)+1)+"-"+
               String.valueOf(cal.get(Calendar.DAY_OF_MONTH));
         administeringHTML.setTextTxtWorkingDate(wd);
         cal.setTime(timeStarted);
         String tst=String.valueOf(cal.get(Calendar.HOUR))+":"+
               String.valueOf(cal.get(Calendar.MINUTE));
         administeringHTML.setTextTxtTimeStarted(tst);
         cal.setTime(timeFinished);
         String tfn=String.valueOf(cal.get(Calendar.HOUR))+":"+
               String.valueOf(cal.get(Calendar.MINUTE));
         administeringHTML.setTextTxtTimeFinished(tfn);
         double mny=(double)(Math.round(
               ham.calculate(calculateTimeDifferenceInHours(timeStarted,timeFinished),
            prj.getMoneyPerHour(),pr.getRate())*100))/100;
         administeringHTML.setTextTxtMoney(String.valueOf(mny));
      } catch (Exception ex) {}

      // image to get details on work sheet
      HTMLImageElement detailsImg=administeringHTML.getElementImgDetails();
      detailsImg.setName(workSheetID);

      // image to modify work sheet information
      HTMLImageElement modifyImg=administeringHTML.getElementImgModify();
      modifyImg.setName(workSheetID);

      // image to delete work sheet
      HTMLImageElement deleteImg=administeringHTML.getElementImgDelete();
      deleteImg.setName(workSheetID);

      HTMLTableRowElement row=(HTMLTableRowElement)administeringHTML.
         getElementTemplateRow().cloneNode(true);

      return row;
   }

   private double calculateTimeDifferenceInHours (Time timeStarted,
         Time timeFinished) {
      long ts=timeStarted.getTime();
      long tf=timeFinished.getTime();
      long diffInMilliseconds=tf-ts;
      double diffInHours=diffInMilliseconds/3600000.0;
      return diffInHours;
   }

   class HoursAndMoney {
      private double hours=0;
      private double money=0;

      public double calculate (double hrs,double moneyPerHour,double payRate) {
         hours+=hrs;
         double dm=hrs*moneyPerHour*payRate;
         money+=dm;
         return dm;
      }

      public double getHours () {
         return hours;
      }

      public double getMoney () {
         return money;
      }
   }

}

