/*
 *  projectManagement
 *
 *  Enhydra super-servlet presentation object
 *
 */

package projectmanagement.presentation.worksheets;

import projectmanagement.spec.employee.*;
import projectmanagement.spec.project.*;
import projectmanagement.presentation.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

// Enhydra SuperServlet imports
import com.lutris.appserver.server.httpPresentation.HttpPresentation;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import org.enhydra.xml.xmlc.XMLObject;

// Standard imports
import java.io.IOException;
import java.util.Calendar;

/**
* Generates the blank HTML page.
*/
public class Controls extends BasePO {

   /**
    * Constants
    */
   private static String EMPLOYEE = "employee";
   private static String PROJECT = "project";
   private static String YEAR = "year";
   private static String MONTH = "month";

   private static String IS_PERSONAL = "isPersonal";

   /**
    * Superclass method override. Returns 1 or 2, depending on action.
    */
   protected int getRequiredAuthLevel() {
      String isPersonal="";
      try {
         isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      } catch (Exception ex) {}

      if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
         return 1;
      } else {
         return 2;
      }
   }

   /**
    *  Default event. Just show the page.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {

      ControlsHTML page=new ControlsHTML();

      String employee = this.getComms().request.getParameter(EMPLOYEE);
      HTMLSelectElement sel=page.getElementCboEmployee();
      fillEmployeeSelection(page,sel,employee);

      String project = this.getComms().request.getParameter(PROJECT);
      sel=page.getElementCboProject();
      fillProjectSelection(page,sel,project);

      String year = this.getComms().request.getParameter(YEAR);
      String month = this.getComms().request.getParameter(MONTH);

      if (year==null) {
         year=String.valueOf(Calendar.getInstance().get(Calendar.YEAR));
      }
      HTMLInputElement txtYear=page.getElementTxtYear();
      txtYear.setValue(year);

      if (month==null) {
         month=String.valueOf(Calendar.getInstance().get(Calendar.MONTH)+1);
      }
      fillMonths(page,month);

      String isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
         page.getElementIsPersonal().setValue("true");
      }

      return page;
   }

   private void fillEmployeeSelection (ControlsHTML page,HTMLSelectElement cboEmployees,
         String selectedEmployeeID) throws ProjectManagementPresentationException {
      HTMLOptionElement optEmployee = page.getElementOptEmployee();
   
      String isPersonal="";
      try {
         isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      } catch (Exception ex) {}
      if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
         try {
            selectedEmployeeID=getUser().getHandle();
         } catch (Exception ex) {}
      }

      try {
   EmployeeManager employeeManager = EmployeeManagerFactory.getEmployeeManager("projectmanagement.business.employee.EmployeeManagerImpl");
   Employee[] employees=employeeManager.getAllEmployees();
     	
      	   // Remove the dummy storyboard text from the prototype HTML
      optEmployee.removeChild(optEmployee.getFirstChild());

     
     
         // set the text to select all employees - default option
         HTMLOptionElement clonedOption=null;
         Node optionTextNode=null;
         if (isPersonal==null || !isPersonal.equalsIgnoreCase("true")) {
            clonedOption = (HTMLOptionElement)optEmployee.cloneNode(true);
            clonedOption.setValue("");
            String all="All";
            optionTextNode = clonedOption.getOwnerDocument().
               createTextNode(all);
            clonedOption.appendChild(optionTextNode);
             clonedOption.setAttribute("selected","selected");
            // Do only a shallow copy of the option as we don't want the text child
            // of the node option
            cboEmployees.appendChild(clonedOption);
         }

         // set all employees
   
   
         if (employees!=null) {
            for (int i=0; i<employees.length; i++) {
               Employee e=employees[i];
               // if this is personal, fill the combo only with that employ
               if (isPersonal!=null && isPersonal.equalsIgnoreCase("true") &&
                     !e.getHandle().equals(selectedEmployeeID)) {
                  continue;
               }

               // Now populate the combo with employees
               // This algorithm is obscure because options
               // are not normal HTML elements
               // First populate the option value (the employee database ID).
               //  Then append a text child as the option
               // text, which is what is displayed as the text
               // in each row of the select box
               clonedOption = (HTMLOptionElement)optEmployee.cloneNode(true);
               clonedOption.setValue(e.getHandle());
               optionTextNode = clonedOption.getOwnerDocument().
                   createTextNode(e.getFirstName()+" "+e.getLastName());
               clonedOption.appendChild(optionTextNode);
               if (selectedEmployeeID!=null && e.getHandle().equals(selectedEmployeeID)) {
                   clonedOption.setAttribute("selected","selected");
               }
               // Do only a shallow copy of the option as we don't want the text child
               // of the node option
               cboEmployees.appendChild(clonedOption);
            }
         }
         cboEmployees.removeChild(optEmployee);
      } catch(NullPointerException ex) {
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of employees: " + ex);
         throw new ProjectManagementPresentationException("Error populating employee list: ", ex);
      }

      
   }

   private void fillProjectSelection (ControlsHTML page,HTMLSelectElement cboProjects,
         String selectedProjectID) throws ProjectManagementPresentationException {
      HTMLOptionElement optProject = page.getElementOptProject();
      // Remove the dummy storyboard text from the prototype HTML
      optProject.removeChild(optProject.getFirstChild());

      String isPersonal="";
      try {
         isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      } catch (Exception ex) {}

      try {
 
         // set the text to select all projects - default option
         HTMLOptionElement clonedOption = (HTMLOptionElement)optProject.cloneNode(true);
         clonedOption.setValue("");
         String all="All";
         Node optionTextNode = clonedOption.getOwnerDocument().
            createTextNode(all);
         clonedOption.appendChild(optionTextNode);
          clonedOption.setAttribute("selected","selected");
         // Do only a shallow copy of the option as we don't want the text child
         // of the node option
         cboProjects.appendChild(clonedOption);

         // set all projects
         Project[] projects=null;
         ProjectManager projectManager = ProjectManagerFactory.getProjectManager("projectmanagement.business.project.ProjectManagerImpl");      
    
         if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
            // IMPLEMENT ME!!!
         projects=projectManager.getAllProjects();
        
         } else {
         projects=projectManager.getAllProjects();
         }
         if (projects!=null) {
            for (int i=0; i<projects.length; i++) {
               Project p=projects[i];
               // Now populate the combo with projects
               // This algorithm is obscure because options
               // are not normal HTML elements
               // First populate the option value (the project database ID).
               //  Then append a text child as the option
               // text, which is what is displayed as the text
               // in each row of the select box
               clonedOption = (HTMLOptionElement)optProject.cloneNode(true);
               clonedOption.setValue(p.getHandle());
               optionTextNode = clonedOption.getOwnerDocument().
                   createTextNode(p.getName());
               clonedOption.appendChild(optionTextNode);
               if (selectedProjectID!=null && p.getHandle().equals(selectedProjectID)) {
                   clonedOption.setAttribute("selected","selected");
               }
               // Do only a shallow copy of the option as we don't want the text child
               // of the node option
               cboProjects.appendChild(clonedOption);
            }
         }
      cboProjects.removeChild(optProject);
     
      } catch(NullPointerException ex) {
      } catch (Exception ex) {
             ex.printStackTrace();
         this.writeDebugMsg("Error populating list of projects: " + ex);
         throw new ProjectManagementPresentationException("Error populating project list: ", ex);
      }

     
   }

   private void fillMonths(ControlsHTML page,String selectedMonthID)
         throws ProjectManagementPresentationException {
      HTMLSelectElement cboMonth=page.getElementCboMonth();
      HTMLOptionElement optMonth = page.getElementOptMonth();
      // Remove the dummy storyboard text from the prototype HTML
      optMonth.removeChild(optMonth.getFirstChild());

      try {
         // set all months
         HTMLOptionElement clonedOption = (HTMLOptionElement)optMonth.cloneNode(true);
         String all="All";
         clonedOption.setValue("");
         Node optionTextNode = clonedOption.getOwnerDocument().
            createTextNode(all);
         clonedOption.appendChild(optionTextNode);
         cboMonth.appendChild(clonedOption);

         for (int i=1; i<=12; i++) {
            // Now populate the combo with months
            // This algorithm is obscure because options
            // are not normal HTML elements
            // First populate the option value (the month database ID).
            //  Then append a text child as the option
            // text, which is what is displayed as the text
            // in each row of the select box
            clonedOption = (HTMLOptionElement)optMonth.cloneNode(true);
            String curMonth=String.valueOf(i);
            clonedOption.setValue(curMonth);
            optionTextNode = clonedOption.getOwnerDocument().
               createTextNode(curMonth);
            clonedOption.appendChild(optionTextNode);
            if (selectedMonthID!=null && curMonth.equals(selectedMonthID)) {
                clonedOption.setAttribute("selected","selected");
            }
            // Do only a shallow copy of the option as we don't want the text child
            // of the node option
            cboMonth.appendChild(clonedOption);
         }
      cboMonth.removeChild(optMonth);
      } catch(NullPointerException ex) {
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of months: " + ex);
         throw new ProjectManagementPresentationException("Error populating month list: ", ex);
      }

      
   }
}
