package projectmanagement.presentation.worksheets;

import projectmanagement.presentation.*;

import projectmanagement.spec.*;
import projectmanagement.spec.timewage.*;
import projectmanagement.spec.project.*;
import projectmanagement.spec.employee.*;
import com.lutris.appserver.server.httpPresentation.*;
import org.enhydra.xml.xmlc.XMLObject;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import java.sql.Date;
import java.sql.Time;
import java.util.*;


/**
 * Manages all actions on worksheets.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class Edit extends BasePO {

   /**
    * Constants
    */
   private static String EMPLOYEE = "cboEmployee";
   private static String PROJECT = "cboProject";
   private static String WORKING_DATE_YYYY = "txtWorkingDateYYYY";
   private static String WORKING_DATE_MM = "txtWorkingDateMM";
   private static String WORKING_DATE_DD = "txtWorkingDateDD";
   private static String TIME_STARTED_hh = "txtTimeStartedhh";
   private static String TIME_STARTED_mm = "txtTimeStartedmm";
   private static String TIME_FINISHED_hh = "txtTimeFinishedhh";
   private static String TIME_FINISHED_mm = "txtTimeFinishedmm";
   private static String PERSONAL_COMMENT= "txtPersonalComment";
   private static String COMMENT_FOR_CLIENT = "txtCommentForClient";

   private static String WORK_SHEET_ID = "workSheetID";
   private static String SELECTED_EMPLOYEE_ID = "selectedEmployeeID";
   private static String CONTEXT_PAGE = "Context.po";
   private static String ADD = "add";
   private static String DELETE = "delete";
   private static String MODIFY = "modify";
   private static String DETAILS = "showDetailsPage";
   private static String POPULATE_PROJECTS = "populateProjects";
   private static String IS_PERSONAL = "isPersonal";
   private static String EVENT = "event";

   /**
    * Superclass method override. Returns 1 or 2, depending on action.
    */
   protected int getRequiredAuthLevel() {
      String isPersonal="";
      try {
         isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      } catch (Exception ex) {}

      if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
         return 1;
      } else {
         return 2;
      }
   }

   /**
    *  Default event. Just show the page for editing.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {
      return showModifyPage(null,false);
   }

   /**
    * handle show add worksheet page event.
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleShowAddPage()
      throws HttpPresentationException {
      return showAddPage(null);
   }

   /**
    * handle show details worksheet page event.
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleShowDetailsPage ()
         throws HttpPresentationException {
      return showModifyPage(null,true);
   }

   /*
    * Modifies an existing worksheet
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleModify()
         throws HttpPresentationException {
      String workSheetID = this.getComms().request.getParameter(WORK_SHEET_ID);
      WorkSheet workSheet=null;
      // Try to get the worksheet by its ID
      try {
     WorkSheetManager workSheetManager = WorkSheetManagerFactory.
               getWorkSheetManager("projectmanagement.business.timewage.WorkSheetManagerImpl"); 
      workSheet = workSheetManager.findWorkSheetByID(workSheetID);
   
      } catch(Exception ex) {
         this.getSessionData().setUserMessage("Please choose a valid worksheet to edit");
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      }

      try {
         saveWorkSheet(workSheet);
         String isPersonal="";
         try {
            isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
         } catch (Exception ex) {}
         String redirectTo=CONTEXT_PAGE;
         if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
            redirectTo=redirectTo+"?isPersonal=true";
         }
         throw new ClientPageRedirectException(redirectTo);
      } catch(Exception ex) {
         return showModifyPage("To modify this worksheet, you must fill out fields properly!",false);
      }
   }

   /*
    * Adds a worksheet to the worksheet database
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleAdd()
         throws HttpPresentationException {
      try {
  WorkSheetManager workSheetManager = WorkSheetManagerFactory.
               getWorkSheetManager("projectmanagement.business.timewage.WorkSheetManagerImpl");      
      WorkSheet workSheet = workSheetManager.getWorkSheet();
      
               saveWorkSheet(workSheet);

         String isPersonal="";
         try {
            isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
   /*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page 
 */ 
     } catch(NullPointerException ex) {
          return showAddPage("You cannot add works sheets while runing ProjectManagement_pres");
                
         } catch (Exception ex) {}
         String redirectTo=CONTEXT_PAGE;
         if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
            redirectTo=redirectTo+"?isPersonal=true";
         }
         throw new ClientPageRedirectException(redirectTo);

      } catch(Exception ex) {
         return showAddPage("To add this worksheet, you must fill out fields properly!");
      }
   }

   /*
    * Deletes a WorkSheet from the WorkSheet database
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handleDelete()
         throws HttpPresentationException, ProjectManagementPresentationException {
      String workSheetID = this.getComms().request.getParameter(WORK_SHEET_ID);

      try {
  WorkSheetManager workSheetManager = WorkSheetManagerFactory.
               getWorkSheetManager("projectmanagement.business.timewage.WorkSheetManagerImpl");      
      WorkSheet workSheet = workSheetManager.findWorkSheetByID(workSheetID);
        
         String workSheetName = "["+workSheet.getPayRate().getEmployee().getFirstName()+" "+
            workSheet.getPayRate().getEmployee().getLastName()+","+
            workSheet.getPayRate().getProject().getName()+"]";
         workSheet.delete();
         this.getSessionData().setUserMessage("The worksheet, " + workSheetName + ", was deleted");
         // Catch any possible database exception as well as the null pointer
         //  exception if the worksheet is not found and is null after findWorkSheetByID
      } catch(Exception ex) {
         this.getSessionData().setUserMessage("Please choose a valid worksheet to delete");
      }

      String isPersonal="";
      try {
         isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      } catch (Exception ex) {}
      String redirectTo=CONTEXT_PAGE;
      if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
         redirectTo=redirectTo+"?isPersonal=true";
      }
      throw new ClientPageRedirectException(redirectTo);

   }

   /*
    * Populates the projects combo based on the selected value of
    * employee combo.
    *
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject handlePopulateProjects ()
         throws HttpPresentationException {
      String workSheetID = this.getComms().request.getParameter(WORK_SHEET_ID);
      XMLObject page=showAddPage(null);
      if (workSheetID!=null && workSheetID.length()>0) {
         ((EditHTML)page).getElementWorkSheetID().setValue(workSheetID);
         ((EditHTML)page).getElementEvent().setValue(MODIFY);
      }
      return page;
   }

   /**
    * Produce HTML for this PO, populated by the worksheet information
    * that the user wants to edit
    *
    * @param errorMsg, the error messages
    * @param disabled, if controls are disabled
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject showModifyPage(String errorMsg,boolean disabled)
         throws HttpPresentationException, ProjectManagementPresentationException {

      String employee = this.getComms().request.getParameter(EMPLOYEE);
      String project = this.getComms().request.getParameter(PROJECT);
      String workingDateYYYY = this.getComms().request.getParameter(WORKING_DATE_YYYY);
      String workingDateMM = this.getComms().request.getParameter(WORKING_DATE_MM);
      String workingDateDD = this.getComms().request.getParameter(WORKING_DATE_DD);
      String timeStartedhh = this.getComms().request.getParameter(TIME_STARTED_hh);
      String timeStartedmm = this.getComms().request.getParameter(TIME_STARTED_mm);
      String timeFinishedhh = this.getComms().request.getParameter(TIME_FINISHED_hh);
      String timeFinishedmm = this.getComms().request.getParameter(TIME_FINISHED_mm);
      String personalComment = this.getComms().request.getParameter(PERSONAL_COMMENT);
      String commentForClient = this.getComms().request.getParameter(COMMENT_FOR_CLIENT);

      String workSheetID = this.getComms().request.getParameter(WORK_SHEET_ID);
      // Instantiate the page object
      EditHTML page = new EditHTML();

      WorkSheet workSheet = null;

      try {
  WorkSheetManager workSheetManager = WorkSheetManagerFactory.
               getWorkSheetManager("projectmanagement.business.timewage.WorkSheetManagerImpl"); 
        workSheet = workSheetManager.findWorkSheetByID(workSheetID); 
  /*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run ProjectManagement_pres )
 * We need  to allow ProjectManagement_pres to be functional ,  response
 * will be default HTML page 
 */ 
 
      } catch(NullPointerException ex) {
           page.setTextErrorText("This is a default HTML page");
           return page;
         // Catch any possible database exception in findWorkSheetByID()
      } catch(ProjectManagementException ex) {
         this.getSessionData().setUserMessage("Please choose a valid worksheet to edit");
         throw new ClientPageRedirectException(CONTEXT_PAGE);
      }

      try {
         // If we received a valid workSheetID then try to show the worksheet's contents,
         // otherwise try to use the HTML input parameters
         page.getElementWorkSheetID().setValue(workSheet.getHandle());

         HTMLSelectElement sel=page.getElementCboEmployee();
         sel.setDisabled(disabled);

         if (null == employee || employee.length() == 0) {
            employee=workSheet.getPayRate().getEmployee().getHandle();
         }

         fillEmployeeSelection(page,sel,employee);

         sel=page.getElementCboProject();
         if (null == project || project.length() == 0) {
            project=workSheet.getPayRate().getProject().getHandle();
         }
         fillProjectSelection(page,sel,employee,project);
         sel.setDisabled(disabled);

         HTMLInputElement el=page.getElementTxtWorkingDateYYYY();
         el.setDisabled(disabled);
         if (null != workingDateYYYY && workingDateYYYY.length() != 0) {
            el.setValue(workingDateYYYY);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(workSheet.getWorkingDate());
            el.setValue(String.valueOf(cal.get(Calendar.YEAR)));
         }

         el=page.getElementTxtWorkingDateMM();
         el.setDisabled(disabled);
         if (null != workingDateMM && workingDateMM.length() != 0) {
            el.setValue(workingDateMM);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(workSheet.getWorkingDate());
            el.setValue(String.valueOf(cal.get(Calendar.MONTH)+1));
         }

         el=page.getElementTxtWorkingDateDD();
         el.setDisabled(disabled);
         if (null != workingDateDD && workingDateDD.length() != 0) {
            el.setValue(workingDateDD);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(workSheet.getWorkingDate());
            el.setValue(String.valueOf(cal.get(Calendar.DAY_OF_MONTH)));
         }

         el=page.getElementTxtTimeStartedhh();
         el.setDisabled(disabled);
         if (null != timeStartedhh && timeStartedhh.length() != 0) {
            el.setValue(timeStartedhh);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(workSheet.getTimeStarted());
            el.setValue(String.valueOf(cal.get(Calendar.HOUR_OF_DAY)));
         }

         el=page.getElementTxtTimeStartedmm();
         el.setDisabled(disabled);
         if (null != timeStartedmm && timeStartedmm.length() != 0) {
            el.setValue(timeStartedmm);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(workSheet.getTimeStarted());
            el.setValue(String.valueOf(cal.get(Calendar.MINUTE)));
         }

         el=page.getElementTxtTimeFinishedhh();
         el.setDisabled(disabled);
         if (null != timeFinishedhh && timeFinishedhh.length() != 0) {
            el.setValue(timeFinishedhh);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(workSheet.getTimeFinished());
            el.setValue(String.valueOf(cal.get(Calendar.HOUR_OF_DAY)));
         }

         el=page.getElementTxtTimeFinishedmm();
         el.setDisabled(disabled);
         if (null != timeFinishedmm && timeFinishedmm.length() != 0) {
            el.setValue(timeFinishedmm);
         } else {
            Calendar cal=new GregorianCalendar();
            cal.setTime(workSheet.getTimeFinished());
            el.setValue(String.valueOf(cal.get(Calendar.MINUTE)));
         }

         //HTMLTextAreaElement tael=page.getElementTxtPersonalComment();
         el=page.getElementTxtPersonalComment();
         el.setDisabled(disabled);
         if (null != personalComment && personalComment.length() != 0) {
            el.setValue(personalComment);
         } else {
            el.setValue(workSheet.getPersonalComment());
         }

         el=page.getElementTxtCommentForClient();
         el.setDisabled(disabled);
         if (null != commentForClient && commentForClient.length() != 0) {
            el.setValue(commentForClient);
         } else {
            el.setValue(workSheet.getCommentForClient());
         }

         el=page.getElementBtnSave();
         el.setDisabled(disabled);

         if (null == errorMsg) {
            page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
         } else {
            page.setTextErrorText(errorMsg);
         }
      } catch(ProjectManagementException ex) {
         throw new ProjectManagementPresentationException("Error populating page for worksheet editing", ex);
      }

      page.getElementEvent().setValue(MODIFY);
      return page;
   }

   /**
    * Produce HTML for this PO
    *
    * @param errorMsg, the error messages
    * @return html document
    * @exception HttpPresentationException
    */
   public XMLObject showAddPage(String errorMsg)
         throws HttpPresentationException, ProjectManagementPresentationException {
    // Instantiate the page object
      EditHTML page = new EditHTML();
try{ 
      String employee = this.getComms().request.getParameter(EMPLOYEE);
      String project = this.getComms().request.getParameter(PROJECT);
      String workingDateYYYY = this.getComms().request.getParameter(WORKING_DATE_YYYY);
      String workingDateMM = this.getComms().request.getParameter(WORKING_DATE_MM);
      String workingDateDD = this.getComms().request.getParameter(WORKING_DATE_DD);
      String timeStartedhh = this.getComms().request.getParameter(TIME_STARTED_hh);
      String timeStartedmm = this.getComms().request.getParameter(TIME_STARTED_mm);
      String timeFinishedhh = this.getComms().request.getParameter(TIME_FINISHED_hh);
      String timeFinishedmm = this.getComms().request.getParameter(TIME_FINISHED_mm);
      String personalComment = this.getComms().request.getParameter(PERSONAL_COMMENT);
      String commentForClient = this.getComms().request.getParameter(COMMENT_FOR_CLIENT);

     

      // set the worksheet id value to empty str
      page.getElementWorkSheetID().setValue("");


      HTMLSelectElement sel=page.getElementCboEmployee();
      fillEmployeeSelection(page,sel,employee);

      if (employee==null) {
         sel.setSelectedIndex(0);
         HTMLOptionElement sopt=(HTMLOptionElement)sel.getOptions().item(0);
         if (sopt!=null) {
            employee=sopt.getValue();
         }
      }

      sel=page.getElementCboProject();
      fillProjectSelection(page,sel,employee,project);

      Calendar cal=new GregorianCalendar();
      HTMLInputElement el=page.getElementTxtWorkingDateYYYY();
      if (null!=workingDateYYYY) {
         el.setValue(workingDateYYYY);
      } else {
         el.setValue(String.valueOf(cal.get(Calendar.YEAR)));
      }

      el=page.getElementTxtWorkingDateMM();
      if (null!=workingDateMM) {
         el.setValue(workingDateMM);
      } else {
         el.setValue(String.valueOf(cal.get(Calendar.MONTH)+1));
      }

      el=page.getElementTxtWorkingDateDD();
      if (null!=workingDateDD) {
         el.setValue(workingDateDD);
      } else {
         el.setValue(String.valueOf(cal.get(Calendar.DAY_OF_MONTH)));
      }

      el=page.getElementTxtTimeStartedhh();
      if (null != timeStartedhh) {
         el.setValue(timeStartedhh);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtTimeStartedmm();
      if (null != timeStartedmm) {
         el.setValue(timeStartedmm);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtTimeFinishedhh();
      if (null != timeFinishedhh) {
         el.setValue(timeFinishedhh);
      } else {
         el.setValue("");
      }

      el=page.getElementTxtTimeFinishedmm();
      if (null != timeFinishedmm) {
         el.setValue(timeFinishedmm);
      } else {
         el.setValue("");
      }

      //HTMLTextAreaElement tael=page.getElementTxtPersonalComment();
      el=page.getElementTxtPersonalComment();
      if (null != personalComment) {
         el.setValue(personalComment);
      } else {
         el.setValue("");
      }
      el=page.getElementTxtCommentForClient();
      if (null != commentForClient) {
         el.setValue(commentForClient);
      } else {
         el.setValue("");
      }


      if (null == errorMsg) {
         page.getElementErrorText().getParentNode().removeChild(page.getElementErrorText());
      } else {
         page.setTextErrorText(errorMsg);
      }
      }catch(NullPointerException e){page.setTextErrorText(errorMsg);}
      return page;
   }

   /**
    * Method to save a new or existing worksheet to the database
    *
    * @param workSheet, the worksheet to be saved
    * @return html document
    * @exception HttpPresentationException
    */
   protected void saveWorkSheet(WorkSheet theWorkSheet)
         throws HttpPresentationException,NullPointerException {

      String employee = this.getComms().request.getParameter(EMPLOYEE);
      String project = this.getComms().request.getParameter(PROJECT);
      String workingDateYYYY = this.getComms().request.getParameter(WORKING_DATE_YYYY);
      String workingDateMM = this.getComms().request.getParameter(WORKING_DATE_MM);
      String workingDateDD = this.getComms().request.getParameter(WORKING_DATE_DD);
      String timeStartedhh = this.getComms().request.getParameter(TIME_STARTED_hh);
      String timeStartedmm = this.getComms().request.getParameter(TIME_STARTED_mm);
      String timeFinishedhh = this.getComms().request.getParameter(TIME_FINISHED_hh);
      String timeFinishedmm = this.getComms().request.getParameter(TIME_FINISHED_mm);
      String personalComment = this.getComms().request.getParameter(PERSONAL_COMMENT);
      String commentForClient = this.getComms().request.getParameter(COMMENT_FOR_CLIENT);

      if (isNullField(employee) || isNullField(project) ||
          !areDateFieldsValid(workingDateYYYY,workingDateMM,workingDateDD) ||
          !areTimeFieldsValid(timeStartedhh,timeStartedmm) ||
          !areTimeFieldsValid(timeFinishedhh,timeFinishedmm)) {
         throw new ProjectManagementPresentationException("Error adding worksheet",new Exception());
      }

      try {
         String workingDate=workingDateYYYY+"-"+workingDateMM+"-"+workingDateDD;
         String timeStarted=timeStartedhh+":"+timeStartedmm+":"+"00";
         String timeFinished=timeFinishedhh+":"+timeFinishedmm+":"+"00";
     
       PayRateManager payRateManager = PayRateManagerFactory.
          getPayRateManager("projectmanagement.business.timewage.PayRateManagerImpl");
       PayRate payRate = payRateManager.
          findPayRateByEmployeeProjectAndDate(employee,project,Date.valueOf(workingDate));
     
         theWorkSheet.setPayRate(payRate);
         theWorkSheet.setWorkingDate(Date.valueOf(workingDate));
         Time ts=Time.valueOf(timeStarted);
         Time tf=Time.valueOf(timeFinished);
         if (tf.getTime()<ts.getTime()) {
            throw new ProjectManagementPresentationException("Error adding worksheet - times are not properly set",null);
         }
         theWorkSheet.setTimeStarted(ts);
         theWorkSheet.setTimeFinished(tf);
         theWorkSheet.setPersonalComment(personalComment);
         theWorkSheet.setCommentForClient(commentForClient);

         theWorkSheet.save();
      } catch(NullPointerException ex) {
      	throw new NullPointerException();
      } catch(Exception ex) {
         throw new ProjectManagementPresentationException("Error adding worksheet", ex);
      }
   }

   private void fillEmployeeSelection (EditHTML page,HTMLSelectElement cboEmployees,
         String selectedEmployeeID) throws ProjectManagementPresentationException {

      String isPersonal="";
      try {
         isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      } catch (Exception ex) {}
      if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
         HTMLInputElement ip=page.getElementIsPersonal();
         ip.setValue("true");
         HTMLAnchorElement a=page.getElementAncContext();
         a.setHref("Context.po?isPersonal=true");
         try {
            selectedEmployeeID=getUser().getHandle();
         } catch (Exception ex) {}
      }

      HTMLOptionElement optEmployee = page.getElementOptEmployee();
      // Remove the dummy storyboard text from the prototype HTML
      optEmployee.removeChild(optEmployee.getFirstChild());

      try {
      EmployeeManager employeeManager = EmployeeManagerFactory.getEmployeeManager("projectmanagement.business.employee.EmployeeManagerImpl");
      Employee[] employees=employeeManager.getAllEmployees();
         if (employees!=null) {
            for (int i=0; i<employees.length; i++) {
               Employee e=employees[i];
               // if this is personal, fill the combo only with that employ
               if (isPersonal!=null && isPersonal.equalsIgnoreCase("true") &&
                     !e.getHandle().equals(selectedEmployeeID)) {
                  continue;
               }
               // Now populate the combo with employees
               // This algorithm is obscure because options
               // are not normal HTML elements
               // First populate the option value (the employee database ID).
               //  Then append a text child as the option
               // text, which is what is displayed as the text
               // in each row of the select box
               HTMLOptionElement clonedOption = (HTMLOptionElement)optEmployee.cloneNode(true);
               clonedOption.setValue(e.getHandle());
               Node optionTextNode = clonedOption.getOwnerDocument().
                   createTextNode(e.getFirstName()+" "+e.getLastName());
               clonedOption.appendChild(optionTextNode);
               if (e.getHandle().equals(selectedEmployeeID)) {
                   clonedOption.setAttribute("selected","selected");
               }
               // Do only a shallow copy of the option as we don't want the text child
               // of the node option
               cboEmployees.appendChild(clonedOption);
               // Alternative way to insert nodes below
               // insertBefore(newNode, oldNode);
               // discSelect.insertBefore(clonedOption, templateOption);
            }
         }
      cboEmployees.removeChild(optEmployee);
 
      } catch(NullPointerException ex) {
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of employees: " + ex);
         throw new ProjectManagementPresentationException("Error populating employee list: ", ex);
      }

   }

   private void fillProjectSelection (EditHTML page,HTMLSelectElement cboProjects,
         String selectedEmployeeID,String selectedProjectID)
         throws ProjectManagementPresentationException {

      HTMLOptionElement optProject = page.getElementOptProject();
      // Remove the dummy storyboard text from the prototype HTML
      optProject.removeChild(optProject.getFirstChild());

      try {
         PayRate[] payRates;
    PayRateManager payRateManager = PayRateManagerFactory.
          getPayRateManager("projectmanagement.business.timewage.PayRateManagerImpl");
        
       
         if (selectedEmployeeID==null) {
            payRates=payRateManager.getAllPayRates();
         } else {
            payRates=payRateManager.getAllPayRatesForEmployee(selectedEmployeeID,true);
         }
         if (payRates!=null) {
            for (int i=0; i<payRates.length; i++) {
               Project p=payRates[i].getProject();
               // Now populate the combo with projects
               // This algorithm is obscure because options
               // are not normal HTML elements
               // First populate the option value (the project database ID).
               //  Then append a text child as the option
               // text, which is what is displayed as the text
               // in each row of the select box
               HTMLOptionElement clonedOption = (HTMLOptionElement)optProject.cloneNode(true);
               clonedOption.setValue(p.getHandle());
               Node optionTextNode = clonedOption.getOwnerDocument().
                   createTextNode(p.getName());
               clonedOption.appendChild(optionTextNode);
               if (p.getHandle().equals(selectedProjectID)) {
                   clonedOption.setAttribute("selected","selected");
               }
               // Do only a shallow copy of the option as we don't want the text child
               // of the node option
               cboProjects.appendChild(clonedOption);
               // Alternative way to insert nodes below
               // insertBefore(newNode, oldNode);
               // discSelect.insertBefore(clonedOption, templateOption);
            }
         }
      cboProjects.removeChild(optProject);
      } catch(NullPointerException ex) {      
      } catch (Exception ex) {
ex.printStackTrace();
         this.writeDebugMsg("Error populating list of projects: " + ex);
         throw new ProjectManagementPresentationException("Error populating project list: ", ex);
      }

      
   }

   private boolean areTimeFieldsValid (String hh,String mm) {
      if (isNullField(hh) || isNullField(mm)) {
         return false;
      }
      try {
         int h=Integer.valueOf(hh).intValue();
         int m=Integer.valueOf(mm).intValue();
         if (h<0 || h>23 || m<0 || m>59) {
            return false;
         }
      } catch (Exception ex) {
         return false;
      }
      return true;
   }
}

