/*
 *  projectManagement
 *
 *  Enhydra super-servlet presentation object
 *
 */

package projectmanagement.presentation.worksheets;

import projectmanagement.spec.employee.*;
import projectmanagement.spec.project.*;
import projectmanagement.presentation.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

// Enhydra SuperServlet imports
import com.lutris.appserver.server.httpPresentation.HttpPresentation;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import org.enhydra.xml.xmlc.XMLObject;

// Standard imports
import java.io.IOException;
import java.util.Calendar;

/**
* Generates the blank HTML page.
*/
public class Search extends BasePO {

   private static String EMPLOYEE = "employee";
   private static String PROJECT = "project";
   private static String FROMYEAR = "fromyear";
   private static String FROMMONTH = "frommonth";
   private static String FROMDAY = "fromday";
   private static String TOYEAR = "toyear";
   private static String TOMONTH = "tomonth";
   private static String TODAY = "today";

   private static String IS_PERSONAL = "isPersonal";

    /**
    * Superclass method override. Returns 1 or 2, depending on action.
    */
   protected int getRequiredAuthLevel() {
      String isPersonal="";
      try {
         isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      } catch (Exception ex) {}

      if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
         return 1;
      } else {
         return 2;
      }
   }

   /**
    *  Default event. Just show the page.
    */
   public XMLObject handleDefault()
         throws HttpPresentationException {
      SearchHTML page=new SearchHTML();

      String employee = this.getComms().request.getParameter(EMPLOYEE);
      HTMLSelectElement sel=page.getElementCboEmployee();
      
  
      fillEmployeeSelection(page,sel,employee);
     
      String project = this.getComms().request.getParameter(PROJECT);
      sel=page.getElementCboProject();
      fillProjectSelection(page,sel,project);

      fillFromDays(page);
      fillFromMonths(page);
      fillFromYears(page);

      fillToDays(page);
      fillToMonths(page);
      fillToYears(page);


      return page;
   }

   private void fillEmployeeSelection (SearchHTML page,HTMLSelectElement cboEmployees,
         String selectedEmployeeID) throws ProjectManagementPresentationException {
      HTMLOptionElement optEmployee = page.getElementOptEmployee();
  try {
      EmployeeManager employeeManager = EmployeeManagerFactory.getEmployeeManager("projectmanagement.business.employee.EmployeeManagerImpl");
      Employee[] employees=employeeManager.getAllEmployees();
 
      // Remove the dummy storyboard text from the prototype HTML
      optEmployee.removeChild(optEmployee.getFirstChild());

      String isPersonal="";
     
         isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
      
      if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
         try {
            selectedEmployeeID=getUser().getHandle();
         } catch (Exception ex) {}
      }

     
         // set the text to select all employees - default option
         HTMLOptionElement clonedOption=null;
         Node optionTextNode=null;
         if (isPersonal==null || !isPersonal.equalsIgnoreCase("true")) {
            clonedOption = (HTMLOptionElement)optEmployee.cloneNode(true);
            clonedOption.setValue("");
            String all="All";
            optionTextNode = clonedOption.getOwnerDocument().
               createTextNode(all);
            clonedOption.appendChild(optionTextNode);
             clonedOption.setAttribute("selected","selected");
            // Do only a shallow copy of the option as we don't want the text child
            // of the node option
            cboEmployees.appendChild(clonedOption);
         }

         // set all employees

          if (employees!=null) {
            for (int i=0; i<employees.length; i++) {
               Employee e=employees[i];
               // if this is personal, fill the combo only with that employ
               if (isPersonal!=null && isPersonal.equalsIgnoreCase("true") &&
                     !e.getHandle().equals(selectedEmployeeID)) {
                  continue;
               }

               // Now populate the combo with employees
               // This algorithm is obscure because options
               // are not normal HTML elements
               // First populate the option value (the employee database ID).
               //  Then append a text child as the option
               // text, which is what is displayed as the text
               // in each row of the select box
               clonedOption = (HTMLOptionElement)optEmployee.cloneNode(true);
               clonedOption.setValue(e.getHandle());
               optionTextNode = clonedOption.getOwnerDocument().
                   createTextNode(e.getFirstName()+" "+e.getLastName());
               clonedOption.appendChild(optionTextNode);
               if (selectedEmployeeID!=null && e.getHandle().equals(selectedEmployeeID)) {
                  clonedOption.setAttribute("selected","selected");
               }
               // Do only a shallow copy of the option as we don't want the text child
               // of the node option
               cboEmployees.appendChild(clonedOption);
            }
         }
         if ( this.getComms().request.getParameter("checkFromDate") != null && this.getComms().request.getParameter("checkFromDate").equals("false") ) {
            ( (HTMLElement)page.getElementColorFrom() ).setAttribute("color","red");
          }
          if ( this.getComms().request.getParameter("checkToDate") != null && this.getComms().request.getParameter("checkToDate").equals("false") ) {
            ( (HTMLElement)page.getElementColorTo() ).setAttribute("color","red");
          }
      } catch(NullPointerException ex) {
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of employees: " + ex);
         throw new ProjectManagementPresentationException("Error populating employee list: ", ex);
      }

      cboEmployees.removeChild(optEmployee);
   }

   private void fillProjectSelection (SearchHTML page,HTMLSelectElement cboProjects,
         String selectedProjectID) throws ProjectManagementPresentationException {
      HTMLOptionElement optProject = page.getElementOptProject();
      try {
         Project[] projects=null;
       ProjectManager projectManager = ProjectManagerFactory.getProjectManager("projectmanagement.business.project.ProjectManagerImpl");

      // Remove the dummy storyboard text from the prototype HTML
      optProject.removeChild(optProject.getFirstChild());
     
      String isPersonal="";
      
         isPersonal=this.getComms().request.getParameter(IS_PERSONAL);
    

     
         // set the text to select all projects - default option
         HTMLOptionElement clonedOption = (HTMLOptionElement)optProject.cloneNode(true);
         clonedOption.setValue("");
         String all="All";
         Node optionTextNode = clonedOption.getOwnerDocument().
            createTextNode(all);
         clonedOption.appendChild(optionTextNode);
          clonedOption.setAttribute("selected","selected");
         // Do only a shallow copy of the option as we don't want the text child
         // of the node option
         cboProjects.appendChild(clonedOption);

         // set all projects
          
         if (isPersonal!=null && isPersonal.equalsIgnoreCase("true")) {
            // IMPLEMENT ME!!!
            projects=projectManager.getAllProjects();
         } else {
            projects=projectManager.getAllProjects();
         }
         if (projects!=null) {
            for (int i=0; i<projects.length; i++) {
               Project p=projects[i];
               // Now populate the combo with projects
               // This algorithm is obscure because options
               // are not normal HTML elements
               // First populate the option value (the project database ID).
               //  Then append a text child as the option
               // text, which is what is displayed as the text
               // in each row of the select box
               clonedOption = (HTMLOptionElement)optProject.cloneNode(true);
               clonedOption.setValue(p.getHandle());
               optionTextNode = clonedOption.getOwnerDocument().
                   createTextNode(p.getName());
               clonedOption.appendChild(optionTextNode);
               if (selectedProjectID!=null && p.getHandle().equals(selectedProjectID)) {
                   clonedOption.setAttribute("selected","selected");
               }
               // Do only a shallow copy of the option as we don't want the text child
               // of the node option
               cboProjects.appendChild(clonedOption);
            }
          }
         cboProjects.removeChild(optProject);
      } catch(NullPointerException ex) {
      } catch (Exception ex) {
        ex.printStackTrace();
         this.writeDebugMsg("Error populating list of projects: " + ex);
         throw new ProjectManagementPresentationException("Error populating project list: ", ex);
      }

      
   }

   private void fillFromDays(SearchHTML page)
         throws ProjectManagementPresentationException {
      HTMLSelectElement cboFromDay=page.getElementCboFromDay();
      HTMLOptionElement optFromDay = page.getElementOptFromDay();
      // Remove the dummy storyboard text from the prototype HTML
      optFromDay.removeChild(optFromDay.getFirstChild());
      try {
         // set all days
         HTMLOptionElement clonedOption = (HTMLOptionElement)optFromDay.cloneNode(true);
         Node optionTextNode;
         /*
         String all="All";
         clonedOption.setValue("");
         Node optionTextNode = clonedOption.getOwnerDocument().
            createTextNode(all);
         clonedOption.appendChild(optionTextNode);
         cboFromDay.appendChild(clonedOption);
         */

         for (int i=1; i<=31; i++) {
            // Now populate the combo with days
            // This algorithm is obscure because options
            // are not normal HTML elements
            // First populate the option value (the day database ID).
            //  Then append a text child as the option
            // text, which is what is displayed as the text
            // in each row of the select box
            clonedOption = (HTMLOptionElement)optFromDay.cloneNode(true);
            String curDay=String.valueOf(i);
            clonedOption.setValue(curDay);
            optionTextNode = clonedOption.getOwnerDocument().
               createTextNode(curDay);
            clonedOption.appendChild(optionTextNode);
            if ( i==Calendar.getInstance().get(Calendar.DAY_OF_MONTH) ) {
                clonedOption.setAttribute("selected","selected");
            }

            // Do only a shallow copy of the option as we don't want the text child
            // of the node option
            cboFromDay.appendChild(clonedOption);
           
         }
       cboFromDay.removeChild(optFromDay);
      } catch(NullPointerException ex) {
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of days: " + ex);
         throw new ProjectManagementPresentationException("Error populating day list: ", ex);
      }

      
   }

   private void fillFromMonths(SearchHTML page)
         throws ProjectManagementPresentationException {
      HTMLSelectElement cboFromMonth=page.getElementCboFromMonth();
      HTMLOptionElement optFromMonth = page.getElementOptFromMonth();
      // Remove the dummy storyboard text from the prototype HTML
      optFromMonth.removeChild(optFromMonth.getFirstChild());
      try {
 
         // set all months
         HTMLOptionElement clonedOption = (HTMLOptionElement)optFromMonth.cloneNode(true);
         Node optionTextNode;
         /*
         String all="All";
         clonedOption.setValue("");
         Node optionTextNode = clonedOption.getOwnerDocument().
            createTextNode(all);
         clonedOption.appendChild(optionTextNode);
         cboFromMonth.appendChild(clonedOption);
         */

         for (int i=1; i<=12; i++) {
            // Now populate the combo with months
            // This algorithm is obscure because options
            // are not normal HTML elements
            // First populate the option value (the month database ID).
            //  Then append a text child as the option
            // text, which is what is displayed as the text
            // in each row of the select box
            clonedOption = (HTMLOptionElement)optFromMonth.cloneNode(true);
            String curMonth=String.valueOf(i);
            clonedOption.setValue(curMonth);
            optionTextNode = clonedOption.getOwnerDocument().
               createTextNode(curMonth);
            clonedOption.appendChild(optionTextNode);
            if( i==Calendar.getInstance().get(Calendar.MONTH)+1 ) {
                clonedOption.setAttribute("selected","selected");
            }

            // Do only a shallow copy of the option as we don't want the text child
            // of the node option
            cboFromMonth.appendChild(clonedOption);
         }
     cboFromMonth.removeChild(optFromMonth);
      } catch(NullPointerException ex) {      
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of months: " + ex);
         throw new ProjectManagementPresentationException("Error populating month list: ", ex);
      }
   }

   private void fillFromYears(SearchHTML page)
         throws ProjectManagementPresentationException {
      HTMLSelectElement cboFromYear=page.getElementCboFromYear();
      HTMLOptionElement optFromYear = page.getElementOptFromYear();
      // Remove the dummy storyboard text from the prototype HTML
      optFromYear.removeChild(optFromYear.getFirstChild());
      try {
         // set all years
         HTMLOptionElement clonedOption = (HTMLOptionElement)optFromYear.cloneNode(true);
         Node optionTextNode;
         /*
         String all="All";
         clonedOption.setValue("");
         Node optionTextNode = clonedOption.getOwnerDocument().
            createTextNode(all);
         clonedOption.appendChild(optionTextNode);
         cboFromYear.appendChild(clonedOption);
         */

         for (int i=1998; i<=2008; i++) {
            // Now populate the combo with years
            // This algorithm is obscure because options
            // are not normal HTML elements
            // First populate the option value (the year database ID).
            //  Then append a text child as the option
            // text, which is what is displayed as the text
            // in each row of the select box
            clonedOption = (HTMLOptionElement)optFromYear.cloneNode(true);
            String curYear=String.valueOf(i);
            clonedOption.setValue(curYear);
            optionTextNode = clonedOption.getOwnerDocument().
               createTextNode(curYear);
            clonedOption.appendChild(optionTextNode);
            if( i==Calendar.getInstance().get(Calendar.YEAR) ) {
                clonedOption.setAttribute("selected","selected");
            }

            // Do only a shallow copy of the option as we don't want the text child
            // of the node option
            cboFromYear.appendChild(clonedOption);
         }
         cboFromYear.removeChild(optFromYear);
 
      } catch(NullPointerException ex) {    
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of years: " + ex);
         throw new ProjectManagementPresentationException("Error populating year list: ", ex);
      }

      
   }


   private void fillToDays(SearchHTML page)
         throws ProjectManagementPresentationException {
      HTMLSelectElement cboToDay=page.getElementCboToDay();
      HTMLOptionElement optToDay = page.getElementOptToDay();
      // Remove the dummy storyboard text to the prototype HTML
      optToDay.removeChild(optToDay.getFirstChild());
      try {
         // set all days
         HTMLOptionElement clonedOption = (HTMLOptionElement)optToDay.cloneNode(true);
         Node optionTextNode;
         /*
         String all="All";
         clonedOption.setValue("");
         Node optionTextNode = clonedOption.getOwnerDocument().
            createTextNode(all);
         clonedOption.appendChild(optionTextNode);
         cboToDay.appendChild(clonedOption);
         */

         for (int i=1; i<=31; i++) {
            // Now populate the combo with days
            // This algorithm is obscure because options
            // are not normal HTML elements
            // First populate the option value (the day database ID).
            //  Then append a text child as the option
            // text, which is what is displayed as the text
            // in each row of the select box
            clonedOption = (HTMLOptionElement)optToDay.cloneNode(true);
            String curDay=String.valueOf(i);
            clonedOption.setValue(curDay);
            optionTextNode = clonedOption.getOwnerDocument().
               createTextNode(curDay);
            clonedOption.appendChild(optionTextNode);
            if( i==Calendar.getInstance().get(Calendar.DAY_OF_MONTH) ) {
                clonedOption.setAttribute("selected","selected");
            }

            // Do only a shallow copy of the option as we don't want the text child
            // of the node option
            cboToDay.appendChild(clonedOption);
         }
          cboToDay.removeChild(optToDay);
      } catch(NullPointerException ex) {        
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of days: " + ex);
         throw new ProjectManagementPresentationException("Error populating day list: ", ex);
      }

     
   }

   private void fillToMonths(SearchHTML page)
         throws ProjectManagementPresentationException {
      HTMLSelectElement cboToMonth=page.getElementCboToMonth();
      HTMLOptionElement optToMonth = page.getElementOptToMonth();
      // Remove the dummy storyboard text to the prototype HTML
      optToMonth.removeChild(optToMonth.getFirstChild());
      try {
         // set all months
         HTMLOptionElement clonedOption = (HTMLOptionElement)optToMonth.cloneNode(true);
         Node optionTextNode;
         /*
         String all="All";
         clonedOption.setValue("");
         Node optionTextNode = clonedOption.getOwnerDocument().
            createTextNode(all);
         clonedOption.appendChild(optionTextNode);
         cboToMonth.appendChild(clonedOption);
         */

         for (int i=1; i<=12; i++) {
            // Now populate the combo with months
            // This algorithm is obscure because options
            // are not normal HTML elements
            // First populate the option value (the month database ID).
            //  Then append a text child as the option
            // text, which is what is displayed as the text
            // in each row of the select box
            clonedOption = (HTMLOptionElement)optToMonth.cloneNode(true);
            String curMonth=String.valueOf(i);
            clonedOption.setValue(curMonth);
            optionTextNode = clonedOption.getOwnerDocument().
               createTextNode(curMonth);
            clonedOption.appendChild(optionTextNode);
            if( i==Calendar.getInstance().get(Calendar.MONTH)+1 ) {
                clonedOption.setAttribute("selected","selected");
            }
            // Do only a shallow copy of the option as we don't want the text child
            // of the node option
            cboToMonth.appendChild(clonedOption);
         }
      cboToMonth.removeChild(optToMonth);
      } catch(NullPointerException ex) {  
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of months: " + ex);
         throw new ProjectManagementPresentationException("Error populating month list: ", ex);
      }

      
   }

   private void fillToYears(SearchHTML page)
         throws ProjectManagementPresentationException {
      HTMLSelectElement cboToYear=page.getElementCboToYear();
      HTMLOptionElement optToYear = page.getElementOptToYear();
      // Remove the dummy storyboard text to the prototype HTML
      optToYear.removeChild(optToYear.getFirstChild());
      try {
         // set all years
         HTMLOptionElement clonedOption = (HTMLOptionElement)optToYear.cloneNode(true);
         Node optionTextNode;
         /*
         String all="All";
         clonedOption.setValue("");
         Node optionTextNode = clonedOption.getOwnerDocument().
            createTextNode(all);
         clonedOption.appendChild(optionTextNode);
         cboToYear.appendChild(clonedOption);
         */

         for (int i=1998; i<=2008; i++) {
            // Now populate the combo with years
            // This algorithm is obscure because options
            // are not normal HTML elements
            // First populate the option value (the year database ID).
            //  Then append a text child as the option
            // text, which is what is displayed as the text
            // in each row of the select box
            clonedOption = (HTMLOptionElement)optToYear.cloneNode(true);
            String curYear=String.valueOf(i);
            clonedOption.setValue(curYear);
            optionTextNode = clonedOption.getOwnerDocument().
               createTextNode(curYear);
            clonedOption.appendChild(optionTextNode);
            if ( i==Calendar.getInstance().get(Calendar.YEAR) ) {
               clonedOption.setAttribute("selected","selected");
            }
            // Do only a shallow copy of the option as we don't want the text child
            // of the node option
            cboToYear.appendChild(clonedOption);
         }
          cboToYear.removeChild(optToYear);
       } catch(NullPointerException ex) {  
      } catch (Exception ex) {
         this.writeDebugMsg("Error populating list of years: " + ex);
         throw new ProjectManagementPresentationException("Error populating year list: ", ex);
      }

     
   }


}