<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet version="1.0"
				xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xslt"
                xmlns:xalan-ext="http://xml.apache.org/xalan">
                
<xsl:output method="text" encoding="UTF-8" media-type="text/plain"/>
	
<xsl:template match ="/">
CREATE PROCEDURE create_oid @returnOid decimal(19,0) output AS

BEGIN

  declare @OID_MIN_VALUE DECIMAL(19,0)           -- minimal OID value constant
  declare @OID_INCREMENT_VALUE DECIMAL(19,0)     -- increment for OIDs constant
  declare @TRY_COUNT DECIMAL(2)                  -- number of trys in creating oid when error occurs
  declare @is_not_ok BIT                         -- oid is not successfuly created (0=0,1=1)
  declare @i INT
  declare @errno INT                            -- error code number
  
  set @OID_MIN_VALUE = 1      
  set @OID_INCREMENT_VALUE = 1    
  set @TRY_COUNT = 3      
  set @is_not_ok = 1       
  set @i = 1
  set @returnOid = -1        

  
  WHILE (@is_not_ok = 1 AND @i &lt;= @TRY_COUNT)
  BEGIN
    SELECT @returnOid = (SELECT NEXT
                         FROM OBJECTID);

    set @errno = @@ERROR
    IF(@errno = 0 AND (@returnOid is null OR @returnOid = -1) )-- no data found
    BEGIN
      BEGIN TRANSACTION
      INSERT INTO OBJECTID
      VALUES(@OID_MIN_VALUE)
      COMMIT
      set @returnOid = @OID_MIN_VALUE
    END

    ELSE IF(@errno = '512') -- too meny rows
    BEGIN
    print @errno
       BEGIN TRANSACTION
      DELETE FROM OBJECTID
      WHERE NEXT != (SELECT MAX(NEXT) 
                     FROM OBJECTID);
      COMMIT
    END

    ELSE IF(@errno != 0 AND @errno != 512) -- others
    BEGIN
      raiserror ('Unable to find OID. Error during the select!',16,-1)
      return(-1)
    END

    IF(@returnOid != -1)
    BEGIN
      BEGIN TRANSACTION
      UPDATE OBJECTID
      SET NEXT = @returnOid + @OID_INCREMENT_VALUE
      WHERE NEXT = @returnOid;
       
      IF (@@ROWCOUNT = 1)
      BEGIN
        set @is_not_ok = 0;
        COMMIT
      END
      ELSE
        ROLLBACK
    END;
    
    set @i = @i + 1;
  END
  
  IF(@is_not_ok = 1) 
  BEGIN 
    raiserror('Unable to find OID!',16,-1)
    return(-1)
  END
  
   
END
GO



CREATE PROCEDURE create_oid_per_table @table_name_i varchar(60), @returnOid decimal(19,0) output AS

BEGIN

  declare @OID_MIN_VALUE DECIMAL(19,0)           -- minimal OID value constant
  declare @OID_INCREMENT_VALUE DECIMAL(19,0)     -- increment for OIDs constant
  declare @TRY_COUNT DECIMAL(2)                  -- number of trys in creating oid when error occurs
  declare @is_not_ok BIT                         -- oid is not successfuly created (0=0,1=1)
  declare @i INT
  declare @errno INT                            -- error code number
  
  set @OID_MIN_VALUE = 1      
  set @OID_INCREMENT_VALUE = 1    
  set @TRY_COUNT = 3      
  set @is_not_ok = 1       
  set @i = 1
  set @returnOid = -1        

  
  IF (@table_name_i IS NULL OR rtrim(@table_name_i) = '')
  BEGIN
     raiserror ('Table name is missing!',16,-1)
     return(-1)
  END
  
  WHILE (@is_not_ok = 1 AND @i &lt;= @TRY_COUNT)
  BEGIN
    SELECT @returnOid = (SELECT NEXT
                         FROM OBJECTID
                         WHERE table_name = @table_name_i);

    set @errno = @@ERROR
    IF(@errno = 0 AND (@returnOid is null OR @returnOid = -1) )-- no data found
    BEGIN
      BEGIN TRANSACTION
      INSERT INTO OBJECTID
      VALUES(@OID_MIN_VALUE,@table_name_i)
      COMMIT
      set @returnOid = @OID_MIN_VALUE
    END

    ELSE IF(@errno = '512') -- too meny rows
    BEGIN
    print @errno
       BEGIN TRANSACTION
      DELETE FROM OBJECTID
      WHERE table_name = @table_name_i
      AND NEXT != (SELECT MAX(NEXT) 
                   FROM OBJECTID WHERE table_name = @table_name_i);
      COMMIT
    END

    ELSE IF(@errno != 0 AND @errno != 512) -- others
    BEGIN
      raiserror ('Unable to find OID. Error during the select!',16,-1)
      return(-1)
    END

    IF(@returnOid != -1)
    BEGIN
      BEGIN TRANSACTION
      UPDATE OBJECTID
      SET NEXT = @returnOid + @OID_INCREMENT_VALUE
      WHERE table_name = @table_name_i
      AND NEXT = @returnOid;
       
      IF (@@ROWCOUNT = 1)
      BEGIN
        set @is_not_ok = 0;
        COMMIT
      END
      ELSE
        ROLLBACK
    END;
    
    set @i = @i + 1;
  END
  
  IF(@is_not_ok = 1) 
  BEGIN 
    raiserror('Unable to find OID!',16,-1)
    return(-1)
  END
  
   
END
GO


</xsl:template>

</xsl:stylesheet>
