<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet version="1.0"
				xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xslt"
                xmlns:xalan-ext="http://xml.apache.org/xalan">
                
<xsl:output method="text" encoding="UTF-8" media-type="text/plain"/>
	
<xsl:template match ="/">
CREATE OR REPLACE FUNCTION create_oid RETURN NUMBER IS

  OID_MIN_VALUE NUMBER(19,0) := 1;        -- minimal OID value constant
  OID_INCREMENT_VALUE NUMBER(19,0) := 1;  -- increment for OIDs constant
  TRY_COUNT NUMBER(2) := 3;               -- number of trys in creating oid when error occurs

  is_not_ok BOOLEAN := TRUE;              -- oid is not successfuly created
  returnOid NUMBER(19,0) := -1;           -- return value
  i NUMBER(2) := 1;                       -- loop counter
  row_count NUMBER(2) := 0;               -- updated row counter
  
  OID_CREATION_ERROR EXCEPTION;
  PRAGMA EXCEPTION_INIT(OID_CREATION_ERROR, -20701);

BEGIN

  WHILE (is_not_ok AND i &lt;= TRY_COUNT) LOOP
    BEGIN
       SELECT NEXT
       INTO returnOid
       FROM OBJECTID;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
       INSERT INTO OBJECTID
       VALUES(OID_MIN_VALUE);
       COMMIT;
       returnOid := OID_MIN_VALUE;

      WHEN TOO_MANY_ROWS THEN
        DELETE FROM OBJECTID
        WHERE NEXT != (SELECT MAX(NEXT)
                       FROM OBJECTID);
       COMMIT;
	   returnOid := -1;
	   
      WHEN OTHERS THEN
        RAISE_APPLICATION_ERROR(-20701, 'Unable to find OID. Error during the select!');
    END;

	BEGIN
	  IF(returnOid != -1) THEN
	    UPDATE OBJECTID
	    SET NEXT = returnOid + OID_INCREMENT_VALUE
	    WHERE NEXT = returnOid;
		row_count := SQL%ROWCOUNT;
	
	    IF (row_count = 1) THEN
		  COMMIT;
	      is_not_ok := FALSE;
	    END IF;
	  END IF;

	EXCEPTION
      WHEN OTHERS THEN
	    is_not_ok := TRUE;
	END;
	
    i := i + 1;
  END LOOP;

  IF(is_not_ok) THEN
     RAISE_APPLICATION_ERROR(-20701, 'Unable to find OID!');
  END IF;

  RETURN returnOid;

END create_oid;


CREATE OR REPLACE FUNCTION create_oid_per_table(table_name_i VARCHAR2) RETURN NUMBER IS

  OID_MIN_VALUE NUMBER(19,0) := 1;        -- minimal OID value constant
  OID_INCREMENT_VALUE NUMBER(19,0) := 1;  -- increment for OIDs constant
  TRY_COUNT NUMBER(2) := 3;               -- number of trys in creating oid when error occurs

  is_not_ok BOOLEAN := TRUE;              -- oid is not successfuly created
  returnOid NUMBER(19,0) := -1;           -- return value
  i NUMBER(2) := 1;                       -- loop counter
  row_count NUMBER(2) := 0;               -- updated row counter
  

  OID_CREATION_ERROR EXCEPTION;
  PRAGMA EXCEPTION_INIT(OID_CREATION_ERROR, -20701);

BEGIN
  IF(table_name_i IS NULL OR trim(table_name_i) = '') THEN
    RAISE_APPLICATION_ERROR(-20701, 'Table name is missing!');
  END IF;

  WHILE (is_not_ok AND i &lt;= TRY_COUNT) LOOP
    BEGIN
       SELECT NEXT
       INTO returnOid
       FROM OBJECTID
       WHERE table_name = table_name_i;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
       INSERT INTO OBJECTID
       VALUES(OID_MIN_VALUE,table_name_i);
       COMMIT;
       returnOid := OID_MIN_VALUE;

      WHEN TOO_MANY_ROWS THEN
        DELETE FROM OBJECTID
        WHERE table_name = table_name_i
        AND NEXT != (SELECT MAX(NEXT)
                     FROM OBJECTID WHERE table_name = table_name_i);
       COMMIT;
	   returnOid := -1;
	   
      WHEN OTHERS THEN
        RAISE_APPLICATION_ERROR(-20701, 'Unable to find OID. Error during the select!');
    END;

	BEGIN
	  IF(returnOid != -1) THEN
	    UPDATE OBJECTID
	    SET NEXT = returnOid + OID_INCREMENT_VALUE
	    WHERE table_name = table_name_i
	    AND NEXT = returnOid;
		row_count := SQL%ROWCOUNT;
	
	    IF (row_count = 1) THEN
		  COMMIT;
	      is_not_ok := FALSE;
	    END IF;
	  END IF;

	EXCEPTION
      WHEN OTHERS THEN
	    is_not_ok := TRUE;
	END;
	
    i := i + 1;
  END LOOP;

  IF(is_not_ok) THEN
     RAISE_APPLICATION_ERROR(-20701, 'Unable to find OID!');
  END IF;

  RETURN returnOid;

END Create_Oid_Per_Table;

</xsl:template>

</xsl:stylesheet>
