<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet version="1.0"
				xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xslt"
                xmlns:xalan-ext="http://xml.apache.org/xalan">
                
<xsl:output method="text" encoding="UTF-8" media-type="text/plain"/>
	
<xsl:template match ="/">

CREATE OR REPLACE FUNCTION create_oid()
  RETURNS "numeric" AS
$$
DECLARE

  OID_MIN_VALUE NUMERIC(19,0) := 1;        -- minimal OID value constant
  OID_INCREMENT_VALUE NUMERIC(19,0) := 1;  -- increment for OIDs constant
  TRY_COUNT NUMERIC(2) := 3;               -- number of trys in creating oid when error occurs

  is_not_ok BOOLEAN := TRUE;               -- oid is not successfuly created
  returnOid NUMERIC(19,0) := -1;           -- return value
  i NUMERIC(2) := 1;                       -- loop counter
  row_count_v NUMERIC(2) := 0;             -- updated row counter
  
BEGIN

  WHILE (is_not_ok = TRUE AND i &lt;= TRY_COUNT) LOOP

    SELECT count(NEXT), max(NEXT)
    INTO row_count_v, returnOid
    FROM OBJECTID

    IF(row_count_v = 0) THEN
      INSERT INTO OBJECTID
      VALUES(OID_MIN_VALUE);
      returnOid := OID_MIN_VALUE;

    ELSIF(row_count_v > 1) THEN
      DELETE FROM OBJECTID
      WHERE NEXT != (SELECT MAX(NEXT)
	                   FROM OBJECTID);
      returnOid := -1;

    ELSIF(row_count_v &lt; 0) THEN
      RAISE EXCEPTION 'Unable to find OID. Error during the select!';

    END IF;

    BEGIN
      IF(returnOid != -1) THEN
        UPDATE OBJECTID
        SET NEXT = returnOid + OID_INCREMENT_VALUE
        WHERE NEXT = returnOid;
      
        GET DIAGNOSTICS row_count_v = ROW_COUNT;
  
        IF (row_count_v = 1) THEN
          is_not_ok := FALSE;
        END IF;
      END IF;

    EXCEPTION
      WHEN OTHERS THEN
      is_not_ok := TRUE;
    END;
  
    i := i + 1;
  END LOOP;

  IF(is_not_ok) THEN
     RAISE EXCEPTION 'Unable to find OID!';
  END IF;

  RETURN returnOid;

END;
$$
  LANGUAGE 'plpgsql' VOLATILE;
  
  
CREATE OR REPLACE FUNCTION create_oid_per_table(table_name_i VARCHAR)
  RETURNS "numeric" AS
$$
DECLARE

  OID_MIN_VALUE NUMERIC(19,0) := 1;        -- minimal OID value constant
  OID_INCREMENT_VALUE NUMERIC(19,0) := 1;  -- increment for OIDs constant
  TRY_COUNT NUMERIC(2) := 3;               -- number of trys in creating oid when error occurs

  is_not_ok BOOLEAN := TRUE;               -- oid is not successfuly created
  returnOid NUMERIC(19,0) := -1;           -- return value
  i NUMERIC(2) := 1;                       -- loop counter
  row_count_v NUMERIC(2) := 0;             -- updated row counter
  
BEGIN
  IF(table_name_i IS NULL OR trim(table_name_i) = '') THEN
    RAISE EXCEPTION 'Table name is missing!';
  END IF;

  WHILE (is_not_ok = TRUE AND i &lt;= TRY_COUNT) LOOP

    SELECT count(NEXT), max(NEXT)
    INTO row_count_v, returnOid
    FROM OBJECTID
    WHERE TABLE_NAME = table_name_i;
       
    IF(row_count_v = 0) THEN
      INSERT INTO OBJECTID
      VALUES(OID_MIN_VALUE,table_name_i);
      returnOid := OID_MIN_VALUE;

    ELSIF(row_count_v > 1) THEN
      DELETE FROM OBJECTID
      WHERE table_name = table_name_i
      AND NEXT != (SELECT MAX(NEXT)
	           FROM OBJECTID WHERE table_name = table_name_i);
      returnOid := -1;

    ELSIF(row_count_v &lt; 0) THEN
      RAISE EXCEPTION 'Unable to find OID. Error during the select!';

    END IF;

    BEGIN
      IF(returnOid != -1) THEN
        UPDATE OBJECTID
        SET NEXT = returnOid + OID_INCREMENT_VALUE
        WHERE table_name = table_name_i
        AND NEXT = returnOid;
      
        GET DIAGNOSTICS row_count_v = ROW_COUNT;
  
        IF (row_count_v = 1) THEN
          is_not_ok := FALSE;
        END IF;
      END IF;

    EXCEPTION
      WHEN OTHERS THEN
      is_not_ok := TRUE;
    END;
  
    i := i + 1;
  END LOOP;

  IF(is_not_ok) THEN
     RAISE EXCEPTION 'Unable to find OID!';
  END IF;

  RETURN returnOid;

END;
$$
  LANGUAGE 'plpgsql' VOLATILE;
  

</xsl:template>

</xsl:stylesheet>
