/*
 * jsrsClientServerHTTPLib.js - javascript library to make make asynchronous
 * remote calls to server without client page refresh (based off Brent
 * Ashley's [jsrs@megahuge.com] jsrsClient.js, v.2.13).
 * See http://www.ashleyit.com/rs/jsrs/ for details.
 *
 * see jsrsLicense.txt for copyright and license information
 *
 * changes:
 * 08/20/2001 - revamped to interact with the Barracuda ParamGateway servlet; the
 *      script submits a form to the servlet, which saves all the form values in
 *      session state, and then returns a url to the client (which can then do
 *      a location.replace on that url).
 */

// callback pool needs global scope
var jsrsContextPoolSize = 0;
var jsrsContextMaxPool = 10;
var jsrsContextPool = new Array();
var jsrsBrowser = jsrsBrowserSniff();
var jsrsPOST = true;
var jsrsOnsubmitInvoked = false;                //csc_032105_1 - TODO: technically, this really is not quite right; we ought to be storing these values on a per-targetForm basis (like in a hashmap or something), but I'm not sure how to do that in javascript - so this will probably not work if we have multiple forms on a single page where they can be submitted without causing the entire page to reload
var jsrsSubmitted = false;                      //csc_032105_1

//--------------------- Public functions ----------------------------
/**
 * Used to submit a form without causing a screen refresh
 */
function cshl_SubmitAndReplace(targetForm) {
    if (jsrsSubmitted) return true;             //csc_032105_1
    cshl_InvokeOnsubmit(targetForm);            //csc_032105_1
    jsrsSubmit(targetForm, myCallback, false);
    jsrsSubmitted = true;                       //csc_032105_1
}

//csc_032105_1 - added
/**
 * Used to invoke the onsubmit handler - returning true means its ok to continue
 * No matter how many times you invoke this method within a page, it will not be
 * called again once it has sucessfully returned true (ie. data validated, etc).
 * If you want to call it more than that, you will need to programmatically reset
 * the form using cshl_Reset()
 */
function cshl_InvokeOnsubmit(targetForm) {
    if (jsrsOnsubmitInvoked) return true;                               //this is here to make sure we only call this function once
    var result = (targetForm.onsubmit) ? args[0].onsubmit() : true;     //figure out the result
    jsrsOnsubmitInvoked = result;                                       //note that by setting the status to the result, we ensure that if the onsubmit() fails (ie. invalid data), repeated calls will still invoke the handler until it gets a success status (after which you have to manually reset to reinvoke)
    return result;
}

//csc_032105_1 - added
/**
 * You can use this to programatically "reset" the form (so it can be submitted again)
 */
function cshl_Reset(targetForm) {
    jsrsOnsubmitInvoked = false;
    jsrsSubmitted = false;
    return true;
}

//--------------------- Utility functions ---------------------------
/**
 * callback function that receives the resulting redirect URL and replaces the
 * URL accordingly
 */
function myCallback(result) {
    location.replace(result+'?$csjs=true'); //&$u='+generateUniqueString());  //adding script stuff prevents an extra round trip to the server
    //location.replace(result);
}

// constructor for context object
function jsrsContextObj( contextID ){

  // properties
  this.id = contextID;
  this.busy = true;
  this.callback = null;
  this.container = contextCreateContainer( contextID );

  // methods
  this.GET = contextGET;
  this.POST = contextPOST;
  this.getPayload = contextGetPayload;
  this.setVisibility = contextSetVisibility;
}

//  method functions are not privately scoped
//  because Netscape's debugger chokes on private functions
function contextCreateContainer( containerName ){
  // creates hidden container to receive server data
  var container;
  switch( jsrsBrowser ) {
    case 'NS':
      container = new Layer(100);
      container.name = containerName;
      container.visibility = 'hide';
      container.clip.width = 100;
      container.clip.height = 100;
      break;

    case 'IE':
      document.body.insertAdjacentHTML("afterBegin", '<div id="DIV' + containerName + '"></div>');
      var div = document.all("DIV" + containerName);
      var html = '<iframe name="' + containerName + '" src=""></iframe>';
      div.innerHTML = html;
      div.style.display = 'none';
      container = window.frames[ containerName ];
      break;

    case 'MOZ':
      var div = document.createElement('DIV');
      div.id = "DIV" + containerName;
      document.body.appendChild(div);
      var iframe = document.createElement('IFRAME');
      iframe.name = containerName;
      div.appendChild(iframe);
      div.style.display = 'none';
      container = iframe;
      break;
  }
  return container;
}

function contextPOST(sourceForm, vis) {
    //preliminary stuff
    var unique = generateUniqueString();
    var doc = window.frames[this.container.name].document; //formerly (jsrsBrowser == "IE" ) ? this.container.document : this.container.contentDocument;, but window.frames works for both
    var sep = "?";
    if (sourceForm.action.indexOf(sep)>-1) sep = "&amp;"; //formerly '&', but it is illegal in html...must use special character (jrk_120201.10:51)

    //start building the form
    doc.open();
    doc.writeln('<html><head><title></title></head><body>');
    doc.writeln('<form name="jsrsForm" method="post" action="'+sourceForm.action+sep+'U='+unique+'"><div>');
    doc.writeln('<input type="hidden" name="pm_cid" value="' + this.id + '">');

    // add in form's input elements
    for (var i=0; i<sourceForm.elements.length; i++) {
        var element = sourceForm.elements[i];

        //note that we skip buttons...this is because these are not really
        //needed for the post information
        if ((element.type=="submit") ||
            (element.type=="reset") ||
            (element.type=="button")) continue;

        //select controls
        if ((element.type=="select-one") || (element.type=="select-multiple")) {
            if (element.type=="select-one") doc.write('<select ');
            else doc.write('<select multiple ');
            doc.writeln('name="'+element.name+'">');

            var options = element.options;
            for (var j=0; j<options.length; j++) {
                var opt = options[j];
                doc.write('<option value="'+opt.value+'" ');
                if (opt.selected) doc.write('selected="selected" ');
                doc.writeln('>'+opt.text+'</option>');
            }
            doc.writeln('</select>');

        //everything else
        } else {
            doc.write('<input ');
            doc.write('type="'+element.type+'" ');
            doc.write('name="'+element.name+'" ');
            doc.write('value="'+element.value+'" ');
            if ((element.type=="checkbox") || (element.type=="radio")) {
                if (element.checked) doc.write('checked="checked" ');
            }
            doc.writeln('>');
        }
    }

    //finish up
    doc.writeln('<input type="submit" value="Finish Submit!" name="jsrsSubmit">');
    doc.writeln('</div></form>');
    doc.writeln('</body></html>');
    doc.close();
    if (!vis) doc.forms['jsrsForm'].submit();
}

function contextGET(sourceForm) {
    //preliminary stuff
    var unique = generateUniqueString();
    var sep = "?";
//csc_031705_1    if (sourceForm.action.indexOf(sep)>-1) sep = "&";
    if (sourceForm.action.indexOf(sep)>-1) sep = "&amp;"; //csc_031705_1 - copied from the post section up above

    var URL = sourceForm.action+sep+'U='+unique;

    //add in the context id
    URL += '&pm_cid='+this.id;

    // add in form's input elements
    for (var i=0; i<sourceForm.elements.length; i++) {
        var element = sourceForm.elements[i];

        //note that we skip buttons...this is because these are not really
        //needed for the post information
        if ((element.type=="submit") ||
            (element.type=="reset") ||
            (element.type=="button")) continue;

        //select controls
        if ((element.type=="select-one") || (element.type=="select-multiple")) {
            if (element.selectedIndex==-1) continue;
            var options = element.options;
            for (var j=0; j<options.length; j++) {
                var opt = options[j];
                if (opt.selected) URL += '&'+element.name+'='+opt.value;
            }

        //everything else
        } else if ((element.type=="checkbox") || (element.type=="radio")) {
            if (element.checked) URL += '&'+element.name+'='+element.value;
        } else {
            URL += '&'+element.name+'='+element.value;
        }
    }

  // make the call
  switch( jsrsBrowser ) {
    case 'NS':
      this.container.src = URL;
      break;
    case 'IE':
      this.container.document.location.replace(URL);
      break;
    case 'MOZ':
      this.container.src = '';
      this.container.src = URL;
      break;
  }

}

function contextGetPayload(){
  switch( jsrsBrowser ) {
    case 'NS':
      return this.container.document.forms['jsrs_Form'].elements['jsrs_Payload'].value;
      break;
    case 'IE':
      return this.container.document.forms['jsrs_Form']['jsrs_Payload'].value;
      break;
    case 'MOZ':
      return window.frames[this.container.name].document.forms['jsrs_Form']['jsrs_Payload'].value;
      break;
  }
}

function contextSetVisibility( vis ){
  switch( jsrsBrowser ) {
    case 'NS':
      this.container.visibility = (vis)? 'show' : 'hide';
      break;
    case 'IE':
      document.all("DIV" + this.id).style.display = (vis)? 'block' : 'none';
      break;
    case 'MOZ':
      //NOTE: Mozilla now supports form submission from inside blocks set to display:none;. Should be available as of Netscape6.2, but leaving use of "visibility" for backward compatibility
      document.getElementById("DIV" + this.id).style.visibility = (vis)? 'visible' : 'hidden';
      this.container.width = (vis)? '250px' : 0;
      this.container.height = (vis)? '100px' : 0;
      break;
  }
}

// end of context constructor

function jsrsGetContextID(){
  var contextObj;
  for (var i = 1; i <= jsrsContextPoolSize; i++){
    contextObj = jsrsContextPool[ 'jsrs' + i ];
    if ( !contextObj.busy ){
      contextObj.busy = true;
      return contextObj.id;
    }
  }
  // if we got here, there are no existing free contexts
  if ( jsrsContextPoolSize <= jsrsContextMaxPool ){
    // create new context
    var contextID = "jsrs" + (jsrsContextPoolSize + 1);
    jsrsContextPool[ contextID ] = new jsrsContextObj( contextID );
    jsrsContextPoolSize++;
    return contextID;
  } else {
    alert( "jsrs Error:  context pool full" );
    return null;
  }
}

function jsrsSubmit(sourceForm, callback, visibility) {
 // submit a form to the server from client code without causing a new page
 // to be loaded
 //
 // sourceForm  - the sourceForm
 // callback    - function to call on return
 //               or null if no return needed
 //               (passes returned string to callback)
 // visibility  - optional boolean to make container visible for debugging

  // get context
  var contextObj = jsrsContextPool[ jsrsGetContextID() ];
  contextObj.callback = callback;

  var vis = (visibility == null)? false : visibility;
  contextObj.setVisibility( vis );


  if ( jsrsPOST && ((jsrsBrowser == 'IE') || (jsrsBrowser == 'MOZ'))){
    contextObj.POST(sourceForm, vis);
  } else {
    contextObj.GET(sourceForm);
  }

  return contextObj.id;
}

function jsrsLoaded( contextID ){
  // get context object and invoke callback
  var contextObj = jsrsContextPool[ contextID ];
  if( contextObj.callback != null){
    contextObj.callback( jsrsUnescape( contextObj.getPayload() ), contextID );
  }
  // clean up and return context to pool
  contextObj.callback = null;
  contextObj.busy = false;
}

function jsrsError( contextID, str ){
  alert( unescape(str) );
  jsrsContextPool[ contextID ].busy = false
}

function jsrsEscapeQQ( thing ){
  return thing.replace(/'"'/g, '\\"');
}

function jsrsUnescape( str ){
  // payload has slashes escaped with whacks
  return str.replace( /\\\//g, "/" );
}

function jsrsBrowserSniff(){
  if (document.layers) return "NS";
  if (document.all) return "IE";
  if (document.getElementById) return "MOZ";
  return "OTHER";
}

/////////////////////////////////////////////////
//
// user functions

function jsrsArrayFromString( s, delim ){
  // rebuild an array returned from server as string
  // optional delimiter defaults to ~
  var d = (delim == null)? '~' : delim;
  return s.split(d);
}

function jsrsDebugInfo(){
  // use for debugging by attaching to f1 (works with IE)
  // with onHelp = "return jsrsDebugInfo();" in the body tag
  var doc = window.open().document;
  doc.open;
  doc.write('<div id="DebugInfo">Pool Size: ' + jsrsContextPoolSize + '<div style="font:bold .8em Arial,Helvetica,sans-serif;">');
  for( var i in jsrsContextPool ){
    var contextObj = jsrsContextPool[i];
    doc.write('<hr><div>' + contextObj.id + ' : ' + (contextObj.busy ? 'busy' : 'available') + '</div>');
    doc.write('<div>' + contextObj.container.document.location.pathname + '</div>');
    doc.write('<div>' + contextObj.container.document.location.search + '</div>');
    doc.write('<div style="border:1px solid black;">' + contextObj.container.document.body.innerHTML + '</div>');
  }
  doc.write('</div></div>');
  doc.close();
  return false;
}

function generateUniqueString() {
    var d = new Date();
    return d.getTime() + '' + Math.floor(1000 * Math.random());
}
